@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -197,7 +201,8 @@ ov0SetRegion( CoreLayer                  *layer,
               CoreLayerRegionConfigFlags  updated,
               CoreSurface                *surface,
               CorePalette                *palette,
-              CoreSurfaceBufferLock      *lock )
+              CoreSurfaceBufferLock      *left_lock,
+              CoreSurfaceBufferLock      *right_lock )
 {
      ATI128DriverData    *adrv = (ATI128DriverData*) driver_data;
      ATIOverlayLayerData *aov0 = (ATIOverlayLayerData*) layer_data;
@@ -205,7 +210,7 @@ ov0SetRegion( CoreLayer                  *layer,
      /* remember configuration */
      aov0->config = *config;
 
-     ov0_calc_regs( adrv, aov0, config, surface, lock );
+     ov0_calc_regs( adrv, aov0, config, surface, left_lock );
      ov0_set_regs( adrv, aov0 );
 
      /* enable overlay */
@@ -236,14 +241,17 @@ ov0FlipRegion( CoreLayer             *layer,
                void                  *region_data,
                CoreSurface           *surface,
                DFBSurfaceFlipFlags    flags,
-               CoreSurfaceBufferLock *lock )
+               const DFBRegion       *left_update,
+               CoreSurfaceBufferLock *left_lock,
+               const DFBRegion       *right_update,
+               CoreSurfaceBufferLock *right_lock )
 {
      ATI128DriverData    *adrv = (ATI128DriverData*) driver_data;
      ATIOverlayLayerData *aov0 = (ATIOverlayLayerData*) layer_data;
 
      dfb_surface_flip( surface, false );
 
-     ov0_calc_regs( adrv, aov0, &aov0->config, surface, lock );
+     ov0_calc_regs( adrv, aov0, &aov0->config, surface, left_lock );
      ov0_set_regs( adrv, aov0 );
 
      return DFB_OK;
