@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <dfb_types.h>
@@ -1556,6 +1560,8 @@ driver_init_device( CoreGraphicsDevice *device,
 
      /* 3D Rage Pro is the first chip that supports auto fast fill/block write. */
      if (mdev->chip >= CHIP_3D_RAGE_PRO) {
+          u32 cfg_mem_type = mach64_in32( mmio, CONFIG_STAT0 ) & CFG_MEM_TYPE;
+
           mdev->hw_debug = mach64_in32( mmio, HW_DEBUG );
 
           /* Save original HW_DEBUG. */
@@ -1564,7 +1570,8 @@ driver_init_device( CoreGraphicsDevice *device,
           /* Enable auto fast fill and fast fill/block write scissoring. */
           mdev->hw_debug &= ~(AUTO_FF_DIS | INTER_PRIM_DIS);
 
-          if ((mach64_in32( mmio, CONFIG_STAT0 ) & CFG_MEM_TYPE) == CFG_MEM_TYPE_SGRAM) {
+          if (cfg_mem_type == CFG_MEM_TYPE_SGRAM ||
+              (mdev->chip == CHIP_3D_RAGE_PRO && cfg_mem_type == CFG_MEM_TYPE_WRAM)) {
                /* Enable auto block write and auto color register updates. */
                mdev->hw_debug &= ~(AUTO_BLKWRT_DIS | AUTO_BLKWRT_COLOR_DIS);
 
