@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -257,7 +261,8 @@ ovSetRegion( CoreLayer                  *layer,
              CoreLayerRegionConfigFlags  updated,
              CoreSurface                *surface,
              CorePalette                *palette,
-             CoreSurfaceBufferLock      *lock )
+             CoreSurfaceBufferLock      *left_lock,
+             CoreSurfaceBufferLock      *right_lock )
 {
      Mach64DriverData       *mdrv = (Mach64DriverData*) driver_data;
      Mach64OverlayLayerData *mov  = (Mach64OverlayLayerData*) layer_data;
@@ -269,8 +274,8 @@ ovSetRegion( CoreLayer                  *layer,
           ov_reset( mdrv );
 
      if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT | CLRCF_SOURCE | CLRCF_DEST | CLRCF_OPTIONS)) {
-          ov_calc_buffer( mdrv, mov, config, surface, lock );
-          ov_calc_regs( mdrv, mov, config, surface, lock );
+          ov_calc_buffer( mdrv, mov, config, surface, left_lock );
+          ov_calc_regs( mdrv, mov, config, surface, left_lock );
           ov_set_buffer( mdrv, mov );
           ov_set_regs( mdrv, mov );
      }
@@ -322,12 +327,16 @@ ovFlipRegion( CoreLayer             *layer,
               void                  *region_data,
               CoreSurface           *surface,
               DFBSurfaceFlipFlags    flags,
-              CoreSurfaceBufferLock *lock )
+              const DFBRegion       *left_update,
+              CoreSurfaceBufferLock *left_lock,
+              const DFBRegion       *right_update,
+              CoreSurfaceBufferLock *right_lock )
+
 {
      Mach64DriverData       *mdrv = (Mach64DriverData*) driver_data;
      Mach64OverlayLayerData *mov  = (Mach64OverlayLayerData*) layer_data;
 
-     ov_calc_buffer( mdrv, mov, &mov->config, surface, lock );
+     ov_calc_buffer( mdrv, mov, &mov->config, surface, left_lock );
      ov_set_buffer( mdrv, mov );
 
      dfb_surface_flip( surface, false );
