@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -179,7 +183,8 @@ spicSetRegion( CoreLayer                  *layer,
                CoreLayerRegionConfigFlags  updated,
                CoreSurface                *surface,
                CorePalette                *palette,
-               CoreSurfaceBufferLock      *lock )
+               CoreSurfaceBufferLock      *left_lock,
+               CoreSurfaceBufferLock      *right_lock )
 {
      MatroxDriverData    *mdrv  = (MatroxDriverData*) driver_data;
      MatroxSpicLayerData *mspic = (MatroxSpicLayerData*) layer_data;
@@ -206,7 +211,7 @@ spicSetRegion( CoreLayer                  *layer,
 
      if (updated & (CLRCF_WIDTH | CLRCF_HEIGHT | CLRCF_FORMAT | CLRCF_SURFACE_CAPS |
                     CLRCF_OPTIONS | CLRCF_OPACITY | CLRCF_SURFACE)) {
-          spic_calc_buffer( mdrv, mspic, surface, lock );
+          spic_calc_buffer( mdrv, mspic, surface, left_lock );
           spic_set_buffer( mdrv, mspic );
 
           mspic->regs.c2DATACTL = mga_in32( mmio, C2DATACTL );
@@ -261,12 +266,15 @@ spicFlipRegion( CoreLayer             *layer,
                 void                  *region_data,
                 CoreSurface           *surface,
                 DFBSurfaceFlipFlags    flags,
-                CoreSurfaceBufferLock *lock )
+                const DFBRegion       *left_update,
+                CoreSurfaceBufferLock *left_lock,
+                const DFBRegion       *right_update,
+                CoreSurfaceBufferLock *right_lock )
 {
      MatroxDriverData    *mdrv  = (MatroxDriverData*) driver_data;
      MatroxSpicLayerData *mspic = (MatroxSpicLayerData*) layer_data;
 
-     spic_calc_buffer( mdrv, mspic, surface, lock );
+     spic_calc_buffer( mdrv, mspic, surface, left_lock );
      spic_set_buffer( mdrv, mspic );
 
      dfb_surface_flip( surface, false );
