@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __NVIDIA_ACCEL_H__
 #define __NVIDIA_ACCEL_H__
 
@@ -149,10 +153,8 @@ static inline void
 nv_emitdma( NVidiaDriverData *nvdrv, NVidiaDeviceData *nvdev )
 {
      if (nvdev->dma_put != nvdev->dma_cur) {
-          volatile u8 scratch;
-          
           /* flush MTRR buffers */
-          scratch = nv_in8( nvdrv->fb_base, 0 );
+          nv_in8( nvdrv->fb_base, 0 );
           nv_out32( nvdrv->mmio_base, DMA_PUT, nvdev->dma_cur << 2 );
           
           nvdev->dma_put = nvdev->dma_cur;
