@@ -34,7 +34,7 @@
 
 #include <core/state.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <gfx/convert.h>
 
@@ -77,7 +77,7 @@ void r200_restore( RadeonDriverData *rdrv, RadeonDeviceData *rdev )
 {
      volatile u8 *mmio = rdrv->mmio_base;
      
-     radeon_waitfifo( rdrv, rdev, 19 );
+     radeon_waitfifo( rdrv, rdev, 15 );
      /* enable caches */
      radeon_out32( mmio, RB2D_DSTCACHE_MODE, RB2D_DC_2D_CACHE_AUTOFLUSH     |
                                              RB2D_DC_3D_CACHE_AUTOFLUSH     |
@@ -105,32 +105,27 @@ void r200_restore( RadeonDriverData *rdrv, RadeonDeviceData *rdev )
      radeon_out32( mmio, RB3D_ZSTENCILCNTL, Z_TEST_ALWAYS );
      radeon_out32( mmio, RB3D_ROPCNTL, ROP_XOR );
      radeon_out32( mmio, R200_PP_TXFORMAT_X_0, 0 );
-     radeon_out32( mmio, R200_PP_BORDER_COLOR_0, 0 );
-     /* set YUV422 color buffer */
-     radeon_out32( mmio, R200_PP_TXFILTER_1, 0 );
-     radeon_out32( mmio, R200_PP_TXFORMAT_1, R200_TXFORMAT_VYUY422 );
      radeon_out32( mmio, R200_PP_TXFORMAT_X_1, 0 );
-     radeon_out32( mmio, R200_PP_BORDER_COLOR_1, 0 );
 }
 
 void r200_set_destination( RadeonDriverData *rdrv,
                            RadeonDeviceData *rdev,
                            CardState        *state )
 {
-     CoreSurface   *surface = state->destination;
-     SurfaceBuffer *buffer  = surface->back_buffer;
-     volatile u8   *mmio    = rdrv->mmio_base;
-     u32            offset;
-     u32            pitch;
+     CoreSurface       *surface = state->destination;
+     CoreSurfaceBuffer *buffer  = state->dst.buffer;
+     volatile u8       *mmio    = rdrv->mmio_base;
+     u32                offset;
+     u32                pitch;
     
      if (RADEON_IS_SET( DESTINATION ))
           return;
      
-     D_ASSERT( (buffer->video.offset % 32) == 0 );
-     D_ASSERT( (buffer->video.pitch % 32) == 0 );
+     D_ASSERT( (state->dst.offset % 32) == 0 );
+     D_ASSERT( (state->dst.pitch % 32) == 0 );
 
-     offset = radeon_buffer_offset( rdev, buffer );
-     pitch  = buffer->video.pitch;
+     offset = radeon_buffer_offset( rdev, &state->dst );
+     pitch  = state->dst.pitch;
     
      if (rdev->dst_offset != offset        ||
          rdev->dst_pitch  != pitch         ||
@@ -187,16 +182,16 @@ void r200_set_destination( RadeonDriverData *rdrv,
                case DSPF_I420:
                     rdev->gui_master_cntl = GMC_DST_8BPP;
                     rdev->rb3d_cntl = COLOR_FORMAT_RGB8;
-                    rdev->dst_offset_cb = offset + pitch * surface->height;
+                    rdev->dst_offset_cb = offset + pitch * surface->config.size.h;
                     rdev->dst_offset_cr = rdev->dst_offset_cb + 
-                                          pitch/2 * surface->height/2;
+                                          pitch/2 * surface->config.size.h/2;
                     break;
                case DSPF_YV12:
                     rdev->gui_master_cntl = GMC_DST_8BPP;
                     rdev->rb3d_cntl = COLOR_FORMAT_RGB8;
-                    rdev->dst_offset_cr = offset + pitch * surface->height;
+                    rdev->dst_offset_cr = offset + pitch * surface->config.size.h;
                     rdev->dst_offset_cb = rdev->dst_offset_cr +
-                                          pitch/2 * surface->height/2;
+                                          pitch/2 * surface->config.size.h/2;
                     break;
                default:
                     D_BUG( "unexpected pixelformat" );
@@ -239,14 +234,14 @@ void r200_set_source( RadeonDriverData *rdrv,
                       RadeonDeviceData *rdev,
                       CardState        *state )
 {
-     CoreSurface   *surface  = state->source;
-     SurfaceBuffer *buffer   = surface->front_buffer;
-     volatile u8   *mmio     = rdrv->mmio_base;
-     u32            txformat = R200_TXFORMAT_NON_POWER2;
-     u32            txfilter = R200_MAG_FILTER_LINEAR  |
-                               R200_MIN_FILTER_LINEAR  |
-                               R200_CLAMP_S_CLAMP_LAST |
-                               R200_CLAMP_T_CLAMP_LAST;
+     CoreSurface       *surface  = state->source;
+     CoreSurfaceBuffer *buffer   = state->src.buffer;
+     volatile u8       *mmio     = rdrv->mmio_base;
+     u32                txformat = R200_TXFORMAT_NON_POWER2;
+     u32                txfilter = R200_MAG_FILTER_LINEAR  |
+                                   R200_MIN_FILTER_LINEAR  |
+                                   R200_CLAMP_S_CLAMP_LAST |
+                                   R200_CLAMP_T_CLAMP_LAST;
      
      if (RADEON_IS_SET( SOURCE )) {
           if ((state->blittingflags & DSBLIT_DEINTERLACE) ==
@@ -254,13 +249,13 @@ void r200_set_source( RadeonDriverData *rdrv,
                return;
      }
 
-     D_ASSERT( (buffer->video.offset % 32) == 0 );
-     D_ASSERT( (buffer->video.pitch % 32) == 0 );
+     D_ASSERT( (state->src.offset % 32) == 0 );
+     D_ASSERT( (state->src.pitch % 32) == 0 );
      
-     rdev->src_offset = radeon_buffer_offset( rdev, buffer );
-     rdev->src_pitch  = buffer->video.pitch;
-     rdev->src_width  = surface->width;
-     rdev->src_height = surface->height;
+     rdev->src_offset = radeon_buffer_offset( rdev, &state->src );
+     rdev->src_pitch  = state->src.pitch;
+     rdev->src_width  = surface->config.size.w;
+     rdev->src_height = surface->config.size.h;
 
      switch (buffer->format) {
           case DSPF_LUT8:
@@ -358,7 +353,7 @@ void r200_set_source( RadeonDriverData *rdrv,
 
      if (state->blittingflags & DSBLIT_DEINTERLACE) { 
           rdev->src_height /= 2;
-          if (surface->caps & DSCAPS_SEPARATED) {
+          if (surface->config.caps & DSCAPS_SEPARATED) {
                if (surface->field) {
                     rdev->src_offset    += rdev->src_height * rdev->src_pitch;
                     rdev->src_offset_cr += rdev->src_height * rdev->src_pitch/4;
@@ -383,7 +378,7 @@ void r200_set_source( RadeonDriverData *rdrv,
                                            ((rdev->src_width-1) & 0xffff) );
      radeon_out32( mmio, R200_PP_TXPITCH_0, rdev->src_pitch - 32 );
      radeon_out32( mmio, R200_PP_TXOFFSET_0, rdev->src_offset );
-     
+
      if (rdev->src_format != buffer->format)
           RADEON_UNSET( BLITTING_FLAGS );
      rdev->src_format = buffer->format;
@@ -391,6 +386,95 @@ void r200_set_source( RadeonDriverData *rdrv,
      RADEON_SET( SOURCE );
 }
 
+void r200_set_source_mask( RadeonDriverData *rdrv,
+                           RadeonDeviceData *rdev,
+                           CardState        *state )
+{
+     CoreSurface       *surface  = state->source_mask;
+     CoreSurfaceBuffer *buffer   = state->src_mask.buffer;
+     volatile u8       *mmio     = rdrv->mmio_base;
+     u32                txformat = R200_TXFORMAT_NON_POWER2;
+     u32                txfilter = R200_MAG_FILTER_LINEAR  |
+                                   R200_MIN_FILTER_LINEAR  |
+                                   R200_CLAMP_S_CLAMP_LAST |
+                                   R200_CLAMP_T_CLAMP_LAST;
+     
+     if (RADEON_IS_SET( SOURCE_MASK )) {
+          if ((state->blittingflags & DSBLIT_DEINTERLACE) ==
+              (rdev->blittingflags  & DSBLIT_DEINTERLACE))
+               return;
+     }
+
+     D_ASSERT( (state->src_mask.offset % 32) == 0 );
+     D_ASSERT( (state->src_mask.pitch % 32) == 0 );
+     
+     rdev->msk_format = buffer->format;
+     rdev->msk_offset = radeon_buffer_offset( rdev, &state->src_mask );
+     rdev->msk_pitch  = state->src_mask.pitch;
+     rdev->msk_width  = surface->config.size.w;
+     rdev->msk_height = surface->config.size.h;
+
+     switch (buffer->format) {
+          case DSPF_A8:
+               txformat |= R200_TXFORMAT_I8 |
+                           R200_TXFORMAT_ALPHA_IN_MAP;
+               break;
+          case DSPF_RGB332:
+               txformat |= R200_TXFORMAT_RGB332;
+               break;
+          case DSPF_RGB444:
+               txformat |= R200_TXFORMAT_ARGB4444;
+               break;     
+          case DSPF_ARGB4444:
+               txformat |= R200_TXFORMAT_ARGB4444 |
+                           R200_TXFORMAT_ALPHA_IN_MAP;
+               break;
+          case DSPF_RGB555:
+               txformat |= R200_TXFORMAT_ARGB1555;
+               break; 
+          case DSPF_ARGB1555:
+               txformat |= R200_TXFORMAT_ARGB1555 |
+                           R200_TXFORMAT_ALPHA_IN_MAP;
+               break;
+          case DSPF_RGB16:
+               txformat |= R200_TXFORMAT_RGB565;
+               break;
+          case DSPF_RGB32:
+               txformat |= R200_TXFORMAT_ARGB8888;
+               break;
+          case DSPF_ARGB:
+          case DSPF_AiRGB:
+               txformat |= R200_TXFORMAT_ARGB8888 |
+                           R200_TXFORMAT_ALPHA_IN_MAP;
+               break;
+          default:
+               D_BUG( "unexpected pixelformat" );
+               return;
+     }
+
+     if (state->blittingflags & DSBLIT_DEINTERLACE) { 
+          rdev->msk_height /= 2;
+          if (surface->config.caps & DSCAPS_SEPARATED) {
+               if (surface->field)
+                    rdev->msk_offset += rdev->msk_height * rdev->msk_pitch;
+          } else {
+               if (surface->field)
+                    rdev->msk_offset += rdev->msk_pitch;
+               rdev->msk_pitch *= 2;
+          }
+     }
+
+     radeon_waitfifo( rdrv, rdev, 5 );
+     radeon_out32( mmio, R200_PP_TXFILTER_1, txfilter );
+     radeon_out32( mmio, R200_PP_TXFORMAT_1, txformat );
+     radeon_out32( mmio, R200_PP_TXSIZE_1, ((rdev->msk_height-1) << 16) | 
+                                           ((rdev->msk_width-1) & 0xffff) );
+     radeon_out32( mmio, R200_PP_TXPITCH_1, rdev->msk_pitch - 32 );
+     radeon_out32( mmio, R200_PP_TXOFFSET_1, rdev->msk_offset );
+
+     RADEON_SET( SOURCE_MASK );
+}
+
 void r200_set_clip( RadeonDriverData *rdrv,
                     RadeonDeviceData *rdev,
                     CardState        *state )
@@ -431,9 +515,12 @@ void r200_set_clip( RadeonDriverData *rdrv,
      radeon_out32( (rdrv)->fb_base, (rdev)->yuv422_buffer,      \
                    PIXEL_YUY2( y, u, v ) );                     \
      radeon_in8( (rdrv)->fb_base, (rdev)->yuv422_buffer );      \
-     radeon_waitfifo( rdrv, rdev, 1 );                          \
+     radeon_waitfifo( rdrv, rdev, 3 );                          \
      radeon_out32( (rdrv)->mmio_base, R200_PP_TXOFFSET_1,       \
                    (rdev)->fb_offset + (rdev)->yuv422_buffer ); \
+     radeon_out32( (rdrv)->mmio_base, R200_PP_TXFORMAT_1,       \
+                   R200_TXFORMAT_VYUY422 );                     \
+     radeon_out32( (rdrv)->mmio_base, R200_PP_TXFILTER_1, 0 );  \
 }
 
 void r200_set_drawing_color( RadeonDriverData *rdrv,
@@ -467,7 +554,7 @@ void r200_set_drawing_color( RadeonDriverData *rdrv,
                break;
           case DSPF_A8:
                color2d = color.a;
-               color3d = (color.a << 24) | 0x00ffffff;
+               color3d = (color.a << 24) | 0x00ffffff;              
                break;
           case DSPF_RGB332:
                color2d = PIXEL_RGB332( color.r, color.g, color.b );
@@ -548,9 +635,9 @@ void r200_set_blitting_color( RadeonDriverData *rdrv,
 
      if (state->blittingflags & DSBLIT_COLORIZE &&
          state->blittingflags & DSBLIT_SRC_PREMULTCOLOR) {
-          color.r = ((long) color.r * color.a / 255L);
-          color.g = ((long) color.g * color.a / 255L);
-          color.b = ((long) color.b * color.a / 255L);
+          color.r = color.r * color.a / 255;
+          color.g = color.g * color.a / 255;
+          color.b = color.b * color.a / 255;
      }
 
      switch (rdev->dst_format) {
@@ -626,7 +713,18 @@ void r200_set_blend_function( RadeonDriverData *rdrv,
           else if (dblend == DST_BLEND_GL_ONE_MINUS_DST_ALPHA)
                dblend = DST_BLEND_GL_ZERO;
      }
+     else if (rdev->dst_format == DSPF_A8) {
+          if (sblend == SRC_BLEND_GL_DST_ALPHA)
+               sblend = SRC_BLEND_GL_DST_COLOR;
+          else if (sblend == SRC_BLEND_GL_ONE_MINUS_DST_ALPHA)
+               sblend = SRC_BLEND_GL_ONE_MINUS_DST_COLOR;
 
+          if (dblend == DST_BLEND_GL_DST_ALPHA)
+               dblend = DST_BLEND_GL_DST_COLOR;
+          else if (dblend == DST_BLEND_GL_ONE_MINUS_DST_ALPHA)
+               dblend = DST_BLEND_GL_ONE_MINUS_DST_COLOR;
+     }
+     
      radeon_waitfifo( rdrv, rdev, 1 ); 
      radeon_out32( mmio, RB3D_BLENDCNTL, sblend | dblend );
      
@@ -634,6 +732,33 @@ void r200_set_blend_function( RadeonDriverData *rdrv,
      RADEON_SET( DST_BLEND );
 }
 
+void r200_set_render_options( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev,
+                              CardState        *state )
+{
+     if (RADEON_IS_SET( RENDER_OPTIONS ))
+          return;
+          
+     if (state->render_options & DSRO_MATRIX &&
+        (!state->affine_matrix ||
+          state->matrix[0] != (1<<16) || state->matrix[1] != 0 || state->matrix[2] != 0 ||
+          state->matrix[3] != 0 || state->matrix[4] != (1<<16) || state->matrix[5] != 0)) {
+          rdev->matrix = state->matrix;
+          rdev->affine_matrix = state->affine_matrix;
+     }
+     else {
+          rdev->matrix = NULL;
+     }
+     
+     if ((rdev->render_options & DSRO_ANTIALIAS) != (state->render_options & DSRO_ANTIALIAS)) {
+          RADEON_UNSET( DRAWING_FLAGS );
+          RADEON_UNSET( BLITTING_FLAGS );
+     }     
+     rdev->render_options = state->render_options;
+     
+     RADEON_SET( RENDER_OPTIONS );
+}
+
 /* NOTES:
  * - We use texture unit 0 for blitting functions,
  *          texture unit 1 for drawing functions
@@ -657,22 +782,26 @@ void r200_set_drawingflags( RadeonDriverData *rdrv,
           return;
 
      if (rdev->dst_422) {
-          pp_cntl     |= TEX_1_ENABLE;
-          cblend       = R200_TXC_ARG_C_R1_COLOR;
-     }
-     
-     if (state->drawingflags & DSDRAW_BLEND) {
-          rb3d_cntl   |= ALPHA_BLEND_ENABLE;
+          pp_cntl |= TEX_1_ENABLE;
+          cblend   = R200_TXC_ARG_C_R1_COLOR;
      }
      else if (rdev->dst_format == DSPF_A8) {
-          cblend       = R200_TXC_ARG_C_TFACTOR_ALPHA;
+          cblend = R200_TXC_ARG_C_TFACTOR_ALPHA;
      }
-
+     
+     if (state->drawingflags & DSDRAW_BLEND)
+          rb3d_cntl |= ALPHA_BLEND_ENABLE;
+     
      if (state->drawingflags & DSDRAW_XOR) {
           rb3d_cntl   |= ROP_ENABLE;
           master_cntl |= GMC_ROP3_PATXOR;
-     } else
+     }
+     else {
           master_cntl |= GMC_ROP3_PATCOPY;
+     }
+     
+     if (state->render_options & DSRO_ANTIALIAS)
+          pp_cntl |= ANTI_ALIAS_LINE_POLY;
      
      radeon_waitfifo( rdrv, rdev, 11 );
      radeon_out32( mmio, DP_GUI_MASTER_CNTL, master_cntl );
@@ -756,8 +885,17 @@ void r200_set_blittingflags( RadeonDriverData *rdrv,
           rb3d_cntl |= ALPHA_BLEND_ENABLE;
      }
 
-     if (rdev->dst_format != DSPF_A8) {    
-          if (state->blittingflags & DSBLIT_COLORIZE) {
+     if (rdev->dst_format != DSPF_A8) {
+          if (state->blittingflags & (DSBLIT_SRC_MASK_ALPHA | DSBLIT_SRC_MASK_COLOR)) {
+               if (state->blittingflags & DSBLIT_SRC_MASK_ALPHA)
+                    ablend = R200_TXA_ARG_A_R0_ALPHA | R200_TXA_ARG_B_R1_ALPHA;
+               
+               if (state->blittingflags & DSBLIT_SRC_MASK_COLOR)
+                    cblend = R200_TXC_ARG_A_R0_COLOR | R200_TXC_ARG_B_R1_COLOR;
+                    
+               pp_cntl |= TEX_1_ENABLE | TEX_BLEND_0_ENABLE;
+          }
+          else if (state->blittingflags & DSBLIT_COLORIZE) {
                if (rdev->dst_422) {
                     cblend = (rdev->src_format == DSPF_A8)
                              ? (R200_TXC_ARG_C_R1_COLOR)
@@ -780,13 +918,29 @@ void r200_set_blittingflags( RadeonDriverData *rdrv,
             
                pp_cntl |= TEX_BLEND_0_ENABLE;
           }
+          else if (state->blittingflags & DSBLIT_SRC_PREMULTIPLY) {
+               cblend = (rdev->src_format == DSPF_A8)
+                        ? (R200_TXC_ARG_C_R0_ALPHA)
+                        : (R200_TXC_ARG_A_R0_COLOR | R200_TXC_ARG_B_R0_ALPHA);
+            
+               pp_cntl |= TEX_BLEND_0_ENABLE;
+          }
      } /* DSPF_A8 */
      else {
-          if (state->blittingflags & (DSBLIT_BLEND_COLORALPHA |
-                                      DSBLIT_BLEND_ALPHACHANNEL))
-               cblend = R200_TXC_ARG_C_TFACTOR_COLOR;
-          else
+          if (state->blittingflags & DSBLIT_SRC_MASK_ALPHA) {
+               ablend = R200_TXA_ARG_A_R0_ALPHA | R200_TXA_ARG_B_R1_ALPHA;
+               cblend = R200_TXC_ARG_A_R0_ALPHA | R200_TXC_ARG_B_R1_ALPHA;
+               pp_cntl |= TEX_1_ENABLE;
+          }
+          else if (state->blittingflags & DSBLIT_BLEND_COLORALPHA) {
+               if (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
+                    cblend = R200_TXC_ARG_A_R0_ALPHA | R200_TXC_ARG_B_TFACTOR_ALPHA;
+               else
+                    cblend = R200_TXC_ARG_C_TFACTOR_ALPHA;
+          }
+          else {
                cblend = R200_TXC_ARG_C_R0_ALPHA;
+          }
 
           pp_cntl |= TEX_BLEND_0_ENABLE;
      }
@@ -799,8 +953,13 @@ void r200_set_blittingflags( RadeonDriverData *rdrv,
      if (state->blittingflags & DSBLIT_XOR) {
           master_cntl |= GMC_ROP3_XOR;
           rb3d_cntl   |= ROP_ENABLE; 
-     } else
+     } 
+     else {
           master_cntl |= GMC_ROP3_SRCCOPY;
+     }
+     
+     if (state->render_options & DSRO_ANTIALIAS)
+          pp_cntl |= ANTI_ALIAS_POLY;
 
      radeon_waitfifo( rdrv, rdev, 12 );
      radeon_out32( mmio, CLR_CMP_CNTL, cmp_cntl );
