@@ -34,7 +34,7 @@
 
 #include <core/state.h>
 #include <core/gfxcard.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <gfx/convert.h>
 
@@ -297,23 +297,23 @@ void r300_set_destination( RadeonDriverData *rdrv,
                            RadeonDeviceData *rdev,
                            CardState        *state )
 {
-     CoreSurface   *surface = state->destination;
-     SurfaceBuffer *buffer  = surface->back_buffer;
-     volatile u8   *mmio    = rdrv->mmio_base;
-     u32            offset;
-     u32            pitch;
-     u32            format  = 0;
-     bool           dst_422 = false;
+     CoreSurface       *surface = state->destination;
+     CoreSurfaceBuffer *buffer  = state->dst.buffer;
+     volatile u8       *mmio    = rdrv->mmio_base;
+     u32                offset;
+     u32                pitch;
+     u32                format  = 0;
+     bool               dst_422 = false;
     
      if (RADEON_IS_SET( DESTINATION ))
           return;
      
-     D_ASSERT( (buffer->video.offset % 32) == 0 );
-     D_ASSERT( (buffer->video.pitch % 64) == 0 );
-
-     offset = radeon_buffer_offset( rdev, buffer );
-     pitch  = buffer->video.pitch;
+     D_ASSERT( (state->dst.offset % 32) == 0 );
+     D_ASSERT( (state->dst.pitch % 64) == 0 );
 
+     offset = radeon_buffer_offset( rdev, &state->dst );
+     pitch  = state->dst.pitch;
+     
      if (rdev->dst_offset != offset        ||
          rdev->dst_pitch  != pitch         ||
          rdev->dst_format != buffer->format)
@@ -359,16 +359,16 @@ void r300_set_destination( RadeonDriverData *rdrv,
                case DSPF_I420:
                     format = R300_COLOR_FORMAT_RGB8;
                     rdev->gui_master_cntl = GMC_DST_8BPP;
-                    rdev->dst_offset_cb = offset + pitch * surface->height;
+                    rdev->dst_offset_cb = offset + pitch * surface->config.size.h;
                     rdev->dst_offset_cr = rdev->dst_offset_cb + 
-                                          pitch/2 * surface->height/2;
+                                          pitch/2 * surface->config.size.h/2;
                     break;
                case DSPF_YV12:
                     format = R300_COLOR_FORMAT_RGB8;
                     rdev->gui_master_cntl = GMC_DST_8BPP;
-                    rdev->dst_offset_cr = offset + pitch * surface->height;
+                    rdev->dst_offset_cr = offset + pitch * surface->config.size.h;
                     rdev->dst_offset_cb = rdev->dst_offset_cr +
-                                          pitch/2 * surface->height/2;
+                                          pitch/2 * surface->config.size.h/2;
                     break;
                default:
                     D_BUG( "unexpected pixelformat" );
@@ -417,15 +417,15 @@ void r300_set_source( RadeonDriverData *rdrv,
                       RadeonDeviceData *rdev,
                       CardState        *state )
 {
-     CoreSurface   *surface  = state->source;
-     SurfaceBuffer *buffer   = surface->front_buffer;
-     volatile u8   *mmio     = rdrv->mmio_base;
-     u32            txformat = 0;
-     u32            txfilter = (R300_TX_CLAMP_TO_EDGE << R300_TX_WRAP_S_SHIFT) |
-                               (R300_TX_CLAMP_TO_EDGE << R300_TX_WRAP_T_SHIFT) |
-                               (R300_TX_CLAMP_TO_EDGE << R300_TX_WRAP_Q_SHIFT) |
-                                R300_TX_MAG_FILTER_LINEAR                      |
-                                R300_TX_MIN_FILTER_LINEAR;
+     CoreSurface       *surface  = state->source;
+     CoreSurfaceBuffer *buffer   = state->src.buffer;
+     volatile u8       *mmio     = rdrv->mmio_base;
+     u32                txformat = 0;
+     u32                txfilter = (R300_TX_CLAMP_TO_EDGE << R300_TX_WRAP_S_SHIFT) |
+                                   (R300_TX_CLAMP_TO_EDGE << R300_TX_WRAP_T_SHIFT) |
+                                   (R300_TX_CLAMP_TO_EDGE << R300_TX_WRAP_Q_SHIFT) |
+                                    R300_TX_MAG_FILTER_LINEAR                      |
+                                    R300_TX_MIN_FILTER_LINEAR;
 
      if (RADEON_IS_SET( SOURCE )) {
           if ((state->blittingflags & DSBLIT_DEINTERLACE) ==
@@ -433,13 +433,13 @@ void r300_set_source( RadeonDriverData *rdrv,
                return;
      }
 
-     D_ASSERT( (buffer->video.offset % 32) == 0 );
-     D_ASSERT( (buffer->video.pitch % 64) == 0 );
+     D_ASSERT( (state->src.offset % 32) == 0 );
+     D_ASSERT( (state->src.pitch % 64) == 0 );
      
-     rdev->src_offset = radeon_buffer_offset( rdev, buffer );
-     rdev->src_pitch  = buffer->video.pitch;
-     rdev->src_width  = surface->width;
-     rdev->src_height = surface->height;
+     rdev->src_offset = radeon_buffer_offset( rdev, &state->src );
+     rdev->src_pitch  = state->src.pitch;
+     rdev->src_width  = surface->config.size.w;
+     rdev->src_height = surface->config.size.h;
 
      switch (buffer->format) {
           case DSPF_LUT8:
@@ -535,7 +535,7 @@ void r300_set_source( RadeonDriverData *rdrv,
 
      if (state->blittingflags & DSBLIT_DEINTERLACE) { 
           rdev->src_height /= 2;
-          if (surface->caps & DSCAPS_SEPARATED) {
+          if (surface->config.caps & DSCAPS_SEPARATED) {
                if (surface->field) {
                     rdev->src_offset    += rdev->src_height * rdev->src_pitch;
                     rdev->src_offset_cr += rdev->src_height * rdev->src_pitch/4;
@@ -566,7 +566,6 @@ void r300_set_source( RadeonDriverData *rdrv,
           radeon_out32( mmio, R300_TX_PITCH_0, rdev->src_pitch / 
                                                DFB_BYTES_PER_PIXEL(buffer->format) - 8 );
           radeon_out32( mmio, R300_TX_OFFSET_0, rdev->src_offset ); 
-          //radeon_out32( mmio, R300_TX_BORDER_COLOR_0, 0 );
      }
 
      if (rdev->src_format != buffer->format)
@@ -754,13 +753,6 @@ void r300_set_blitting_color( RadeonDriverData *rdrv,
      if (RADEON_IS_SET( COLOR ) && RADEON_IS_SET( BLITTING_FLAGS ))
           return;
 
-     if (state->blittingflags & DSBLIT_COLORIZE &&
-         state->blittingflags & DSBLIT_SRC_PREMULTCOLOR) {
-          color.r = ((long) color.r * color.a / 255L);
-          color.g = ((long) color.g * color.a / 255L);
-          color.b = ((long) color.b * color.a / 255L);
-     }
-
      switch (rdev->dst_format) {
           case DSPF_A8:
                color.r = color.g = color.b = 0xff;
@@ -786,10 +778,30 @@ void r300_set_blitting_color( RadeonDriverData *rdrv,
                break;
      }
      
-     rdev->color[0] = (float)color.r/255.0;
+     /*rdev->color[0] = (float)color.r/255.0;
      rdev->color[1] = (float)color.g/255.0;
      rdev->color[2] = (float)color.b/255.0;
-     rdev->color[3] = (float)color.a/255.0;
+     rdev->color[3] = (float)color.a/255.0;*/
+
+     if (R300_HAS_3DREGS()) {
+          u32 argb;
+
+          argb = (state->blittingflags & DSBLIT_BLEND_COLORALPHA) ? (color.a << 24) : 0xff000000;
+          if (state->blittingflags & DSBLIT_COLORIZE &&
+              state->blittingflags & (DSBLIT_SRC_PREMULTCOLOR | DSBLIT_BLEND_COLORALPHA)) {
+               argb |= PIXEL_RGB32( (long)color.r * color.a / 255L,
+                                    (long)color.g * color.a / 255L,
+                                    (long)color.b * color.a / 255L );
+          }
+          else {
+               argb |= (state->blittingflags & DSBLIT_COLORIZE)
+                       ? PIXEL_RGB32( color.r, color.g, color.b )
+                       : PIXEL_RGB32( color.a, color.a, color.a );
+          }
+
+          radeon_waitfifo( rdrv, rdev, 1 );
+          radeon_out32( rdrv->mmio_base, R300_RB3D_BLENDCOLOR, argb );
+     }     
      
      RADEON_SET( COLOR );
 }
@@ -865,6 +877,35 @@ r300_set_blend_function( RadeonDriverData *rdrv,
      RADEON_SET( DST_BLEND );
 }
 
+void r300_set_render_options( RadeonDriverData *rdrv,
+                              RadeonDeviceData *rdev,
+                              CardState        *state )
+{
+     if (RADEON_IS_SET( RENDER_OPTIONS ))
+          return;
+          
+     if (state->render_options & DSRO_MATRIX &&
+        (!state->affine_matrix ||
+          state->matrix[0] != (1<<16) || state->matrix[1] != 0 || state->matrix[2] != 0 ||
+          state->matrix[3] != 0 || state->matrix[4] != (1<<16) || state->matrix[5] != 0)) {
+          rdev->matrix = state->matrix;
+          rdev->affine_matrix = state->affine_matrix;
+     }
+     else {
+          rdev->matrix = NULL;
+     }
+     
+     /* TODO: antialiasing */
+#if 0
+     radeon_waitfifo( rdrv, rdev, 1 );
+     radeon_out32( rdrv->mmio_base, R300_GB_AA_CONFIG, 
+                   (state->render_options & DSRO_ANTIALIAS) ? R300_AA_ENABLE : 0 );
+#endif
+     rdev->render_options = state->render_options & ~DSRO_ANTIALIAS;
+     
+     RADEON_SET( RENDER_OPTIONS );
+}
+
 void r300_set_drawingflags( RadeonDriverData *rdrv,
                             RadeonDeviceData *rdev,
                             CardState        *state )
@@ -963,16 +1004,30 @@ void r300_set_blittingflags( RadeonDriverData *rdrv,
      
      if (RADEON_IS_SET( BLITTING_FLAGS ))
           return;
-      
-     if (state->blittingflags & (DSBLIT_BLEND_COLORALPHA |
-                                 DSBLIT_BLEND_ALPHACHANNEL)) {
-          rb3d_blend = R300_BLEND_ENABLE      | R300_BLEND_UNKNOWN |
-                       R300_BLEND_NO_SEPARATE | rdev->rb3d_blend;
+     
+     if (state->blittingflags & (DSBLIT_BLEND_ALPHACHANNEL | DSBLIT_BLEND_COLORALPHA | 
+                                 DSBLIT_COLORIZE           | DSBLIT_SRC_PREMULTCOLOR)) {
+          rb3d_blend = R300_BLEND_ENABLE | R300_BLEND_UNKNOWN | R300_BLEND_NO_SEPARATE;
+          
+          if (state->blittingflags & DSBLIT_BLEND_ALPHACHANNEL)
+               rb3d_blend |= rdev->rb3d_blend;
+          else
+               rb3d_blend |= R300_SRC_BLEND_GL_ONE | R300_DST_BLEND_GL_ZERO;
+
+          if (state->blittingflags & DSBLIT_BLEND_COLORALPHA) {
+               rb3d_blend &= ~(R300_SRC_BLEND_MASK | R300_DST_BLEND_MASK);
+               rb3d_blend |= R300_SRC_BLEND_GL_CONST_ALPHA |
+                             R300_DST_BLEND_GL_ONE_MINUS_CONST_ALPHA;
+          }
+          if (state->blittingflags & (DSBLIT_COLORIZE | DSBLIT_SRC_PREMULTCOLOR)) {
+               rb3d_blend &= ~R300_SRC_BLEND_MASK;
+               rb3d_blend |= R300_SRC_BLEND_GL_CONST_COLOR;
+          }
      }
      else {
           rb3d_blend = R300_SRC_BLEND_GL_ONE | R300_DST_BLEND_GL_ZERO;
      }
-
+     
      if (state->blittingflags & DSBLIT_SRC_COLORKEY) {
           txfilter1 |= R300_CHROMA_KEY_FORCE;
           cmp_cntl   = SRC_CMP_EQ_COLOR | CLR_CMP_SRC_SOURCE;
