@@ -3,7 +3,6 @@
 #endif
 
 #include <stdio.h>
-#include <jpeglib.h>
 
 #undef HAVE_STDLIB_H
 
@@ -41,7 +40,6 @@ DFB_GRAPHICS_DRIVER( sh7722 )
 
 #include "sh7722.h"
 #include "sh7722_blt.h"
-#include "sh7722_jpeglib.h"
 #include "sh7722_layer.h"
 #include "sh7722_lcd.h"
 #include "sh7722_multi.h"
@@ -54,8 +52,11 @@ DFB_GRAPHICS_DRIVER( sh7722 )
 #include <sys/mman.h>
 #endif
 
+/* libshbeu */
+#include <shbeu/shbeu.h>
+#include <uiomux/uiomux.h>
 
-D_DEBUG_DOMAIN( SH7722_Driver, "SH7722/Driver", "Renesas SH7722 Driver" );
+D_DEBUG_DOMAIN( SH7722_Driver, "SH772x/Driver", "Renesas SH7722 Driver" );
 
 /**********************************************************************************************************************/
 
@@ -119,14 +120,22 @@ driver_init_driver( CoreGraphicsDevice  *device,
                               PROT_READ | PROT_WRITE,
                               MAP_SHARED, sdrv->gfx_fd, 0 );
      if (sdrv->gfx_shared == MAP_FAILED) {
-          D_PERROR( "SH7722/Driver: Could not map shared area!\n" );
+          D_PERROR( "SH772x/Driver: Could not map shared area!\n" );
           close( sdrv->gfx_fd );
           return DFB_INIT;
      }
 
      sdrv->mmio_base = dfb_gfxcard_map_mmio( device, 0, -1 );
      if (!sdrv->mmio_base) {
-          D_PERROR( "SH7722/Driver: Could not map MMIO area!\n" );
+          D_PERROR( "SH772x/Driver: Could not map MMIO area!\n" );
+          munmap( (void*) sdrv->gfx_shared, direct_page_align( sizeof(SH772xGfxSharedArea) ) );
+          close( sdrv->gfx_fd );
+          return DFB_INIT;
+     }
+     /* libshbeu */
+     sdrv->shbeu = shbeu_open();
+     if (!sdrv->shbeu) {
+          D_PERROR( "SH772x/Driver: Could not initialize libshbeu" );
           munmap( (void*) sdrv->gfx_shared, direct_page_align( sizeof(SH772xGfxSharedArea) ) );
           close( sdrv->gfx_fd );
           return DFB_INIT;
@@ -147,18 +156,6 @@ driver_init_driver( CoreGraphicsDevice  *device,
                funcs->Blit              = sh7722Blit;
                funcs->StretchBlit       = sh7722StretchBlit;
                funcs->FlushTextureCache = sh7722FlushTextureCache;
-
-               /* Initialize JPEG library. */
-#if 0
-               ret = SH7722_JPEG_Initialize();
-               if (ret) {
-                    D_DERROR( ret, "SH7722/Driver: JPEG initialization failed!\n" );
-                    dfb_gfxcard_unmap_mmio( device, sdrv->mmio_base, -1 );
-                    munmap( (void*) sdrv->gfx_shared, direct_page_align( sizeof(SH772xGfxSharedArea) ) );
-                    close( sdrv->gfx_fd );
-                    return DFB_INIT;
-               }
-#endif
                break;
 
           case SH7723GFX_SHARED_MAGIC:
@@ -241,41 +238,41 @@ driver_init_device( CoreGraphicsDevice *device,
       */
 #ifdef SH772X_FBDEV_SUPPORT
      { 
-     	  struct fb_fix_screeninfo fsi;
-     	  struct fb_var_screeninfo vsi;
-		  int fbdev;
+            struct fb_fix_screeninfo fsi;
+            struct fb_var_screeninfo vsi;
+            int fbdev;
 
-		  if ((fbdev = open("/dev/fb0", O_RDONLY)) < 0) {
-			   D_ERROR( "SH7722/Driver: Can't open fbdev to get LCDC info!\n" );
-			   return DFB_FAILURE;
-		  }
+            if ((fbdev = open("/dev/fb0", O_RDWR)) < 0) {
+                  D_ERROR( "SH772x/Driver: Can't open fbdev to get LCDC info!\n" );
+                  return DFB_FAILURE;
+            }
 
-		  if (ioctl(fbdev, FBIOGET_FSCREENINFO, &fsi) < 0) {
-			   D_ERROR( "SH7722/Driver: FBIOGET_FSCREEINFO failed.\n" );
-			   close(fbdev);
-			   return DFB_FAILURE;
-		  }
+            if (ioctl(fbdev, FBIOGET_FSCREENINFO, &fsi) < 0) {
+                  D_ERROR( "SH772x/Driver: FBIOGET_FSCREEINFO failed.\n" );
+                  close(fbdev);
+                  return DFB_FAILURE;
+            }
 
-		  if (ioctl(fbdev, FBIOGET_VSCREENINFO, &vsi) < 0) {
-			   D_ERROR( "SH7722/Driver: FBIOGET_VSCREEINFO failed.\n" );
-			   close(fbdev);
-			   return DFB_FAILURE;
-		  }
+            if (ioctl(fbdev, FBIOGET_VSCREENINFO, &vsi) < 0) {
+                  D_ERROR( "SH772x/Driver: FBIOGET_VSCREEINFO failed.\n" );
+                  close(fbdev);
+                  return DFB_FAILURE;
+            }
+
+            sdev->lcd_width  = vsi.xres;
+            sdev->lcd_height = vsi.yres;
+            sdev->lcd_pitch  = fsi.line_length;
+            sdev->lcd_size   = fsi.smem_len;
+            sdev->lcd_offset = 0;
+            sdev->lcd_phys   = fsi.smem_start;
+            sdrv->lcd_virt   = mmap(NULL, fsi.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED,
+                                            fbdev, 0);
+            if (sdrv->lcd_virt == MAP_FAILED) {
+                  D_PERROR( "SH772x/Driver: mapping fbdev failed.\n" );
+                  close(fbdev);
+                  return DFB_FAILURE;
+            }
 
-		  sdev->lcd_width  = vsi.xres;
-		  sdev->lcd_height = vsi.yres;
-		  sdev->lcd_pitch  = fsi.line_length;
-		  sdev->lcd_size   = fsi.smem_len;
-		  sdev->lcd_offset = 0;
-		  sdev->lcd_phys   = fsi.smem_start;
-#if 0
-		  sdrv->lcd_virt   = mmap(NULL, fsi.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED,
-				  				  fbdev, 0);
-		  if (sdrv->lcd_virt == MAP_FAILED) {
-			   D_PERROR( "SH7722/Driver: mapping fbdev failed.\n" );
-			   close(fbdev);
-			   return DFB_FAILURE;
-		  }
 
           /* Clear LCD buffer. */
           switch (sdev->lcd_format) {
@@ -292,9 +289,11 @@ driver_init_device( CoreGraphicsDevice *device,
                     D_BUG( "unsupported format" );
                     return DFB_BUG;
           }
-#endif
 
-		  close(fbdev);
+          /* Register the framebuffer with UIOMux */
+          uiomux_register (sdrv->lcd_virt, sdev->lcd_phys, sdev->lcd_size);
+
+          close(fbdev);
      }     
 #else
      sdev->lcd_width  = SH7722_LCD_WIDTH;
@@ -304,7 +303,7 @@ driver_init_device( CoreGraphicsDevice *device,
      sdev->lcd_offset = dfb_gfxcard_reserve_memory( device, sdev->lcd_size );
 
      if (sdev->lcd_offset < 0) {
-          D_ERROR( "SH7722/Driver: Allocating %d bytes for the LCD buffer failed!\n", sdev->lcd_size );
+          D_ERROR( "SH772x/Driver: Allocating %d bytes for the LCD buffer failed!\n", sdev->lcd_size );
           return DFB_FAILURE;
      }
 
@@ -315,7 +314,7 @@ driver_init_device( CoreGraphicsDevice *device,
      sdrv->lcd_virt = dfb_gfxcard_memory_virtual( device, sdev->lcd_offset );
 #endif
 
-     D_INFO( "SH7722/LCD: Allocated %dx%d %s Buffer (%d bytes) at 0x%08lx (%p)\n",
+     D_INFO( "SH772x/LCD: Allocated %dx%d %s Buffer (%d bytes) at 0x%08lx (%p)\n",
              sdev->lcd_width, sdev->lcd_height, dfb_pixelformat_name(sdev->lcd_format),
              sdev->lcd_size, sdev->lcd_phys, sdrv->lcd_virt );
 
@@ -379,16 +378,6 @@ driver_init_device( CoreGraphicsDevice *device,
                return DFB_BUG;
      }
 
-
-     /* Wait for idle BEU. */
-     while (SH7722_GETREG32( sdrv, BSTAR ) & 1);
-
-     /* Disable all inputs. */
-     SH7722_SETREG32( sdrv, BESTR, 0 );
-
-     /* Disable all multi windows. */
-     SH7722_SETREG32( sdrv, BMWCR0, SH7722_GETREG32( sdrv, BMWCR0 ) & ~0xf );
-
 #ifndef SH772X_FBDEV_SUPPORT
      /* Clear LCD buffer. */
      switch (sdev->lcd_format) {
@@ -414,12 +403,15 @@ driver_init_device( CoreGraphicsDevice *device,
      /* Set output pixel format of the BEU. */
      switch (sdev->lcd_format) {
           case DSPF_RGB16:
-               SH7722_SETREG32( sdrv, BPKFR, BPKFR_RY_RGB | WPCK_RGB16 );
+               sdev->shbeu_dest.s.pc = NULL;
+               sdev->shbeu_dest.s.pa = NULL;
+               sdev->shbeu_dest.s.format = REN_RGB565;
                break;
 
           case DSPF_NV16:
-               SH7722_SETREG32( sdrv, BPKFR, BPKFR_RY_RGB | BPKFR_TE_ENABLED | CHDS_YCBCR422 );
-               SH7722_SETREG32( sdrv, BDACR, sdev->lcd_phys + sdev->lcd_height * sdev->lcd_pitch );
+               sdev->shbeu_dest.s.pc = sdrv->lcd_virt + sdev->lcd_height * sdev->lcd_pitch;
+               sdev->shbeu_dest.s.pa = NULL;
+               sdev->shbeu_dest.s.format = REN_NV16;
                break;
 
           default:
@@ -427,13 +419,6 @@ driver_init_device( CoreGraphicsDevice *device,
                return DFB_BUG;
      }
 
-     SH7722_SETREG32( sdrv, BPROCR, 0x00000000 );
-
-     /* Have BEU render into LCD buffer. */
-     SH7722_SETREG32( sdrv, BBLCR1, MT_MEMORY );
-     SH7722_SETREG32( sdrv, BDAYR, sdev->lcd_phys & 0xfffffffc );
-     SH7722_SETREG32( sdrv, BDMWR, sdev->lcd_pitch & 0x0003fffc );
-
 #ifndef SH772X_FBDEV_SUPPORT
      /* Setup LCD controller to show the buffer. */
      sh7722_lcd_setup( sdrv, sdev->lcd_width, sdev->lcd_height,
@@ -443,6 +428,15 @@ driver_init_device( CoreGraphicsDevice *device,
      /* Initialize BEU lock. */
      fusion_skirmish_init( &sdev->beu_lock, "BEU", dfb_core_world(sdrv->core) );
 
+     /* libshbeu */
+     sdev->shbeu_dest.s.py = sdrv->lcd_virt;
+     sdev->shbeu_dest.s.w = sdev->lcd_width;
+     sdev->shbeu_dest.s.h =sdev->lcd_height;
+     sdev->shbeu_dest.s.pitch = sdev->lcd_pitch/DFB_BYTES_PER_PIXEL(sdev->lcd_format);
+     sdev->shbeu_dest.alpha = 0xff;
+     sdev->shbeu_dest.x = 0;
+     sdev->shbeu_dest.y = 0;
+     
      return DFB_OK;
 }
 
@@ -470,25 +464,23 @@ driver_close_driver( CoreGraphicsDevice *device,
 
      D_DEBUG_AT( SH7722_Driver, "%s()\n", __FUNCTION__ );
 
-     D_INFO( "SH7722/BLT: %u starts, %u done, %u interrupts, %u wait_idle, %u wait_next, %u idle\n",
+     D_INFO( "SH772x/BLT: %u starts, %u done, %u interrupts, %u wait_idle, %u wait_next, %u idle\n",
              shared->num_starts, shared->num_done, shared->num_interrupts,
              shared->num_wait_idle, shared->num_wait_next, shared->num_idle );
 
-     D_INFO( "SH7722/BLT: %u words, %u words/start, %u words/idle, %u starts/idle\n",
+     D_INFO( "SH772x/BLT: %u words, %u words/start, %u words/idle, %u starts/idle\n",
              shared->num_words,
              shared->num_words  / shared->num_starts,
              shared->num_words  / shared->num_idle,
              shared->num_starts / shared->num_idle );
 
-     /* Shutdown JPEG library. */
-#if 0
-     SH7722_JPEG_Shutdown();
-#endif
-
      /* Unmap shared area. */
      munmap( (void*) sdrv->gfx_shared, direct_page_align( sizeof(SH772xGfxSharedArea) ) );
 
      /* Close Drawing Engine device. */
      close( sdrv->gfx_fd );
+
+     /* libshbeu */
+     shbeu_close(sdrv->shbeu);
 }
 
