@@ -9,7 +9,6 @@
 #define VEU_REG_BASE     0xFE920000
 #define SH7722_BEU_BASE  0xFE930000
 #define LCDC_REG_BASE    0xFE940000
-#define JPEG_REG_BASE    0xFEA00000
 
 
 /******************************************************************************
@@ -418,207 +417,4 @@
 #define LCDC_LDRCR            (LCDC_REG_BASE + 0x0484)
 
 
-/******************************************************************************
- * JPEG
- */
-
-/* JPEG code mode register */
-#define JCMOD                 (JPEG_REG_BASE + 0x0000)
-
-/* JPEG code command register */
-#define JCCMD                 (JPEG_REG_BASE + 0x0004)
-
-/* JPEG code status register */
-#define JCSTS                 (JPEG_REG_BASE + 0x0008)
-
-/* JPEG code quantization table number register */
-#define JCQTN                 (JPEG_REG_BASE + 0x000C)
-
-/* JPEG code Huffman table number register */
-#define JCHTN                 (JPEG_REG_BASE + 0x0010)
-
-/* JPEG code DRI upper register */
-#define JCDRIU                (JPEG_REG_BASE + 0x0014)
-
-/* JPEG code DRI lower register */
-#define JCDRID                (JPEG_REG_BASE + 0x0018)
-
-/* JPEG code vertical size upper register */
-#define JCVSZU                (JPEG_REG_BASE + 0x001C)
-
-/* JPEG code vertical size lower register */
-#define JCVSZD                (JPEG_REG_BASE + 0x0020)
-
-/* JPEG code horizontal size upper register */
-#define JCHSZU                (JPEG_REG_BASE + 0x0024)
-
-/* JPEG code horizontal size lower register */
-#define JCHSZD                (JPEG_REG_BASE + 0x0028)
-
-/* JPEG code data count upper register */
-#define JCDTCU                (JPEG_REG_BASE + 0x002C)
-
-/* JPEG code data count middle register */
-#define JCDTCM                (JPEG_REG_BASE + 0x0030)
-
-/* JPEG code data count lower register */
-#define JCDTCD                (JPEG_REG_BASE + 0x0034)
-
-/* JPEG interrupt enable register */
-#define JINTE                 (JPEG_REG_BASE + 0x0038)
-
-/* JPEG interrupt status register */
-#define JINTS                 (JPEG_REG_BASE + 0x003C)
-
-/* JPEG code decode error register */
-#define JCDERR                (JPEG_REG_BASE + 0x0040)
-
-/* JPEG code reset register */
-#define JCRST                 (JPEG_REG_BASE + 0x0044)
-
-/* JPEG interface control register */
-#define JIFCNT                (JPEG_REG_BASE + 0x0060)
-
-/* JPEG interface encoding control register */
-#define JIFECNT               (JPEG_REG_BASE + 0x0070)
-
-/* JPEG interface encode source Y address register 1 */
-#define JIFESYA1              (JPEG_REG_BASE + 0x0074)
-
-/* JPEG interface encode source C address register 1 */
-#define JIFESCA1              (JPEG_REG_BASE + 0x0078)
-
-/* JPEG interface encode source Y address register 2 */
-#define JIFESYA2              (JPEG_REG_BASE + 0x007C)
-
-/* JPEG interface encode source C address register 2 */
-#define JIFESCA2              (JPEG_REG_BASE + 0x0080)
-
-/* JPEG interface encode source memory width register */
-#define JIFESMW               (JPEG_REG_BASE + 0x0084)
-
-/* JPEG interface encode source vertical size register */
-#define JIFESVSZ              (JPEG_REG_BASE + 0x0088)
-
-/* JPEG interface encode source horizontal size register */
-#define JIFESHSZ              (JPEG_REG_BASE + 0x008C)
-
-/* JPEG interface encode destination address register 1 */
-#define JIFEDA1               (JPEG_REG_BASE + 0x0090)
-
-/* JPEG interface encode destination address register 2 */
-#define JIFEDA2               (JPEG_REG_BASE + 0x0094)
-
-/* JPEG interface encode data reload size register */
-#define JIFEDRSZ              (JPEG_REG_BASE + 0x0098)
-
-/* JPEG interface decoding control register */
-#define JIFDCNT               (JPEG_REG_BASE + 0x00A0)
-
-/* JPEG interface decode source address register 1 */
-#define JIFDSA1               (JPEG_REG_BASE + 0x00A4)
-
-/* JPEG interface decode source address register 2 */
-#define JIFDSA2               (JPEG_REG_BASE + 0x00A8)
-
-/* JPEG interface decode data reload size register */
-#define JIFDDRSZ              (JPEG_REG_BASE + 0x00AC)
-
-/* JPEG interface decode destination memory width register */
-#define JIFDDMW               (JPEG_REG_BASE + 0x00B0)
-
-/* JPEG interface decode destination vertical size register */
-#define JIFDDVSZ              (JPEG_REG_BASE + 0x00B4)
-
-/* JPEG interface decode destination horizontal size register */
-#define JIFDDHSZ              (JPEG_REG_BASE + 0x00B8)
-
-/* JPEG interface decode destination Y address register 1 */
-#define JIFDDYA1              (JPEG_REG_BASE + 0x00BC)
-
-/* JPEG interface decode destination C address register 1 */
-#define JIFDDCA1              (JPEG_REG_BASE + 0x00C0)
-
-/* JPEG interface decode destination Y address register 2 */
-#define JIFDDYA2              (JPEG_REG_BASE + 0x00C4)
-
-/* JPEG interface decode destination C address register 2 */
-#define JIFDDCA2              (JPEG_REG_BASE + 0x00C8)
-
-
-/* JPEG code quantization table 0 register */
-#define JCQTBL0(n)               (JPEG_REG_BASE + 0x10000 + (((n)*4) & 0x3C)) // to 0x1003C
-
-/* JPEG code quantization table 1 register */
-#define JCQTBL1(n)               (JPEG_REG_BASE + 0x10040 + (((n)*4) & 0x3C)) // to 0x1007C
-
-/* JPEG code quantization table 2 register */
-#define JCQTBL2(n)               (JPEG_REG_BASE + 0x10080 + (((n)*4) & 0x3C)) // to 0x100BC
-
-/* JPEG code quantization table 3 register */
-#define JCQTBL3(n)               (JPEG_REG_BASE + 0x100C0 + (((n)*4) & 0x3C)) // to 0x100FC
-
-/* JPEG code Huffman table DC0 register */
-#define JCHTBD0(n)               (JPEG_REG_BASE + 0x10100 + (((n)*4) & 0x1C)) // to 0x1010C
-
-/* JPEG code Huffman table DC0 register */
-//#define JCHTBD1(n)               (JPEG_REG_BASE + 0x10110 + (((n)*4) & 0x0C)) // to 0x10118
-
-/* JPEG code Huffman table AC0 register */
-#define JCHTBA0(n)               (JPEG_REG_BASE + 0x10120 + (((n)*4) & 0xFC)) // to 0x1012C
-
-/* JPEG code Huffman table AC0 register */
-//#define JCHTBA1(n)               (JPEG_REG_BASE + 0x10130 + (((n)*4) & 0x0C)) // to 0x101D0
-
-/* JPEG code Huffman table DC1 register */
-#define JCHTBD1(n)               (JPEG_REG_BASE + 0x10200 + (((n)*4) & 0x1C)) // to 0x1020C
-
-/* JPEG code Huffman table DC1 register */
-//#define JCHTBD3(n)               (JPEG_REG_BASE + 0x10210 + (((n)*4) & 0x0C)) // to 0x10218
-
-/* JPEG code Huffman table AC1 register */
-#define JCHTBA1(n)               (JPEG_REG_BASE + 0x10220 + (((n)*4) & 0xFC)) // to 0x1022C
-
-/* JPEG code Huffman table AC1 register */
-//#define JCHTBA3(n)               (JPEG_REG_BASE + 0x10230 + (((n)*4) & 0xFC)) // to 0x102D0 
-
-
-#define JCCMD_START           0x00000001
-#define JCCMD_RESTART         0x00000002
-#define JCCMD_END             0x00000004
-#define JCCMD_LCMD2           0x00000100
-#define JCCMD_LCMD1           0x00000200
-#define JCCMD_RESET           0x00000080
-#define JCCMD_READ_RESTART    0x00000400
-#define JCCMD_WRITE_RESTART   0x00000800
-#define JCMOD_DSP_ENCODE      0x00000000
-#define JCMOD_DSP_DECODE      0x00000008
-#define JCMOD_INPUT_CTRL      0x00000080  // must always be set
-#define JIFCNT_VJSEL_JPU      0x00000000
-#define JIFCNT_VJSEL_VPU      0x00000002
-
-#define JIFECNT_LINEBUF_MODE  0x00000002
-#define JIFECNT_SWAP_1234     0x00000000
-#define JIFECNT_SWAP_2143     0x00000010
-#define JIFECNT_SWAP_3412     0x00000020
-#define JIFECNT_SWAP_4321     0x00000030
-#define JIFECNT_RELOAD_ENABLE 0x00000040
-
-#define JIFDCNT_LINEBUF_MODE  0x00000001
-#define JIFDCNT_SWAP_1234     0x00000000
-#define JIFDCNT_SWAP_2143     0x00000002
-#define JIFDCNT_SWAP_3412     0x00000004
-#define JIFDCNT_SWAP_4321     0x00000006
-#define JIFDCNT_RELOAD_ENABLE 0x00000008
-
-#define JINTS_MASK            0x00007C68
-#define JINTS_INS3_HEADER     0x00000008
-#define JINTS_INS5_ERROR      0x00000020
-#define JINTS_INS6_DONE       0x00000040
-#define JINTS_INS10_XFER_DONE 0x00000400
-#define JINTS_INS11_LINEBUF0  0x00000800
-#define JINTS_INS12_LINEBUF1  0x00001000
-#define JINTS_INS13_LOADED    0x00002000
-#define JINTS_INS14_RELOAD    0x00004000
-
 #endif
