@@ -31,7 +31,7 @@ struct uc_ovl_vinfo {
     int level;                      // Position in the DirectFB layer stack
                                     // < 0 = underlay mode, > 0 = overlay mode
     DFBColorAdjustment adj;         // Color adjustment (brightness etc)
-    DFBColor dst_key;               // Destination color key
+    DFBColorKey dst_key;            // Destination color key
     bool dstkey_enabled;            // Destination color key is enabled
 };
 
@@ -53,6 +53,7 @@ typedef struct _UcOverlayData {
     int                 field;
 
     CoreSurface        *surface;
+    CoreSurfaceBufferLock *lock;
     
     int                 opacity_primary; // overlay opacity if primary is logically
                                          // above or -1 if primary has alpha channel
@@ -79,7 +80,7 @@ void uc_ovl_map_v1_control(DFBSurfacePixelFormat format, int sw,
                            u32* control, u32* fifo);
 u32 uc_ovl_map_fifo(u8 depth, u8 pre_thr, u8 thr);
 void uc_ovl_map_adjustment(DFBColorAdjustment* adj, u32* a1, u32* a2);
-u32 uc_ovl_map_colorkey(DFBColor* c);
+u32 uc_ovl_map_colorkey(DFBColorKey* c);
 
 // Video engine - setting functions (uc_ovl_hwset.c)
 
@@ -87,7 +88,8 @@ void uc_ovl_setup_fifo(UcOverlayData* ucovl, int scrwidth);
 void uc_ovl_vcmd_wait(volatile u8* vio);
 DFBResult uc_ovl_update(UcDriverData* ucdrv,
                         UcOverlayData* ucovl, int action,
-                        CoreSurface* surface);
+                        CoreSurface* surface,
+                        CoreSurfaceBufferLock* lock);
 DFBResult uc_ovl_set_adjustment(CoreLayer *layer,
                                 void *driver_data,
                                 void *layer_data,
