@@ -83,7 +83,8 @@ void uc_ovl_vcmd_wait(volatile u8* vio)
 DFBResult uc_ovl_update(UcDriverData* ucdrv,
                         UcOverlayData* ucovl,
                         int action,
-                        CoreSurface* surface)
+                        CoreSurface* surface,
+                        CoreSurfaceBufferLock* lock)
 {
      int sw, sh, sp, sfmt;   // Source width, height, pitch and format
      int dx, dy;             // Destination position
@@ -105,7 +106,7 @@ DFBResult uc_ovl_update(UcDriverData* ucdrv,
      u32 v_ctrl, fifo_ctrl;
      u32 alpha = 0;
 
-     int offset = surface->front_buffer->video.offset;
+     int offset = lock->offset;
 
      if (!ucovl->v1.isenabled) return DFB_OK;
 
@@ -126,10 +127,10 @@ DFBResult uc_ovl_update(UcDriverData* ucdrv,
 
      D_ASSERT(surface);
 
-     sw = surface->width;
-     sh = surface->height;
-     sp = surface->front_buffer->video.pitch;
-     sfmt = surface->format;
+     sw = surface->config.size.w;
+     sh = surface->config.size.h;
+     sp = lock->pitch;
+     sfmt = surface->config.format;
 
      if (ucovl->deinterlace) {
           /*if (ucovl->field)
@@ -202,7 +203,7 @@ DFBResult uc_ovl_update(UcDriverData* ucdrv,
           }
 
           uc_ovl_map_buffer(sfmt, offset,
-                            ucovl->v1.ox, ucovl->v1.oy, sw, surface->height, sp, 0/*field*/, &y_start,
+                            ucovl->v1.ox, ucovl->v1.oy, sw, surface->config.size.h, sp, 0/*field*/, &y_start,
                             &u_start, &v_start);
 
           if (field) {
