@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __DIRECTFB_KEYBOARD_H__
 #define __DIRECTFB_KEYBOARD_H__
 
@@ -454,6 +458,18 @@ typedef enum {
      DIKS_CURSOR_UP_RIGHT          = DFB_KEY( SPECIAL, 0x66 ),
      DIKS_CURSOR_DOWN_RIGHT        = DFB_KEY( SPECIAL, 0x67 ),
 
+     DIKS_PIP                      = DFB_KEY( SPECIAL, 0x68 ),
+     DIKS_SWAP                     = DFB_KEY( SPECIAL, 0x69 ),
+     DIKS_FREEZE                   = DFB_KEY( SPECIAL, 0x6A ),
+     DIKS_MOVE                     = DFB_KEY( SPECIAL, 0x6B ),
+
+     DIKS_CALL                     = DFB_KEY( SPECIAL, 0x6C ),
+     DIKS_SPEAKER                  = DFB_KEY( SPECIAL, 0x6D ),
+     DIKS_SAVE                     = DFB_KEY( SPECIAL, 0x6E ),
+     DIKS_REDIAL                   = DFB_KEY( SPECIAL, 0x6F ),
+     DIKS_FLASH                    = DFB_KEY( SPECIAL, 0x70 ),
+     DIKS_HOLD                     = DFB_KEY( SPECIAL, 0x71 ),
+
      /*
       * Unicode private area - DirectFB Function keys
       *
@@ -614,7 +630,163 @@ typedef enum {
      DIKS_CUSTOM96                 = DFB_CUSTOM_KEY( 96 ),
      DIKS_CUSTOM97                 = DFB_CUSTOM_KEY( 97 ),
      DIKS_CUSTOM98                 = DFB_CUSTOM_KEY( 98 ),
-     DIKS_CUSTOM99                 = DFB_CUSTOM_KEY( 99 )
+     DIKS_CUSTOM99                 = DFB_CUSTOM_KEY( 99 ),
+     DIKS_CUSTOM100                = DFB_CUSTOM_KEY( 100 ),
+     DIKS_CUSTOM101                = DFB_CUSTOM_KEY( 101 ),
+     DIKS_CUSTOM102                = DFB_CUSTOM_KEY( 102 ),
+     DIKS_CUSTOM103                = DFB_CUSTOM_KEY( 103 ),
+     DIKS_CUSTOM104                = DFB_CUSTOM_KEY( 104 ),
+     DIKS_CUSTOM105                = DFB_CUSTOM_KEY( 105 ),
+     DIKS_CUSTOM106                = DFB_CUSTOM_KEY( 106 ),
+     DIKS_CUSTOM107                = DFB_CUSTOM_KEY( 107 ),
+     DIKS_CUSTOM108                = DFB_CUSTOM_KEY( 108 ),
+     DIKS_CUSTOM109                = DFB_CUSTOM_KEY( 109 ),
+     DIKS_CUSTOM110                = DFB_CUSTOM_KEY( 110 ),
+     DIKS_CUSTOM111                = DFB_CUSTOM_KEY( 111 ),
+     DIKS_CUSTOM112                = DFB_CUSTOM_KEY( 112 ),
+     DIKS_CUSTOM113                = DFB_CUSTOM_KEY( 113 ),
+     DIKS_CUSTOM114                = DFB_CUSTOM_KEY( 114 ),
+     DIKS_CUSTOM115                = DFB_CUSTOM_KEY( 115 ),
+     DIKS_CUSTOM116                = DFB_CUSTOM_KEY( 116 ),
+     DIKS_CUSTOM117                = DFB_CUSTOM_KEY( 117 ),
+     DIKS_CUSTOM118                = DFB_CUSTOM_KEY( 118 ),
+     DIKS_CUSTOM119                = DFB_CUSTOM_KEY( 119 ),
+     DIKS_CUSTOM120                = DFB_CUSTOM_KEY( 120 ),
+     DIKS_CUSTOM121                = DFB_CUSTOM_KEY( 121 ),
+     DIKS_CUSTOM122                = DFB_CUSTOM_KEY( 122 ),
+     DIKS_CUSTOM123                = DFB_CUSTOM_KEY( 123 ),
+     DIKS_CUSTOM124                = DFB_CUSTOM_KEY( 124 ),
+     DIKS_CUSTOM125                = DFB_CUSTOM_KEY( 125 ),
+     DIKS_CUSTOM126                = DFB_CUSTOM_KEY( 126 ),
+     DIKS_CUSTOM127                = DFB_CUSTOM_KEY( 127 ),
+     DIKS_CUSTOM128                = DFB_CUSTOM_KEY( 128 ),
+     DIKS_CUSTOM129                = DFB_CUSTOM_KEY( 129 ),
+     DIKS_CUSTOM130                = DFB_CUSTOM_KEY( 130 ),
+     DIKS_CUSTOM131                = DFB_CUSTOM_KEY( 131 ),
+     DIKS_CUSTOM132                = DFB_CUSTOM_KEY( 132 ),
+     DIKS_CUSTOM133                = DFB_CUSTOM_KEY( 133 ),
+     DIKS_CUSTOM134                = DFB_CUSTOM_KEY( 134 ),
+     DIKS_CUSTOM135                = DFB_CUSTOM_KEY( 135 ),
+     DIKS_CUSTOM136                = DFB_CUSTOM_KEY( 136 ),
+     DIKS_CUSTOM137                = DFB_CUSTOM_KEY( 137 ),
+     DIKS_CUSTOM138                = DFB_CUSTOM_KEY( 138 ),
+     DIKS_CUSTOM139                = DFB_CUSTOM_KEY( 139 ),
+     DIKS_CUSTOM140                = DFB_CUSTOM_KEY( 140 ),
+     DIKS_CUSTOM141                = DFB_CUSTOM_KEY( 141 ),
+     DIKS_CUSTOM142                = DFB_CUSTOM_KEY( 142 ),
+     DIKS_CUSTOM143                = DFB_CUSTOM_KEY( 143 ),
+     DIKS_CUSTOM144                = DFB_CUSTOM_KEY( 144 ),
+     DIKS_CUSTOM145                = DFB_CUSTOM_KEY( 145 ),
+     DIKS_CUSTOM146                = DFB_CUSTOM_KEY( 146 ),
+     DIKS_CUSTOM147                = DFB_CUSTOM_KEY( 147 ),
+     DIKS_CUSTOM148                = DFB_CUSTOM_KEY( 148 ),
+     DIKS_CUSTOM149                = DFB_CUSTOM_KEY( 149 ),
+     DIKS_CUSTOM150                = DFB_CUSTOM_KEY( 150 ),
+     DIKS_CUSTOM151                = DFB_CUSTOM_KEY( 151 ),
+     DIKS_CUSTOM152                = DFB_CUSTOM_KEY( 152 ),
+     DIKS_CUSTOM153                = DFB_CUSTOM_KEY( 153 ),
+     DIKS_CUSTOM154                = DFB_CUSTOM_KEY( 154 ),
+     DIKS_CUSTOM155                = DFB_CUSTOM_KEY( 155 ),
+     DIKS_CUSTOM156                = DFB_CUSTOM_KEY( 156 ),
+     DIKS_CUSTOM157                = DFB_CUSTOM_KEY( 157 ),
+     DIKS_CUSTOM158                = DFB_CUSTOM_KEY( 158 ),
+     DIKS_CUSTOM159                = DFB_CUSTOM_KEY( 159 ),
+     DIKS_CUSTOM160                = DFB_CUSTOM_KEY( 160 ),
+     DIKS_CUSTOM161                = DFB_CUSTOM_KEY( 161 ),
+     DIKS_CUSTOM162                = DFB_CUSTOM_KEY( 162 ),
+     DIKS_CUSTOM163                = DFB_CUSTOM_KEY( 163 ),
+     DIKS_CUSTOM164                = DFB_CUSTOM_KEY( 164 ),
+     DIKS_CUSTOM165                = DFB_CUSTOM_KEY( 165 ),
+     DIKS_CUSTOM166                = DFB_CUSTOM_KEY( 166 ),
+     DIKS_CUSTOM167                = DFB_CUSTOM_KEY( 167 ),
+     DIKS_CUSTOM168                = DFB_CUSTOM_KEY( 168 ),
+     DIKS_CUSTOM169                = DFB_CUSTOM_KEY( 169 ),
+     DIKS_CUSTOM170                = DFB_CUSTOM_KEY( 170 ),
+     DIKS_CUSTOM171                = DFB_CUSTOM_KEY( 171 ),
+     DIKS_CUSTOM172                = DFB_CUSTOM_KEY( 172 ),
+     DIKS_CUSTOM173                = DFB_CUSTOM_KEY( 173 ),
+     DIKS_CUSTOM174                = DFB_CUSTOM_KEY( 174 ),
+     DIKS_CUSTOM175                = DFB_CUSTOM_KEY( 175 ),
+     DIKS_CUSTOM176                = DFB_CUSTOM_KEY( 176 ),
+     DIKS_CUSTOM177                = DFB_CUSTOM_KEY( 177 ),
+     DIKS_CUSTOM178                = DFB_CUSTOM_KEY( 178 ),
+     DIKS_CUSTOM179                = DFB_CUSTOM_KEY( 179 ),
+     DIKS_CUSTOM180                = DFB_CUSTOM_KEY( 180 ),
+     DIKS_CUSTOM181                = DFB_CUSTOM_KEY( 181 ),
+     DIKS_CUSTOM182                = DFB_CUSTOM_KEY( 182 ),
+     DIKS_CUSTOM183                = DFB_CUSTOM_KEY( 183 ),
+     DIKS_CUSTOM184                = DFB_CUSTOM_KEY( 184 ),
+     DIKS_CUSTOM185                = DFB_CUSTOM_KEY( 185 ),
+     DIKS_CUSTOM186                = DFB_CUSTOM_KEY( 186 ),
+     DIKS_CUSTOM187                = DFB_CUSTOM_KEY( 187 ),
+     DIKS_CUSTOM188                = DFB_CUSTOM_KEY( 188 ),
+     DIKS_CUSTOM189                = DFB_CUSTOM_KEY( 189 ),
+     DIKS_CUSTOM190                = DFB_CUSTOM_KEY( 190 ),
+     DIKS_CUSTOM191                = DFB_CUSTOM_KEY( 191 ),
+     DIKS_CUSTOM192                = DFB_CUSTOM_KEY( 192 ),
+     DIKS_CUSTOM193                = DFB_CUSTOM_KEY( 193 ),
+     DIKS_CUSTOM194                = DFB_CUSTOM_KEY( 194 ),
+     DIKS_CUSTOM195                = DFB_CUSTOM_KEY( 195 ),
+     DIKS_CUSTOM196                = DFB_CUSTOM_KEY( 196 ),
+     DIKS_CUSTOM197                = DFB_CUSTOM_KEY( 197 ),
+     DIKS_CUSTOM198                = DFB_CUSTOM_KEY( 198 ),
+     DIKS_CUSTOM199                = DFB_CUSTOM_KEY( 199 ),
+     DIKS_CUSTOM200                = DFB_CUSTOM_KEY( 200 ),
+     DIKS_CUSTOM201                = DFB_CUSTOM_KEY( 201 ),
+     DIKS_CUSTOM202                = DFB_CUSTOM_KEY( 202 ),
+     DIKS_CUSTOM203                = DFB_CUSTOM_KEY( 203 ),
+     DIKS_CUSTOM204                = DFB_CUSTOM_KEY( 204 ),
+     DIKS_CUSTOM205                = DFB_CUSTOM_KEY( 205 ),
+     DIKS_CUSTOM206                = DFB_CUSTOM_KEY( 206 ),
+     DIKS_CUSTOM207                = DFB_CUSTOM_KEY( 207 ),
+     DIKS_CUSTOM208                = DFB_CUSTOM_KEY( 208 ),
+     DIKS_CUSTOM209                = DFB_CUSTOM_KEY( 209 ),
+     DIKS_CUSTOM210                = DFB_CUSTOM_KEY( 210 ),
+     DIKS_CUSTOM211                = DFB_CUSTOM_KEY( 211 ),
+     DIKS_CUSTOM212                = DFB_CUSTOM_KEY( 212 ),
+     DIKS_CUSTOM213                = DFB_CUSTOM_KEY( 213 ),
+     DIKS_CUSTOM214                = DFB_CUSTOM_KEY( 214 ),
+     DIKS_CUSTOM215                = DFB_CUSTOM_KEY( 215 ),
+     DIKS_CUSTOM216                = DFB_CUSTOM_KEY( 216 ),
+     DIKS_CUSTOM217                = DFB_CUSTOM_KEY( 217 ),
+     DIKS_CUSTOM218                = DFB_CUSTOM_KEY( 218 ),
+     DIKS_CUSTOM219                = DFB_CUSTOM_KEY( 219 ),
+     DIKS_CUSTOM220                = DFB_CUSTOM_KEY( 220 ),
+     DIKS_CUSTOM221                = DFB_CUSTOM_KEY( 221 ),
+     DIKS_CUSTOM222                = DFB_CUSTOM_KEY( 222 ),
+     DIKS_CUSTOM223                = DFB_CUSTOM_KEY( 223 ),
+     DIKS_CUSTOM224                = DFB_CUSTOM_KEY( 224 ),
+     DIKS_CUSTOM225                = DFB_CUSTOM_KEY( 225 ),
+     DIKS_CUSTOM226                = DFB_CUSTOM_KEY( 226 ),
+     DIKS_CUSTOM227                = DFB_CUSTOM_KEY( 227 ),
+     DIKS_CUSTOM228                = DFB_CUSTOM_KEY( 228 ),
+     DIKS_CUSTOM229                = DFB_CUSTOM_KEY( 229 ),
+     DIKS_CUSTOM230                = DFB_CUSTOM_KEY( 230 ),
+     DIKS_CUSTOM231                = DFB_CUSTOM_KEY( 231 ),
+     DIKS_CUSTOM232                = DFB_CUSTOM_KEY( 232 ),
+     DIKS_CUSTOM233                = DFB_CUSTOM_KEY( 233 ),
+     DIKS_CUSTOM234                = DFB_CUSTOM_KEY( 234 ),
+     DIKS_CUSTOM235                = DFB_CUSTOM_KEY( 235 ),
+     DIKS_CUSTOM236                = DFB_CUSTOM_KEY( 236 ),
+     DIKS_CUSTOM237                = DFB_CUSTOM_KEY( 237 ),
+     DIKS_CUSTOM238                = DFB_CUSTOM_KEY( 238 ),
+     DIKS_CUSTOM239                = DFB_CUSTOM_KEY( 239 ),
+     DIKS_CUSTOM240                = DFB_CUSTOM_KEY( 240 ),
+     DIKS_CUSTOM241                = DFB_CUSTOM_KEY( 241 ),
+     DIKS_CUSTOM242                = DFB_CUSTOM_KEY( 242 ),
+     DIKS_CUSTOM243                = DFB_CUSTOM_KEY( 243 ),
+     DIKS_CUSTOM244                = DFB_CUSTOM_KEY( 244 ),
+     DIKS_CUSTOM245                = DFB_CUSTOM_KEY( 245 ),
+     DIKS_CUSTOM246                = DFB_CUSTOM_KEY( 246 ),
+     DIKS_CUSTOM247                = DFB_CUSTOM_KEY( 247 ),
+     DIKS_CUSTOM248                = DFB_CUSTOM_KEY( 248 ),
+     DIKS_CUSTOM249                = DFB_CUSTOM_KEY( 249 ),
+     DIKS_CUSTOM250                = DFB_CUSTOM_KEY( 250 ),
+     DIKS_CUSTOM251                = DFB_CUSTOM_KEY( 251 ),
+     DIKS_CUSTOM252                = DFB_CUSTOM_KEY( 252 ),
+     DIKS_CUSTOM253                = DFB_CUSTOM_KEY( 253 ),
+     DIKS_CUSTOM254                = DFB_CUSTOM_KEY( 254 ),
+     DIKS_CUSTOM255                = DFB_CUSTOM_KEY( 255 )
 } DFBInputDeviceKeySymbol;
 
 /*
