@@ -1,5 +1,6 @@
 #ifndef __DIRECTFB_STRINGS_H__
 #define __DIRECTFB_STRINGS_H__
+#include <directfb.h>
 
 
 struct DFBPixelFormatName {
@@ -43,7 +44,77 @@ struct DFBPixelFormatName {
      { DSPF_ARGB8565, "ARGB8565" }, \
      { DSPF_AVYU, "AVYU" }, \
      { DSPF_VYU, "VYU" }, \
-     { DSPF_UNKNOWN, "UNKNOWN" } \
+     { DSPF_A1_LSB, "A1_LSB" }, \
+     { DSPF_YV16, "YV16" }, \
+     { DSPF_ABGR, "ABGR" }, \
+     { DSPF_RGBAF88871, "RGBAF88871" }, \
+     { DSPF_LUT4, "LUT4" }, \
+     { DSPF_ALUT8, "ALUT8" }, \
+     { DSPF_LUT1, "LUT1" }, \
+     { (DFBSurfacePixelFormat) DSPF_UNKNOWN, "UNKNOWN" } \
+};
+
+
+struct DFBPorterDuffRuleName {
+     DFBSurfacePorterDuffRule rule;
+     const char *name;
+};
+
+#define DirectFBPorterDuffRuleNames(Identifier) struct DFBPorterDuffRuleName Identifier[] = { \
+     { DSPD_CLEAR, "CLEAR" }, \
+     { DSPD_SRC, "SRC" }, \
+     { DSPD_SRC_OVER, "SRC_OVER" }, \
+     { DSPD_DST_OVER, "DST_OVER" }, \
+     { DSPD_SRC_IN, "SRC_IN" }, \
+     { DSPD_DST_IN, "DST_IN" }, \
+     { DSPD_SRC_OUT, "SRC_OUT" }, \
+     { DSPD_DST_OUT, "DST_OUT" }, \
+     { DSPD_SRC_ATOP, "SRC_ATOP" }, \
+     { DSPD_DST_ATOP, "DST_ATOP" }, \
+     { DSPD_ADD, "ADD" }, \
+     { DSPD_XOR, "XOR" }, \
+     { DSPD_DST, "DST" }, \
+     { (DFBSurfacePorterDuffRule) DSPD_NONE, "NONE" } \
+};
+
+
+struct DFBSurfaceCapabilitiesName {
+     DFBSurfaceCapabilities capability;
+     const char *name;
+};
+
+#define DirectFBSurfaceCapabilitiesNames(Identifier) struct DFBSurfaceCapabilitiesName Identifier[] = { \
+     { DSCAPS_PRIMARY, "PRIMARY" }, \
+     { DSCAPS_SYSTEMONLY, "SYSTEMONLY" }, \
+     { DSCAPS_VIDEOONLY, "VIDEOONLY" }, \
+     { DSCAPS_GL, "GL" }, \
+     { DSCAPS_DOUBLE, "DOUBLE" }, \
+     { DSCAPS_SUBSURFACE, "SUBSURFACE" }, \
+     { DSCAPS_INTERLACED, "INTERLACED" }, \
+     { DSCAPS_SEPARATED, "SEPARATED" }, \
+     { DSCAPS_SEPARATED, "SEPARATED" }, \
+     { DSCAPS_STATIC_ALLOC, "STATIC_ALLOC" }, \
+     { DSCAPS_TRIPLE, "TRIPLE" }, \
+     { DSCAPS_PREMULTIPLIED, "PREMULTIPLIED" }, \
+     { DSCAPS_DEPTH, "DEPTH" }, \
+     { DSCAPS_STEREO, "STEREO" }, \
+     { DSCAPS_SHARED, "SHARED" }, \
+     { DSCAPS_ROTATED, "ROTATED" }, \
+     { (DFBSurfaceCapabilities) DSCAPS_NONE, "NONE" } \
+};
+
+
+struct DFBColorSpaceName {
+     DFBSurfaceColorSpace colorspace;
+     const char *name;
+};
+
+#define DirectFBColorSpaceNames(Identifier) struct DFBColorSpaceName Identifier[] = { \
+     { DSCS_RGB, "RGB" }, \
+     { DSCS_BT601, "BT601" }, \
+     { DSCS_BT601_FULLRANGE, "BT601_FULLRANGE" }, \
+     { DSCS_BT709, "BT709" }, \
+     { (DFBSurfaceColorSpace) DSCS_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -58,7 +129,7 @@ struct DFBInputDeviceTypeFlagsName {
      { DIDTF_JOYSTICK, "JOYSTICK" }, \
      { DIDTF_REMOTE, "REMOTE" }, \
      { DIDTF_VIRTUAL, "VIRTUAL" }, \
-     { DIDTF_NONE, "NONE" } \
+     { (DFBInputDeviceTypeFlags) DIDTF_NONE, "NONE" } \
 };
 
 
@@ -74,7 +145,7 @@ struct DFBSurfaceDrawingFlagsName {
      { DSDRAW_DST_PREMULTIPLY, "DST_PREMULTIPLY" }, \
      { DSDRAW_DEMULTIPLY, "DEMULTIPLY" }, \
      { DSDRAW_XOR, "XOR" }, \
-     { DSDRAW_NOFX, "NOFX" } \
+     { (DFBSurfaceDrawingFlags) DSDRAW_NOFX, "NOFX" } \
 };
 
 
@@ -100,12 +171,37 @@ struct DFBSurfaceBlittingFlagsName {
      { DSBLIT_ROTATE180, "ROTATE180" }, \
      { DSBLIT_ROTATE270, "ROTATE270" }, \
      { DSBLIT_COLORKEY_PROTECT, "COLORKEY_PROTECT" }, \
+     { DSBLIT_SRC_COLORKEY_EXTENDED, "SRC_COLORKEY_EXTENDED" }, \
+     { DSBLIT_DST_COLORKEY_EXTENDED, "DST_COLORKEY_EXTENDED" }, \
      { DSBLIT_SRC_MASK_ALPHA, "SRC_MASK_ALPHA" }, \
      { DSBLIT_SRC_MASK_COLOR, "SRC_MASK_COLOR" }, \
-     { DSBLIT_SOURCE2, "SOURCE2" }, \
      { DSBLIT_FLIP_HORIZONTAL, "FLIP_HORIZONTAL" }, \
      { DSBLIT_FLIP_VERTICAL, "FLIP_VERTICAL" }, \
-     { DSBLIT_NOFX, "NOFX" } \
+     { DSBLIT_ROP, "ROP" }, \
+     { DSBLIT_SRC_COLORMATRIX, "SRC_COLORMATRIX" }, \
+     { DSBLIT_SRC_CONVOLUTION, "SRC_CONVOLUTION" }, \
+     { (DFBSurfaceBlittingFlags) DSBLIT_NOFX, "NOFX" } \
+};
+
+
+struct DFBSurfaceFlipFlagsName {
+     DFBSurfaceFlipFlags flag;
+     const char *name;
+};
+
+#define DirectFBSurfaceFlipFlagsNames(Identifier) struct DFBSurfaceFlipFlagsName Identifier[] = { \
+     { DSFLIP_WAIT, "WAIT" }, \
+     { DSFLIP_BLIT, "BLIT" }, \
+     { DSFLIP_ONSYNC, "ONSYNC" }, \
+     { DSFLIP_WAIT, "WAIT" }, \
+     { DSFLIP_PIPELINE, "PIPELINE" }, \
+     { DSFLIP_ONCE, "ONCE" }, \
+     { DSFLIP_QUEUE, "QUEUE" }, \
+     { DSFLIP_FLUSH, "FLUSH" }, \
+     { DSFLIP_SWAP, "SWAP" }, \
+     { DSFLIP_UPDATE, "UPDATE" }, \
+     { DSFLIP_WAITFORSYNC, "WAITFORSYNC" }, \
+     { (DFBSurfaceFlipFlags) DSFLIP_NONE, "NONE" } \
 };
 
 
@@ -126,7 +222,7 @@ struct DFBSurfaceBlendFunctionName {
      { DSBF_DESTCOLOR, "DESTCOLOR" }, \
      { DSBF_INVDESTCOLOR, "INVDESTCOLOR" }, \
      { DSBF_SRCALPHASAT, "SRCALPHASAT" }, \
-     { DSBF_UNKNOWN, "UNKNOWN" } \
+     { (DFBSurfaceBlendFunction) DSBF_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -139,7 +235,7 @@ struct DFBInputDeviceCapabilitiesName {
      { DICAPS_KEYS, "KEYS" }, \
      { DICAPS_AXES, "AXES" }, \
      { DICAPS_BUTTONS, "BUTTONS" }, \
-     { DICAPS_NONE, "NONE" } \
+     { (DFBInputDeviceCapabilities) DICAPS_NONE, "NONE" } \
 };
 
 
@@ -153,7 +249,7 @@ struct DFBDisplayLayerTypeFlagsName {
      { DLTF_VIDEO, "VIDEO" }, \
      { DLTF_STILL_PICTURE, "STILL_PICTURE" }, \
      { DLTF_BACKGROUND, "BACKGROUND" }, \
-     { DLTF_NONE, "NONE" } \
+     { (DFBDisplayLayerTypeFlags) DLTF_NONE, "NONE" } \
 };
 
 
@@ -184,7 +280,9 @@ struct DFBDisplayLayerCapabilitiesName {
      { DLCAPS_SCREEN_POSITION, "SCREEN_POSITION" }, \
      { DLCAPS_SCREEN_SIZE, "SCREEN_SIZE" }, \
      { DLCAPS_CLIP_REGIONS, "CLIP_REGIONS" }, \
-     { DLCAPS_NONE, "NONE" } \
+     { DLCAPS_LR_MONO, "LR_MONO" }, \
+     { DLCAPS_STEREO, "STEREO" }, \
+     { (DFBDisplayLayerCapabilities) DLCAPS_NONE, "NONE" } \
 };
 
 
@@ -199,7 +297,7 @@ struct DFBDisplayLayerBufferModeName {
      { DLBM_BACKSYSTEM, "BACKSYSTEM" }, \
      { DLBM_TRIPLE, "TRIPLE" }, \
      { DLBM_WINDOWS, "WINDOWS" }, \
-     { DLBM_UNKNOWN, "UNKNOWN" } \
+     { (DFBDisplayLayerBufferMode) DLBM_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -209,6 +307,7 @@ struct DFBWindowCapabilitiesName {
 };
 
 #define DirectFBWindowCapabilitiesNames(Identifier) struct DFBWindowCapabilitiesName Identifier[] = { \
+     { DWCAPS_LR_MONO, "LR_MONO" }, \
      { DWCAPS_ALPHACHANNEL, "ALPHACHANNEL" }, \
      { DWCAPS_DOUBLEBUFFER, "DOUBLEBUFFER" }, \
      { DWCAPS_INPUTONLY, "INPUTONLY" }, \
@@ -216,7 +315,29 @@ struct DFBWindowCapabilitiesName {
      { DWCAPS_SUBWINDOW, "SUBWINDOW" }, \
      { DWCAPS_COLOR, "COLOR" }, \
      { DWCAPS_NOFOCUS, "NOFOCUS" }, \
-     { DWCAPS_NONE, "NONE" } \
+     { DWCAPS_LR_MONO, "LR_MONO" }, \
+     { DWCAPS_STEREO, "STEREO" }, \
+     { (DFBWindowCapabilities) DWCAPS_NONE, "NONE" } \
+};
+
+
+struct DFBDisplayLayerOptionsName {
+     DFBDisplayLayerOptions option;
+     const char *name;
+};
+
+#define DirectFBDisplayLayerOptionsNames(Identifier) struct DFBDisplayLayerOptionsName Identifier[] = { \
+     { DLOP_ALPHACHANNEL, "ALPHACHANNEL" }, \
+     { DLOP_FLICKER_FILTERING, "FLICKER_FILTERING" }, \
+     { DLOP_DEINTERLACING, "DEINTERLACING" }, \
+     { DLOP_SRC_COLORKEY, "SRC_COLORKEY" }, \
+     { DLOP_DST_COLORKEY, "DST_COLORKEY" }, \
+     { DLOP_OPACITY, "OPACITY" }, \
+     { DLOP_FIELD_PARITY, "FIELD_PARITY" }, \
+     { DLOP_LR_MONO, "LR_MONO" }, \
+     { DLOP_STEREO, "STEREO" }, \
+     { DLOP_STEREO, "STEREO" }, \
+     { (DFBDisplayLayerOptions) DLOP_NONE, "NONE" } \
 };
 
 
@@ -236,11 +357,13 @@ struct DFBWindowOptionsName {
      { DWOP_KEEP_STACKING, "KEEP_STACKING" }, \
      { DWOP_GHOST, "GHOST" }, \
      { DWOP_INDESTRUCTIBLE, "INDESTRUCTIBLE" }, \
+     { DWOP_INPUTONLY, "INPUTONLY" }, \
+     { DWOP_STEREO_SIDE_BY_SIDE_HALF, "STEREO_SIDE_BY_SIDE_HALF" }, \
      { DWOP_SCALE, "SCALE" }, \
      { DWOP_KEEP_ABOVE, "KEEP_ABOVE" }, \
      { DWOP_KEEP_UNDER, "KEEP_UNDER" }, \
      { DWOP_FOLLOW_BOUNDS, "FOLLOW_BOUNDS" }, \
-     { DWOP_NONE, "NONE" } \
+     { (DFBWindowOptions) DWOP_NONE, "NONE" } \
 };
 
 
@@ -255,7 +378,7 @@ struct DFBScreenCapabilitiesName {
      { DSCCAPS_MIXERS, "MIXERS" }, \
      { DSCCAPS_ENCODERS, "ENCODERS" }, \
      { DSCCAPS_OUTPUTS, "OUTPUTS" }, \
-     { DSCCAPS_NONE, "NONE" } \
+     { (DFBScreenCapabilities) DSCCAPS_NONE, "NONE" } \
 };
 
 
@@ -278,7 +401,9 @@ struct DFBScreenEncoderCapabilitiesName {
      { DSECAPS_CONNECTORS, "CONNECTORS" }, \
      { DSECAPS_SLOW_BLANKING, "SLOW_BLANKING" }, \
      { DSECAPS_RESOLUTION, "RESOLUTION" }, \
-     { DSECAPS_NONE, "NONE" } \
+     { DSECAPS_FRAMING, "FRAMING" }, \
+     { DSECAPS_ASPECT_RATIO, "ASPECT_RATIO" }, \
+     { (DFBScreenEncoderCapabilities) DSECAPS_NONE, "NONE" } \
 };
 
 
@@ -291,7 +416,7 @@ struct DFBScreenEncoderTypeName {
      { DSET_CRTC, "CRTC" }, \
      { DSET_TV, "TV" }, \
      { DSET_DIGITAL, "DIGITAL" }, \
-     { DSET_UNKNOWN, "UNKNOWN" } \
+     { (DFBScreenEncoderType) DSET_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -311,9 +436,9 @@ struct DFBScreenEncoderTVStandardsName {
      { DSETV_PAL_N, "PAL_N" }, \
      { DSETV_PAL_NC, "PAL_NC" }, \
      { DSETV_NTSC_M_JPN, "NTSC_M_JPN" }, \
-     { DSETV_NTSC_443, "NTSC_443" }, \
      { DSETV_DIGITAL, "DIGITAL" }, \
-     { DSETV_UNKNOWN, "UNKNOWN" } \
+     { DSETV_NTSC_443, "NTSC_443" }, \
+     { (DFBScreenEncoderTVStandards) DSETV_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -329,7 +454,7 @@ struct DFBScreenOutputCapabilitiesName {
      { DSOCAPS_CONNECTOR_SEL, "CONNECTOR_SEL" }, \
      { DSOCAPS_SLOW_BLANKING, "SLOW_BLANKING" }, \
      { DSOCAPS_RESOLUTION, "RESOLUTION" }, \
-     { DSOCAPS_NONE, "NONE" } \
+     { (DFBScreenOutputCapabilities) DSOCAPS_NONE, "NONE" } \
 };
 
 
@@ -346,7 +471,8 @@ struct DFBScreenOutputConnectorsName {
      { DSOC_SCART2, "SCART2" }, \
      { DSOC_COMPONENT, "COMPONENT" }, \
      { DSOC_HDMI, "HDMI" }, \
-     { DSOC_UNKNOWN, "UNKNOWN" } \
+     { DSOC_656, "656" }, \
+     { (DFBScreenOutputConnectors) DSOC_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -363,7 +489,7 @@ struct DFBScreenOutputSignalsName {
      { DSOS_YCBCR, "YCBCR" }, \
      { DSOS_HDMI, "HDMI" }, \
      { DSOS_656, "656" }, \
-     { DSOS_NONE, "NONE" } \
+     { (DFBScreenOutputSignals) DSOS_NONE, "NONE" } \
 };
 
 
@@ -377,7 +503,7 @@ struct DFBScreenOutputSlowBlankingSignalsName {
      { DSOSB_4x3, "4x3" }, \
      { DSOSB_FOLLOW, "FOLLOW" }, \
      { DSOSB_MONITOR, "MONITOR" }, \
-     { DSOSB_OFF, "OFF" } \
+     { (DFBScreenOutputSlowBlankingSignals) DSOSB_OFF, "OFF" } \
 };
 
 
@@ -400,7 +526,17 @@ struct DFBScreenOutputResolutionName {
      { DSOR_1400_1050, "1400_1050" }, \
      { DSOR_1600_1200, "1600_1200" }, \
      { DSOR_1920_1080, "1920_1080" }, \
-     { DSOR_UNKNOWN, "UNKNOWN" } \
+     { DSOR_960_540, "960_540" }, \
+     { DSOR_1440_540, "1440_540" }, \
+     { DSOR_800_480, "800_480" }, \
+     { DSOR_1024_600, "1024_600" }, \
+     { DSOR_1366_768, "1366_768" }, \
+     { DSOR_1920_1200, "1920_1200" }, \
+     { DSOR_2560_1440, "2560_1440" }, \
+     { DSOR_2560_1600, "2560_1600" }, \
+     { DSOR_3840_2160, "3840_2160" }, \
+     { DSOR_4096_2160, "4096_2160" }, \
+     { (DFBScreenOutputResolution) DSOR_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -414,7 +550,7 @@ struct DFBScreenMixerCapabilitiesName {
      { DSMCAPS_SUB_LEVEL, "SUB_LEVEL" }, \
      { DSMCAPS_SUB_LAYERS, "SUB_LAYERS" }, \
      { DSMCAPS_BACKGROUND, "BACKGROUND" }, \
-     { DSMCAPS_NONE, "NONE" } \
+     { (DFBScreenMixerCapabilities) DSMCAPS_NONE, "NONE" } \
 };
 
 
@@ -427,7 +563,7 @@ struct DFBScreenMixerTreeName {
      { DSMT_FULL, "FULL" }, \
      { DSMT_SUB_LEVEL, "SUB_LEVEL" }, \
      { DSMT_SUB_LAYERS, "SUB_LAYERS" }, \
-     { DSMT_UNKNOWN, "UNKNOWN" } \
+     { (DFBScreenMixerTree) DSMT_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -447,7 +583,7 @@ struct DFBScreenEncoderTestPictureName {
      { DSETP_RED, "RED" }, \
      { DSETP_BLUE, "BLUE" }, \
      { DSETP_BLACK, "BLACK" }, \
-     { DSETP_OFF, "OFF" } \
+     { (DFBScreenEncoderTestPicture) DSETP_OFF, "OFF" } \
 };
 
 
@@ -459,7 +595,31 @@ struct DFBScreenEncoderScanModeName {
 #define DirectFBScreenEncoderScanModeNames(Identifier) struct DFBScreenEncoderScanModeName Identifier[] = { \
      { DSESM_INTERLACED, "INTERLACED" }, \
      { DSESM_PROGRESSIVE, "PROGRESSIVE" }, \
-     { DSESM_UNKNOWN, "UNKNOWN" } \
+     { (DFBScreenEncoderScanMode) DSESM_UNKNOWN, "UNKNOWN" } \
+};
+
+
+struct DFBScreenEncoderConfigFlagsName {
+     DFBScreenEncoderConfigFlags config_flags;
+     const char *name;
+};
+
+#define DirectFBScreenEncoderConfigFlagsNames(Identifier) struct DFBScreenEncoderConfigFlagsName Identifier[] = { \
+     { DSECONF_NONE, "NONE" }, \
+     { DSECONF_TV_STANDARD, "TV_STANDARD" }, \
+     { DSECONF_TEST_PICTURE, "TEST_PICTURE" }, \
+     { DSECONF_MIXER, "MIXER" }, \
+     { DSECONF_OUT_SIGNALS, "OUT_SIGNALS" }, \
+     { DSECONF_SCANMODE, "SCANMODE" }, \
+     { DSECONF_TEST_COLOR, "TEST_COLOR" }, \
+     { DSECONF_ADJUSTMENT, "ADJUSTMENT" }, \
+     { DSECONF_FREQUENCY, "FREQUENCY" }, \
+     { DSECONF_CONNECTORS, "CONNECTORS" }, \
+     { DSECONF_SLOW_BLANKING, "SLOW_BLANKING" }, \
+     { DSECONF_RESOLUTION, "RESOLUTION" }, \
+     { DSECONF_FRAMING, "FRAMING" }, \
+     { DSECONF_ASPECT_RATIO, "ASPECT_RATIO" }, \
+     { (DFBScreenEncoderConfigFlags) DSECONF_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -478,7 +638,22 @@ struct DFBScreenEncoderFrequencyName {
      { DSEF_30HZ, "30HZ" }, \
      { DSEF_24HZ, "24HZ" }, \
      { DSEF_23_976HZ, "23_976HZ" }, \
-     { DSEF_UNKNOWN, "UNKNOWN" } \
+     { (DFBScreenEncoderFrequency) DSEF_UNKNOWN, "UNKNOWN" } \
+};
+
+
+struct DFBScreenEncoderPictureFramingName {
+     DFBScreenEncoderPictureFraming framing;
+     const char *name;
+};
+
+#define DirectFBScreenEncoderPictureFramingNames(Identifier) struct DFBScreenEncoderPictureFramingName Identifier[] = { \
+     { DSEPF_MONO, "MONO" }, \
+     { DSEPF_STEREO_SIDE_BY_SIDE_HALF, "STEREO_SIDE_BY_SIDE_HALF" }, \
+     { DSEPF_STEREO_TOP_AND_BOTTOM, "STEREO_TOP_AND_BOTTOM" }, \
+     { DSEPF_STEREO_FRAME_PACKING, "STEREO_FRAME_PACKING" }, \
+     { DSEPF_STEREO_SIDE_BY_SIDE_FULL, "STEREO_SIDE_BY_SIDE_FULL" }, \
+     { (DFBScreenEncoderPictureFraming) DSEPF_UNKNOWN, "UNKNOWN" } \
 };
 
 
@@ -492,12 +667,17 @@ struct DFBAccelerationMaskName {
      { DFXL_DRAWRECTANGLE, "DRAWRECTANGLE" }, \
      { DFXL_DRAWLINE, "DRAWLINE" }, \
      { DFXL_FILLTRIANGLE, "FILLTRIANGLE" }, \
+     { DFXL_FILLTRAPEZOID, "FILLTRAPEZOID" }, \
+     { DFXL_FILLQUADRANGLE, "FILLQUADRANGLE" }, \
+     { DFXL_FILLSPAN, "FILLSPAN" }, \
+     { DFXL_DRAWMONOGLYPH, "DRAWMONOGLYPH" }, \
      { DFXL_BLIT, "BLIT" }, \
      { DFXL_STRETCHBLIT, "STRETCHBLIT" }, \
      { DFXL_TEXTRIANGLES, "TEXTRIANGLES" }, \
      { DFXL_BLIT2, "BLIT2" }, \
+     { DFXL_TILEBLIT, "TILEBLIT" }, \
      { DFXL_DRAWSTRING, "DRAWSTRING" }, \
-     { DFXL_NONE, "NONE" } \
+     { (DFBAccelerationMask) DFXL_NONE, "NONE" } \
 };
 
 #endif
