@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 
@@ -86,6 +90,7 @@ typedef unsigned long kernel_ulong_t;
 #include <stdlib.h>
 
 #define DFB_INPUTDRIVER_HAS_AXIS_INFO
+#define DFB_INPUTDRIVER_HAS_SET_CONFIGURATION
 
 #include <directfb.h>
 #include <directfb_keyboard.h>
@@ -104,15 +109,21 @@ typedef unsigned long kernel_ulong_t;
 #include <misc/conf.h>
 #include <misc/util.h>
 
-#ifdef LINUX_INPUT_USE_FBDEV
+#include <sys/socket.h>
+#include <sys/un.h>
+
+/* Exclude hot-plug stub functionality from this input provider. */
+#define DISABLE_INPUT_HOTPLUG_FUNCTION_STUB
+
 #include <fbdev/fbdev.h>
-#endif
 
 #include <core/input_driver.h>
 
 
 DFB_INPUT_DRIVER( linux_input )
 
+D_DEBUG_DOMAIN( Debug_LinuxInput, "Input/Linux", "Linux input driver" );
+
 #ifndef BITS_PER_LONG
 #define BITS_PER_LONG        (sizeof(long) * 8)
 #endif
@@ -123,15 +134,19 @@ DFB_INPUT_DRIVER( linux_input )
 #undef test_bit
 #define test_bit(bit, array) ((array[LONG(bit)] >> OFF(bit)) & 1)
 
+#define MAX_LENGTH_OF_EVENT_STRING 1024
+
 /* compat for 2.4.x kernel - just a compile fix */
 #ifndef HAVE_INPUT_ABSINFO
-struct input_absinfo {
+typedef struct {
         s32 value;
         s32 minimum;
         s32 maximum;
         s32 fuzz;
         s32 flat;
-};
+} Input_AbsInfo;
+#else
+typedef struct input_absinfo Input_AbsInfo;
 #endif
 
 
@@ -143,19 +158,26 @@ typedef struct {
      DirectThread            *thread;
 
      int                      fd;
+     int                      quitpipe[2];
 
+     bool                     has_keys;
      bool                     has_leds;
      unsigned long            led_state[NBITS(LED_CNT)];
      DFBInputDeviceLockState  locks;
 
-#ifdef LINUX_INPUT_USE_FBDEV
-     VirtualTerminal         *vt;
-#endif
+     int                      vt_fd;
 
      int                      dx;
      int                      dy;
 
      bool                     touchpad;
+
+     /* Indice of the associated device_nums and device_names array entry.
+      * Used as the second parameter of the driver_open_device function.
+      */
+     int                      index;
+
+     int                      sensitivity;
 } LinuxInputData;
 
 
@@ -163,6 +185,21 @@ typedef struct {
 
 static int num_devices = 0;
 static char *device_names[MAX_LINUX_INPUT_DEVICES];
+/* The entries with the same index in device_names and device_nums are the same
+ * are used in two different forms (one is "/dev/input/eventX", the other is
+ * X).
+ */
+static int               device_nums[MAX_LINUX_INPUT_DEVICES] = { 0 };
+/* Socket file descriptor for getting udev events. */
+static int               socket_fd = 0;
+/* Pipe file descriptor for terminating the hotplug thread. */
+static int               hotplug_quitpipe[2];
+/* The hot-plug thread that is launched by the launch_hotplug() function. */
+static DirectThread     *hotplug_thread = NULL;
+/* The driver suspended lock mutex. */
+static pthread_mutex_t   driver_suspended_lock;
+/* Flag that indicates if the driver is suspended when true. */
+static bool              driver_suspended = false;
 
 
 static const
@@ -378,7 +415,6 @@ translate_key( unsigned short code )
      return DIKI_UNKNOWN;
 }
 
-#ifdef LINUX_INPUT_USE_FBDEV
 static DFBInputDeviceKeySymbol
 keyboard_get_symbol( int                             code,
                      unsigned short                  value,
@@ -579,7 +615,7 @@ keyboard_read_value( const LinuxInputData *data,
      entry.kb_index = index;
      entry.kb_value = 0;
 
-     if (ioctl( data->vt->fd, KDGKBENT, &entry )) {
+     if (ioctl( data->vt_fd, KDGKBENT, &entry )) {
           D_PERROR("DirectFB/keyboard: KDGKBENT (table: %d, index: %d) "
                     "failed!\n", table, index);
           return 0;
@@ -587,7 +623,6 @@ keyboard_read_value( const LinuxInputData *data,
 
      return entry.kb_value;
 }
-#endif /* LINUX_INPUT_USE_FBDEV */
 
 /*
  * Translates key and button events.
@@ -642,18 +677,19 @@ key_event( const struct input_event *levt,
  * Translates relative axis events.
  */
 static bool
-rel_event( const struct input_event *levt,
+rel_event( const LinuxInputData     *data,
+           const struct input_event *levt,
            DFBInputEvent            *devt )
 {
      switch (levt->code) {
           case REL_X:
                devt->axis = DIAI_X;
-               devt->axisrel = levt->value;
+               devt->axisrel = levt->value * data->sensitivity / 0x100;
                break;
 
           case REL_Y:
                devt->axis = DIAI_Y;
-               devt->axisrel = levt->value;
+               devt->axisrel = levt->value * data->sensitivity / 0x100;
                break;
 
           case REL_Z:
@@ -713,7 +749,8 @@ abs_event( const struct input_event *levt,
  * Translates a Linux input event into a DirectFB input event.
  */
 static bool
-translate_event( const struct input_event *levt,
+translate_event( const LinuxInputData     *data,
+                 const struct input_event *levt,
                  DFBInputEvent            *devt )
 {
      devt->flags     = DIEF_TIMESTAMP;
@@ -724,7 +761,7 @@ translate_event( const struct input_event *levt,
                return key_event( levt, devt );
 
           case EV_REL:
-               return rel_event( levt, devt );
+               return rel_event( data, levt, devt );
 
           case EV_ABS:
                return abs_event( levt, devt );
@@ -739,13 +776,16 @@ translate_event( const struct input_event *levt,
 static void
 set_led( const LinuxInputData *data, int led, int state )
 {
+     int res;
+
      struct input_event levt;
 
      levt.type = EV_LED;
      levt.code = led;
      levt.value = !!state;
 
-     write( data->fd, &levt, sizeof(levt) );
+     res = write( data->fd, &levt, sizeof(levt) );
+     (void)res;
 }
 
 static void
@@ -794,13 +834,18 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
      LinuxInputData    *data = (LinuxInputData*) driver_data;
      int                readlen, status;
      unsigned int       i;
+     int                fdmax;
      struct input_event levt[64];
      fd_set             set;
      struct touchpad_fsm_state fsm_state;
 
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     fdmax = MAX( data->fd, data->quitpipe[0] );
+
      /* Query min/max coordinates. */
      if (data->touchpad) {
-          struct input_absinfo absinfo;
+          Input_AbsInfo absinfo;
 
           touchpad_fsm_init( &fsm_state );
 
@@ -813,11 +858,45 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
           fsm_state.y.max = absinfo.maximum;
      }
 
+     /* Query key states. */
+     if (data->has_keys) {
+          unsigned long keybit[NBITS(KEY_CNT)];
+          unsigned long keystate[NBITS(KEY_CNT)];
+          int i;
+
+          /* get keyboard bits */
+          ioctl( data->fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit );
+
+          /* get key states */
+          ioctl( data->fd, EVIOCGKEY(sizeof(keystate)), keystate );
+
+          /* for each key,
+             synthetize a press or release event depending on the key state */
+          for (i=0; i<=KEY_CNT; i++) {
+               if (test_bit( i, keybit )) {
+                    const int key = translate_key( i );
+
+                    if (DFB_KEY_TYPE(key) == DIKT_IDENTIFIER) {
+                         DFBInputEvent devt;
+
+                         devt.type     = (test_bit( i, keystate )
+                                          ? DIET_KEYPRESS : DIET_KEYRELEASE);
+                         devt.flags    = DIEF_KEYID | DIEF_KEYCODE;
+                         devt.key_id   = key;
+                         devt.key_code = i;
+
+                         dfb_input_dispatch( data->device, &devt );
+                    }
+               }
+          }
+     }
+
      while (1) {
           DFBInputEvent devt = { .type = DIET_UNKNOWN };
 
           FD_ZERO( &set );
           FD_SET( data->fd, &set );
+          FD_SET( data->quitpipe[0], &set );
 
           if (data->touchpad && timeout_is_set( &fsm_state.timeout )) {
                struct timeval time;
@@ -826,18 +905,21 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                if (!timeout_passed( &fsm_state.timeout, &time )) {
                     struct timeval timeout = fsm_state.timeout;
                     timeout_sub( &timeout, &time );
-                    status = select( data->fd + 1, &set, NULL, NULL, &timeout );
+                    status = select( fdmax + 1, &set, NULL, NULL, &timeout );
                } else {
                     status = 0;
                }
           }
           else {
-               status = select( data->fd + 1, &set, NULL, NULL, NULL );
+               status = select( fdmax + 1, &set, NULL, NULL, NULL );
           }
 
           if (status < 0 && errno != EINTR)
                break;
 
+          if (status > 0 && FD_ISSET( data->quitpipe[0], &set ))
+               break;
+
           direct_thread_testcancel( thread );
 
           if (status < 0)
@@ -868,7 +950,7 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                     status = touchpad_fsm( &fsm_state, &levt[i], &temp );
                     if (status < 0) {
                          /* Not handled. Try the direct approach. */
-                         if (!translate_event( &levt[i], &temp ))
+                         if (!translate_event( data, &levt[i], &temp ))
                               continue;
                     }
                     else if (status == 0) {
@@ -877,7 +959,7 @@ linux_input_EventThread( DirectThread *thread, void *driver_data )
                     }
                }
                else {
-                    if (!translate_event( &levt[i], &temp ))
+                    if (!translate_event( data, &levt[i], &temp ))
                          continue;
                }
 
@@ -966,9 +1048,15 @@ get_device_info( int              fd,
      unsigned long relbit[NBITS(REL_CNT)];
      unsigned long absbit[NBITS(ABS_CNT)];
 
+     struct input_id devinfo;
+
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
      /* get device name */
      ioctl( fd, EVIOCGNAME(DFB_INPUT_DEVICE_DESC_NAME_LENGTH - 1), info->desc.name );
 
+     D_DEBUG_AT( Debug_LinuxInput, "  -> name '%s'\n", info->desc.name );
+
      /* set device vendor */
      snprintf( info->desc.vendor,
                DFB_INPUT_DEVICE_DESC_VENDOR_LENGTH, "Linux" );
@@ -982,11 +1070,19 @@ get_device_info( int              fd,
           /* get keyboard bits */
           ioctl( fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit );
 
-	  /**  count typical keyboard keys only */
+          /* count typical keyboard keys only */
           for (i=KEY_Q; i<=KEY_M; i++)
                if (test_bit( i, keybit ))
                     num_keys++;
 
+          /* this might be a keyboard with just cursor keys, typically found
+             on front panels - handle as remote control and make sure not to
+             treat normal (full key) keyboards likewise */
+          if (!num_keys)
+               for (i=KEY_HOME; i<=KEY_PAGEDOWN; i++)
+                    if (test_bit( i, keybit ))
+                         num_ext_keys++;
+
           for (i=KEY_OK; i<KEY_CNT; i++)
                if (test_bit( i, keybit ))
                     num_ext_keys++;
@@ -994,6 +1090,14 @@ get_device_info( int              fd,
           for (i=BTN_MOUSE; i<BTN_JOYSTICK; i++)
                if (test_bit( i, keybit ))
                     num_buttons++;
+
+          if (num_keys || num_ext_keys)
+#ifndef DIRECTFB_DISABLE_DEPRECATED
+               info->desc.caps |= DICAPS_KEYS;
+#else
+               info->desc.caps |= DIDCAPS_KEYS;
+#endif
+
      }
 
      if (test_bit( EV_REL, evbit )) {
@@ -1042,7 +1146,6 @@ get_device_info( int              fd,
      /* A Keyboard, do we have at least some letters? */
      if (num_keys > 20) {
           info->desc.type |= DIDTF_KEYBOARD;
-          info->desc.caps |= DICAPS_KEYS;
 
           info->desc.min_keycode = 0;
           info->desc.max_keycode = 127;
@@ -1051,18 +1154,25 @@ get_device_info( int              fd,
      /* A Remote Control? */
      if (num_ext_keys) {
           info->desc.type |= DIDTF_REMOTE;
-          info->desc.caps |= DICAPS_KEYS;
      }
 
      /* Buttons */
      if (num_buttons) {
+#ifndef DIRECTFB_DISABLE_DEPRECATED
           info->desc.caps       |= DICAPS_BUTTONS;
+#else
+          info->desc.caps       |= DIDCAPS_BUTTONS;
+#endif
           info->desc.max_button  = DIBI_FIRST + num_buttons - 1;
      }
 
      /* Axes */
      if (num_rels || num_abs) {
+#ifndef DIRECTFB_DISABLE_DEPRECATED
           info->desc.caps       |= DICAPS_AXES;
+#else
+          info->desc.caps       |= DIDCAPS_AXES;
+#endif
           info->desc.max_axis    = DIAI_FIRST + MAX(num_rels, num_abs) - 1;
      }
 
@@ -1077,6 +1187,14 @@ get_device_info( int              fd,
           info->prefered_id = DIDID_MOUSE;
      else
           info->prefered_id = DIDID_ANY;
+
+     /* Get VID and PID information */
+     ioctl( fd, EVIOCGID, &devinfo );
+
+     info->desc.vendor_id  = devinfo.vendor;
+     info->desc.product_id = devinfo.product;
+
+     D_DEBUG_AT( Debug_LinuxInput, "  -> ids %d/%d\n", info->desc.vendor_id, info->desc.product_id );
 }
 
 static bool
@@ -1084,9 +1202,12 @@ check_device( const char *device )
 {
      int  fd;
 
+     D_DEBUG_AT( Debug_LinuxInput, "%s( '%s' )\n", __FUNCTION__, device );
+
      /* Check if we are able to open the device */
      fd = open( device, O_RDWR );
      if (fd < 0) {
+          D_DEBUG_AT( Debug_LinuxInput, "  -> open failed!\n" );
           return false;
      }
      else {
@@ -1110,11 +1231,18 @@ check_device( const char *device )
                ioctl( fd, EVIOCGRAB, 0 );
           close( fd );
 
+          if (!info.desc.caps) {
+              D_DEBUG_AT( Debug_LinuxInput, "  -> no caps!\n" );
+              return false;
+          }
+
           if (!dfb_config->linux_input_ir_only ||
               (info.desc.type & DIDTF_REMOTE))
                return true;
      }
 
+     D_DEBUG_AT( Debug_LinuxInput, "  -> returning false!\n" );
+
      return false;
 }
 
@@ -1130,10 +1258,19 @@ driver_get_available( void )
      int   i;
      char *tsdev;
 
-#ifdef LINUX_INPUT_USE_FBDEV
-     if (dfb_system_type() != CORE_FBDEV)
+     if (!(dfb_config->linux_input_force || (dfb_system_type() == CORE_FBDEV) || (dfb_system_type() == CORE_MESA) || (dfb_system_type() == CORE_DRMKMS)  ))
           return 0;
-#endif
+
+     if (dfb_system_type() == CORE_FBDEV && !dfb_config->linux_input_force) {
+          FBDev *dfb_fbdev = (FBDev*) dfb_system_data();
+          D_ASSERT( dfb_fbdev );
+
+          // Only allow USB keyboard and mouse support if the systems driver has
+          // the Virtual Terminal file ("/dev/tty0") open and available for use.
+          // FIXME:  Additional logic needed for system drivers not similar to fbdev?
+          if (!dfb_fbdev->vt || dfb_fbdev->vt->fd < 0)
+               return 0;
+     }
 
      /* Use the devices specified in the configuration. */
      if (fusion_vector_has_elements( &dfb_config->linux_input_devices )) {
@@ -1143,8 +1280,13 @@ driver_get_available( void )
                if (num_devices >= MAX_LINUX_INPUT_DEVICES)
                     break;
 
-               if (check_device( device ))
-                    device_names[num_devices++] = D_STRDUP( device );
+               /* Update the device_names and device_nums array entries too. */
+               if (check_device( device )){
+                    D_ASSERT( device_names[num_devices] == NULL );
+                    device_names[num_devices] = D_STRDUP( device );
+                    device_nums[num_devices] = i;
+                    num_devices++;
+               }
           }
 
           return num_devices;
@@ -1159,12 +1301,21 @@ driver_get_available( void )
 
           snprintf( buf, 32, "/dev/input/event%d", i );
 
+          /* Initialize device_names and device_nums array entries. */
+          device_nums[i] = MAX_LINUX_INPUT_DEVICES;
+          device_names[i] = NULL;
+
           /* Let tslib driver handle its device. */
           if (tsdev && !strcmp( tsdev, buf ))
                continue;
 
-          if (check_device( buf ))
-               device_names[num_devices++] = D_STRDUP( buf );
+          /* Update the device_names and device_nums array entries too. */
+          if (check_device( buf )){
+               D_ASSERT( device_names[num_devices] == NULL );
+               device_names[num_devices] = D_STRDUP( buf );
+               device_nums[num_devices] = i;
+               num_devices++;
+          }
      }
 
      return num_devices;
@@ -1187,6 +1338,461 @@ driver_get_info( InputDriverInfo *info )
      info->version.minor = 1;
 }
 
+/*
+ * Enter the driver suspended state by setting the driver_suspended Boolean
+ * to prevent hotplug events from being handled.
+ */
+static DFBResult
+driver_suspend( void )
+{
+     if (pthread_mutex_lock(&driver_suspended_lock))
+          return DFB_FAILURE;
+
+     driver_suspended = true;
+
+     pthread_mutex_unlock(&driver_suspended_lock);
+
+     return DFB_OK;
+}
+
+/*
+ * Leave the driver suspended state by clearing the driver_suspended Boolean
+ * which will allow hotplug events to be handled again.
+ */
+static DFBResult
+driver_resume( void )
+{
+     if (pthread_mutex_lock(&driver_suspended_lock))
+          return DFB_FAILURE;
+
+     driver_suspended = false;
+
+     pthread_mutex_unlock(&driver_suspended_lock);
+
+     return DFB_OK;
+}
+
+/*
+ * Register /dev/input/eventX device node into the driver.  Called when a new
+ * device node is created. The device node indicated by event_num should never
+ * be registered before registering it into the driver or should be
+ * unregistered by unregister_device_node() beforehand.
+ */
+static DFBResult
+register_device_node( int event_num, int *index)
+{
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     D_ASSERT( index != NULL );
+
+     int i;
+     char buf[32];
+
+     for (i=0; i<MAX_LINUX_INPUT_DEVICES; i++) {
+          if (device_nums[i] == MAX_LINUX_INPUT_DEVICES) {
+               device_nums[i] = event_num;
+               *index = i;
+               num_devices++;
+
+               snprintf( buf, 32, "/dev/input/event%d", event_num);
+               D_ASSERT( device_names[i] == NULL );
+               device_names[i] = D_STRDUP( buf );
+
+               return DFB_OK;
+          }
+     }
+
+     /* Too many input devices plugged in to be handled by linux_input driver. */
+     D_DEBUG_AT( Debug_LinuxInput,
+                 "The amount of devices registered exceeds the limit (%u) "
+                 "supported by linux input provider.\n",
+                 MAX_LINUX_INPUT_DEVICES );
+     return DFB_UNSUPPORTED;
+}
+
+/*
+ * Unregister /dev/input/eventX device node from the driver.  Called when a new
+ * device node is removed.
+ */
+static DFBResult
+unregister_device_node( int event_num, int *index)
+{
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     D_ASSERT( index != NULL );
+
+     int i;
+
+     for (i=0; i<MAX_LINUX_INPUT_DEVICES; i++) {
+          if (device_nums[i] == event_num) {
+               device_nums[i] = MAX_LINUX_INPUT_DEVICES;
+               num_devices--;
+
+               *index = i;
+               D_FREE(device_names[i]);
+               device_names[i] = NULL;
+
+               return DFB_OK;
+          }
+     }
+
+     return DFB_UNSUPPORTED;
+}
+
+/*
+ * Check if /dev/input/eventX is handled by the input device.  If so, return
+ * DFB_OK.  Otherwise, return DFB_UNSUPPORTED.
+ */
+static DFBResult
+is_created( int index, void *data)
+{
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     D_ASSERT( data != NULL );
+
+     if (index < 0 || index >= MAX_LINUX_INPUT_DEVICES) {
+          return DFB_UNSUPPORTED;
+     }
+
+     if (index != ((LinuxInputData *)data)->index) {
+          return DFB_UNSUPPORTED;
+     }
+
+     return DFB_OK;
+}
+
+/*
+ * Indicate that the hotplug detection capability is supported by this input
+ * provider if the Virtual Terminal was opened for use by the systems driver.
+ *
+ * Note:  The systems driver will open the Virtual Terminal file
+ *        ("/dev/tty0") based on the directfbrc commands "vt" and "no-vt".
+ */
+static InputDriverCapability
+get_capability( void )
+{
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     InputDriverCapability   capabilities = IDC_NONE;
+
+     if (!(dfb_config->linux_input_force || (dfb_system_type() == CORE_FBDEV) || (dfb_system_type() == CORE_MESA) ))
+          goto exit;
+
+     if (dfb_system_type() == CORE_FBDEV) {
+          FBDev *dfb_fbdev = (FBDev*) dfb_system_data();
+          D_ASSERT( dfb_fbdev );
+
+          // Only allow USB keyboard and mouse support if the systems driver has
+          // the Virtual Terminal file ("/dev/tty0") open and available for use.
+          // FIXME:  Additional logic needed for system drivers not similar to fbdev?
+          if (!dfb_fbdev->vt || dfb_fbdev->vt->fd < 0) {
+               D_DEBUG_AT( Debug_LinuxInput, "  -> no VT\n" );
+               goto exit;
+          }
+     }
+
+     D_DEBUG_AT( Debug_LinuxInput, "  -> returning HOTPLUG\n" );
+
+     capabilities |= IDC_HOTPLUG;
+
+exit:
+     return capabilities;
+}
+
+/*
+ * Detect udev hotplug events from socket /org/kernel/udev/monitor and act
+ * according to hotplug events received.
+ */
+static void *
+udev_hotplug_EventThread(DirectThread *thread, void * hotplug_data)
+{
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     CoreDFB           *core;
+     void              *driver;
+     HotplugThreadData *data = (HotplugThreadData *)hotplug_data;
+     int                rt;
+     struct sockaddr_un sock_addr;
+     int                fdmax;
+
+     D_ASSERT( data != NULL );
+     D_ASSERT( data->core != NULL );
+     D_ASSERT( data->driver != NULL );
+
+     core = data->core;
+     driver = data->driver;
+
+     /* Free no needed data packet */
+     D_FREE(data);
+
+     /* Open and bind the socket /org/kernel/udev/monitor */
+
+     socket_fd = socket(AF_UNIX, SOCK_DGRAM, 0);
+     if (socket_fd == -1) {
+          D_PERROR( "DirectFB/linux_input: socket() failed: %s\n",
+                    strerror(errno) );
+          goto errorExit;
+     }
+
+     fdmax = MAX( socket_fd, hotplug_quitpipe[0] );
+
+     memset(&sock_addr, 0, sizeof(sock_addr));
+     sock_addr.sun_family = AF_UNIX;
+     strncpy(&sock_addr.sun_path[1],
+             "/org/kernel/udev/monitor",
+             sizeof(sock_addr.sun_path) - 1);
+
+     rt = bind(socket_fd, &sock_addr,
+               sizeof(sock_addr.sun_family)+1+strlen(&sock_addr.sun_path[1]));
+     if (rt < 0) {
+          D_PERROR( "DirectFB/linux_input: bind() failed: %s\n",
+                    strerror(errno) );
+          goto errorExit;
+     }
+
+     while(1) {
+          char      udev_event[MAX_LENGTH_OF_EVENT_STRING];
+          char     *pos;
+          char     *event_cont; //udev event content
+          int       device_num, number_file, recv_len, index;
+          DFBResult ret;
+          fd_set    rset;
+
+          /* get udev event */
+          FD_ZERO(&rset);
+          FD_SET(socket_fd, &rset);
+          FD_SET(hotplug_quitpipe[0], &rset);
+
+          number_file = select(fdmax+1, &rset, NULL, NULL, NULL);
+
+          if (number_file < 0 && errno != EINTR)
+               break;
+
+          if (FD_ISSET( hotplug_quitpipe[0], &rset ))
+               break;
+
+          /* check cancel thread */
+          direct_thread_testcancel( thread );
+
+          if (FD_ISSET(socket_fd, &rset)) {
+               recv_len = recv(socket_fd, udev_event, sizeof(udev_event), 0);
+               if (recv_len <= 0) {
+                    D_DEBUG_AT( Debug_LinuxInput,
+                                "error receiving uevent message: %s\n",
+                                strerror(errno) );
+                    continue;
+               }
+               /* check cancel thread */
+               direct_thread_testcancel( thread );
+          }
+          /* analysize udev event */
+
+          pos = strchr(udev_event, '@');
+          if (pos == NULL)
+               continue;
+
+          /* replace '@' with '\0' to separate event type and event content */
+          *pos = '\0';
+
+          event_cont = pos + 1;
+
+          pos = strstr(event_cont, "/event");
+          if (pos == NULL)
+               continue;
+
+          /* get event device number */
+          device_num = atoi(pos + 6);
+
+          /* Attempt to lock the driver suspended mutex. */
+          pthread_mutex_lock(&driver_suspended_lock);
+          if (driver_suspended)
+          {
+               /* Release the lock and quit handling hotplug events. */
+               D_DEBUG_AT( Debug_LinuxInput, "Driver is suspended\n" );
+               pthread_mutex_unlock(&driver_suspended_lock);
+               continue;
+          }
+
+          /* Handle hotplug events since the driver is not suspended. */
+          if (!strcmp(udev_event, "add")) {
+               D_DEBUG_AT( Debug_LinuxInput,
+                           "Device node /dev/input/event%d is created by udev\n",
+                           device_num);
+
+               ret = register_device_node( device_num, &index);
+               if ( DFB_OK == ret) {
+                    /* Handle the event that the input device node is created */
+                    ret = dfb_input_create_device(index, core, driver);
+
+                    /* If cannot create the device within Linux Input
+                     * provider, inform the user.
+                     */
+                    if ( DFB_OK != ret) {
+                         D_DEBUG_AT( Debug_LinuxInput,
+                                     "Linux/Input: Failed to create the "
+                                     "device for /dev/input/event%d\n",
+                                     device_num );
+                    }
+               }
+          }
+          else if (!strcmp(udev_event, "remove")) {
+               D_DEBUG_AT( Debug_LinuxInput,
+                           "Device node /dev/input/event%d is removed by udev\n",
+                           device_num );
+               ret = unregister_device_node( device_num, &index );
+
+               if ( DFB_OK == ret) {
+                    /* Handle the event that the input device node is removed */
+                    ret = dfb_input_remove_device( index, driver );
+
+                    /* If unable to remove the device within the Linux Input
+                     * provider, just print the info.
+                     */
+                    if ( DFB_OK != ret) {
+                         D_DEBUG_AT( Debug_LinuxInput,
+                                     "Linux/Input: Failed to remove the "
+                                     "device for /dev/input/event%d\n",
+                                     device_num );
+                    }
+               }
+          }
+
+          /* Hotplug event handling is complete so release the lock. */
+          pthread_mutex_unlock(&driver_suspended_lock);
+     }
+
+     D_DEBUG_AT( Debug_LinuxInput,
+                 "Finished hotplug detection thread within Linux Input "
+                 "provider.\n" );
+     return NULL;
+
+errorExit:
+     D_INFO( "Linux/Input: Fail to open udev socket, disable detecting "
+             "hotplug with Linux Input provider\n" );
+
+     if (socket_fd != -1) {
+          close(socket_fd);
+     }
+
+     return NULL;
+}
+
+/*
+ * Stop hotplug detection thread.
+ */
+static DFBResult
+stop_hotplug( void )
+{
+     int res;
+
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     /* Exit immediately if the hotplug thread is not created successfully in
+      * launch_hotplug().
+      */
+     if (!hotplug_thread)
+          goto exit;
+
+     /* Write to the hotplug quit pipe to cause the thread to terminate */
+     res = write( hotplug_quitpipe[1], " ", 1 );
+     (void)res;
+     /* Shutdown the hotplug detection thread. */
+     direct_thread_join(hotplug_thread);
+     direct_thread_destroy(hotplug_thread);
+     close( hotplug_quitpipe[0] );
+     close( hotplug_quitpipe[1] );
+
+     hotplug_thread = NULL;
+
+     /* Destroy the suspended mutex. */
+     pthread_mutex_destroy(&driver_suspended_lock);
+
+     /* shutdown the connection of the socket */
+     if (socket_fd > 0) {
+          int rt = shutdown(socket_fd, SHUT_RDWR);
+          if (rt < 0) {
+               D_PERROR( "DirectFB/linux_input: Socket shutdown failed: %s\n",
+                         strerror(errno) );
+               return DFB_FAILURE;
+          }
+     }
+     if (socket_fd > 0) {
+          close(socket_fd);
+          socket_fd = 0;
+     }
+
+exit:
+     D_DEBUG_AT( Debug_LinuxInput, "%s() closed\n", __FUNCTION__ );
+     return DFB_OK;
+}
+
+/*
+ * Launch hotplug detection thread.
+ */
+static DFBResult
+launch_hotplug(CoreDFB         *core,
+               void            *input_driver)
+{
+     int ret;
+
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
+     HotplugThreadData  *data;
+     DFBResult           result;
+
+     D_ASSERT( core != NULL );
+     D_ASSERT( input_driver != NULL );
+     D_ASSERT( hotplug_thread == NULL );
+
+     data = D_CALLOC(1, sizeof(HotplugThreadData));
+
+     if (!data) {
+          D_OOM();
+          result = DFB_UNSUPPORTED;
+          goto errorExit;
+     }
+
+     data->core        = core;
+     data->driver      = input_driver;
+
+     /* open a pipe to awake the reader thread when we want to quit */
+     ret = pipe( hotplug_quitpipe );
+     if (ret < 0) {
+          D_PERROR( "DirectFB/linux_input: could not open quitpipe for hotplug" );
+          D_FREE( data );
+          result = DFB_INIT;
+          goto errorExit;
+     }
+     socket_fd = 0;
+
+     /* Initialize a mutex used to communicate to the hotplug handling thread
+      * when the driver is suspended.
+      */
+     pthread_mutex_init(&driver_suspended_lock, NULL);
+
+     /* Create a thread to handle hotplug events. */
+     hotplug_thread = direct_thread_create( DTT_INPUT,
+                                             udev_hotplug_EventThread,
+                                             data,
+                                             "Hotplug with Linux Input" );
+     if (!hotplug_thread) {
+          pthread_mutex_destroy(&driver_suspended_lock);
+
+          /* The hotplug thread normally deallocates the HotplugThreadData
+           * memory, however since it could not be created it must be done
+           * here.
+           */
+          D_FREE( data );
+
+          result = DFB_UNSUPPORTED;
+     }
+     else
+          result = DFB_OK;
+
+errorExit:
+     return result;
+}
+
 /*
  * Open the device, fill out information about it,
  * allocate and fill private data, start input thread.
@@ -1199,17 +1805,18 @@ driver_open_device( CoreInputDevice  *device,
                     void            **driver_data )
 {
      int              fd, ret;
+     bool             touchpad;
      unsigned long    ledbit[NBITS(LED_CNT)];
      LinuxInputData  *data;
-#ifdef LINUX_INPUT_USE_FBDEV
-     FBDev           *dfb_fbdev = dfb_system_data();
-#endif
-     bool             touchpad;
+
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
 
      /* open device */
      fd = open( device_names[number], O_RDWR );
      if (fd < 0) {
-          D_PERROR( "DirectFB/linux_input: could not open device" );
+          D_DEBUG_AT( Debug_LinuxInput,
+                      "DirectFB/linux_input: could not open device %s\n",
+                      device_names[number] );
           return DFB_INIT;
      }
 
@@ -1236,12 +1843,33 @@ driver_open_device( CoreInputDevice  *device,
           return D_OOM();
      }
 
-     data->fd     = fd;
-     data->device = device;
-#ifdef LINUX_INPUT_USE_FBDEV
-     data->vt     = dfb_fbdev->vt;
+     data->fd          = fd;
+     data->device      = device;
+#ifndef DIRECTFB_DISABLE_DEPRECATED
+     data->has_keys    = (info->desc.caps & DICAPS_KEYS) != 0;
+#else
+     data->has_keys    = (info->desc.caps & DIDCAPS_KEYS) != 0;
 #endif
-     data->touchpad = touchpad;
+     data->touchpad    = touchpad;
+     data->vt_fd       = -1;
+     data->sensitivity = 0x100;
+
+      /* Track associated entry in device_nums and device_names array. */
+      data->index = number;
+
+     if (info->desc.min_keycode >= 0 && info->desc.max_keycode >= info->desc.min_keycode) {
+          if (dfb_system_type() == CORE_FBDEV) {
+               FBDev *dfb_fbdev = dfb_system_data();
+
+               if (dfb_fbdev->vt)
+                    data->vt_fd = dup( dfb_fbdev->vt->fd );
+          }
+          if (data->vt_fd < 0)
+               data->vt_fd = open( "/dev/tty0", O_RDWR | O_NOCTTY );
+
+          if (data->vt_fd < 0)
+               D_WARN( "no keymap support (requires /dev/tty0 - CONFIG_VT)" );
+     }
 
      /* check if the device has LEDs */
      ret = ioctl( fd, EVIOCGBIT(EV_LED, sizeof(ledbit)), ledbit );
@@ -1257,11 +1885,7 @@ driver_open_device( CoreInputDevice  *device,
           ret = ioctl( fd, EVIOCGLED(sizeof(data->led_state)), data->led_state );
           if (ret < 0) {
                D_PERROR( "DirectFB/linux_input: could not get LED state" );
-               if (dfb_config->linux_input_grab)
-                    ioctl( fd, EVIOCGRAB, 0 );
-               close( fd );
-               D_FREE( data );
-               return DFB_INIT;
+               goto driver_open_device_error;
           }
 
           /* turn off LEDs */
@@ -1270,6 +1894,13 @@ driver_open_device( CoreInputDevice  *device,
           set_led( data, LED_CAPSL, 0 );
      }
 
+     /* open a pipe to awake the reader thread when we want to quit */
+     ret = pipe( data->quitpipe );
+     if (ret < 0) {
+          D_PERROR( "DirectFB/linux_input: could not open quitpipe" );
+          goto driver_open_device_error;
+     }
+
      /* start input thread */
      data->thread = direct_thread_create( DTT_INPUT, linux_input_EventThread, data, "Linux Input" );
 
@@ -1277,6 +1908,16 @@ driver_open_device( CoreInputDevice  *device,
      *driver_data = data;
 
      return DFB_OK;
+
+driver_open_device_error:
+     if (dfb_config->linux_input_grab)
+          ioctl( fd, EVIOCGRAB, 0 );
+     if (data->vt_fd >= 0)
+          close( data->vt_fd );
+     close( fd );
+     D_FREE( data );
+
+     return DFB_INIT;
 }
 
 /*
@@ -1290,7 +1931,7 @@ driver_get_axis_info( CoreInputDevice              *device,
 {
      LinuxInputData *data = (LinuxInputData*) driver_data;
 
-     if (data->touchpad)
+     if (data->touchpad && !dfb_config->linux_input_touch_abs)
           return DFB_OK;
 
      if (axis <= ABS_PRESSURE && axis < DIAI_LAST) {
@@ -1300,7 +1941,7 @@ driver_get_axis_info( CoreInputDevice              *device,
           ioctl( data->fd, EVIOCGBIT(EV_ABS, sizeof(absbit)), absbit );
 
           if (test_bit (axis, absbit)) {
-               struct input_absinfo absinfo;
+               Input_AbsInfo absinfo;
 
                if (ioctl( data->fd, EVIOCGABS(axis), &absinfo ) == 0 &&
                    (absinfo.minimum || absinfo.maximum)) {
@@ -1314,6 +1955,19 @@ driver_get_axis_info( CoreInputDevice              *device,
      return DFB_OK;
 }
 
+static DFBResult
+driver_set_configuration( CoreInputDevice              *device,
+                          void                         *driver_data,
+                          const DFBInputDeviceConfig   *config )
+{
+     LinuxInputData *data = (LinuxInputData*) driver_data;
+
+     if (config->flags & DIDCONF_SENSITIVITY)
+          data->sensitivity = config->sensitivity;
+
+     return DFB_OK;
+}
+
 /*
  * Fetch one entry from the kernel keymap.
  */
@@ -1322,13 +1976,12 @@ driver_get_keymap_entry( CoreInputDevice           *device,
                          void                      *driver_data,
                          DFBInputDeviceKeymapEntry *entry )
 {
-#ifdef LINUX_INPUT_USE_FBDEV
      LinuxInputData             *data = (LinuxInputData*) driver_data;
      int                         code = entry->code;
      unsigned short              value;
      DFBInputDeviceKeyIdentifier identifier;
 
-     if (!data->vt)
+     if (data->vt_fd < 0)
           return DFB_UNSUPPORTED;
 
      /* fetch the base level */
@@ -1375,9 +2028,6 @@ driver_get_keymap_entry( CoreInputDevice           *device,
                                                             DIKSI_ALT_SHIFT );
 
      return DFB_OK;
-#else
-     return DFB_UNSUPPORTED;
-#endif
 }
 
 /*
@@ -1386,12 +2036,18 @@ driver_get_keymap_entry( CoreInputDevice           *device,
 static void
 driver_close_device( void *driver_data )
 {
+     int res;
      LinuxInputData *data = (LinuxInputData*) driver_data;
 
+     D_DEBUG_AT( Debug_LinuxInput, "%s()\n", __FUNCTION__ );
+
      /* stop input thread */
-     direct_thread_cancel( data->thread );
+     res = write( data->quitpipe[1], " ", 1 );
+     (void)res;
      direct_thread_join( data->thread );
      direct_thread_destroy( data->thread );
+     close( data->quitpipe[0] );
+     close( data->quitpipe[1] );
 
      if (data->has_leds) {
           /* restore LED state */
@@ -1404,11 +2060,16 @@ driver_close_device( void *driver_data )
      if (dfb_config->linux_input_grab)
           ioctl( data->fd, EVIOCGRAB, 0 );
 
+  if (data->vt_fd >= 0)
+          close( data->vt_fd );
+
      /* close file */
      close( data->fd );
 
      /* free private data */
      D_FREE( data );
+
+     D_DEBUG_AT( Debug_LinuxInput, "%s() closed\n", __FUNCTION__ );
 }
 
 static bool
@@ -1477,7 +2138,7 @@ touchpad_translate( struct touchpad_fsm_state *state,
      struct touchpad_axis *axis = NULL;
      int abs, rel;
 
-     devt->flags     = DIEF_TIMESTAMP | DIEF_AXISREL;
+     devt->flags     = DIEF_TIMESTAMP | (dfb_config->linux_input_touch_abs ? DIEF_AXISABS : DIEF_AXISREL);
      devt->timestamp = levt->time;
      devt->type      = DIET_AXISMOTION;
 
@@ -1510,6 +2171,7 @@ touchpad_translate( struct touchpad_fsm_state *state,
 
      axis->old     = abs;
      devt->axisrel = rel;
+     devt->axisabs = levt->value;
 
      return 1;
 }
