@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -41,11 +45,9 @@
 
 #include <media/idirectfbimageprovider.h>
 
-#include <core/coredefs.h>
-#include <core/coretypes.h>
-
 #include <core/layers.h>
-#include <core/surface.h>
+
+#include <core/CoreSurface.h>
 
 #include <misc/gfx_util.h>
 #include <direct/interface.h>
@@ -66,10 +68,10 @@ DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, GIF )
 
 
 #ifndef NODEBUG
-#define GIFERRORMSG(x...)     { fprintf( stderr, "(GIFLOADER) "x ); \
+#define GIFERRORMSG(...)     { fprintf( stderr, "(GIFLOADER) " __VA_ARGS__ ); \
                                 fprintf( stderr, "\n" ); }
 #else
-#define GIFERRORMSG(x...)
+#define GIFERRORMSG(...)
 #endif
 
 #define MAXCOLORMAPSIZE 256
@@ -91,9 +93,7 @@ DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBImageProvider, GIF )
  * private data struct of IDirectFBImageProvider_GIF
  */
 typedef struct {
-     int                  ref;      /* reference counter */
-
-     IDirectFBDataBuffer *buffer;
+     IDirectFBImageProvider_data base;
 
      u32                 *image;
      int                  image_width;
@@ -128,11 +128,6 @@ typedef struct {
      int clear_code, end_code;
      int table[2][(1<< MAX_LWZ_BITS)];
      int stack[(1<<(MAX_LWZ_BITS))*2], *sp;
-
-     DIRenderCallback  render_callback;
-     void             *render_callback_ctx;
-
-     CoreDFB *core;
 } IDirectFBImageProvider_GIF_data;
 
 static bool verbose       = false;
@@ -146,22 +141,11 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
 static bool ReadOK( IDirectFBDataBuffer *buffer, void *data, unsigned int len );
 
 
-static DirectResult
-IDirectFBImageProvider_GIF_AddRef  ( IDirectFBImageProvider *thiz );
-
-static DirectResult
-IDirectFBImageProvider_GIF_Release ( IDirectFBImageProvider *thiz );
-
 static DFBResult
 IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
                                      IDirectFBSurface       *destination,
                                      const DFBRectangle     *destination_rect );
 
-static DFBResult
-IDirectFBImageProvider_GIF_SetRenderCallback( IDirectFBImageProvider *thiz,
-                                              DIRenderCallback        callback,
-                                              void                   *context );
-
 static DFBResult
 IDirectFBImageProvider_GIF_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                   DFBSurfaceDescription  *dsc );
@@ -171,6 +155,16 @@ IDirectFBImageProvider_GIF_GetImageDescription( IDirectFBImageProvider *thiz,
                                                 DFBImageDescription    *dsc );
 
 
+static void
+IDirectFBImageProvider_GIF_Destruct( IDirectFBImageProvider *thiz )
+{
+     IDirectFBImageProvider_GIF_data *data =
+                                   (IDirectFBImageProvider_GIF_data*)thiz->priv;
+
+     if (data->image)
+          D_FREE( data->image );
+}
+
 static DFBResult
 Probe( IDirectFBImageProvider_ProbeContext *ctx )
 {
@@ -194,33 +188,35 @@ Construct( IDirectFBImageProvider *thiz,
      buffer = va_arg( tag, IDirectFBDataBuffer * );
      core = va_arg( tag, CoreDFB * );
      va_end( tag );
-     
-     data->ref = 1;
+
+     data->base.ref = 1;
+
+     data->base.buffer = buffer;
+     data->base.core = core;
+
+     buffer->AddRef( buffer );
 
      data->GrayScale   = -1;
      data->transparent = -1;
      data->delayTime   = -1;
 
-     data->core = core;
-     data->buffer = buffer;
-     buffer->AddRef( buffer );
-
      data->image = ReadGIF( data, 1, &data->image_width, &data->image_height,
                             &data->image_transparency, &data->image_colorkey,
                             true, false );
 
      buffer->Release( buffer );
-     data->buffer = NULL;
-     
-     if (!data->image) {
+     data->base.buffer = NULL;
+
+     if (!data->image ||
+         (data->image_height == 0) ||
+         (data->image_width  == 0) ) {
           DIRECT_DEALLOCATE_INTERFACE( thiz );
           return DFB_FAILURE;
      }
 
-     thiz->AddRef = IDirectFBImageProvider_GIF_AddRef;
-     thiz->Release = IDirectFBImageProvider_GIF_Release;
+     data->base.Destruct = IDirectFBImageProvider_GIF_Destruct;
+
      thiz->RenderTo = IDirectFBImageProvider_GIF_RenderTo;
-     thiz->SetRenderCallback = IDirectFBImageProvider_GIF_SetRenderCallback;
      thiz->GetImageDescription = IDirectFBImageProvider_GIF_GetImageDescription;
      thiz->GetSurfaceDescription =
                                IDirectFBImageProvider_GIF_GetSurfaceDescription;
@@ -228,40 +224,6 @@ Construct( IDirectFBImageProvider *thiz,
      return DFB_OK;
 }
 
-static void
-IDirectFBImageProvider_GIF_Destruct( IDirectFBImageProvider *thiz )
-{
-     IDirectFBImageProvider_GIF_data *data =
-                                   (IDirectFBImageProvider_GIF_data*)thiz->priv;
-
-     if (data->image)
-          D_FREE( data->image );
-
-     DIRECT_DEALLOCATE_INTERFACE( thiz );
-}
-
-static DirectResult
-IDirectFBImageProvider_GIF_AddRef( IDirectFBImageProvider *thiz )
-{
-     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
-
-     data->ref++;
-
-     return DFB_OK;
-}
-
-static DirectResult
-IDirectFBImageProvider_GIF_Release( IDirectFBImageProvider *thiz )
-{
-     DIRECT_INTERFACE_GET_DATA(IDirectFBImageProvider_GIF)
-
-     if (--data->ref == 0) {
-          IDirectFBImageProvider_GIF_Destruct( thiz );
-     }
-
-     return DFB_OK;
-}
-
 static DFBResult
 IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
                                      IDirectFBSurface       *destination,
@@ -289,7 +251,7 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
      if (dest_rect) {
           if (dest_rect->w < 1 || dest_rect->h < 1)
                return DFB_INVARG;
-          rect = *dest_rect; 
+          rect = *dest_rect;
           rect.x += dst_data->area.wanted.x;
           rect.y += dst_data->area.wanted.y;
      }
@@ -314,7 +276,7 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
 
           dfb_surface_unlock_buffer( dst_surface, &lock );
 
-          if (data->render_callback) {
+          if (data->base.render_callback) {
                DIRenderCallbackResult r;
 
                rect.x = 0;
@@ -322,7 +284,8 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
                rect.w = data->image_width;
                rect.h = data->image_height;
 
-               r = data->render_callback( &rect, data->render_callback_ctx );
+               r = data->base.render_callback( &rect,
+                                               data->base.render_callback_context );
 
                if (r != DIRCR_OK)
                        return DFB_INTERRUPTED;
@@ -332,19 +295,6 @@ IDirectFBImageProvider_GIF_RenderTo( IDirectFBImageProvider *thiz,
      return DFB_OK;
 }
 
-static DFBResult
-IDirectFBImageProvider_GIF_SetRenderCallback( IDirectFBImageProvider *thiz,
-                                              DIRenderCallback        callback,
-                                              void                   *context )
-{
-     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_GIF)
-
-     data->render_callback     = callback;
-     data->render_callback_ctx = context;
-     
-     return DFB_OK;
-}
-
 static DFBResult
 IDirectFBImageProvider_GIF_GetSurfaceDescription( IDirectFBImageProvider *thiz,
                                                   DFBSurfaceDescription  *dsc )
@@ -442,7 +392,7 @@ static int GetCode(IDirectFBImageProvider_GIF_data *data, int code_size, int fla
           data->buf[0] = data->buf[data->last_byte-2];
           data->buf[1] = data->buf[data->last_byte-1];
 
-          if ((count = GetDataBlock( data->buffer, &data->buf[2] )) == 0) {
+          if ((count = GetDataBlock( data->base.buffer, &data->buf[2] )) == 0) {
                data->done = true;
           }
 
@@ -474,21 +424,21 @@ static int DoExtension( IDirectFBImageProvider_GIF_data *data, int label )
                break;
           case 0xfe:              /* Comment Extension */
                str = "Comment Extension";
-               while (GetDataBlock( data->buffer, (u8*) buf ) != 0) {
+               while (GetDataBlock( data->base.buffer, (u8*) buf ) != 0) {
                     if (showComment)
                          GIFERRORMSG("gif comment: %s", buf );
                     }
                return false;
           case 0xf9:              /* Graphic Control Extension */
                str = "Graphic Control Extension";
-               (void) GetDataBlock( data->buffer, (u8*) buf );
+               (void) GetDataBlock( data->base.buffer, (u8*) buf );
                data->disposal    = (buf[0] >> 2) & 0x7;
                data->inputFlag   = (buf[0] >> 1) & 0x1;
                data->delayTime   = LM_to_uint( buf[1], buf[2] );
                if ((buf[0] & 0x1) != 0) {
                     data->transparent = buf[3];
                }
-               while (GetDataBlock( data->buffer, (u8*) buf ) != 0)
+               while (GetDataBlock( data->base.buffer, (u8*) buf ) != 0)
                     ;
                return false;
           default:
@@ -500,7 +450,7 @@ static int DoExtension( IDirectFBImageProvider_GIF_data *data, int label )
      if (verbose)
           GIFERRORMSG("got a '%s' extension", str );
 
-     while (GetDataBlock( data->buffer, (u8*) buf ) != 0)
+     while (GetDataBlock( data->base.buffer, (u8*) buf ) != 0)
           ;
 
      return false;
@@ -572,7 +522,7 @@ static int LWZReadByte( IDirectFBImageProvider_GIF_data *data, int flag, int inp
                     return -2;
                }
 
-               while ((count = GetDataBlock( data->buffer, buf )) > 0)
+               while ((count = GetDataBlock( data->base.buffer, buf )) > 0)
                     ;
 
                if (count != 0)
@@ -681,7 +631,7 @@ static u32* ReadImage( IDirectFBImageProvider_GIF_data *data, int width, int hei
      /*
      **  Initialize the decompression routines
      */
-     if (! ReadOK( data->buffer, &c, 1 ))
+     if (! ReadOK( data->base.buffer, &c, 1 ))
           GIFERRORMSG("EOF / read error on image data" );
 
      if (LWZReadByte( data, true, c ) < 0)
@@ -698,10 +648,10 @@ static u32* ReadImage( IDirectFBImageProvider_GIF_data *data, int width, int hei
                ;
           return NULL;
      }
-     
+
      // FIXME: allocates four additional bytes because the scaling functions
      //        in src/misc/gfx_util.c have an off-by-one bug which causes
-     //        segfaults on darwin/osx (not on linux)           
+     //        segfaults on darwin/osx (not on linux)
      if ((image = D_MALLOC(width * height * 4 + 4)) == NULL) {
           GIFERRORMSG("couldn't alloc space for image" );
      }
@@ -789,7 +739,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
      int   imageCount = 0;
      char  version[4];
 
-     if (! ReadOK( data->buffer, buf, 6 )) {
+     if (! ReadOK( data->base.buffer, buf, 6 )) {
           GIFERRORMSG("error reading magic number" );
      }
 
@@ -803,7 +753,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
           GIFERRORMSG("bad version number, not '87a' or '89a'" );
      }
 
-     if (! ReadOK(data->buffer,buf,7)) {
+     if (! ReadOK(data->base.buffer,buf,7)) {
           GIFERRORMSG("failed to read screen descriptor" );
      }
 
@@ -815,7 +765,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
      data->AspectRatio     = buf[6];
 
      if (BitSet(buf[4], LOCALCOLORMAP)) {    /* Global Colormap */
-          if (ReadColorMap( data->buffer, data->BitPixel, data->ColorMap )) {
+          if (ReadColorMap( data->base.buffer, data->BitPixel, data->ColorMap )) {
                GIFERRORMSG("error reading global colormap" );
           }
      }
@@ -831,7 +781,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
      data->disposal    = 0;
 
      for (;;) {
-          if (! ReadOK( data->buffer, &c, 1)) {
+          if (! ReadOK( data->base.buffer, &c, 1)) {
                GIFERRORMSG("EOF / read error on image data" );
           }
 
@@ -844,7 +794,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
           }
 
           if (c == '!') {         /* Extension */
-               if (! ReadOK( data->buffer, &c, 1)) {
+               if (! ReadOK( data->base.buffer, &c, 1)) {
                     GIFERRORMSG("OF / read error on extention function code");
                }
                DoExtension( data, c );
@@ -858,7 +808,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
 
           ++imageCount;
 
-          if (! ReadOK( data->buffer, buf, 9 )) {
+          if (! ReadOK( data->base.buffer, buf, 9 )) {
                GIFERRORMSG("couldn't read left/top/width/height");
           }
 
@@ -878,7 +828,7 @@ static u32* ReadGIF( IDirectFBImageProvider_GIF_data *data, int imageNumber,
           }
           else {
                bitPixel = 2 << (buf[8] & 0x07);
-               if (ReadColorMap( data->buffer, bitPixel, localColorMap ))
+               if (ReadColorMap( data->base.buffer, bitPixel, localColorMap ))
                     GIFERRORMSG("error reading local colormap" );
 
                if (*transparency && (key_rgb || !headeronly))
