@@ -1,13 +1,16 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <errno.h>
@@ -32,6 +37,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <stdarg.h>
 
 #include <directfb.h>
 
@@ -44,7 +50,7 @@
 
 #include <core/layers.h>
 #include <core/palette.h>
-#include <core/surfaces.h>
+#include <core/surface.h>
 
 #include <misc/gfx_util.h>
 #include <direct/interface.h>
@@ -58,7 +64,7 @@ Probe( IDirectFBImageProvider_ProbeContext *ctx );
 
 static DFBResult
 Construct( IDirectFBImageProvider *thiz,
-           IDirectFBDataBuffer    *buffer );
+           ... );
 
 #include <direct/interface_implementation.h>
 
@@ -77,8 +83,7 @@ enum {
  * private data struct of IDirectFBImageProvider_MPEG2
  */
 typedef struct {
-     int                  ref;      /* reference counter */
-     IDirectFBDataBuffer *buffer;
+     IDirectFBImageProvider_data   base;
 
      MPEG2_Decoder       *dec;
 
@@ -87,14 +92,11 @@ typedef struct {
      int                  width;
      int                  height;
 
-     __u32               *image;
+     u32                 *image;
+
+     CoreDFB             *core;
 } IDirectFBImageProvider_MPEG2_data;
 
-static DFBResult
-IDirectFBImageProvider_MPEG2_AddRef  ( IDirectFBImageProvider *thiz );
-
-static DFBResult
-IDirectFBImageProvider_MPEG2_Release ( IDirectFBImageProvider *thiz );
 
 static DFBResult
 IDirectFBImageProvider_MPEG2_RenderTo( IDirectFBImageProvider *thiz,
@@ -116,7 +118,7 @@ IDirectFBImageProvider_MPEG2_GetImageDescription( IDirectFBImageProvider *thiz,
 
 
 static int  mpeg2_read_func ( void *buf, int count, void *ctx );
-static void mpeg2_write_func( int x, int y, __u32 argb, void *ctx );
+static void mpeg2_write_func( int x, int y, u32 argb, void *ctx );
 
 static DFBResult
 Probe( IDirectFBImageProvider_ProbeContext *ctx )
@@ -129,16 +131,38 @@ Probe( IDirectFBImageProvider_ProbeContext *ctx )
      return DFB_UNSUPPORTED;
 }
 
+static void
+IDirectFBImageProvider_MPEG2_Destruct( IDirectFBImageProvider *thiz )
+{
+     IDirectFBImageProvider_MPEG2_data *data =
+                              (IDirectFBImageProvider_MPEG2_data*)thiz->priv;
+
+     MPEG2_Close(data->dec);
+
+     /* Deallocate image data. */
+     if (data->image)
+          D_FREE( data->image );
+}
+
 static DFBResult
 Construct( IDirectFBImageProvider *thiz,
-           IDirectFBDataBuffer    *buffer )
+           ... )
 {
      DFBResult ret = DFB_FAILURE;
+     IDirectFBDataBuffer *buffer;
+     CoreDFB             *core;
+     va_list              tag;
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_MPEG2)
 
-     data->ref    = 1;
-     data->buffer = buffer;
+     va_start( tag, thiz );
+     buffer = va_arg( tag, IDirectFBDataBuffer * );
+     core = va_arg( tag, CoreDFB * );
+     va_end( tag );
+       
+     data->base.ref    = 1;
+     data->base.buffer = buffer;
+     data->core   = core;
 
      /* Increase the data buffer reference counter. */
      buffer->AddRef( buffer );
@@ -157,8 +181,8 @@ Construct( IDirectFBImageProvider *thiz,
 
      data->stage = STAGE_IMAGE;
 
-     thiz->AddRef = IDirectFBImageProvider_MPEG2_AddRef;
-     thiz->Release = IDirectFBImageProvider_MPEG2_Release;
+     data->base.Destruct = IDirectFBImageProvider_MPEG2_Destruct;
+
      thiz->RenderTo = IDirectFBImageProvider_MPEG2_RenderTo;
      thiz->SetRenderCallback = IDirectFBImageProvider_MPEG2_SetRenderCallback;
      thiz->GetImageDescription = IDirectFBImageProvider_MPEG2_GetImageDescription;
@@ -178,44 +202,6 @@ error:
      return ret;
 }
 
-static void
-IDirectFBImageProvider_MPEG2_Destruct( IDirectFBImageProvider *thiz )
-{
-     IDirectFBImageProvider_MPEG2_data *data =
-                              (IDirectFBImageProvider_MPEG2_data*)thiz->priv;
-
-     MPEG2_Close(data->dec);
-
-     /* Decrease the data buffer reference counter. */
-     data->buffer->Release( data->buffer );
-
-     /* Deallocate image data. */
-     if (data->image)
-          D_FREE( data->image );
-
-     DIRECT_DEALLOCATE_INTERFACE( thiz );
-}
-
-static DFBResult
-IDirectFBImageProvider_MPEG2_AddRef( IDirectFBImageProvider *thiz )
-{
-     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_MPEG2)
-
-     data->ref++;
-
-     return DFB_OK;
-}
-
-static DFBResult
-IDirectFBImageProvider_MPEG2_Release( IDirectFBImageProvider *thiz )
-{
-     DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_MPEG2)
-
-     if (--data->ref == 0)
-          IDirectFBImageProvider_MPEG2_Destruct( thiz );
-
-     return DFB_OK;
-}
 
 static DFBResult
 IDirectFBImageProvider_MPEG2_RenderTo( IDirectFBImageProvider *thiz,
@@ -225,7 +211,8 @@ IDirectFBImageProvider_MPEG2_RenderTo( IDirectFBImageProvider *thiz,
      DFBResult              ret;
      IDirectFBSurface_data *dst_data;
      CoreSurface           *dst_surface;
-     DFBRectangle           rect = { 0, 0, 0, 0 };
+     DFBRegion              clip;
+     DFBRectangle           rect;
 
      DIRECT_INTERFACE_GET_DATA (IDirectFBImageProvider_MPEG2)
 
@@ -237,9 +224,18 @@ IDirectFBImageProvider_MPEG2_RenderTo( IDirectFBImageProvider *thiz,
      if (!dst_surface)
           return DFB_DESTROYED;
 
-     ret = destination->GetSize( destination, &rect.w, &rect.h );
-     if (ret)
-          return ret;
+     dfb_region_from_rectangle( &clip, &dst_data->area.current );
+
+     if (dest_rect) {
+          if (dest_rect->w < 1 || dest_rect->h < 1)
+               return DFB_INVARG;
+          rect = *dest_rect;
+          rect.w += dst_data->area.wanted.w;
+          rect.h += dst_data->area.wanted.h;
+     }
+     else {
+          rect = dst_data->area.wanted;
+     }
 
      switch (data->stage) {
           case STAGE_END:
@@ -256,18 +252,17 @@ IDirectFBImageProvider_MPEG2_RenderTo( IDirectFBImageProvider *thiz,
      }
 
      /* actual rendering */
-     if (dest_rect == NULL || dfb_rectangle_intersect ( &rect, dest_rect )) {
-          void *dst;
-          int   pitch;
+     if (dfb_rectangle_region_intersects( &rect, &clip )) {
+          CoreSurfaceBufferLock lock;
 
-          ret = destination->Lock( destination, DSLF_WRITE, &dst, &pitch );
+          ret = dfb_surface_lock_buffer( dst_surface, CSBR_BACK, CSAID_CPU, CSAF_WRITE, &lock );
           if (ret)
                return ret;
 
           dfb_scale_linear_32( data->image, data->width, data->height,
-                               dst, pitch, &rect, dst_surface );
+                               lock.addr, lock.pitch, &rect, dst_surface, &clip );
 
-          destination->Unlock( destination );
+          dfb_surface_unlock_buffer( dst_surface, &lock );
      }
 
      return DFB_OK;
@@ -330,7 +325,7 @@ mpeg2_read_func( void *buf, int count, void *ctx )
 }
 
 static void
-mpeg2_write_func( int x, int y, __u32 argb, void *ctx )
+mpeg2_write_func( int x, int y, u32 argb, void *ctx )
 {
      IDirectFBImageProvider_MPEG2_data *data =
                                        (IDirectFBImageProvider_MPEG2_data*) ctx;
