@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #if defined(__dietlibc__) && !defined(_BSD_SOURCE)
 #define _BSD_SOURCE
 #endif
@@ -924,6 +928,7 @@ v4l_to_surface_overlay( CoreSurface *surface, DFBRectangle *rect,
                palette = VIDEO_PALETTE_RGB24;
                break;
           case DSPF_ARGB:
+          case DSPF_ABGR:
           case DSPF_AiRGB:
           case DSPF_RGB32:
                bpp = 32;
