@@ -1,13 +1,14 @@
 /*
-   (c) Copyright 2000-2002  convergence integrated media GmbH.
-   (c) Copyright 2002-2004  convergence GmbH.
+   (c) Copyright 2001-2011  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
               Andreas Hundt <andi@fischlustig.de>,
-              Sven Neumann <neo@directfb.org> and
-              Ville Syrjl <syrjala@sci.fi>.
+              Sven Neumann <neo@directfb.org>,
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,13 +29,26 @@
 #ifndef __DIRECT__BUILD_H__
 #define __DIRECT__BUILD_H__
 
-#define DIRECT_BUILD_DEBUG   (1)
-#define DIRECT_BUILD_DEBUGS  (1)
-#define DIRECT_BUILD_TRACE   (0)
-#define DIRECT_BUILD_TEXT    (1)
-#define DIRECT_BUILD_GETTID  (1)
-#define DIRECT_BUILD_NETWORK (1)
-#define DIRECT_BUILD_STDBOOL (1)
+
+#define DIRECT_OS_LINUX_GNU_LIBC   (0)
+#define DIRECT_OS_LINUX_KERNEL     (1)
+#define DIRECT_OS_PSP              (2)
+#define DIRECT_OS_WIN32            (3)
+#define DIRECT_OS_FAMOS            (4)
+
+
+#define DIRECT_BUILD_DEBUG       (1)
+#define DIRECT_BUILD_DEBUGS      (1)
+#define DIRECT_BUILD_TRACE       (1)
+#define DIRECT_BUILD_TEXT        (1)
+#define DIRECT_BUILD_GETTID      (1)
+#define DIRECT_BUILD_NETWORK     (1)
+#define DIRECT_BUILD_STDBOOL     (1)
+#define DIRECT_BUILD_DYNLOAD     (1)
+#define DIRECT_BUILD_MULTICORE   (1)
+#define DIRECT_BUILD_OSTYPE      (DIRECT_OS_LINUX_GNU_LIBC)
+#define DIRECT_BUILD_GCC_ATOMICS (0)
+
 
 
 #if !DIRECT_BUILD_DEBUGS
