@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,14 +28,13 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __DIRECT__DEBUG_H__
 #define __DIRECT__DEBUG_H__
 
 #include <direct/build.h>
 
-#include <stdio.h>
-#include <errno.h>
-
 #include <direct/clock.h>
 #include <direct/conf.h>
 #include <direct/log.h>
@@ -44,91 +45,106 @@
 #include <direct/types.h>
 
 
-typedef struct {
-     unsigned int   age;
-     bool           enabled;
-     bool           registered;
+// @deprecated
+#define D_DEBUG_DOMAIN( _identifier, _name, _description )                                          \
+     D_LOG_DOMAIN( _identifier, _name, _description )
 
-     const char    *name;
-     const char    *description;
+#ifndef DIRECT_DISABLE_DEPRECATED
 
-     int            name_len;
-} DirectDebugDomain;
 
-void direct_debug_config_domain( const char *name, bool enable );
+// @deprecated
+D_DEBUG_DOMAIN( _direct_debug_deprecated, "-", "deprecated" );
 
+// @deprecated
+#define D_DEBUG( ... )                                                                              \
+     D_DEBUG_AT( _direct_debug_deprecated, __VA_ARGS__ )
+
+#endif
+
+static __inline__ void
+direct_debug_config_domain( const char *name, bool enable )
+{
+     direct_log_domain_config_level( name, enable ? DIRECT_LOG_ALL : DIRECT_LOG_NONE );
+}
 
 #if DIRECT_BUILD_TEXT
 
-#define D_DEBUG_DOMAIN(identifier,name,description)                                  \
-     static DirectDebugDomain identifier __attribute__((unused))                     \
-            = { 0, false, false, name, description, sizeof(name) - 1 }
+void DIRECT_API direct_debug_at_always( DirectLogDomain *domain,
+                                        const char      *format, ... )  D_FORMAT_PRINTF(2);
 
-void direct_debug_at_always( DirectDebugDomain *domain,
-                             const char        *format, ... )  D_FORMAT_PRINTF(2);
-
-#define d_debug_at( domain, x... )      direct_debug_at_always( &domain, x )
+#define d_debug_at( domain, ... )      direct_debug_at_always( &domain, __VA_ARGS__ )
 
 
 #if DIRECT_BUILD_DEBUGS
 
-void direct_debug( const char *format, ... )  D_FORMAT_PRINTF(1);
+/*
+ * Direct v2.0 - Debug Interface
+ *
+ * debug level 1-9  (0 = verbose)
+ *
+ */
+void DIRECT_API direct_debug_log( DirectLogDomain *domain,
+                                  unsigned int     debug_level,
+                                  const char      *format, ... )      D_FORMAT_PRINTF(3);
 
-void direct_debug_at( DirectDebugDomain *domain,
-                      const char        *format, ... )  D_FORMAT_PRINTF(2);
 
-void direct_debug_enter( DirectDebugDomain *domain,
-	      	         const char *func,
-                         const char *file,
-                         int         line,
-                         const char *format, ... )  D_FORMAT_PRINTF(5);
 
-void direct_debug_exit( DirectDebugDomain *domain,
-	      	        const char *func,
-                        const char *file,
-                        int         line,
-                        const char *format, ... )  D_FORMAT_PRINTF(5);
+/* old */
+void DIRECT_API direct_debug_at( DirectLogDomain *domain,
+                                 const char      *format, ... ) D_FORMAT_PRINTF(2);
 
-void direct_break( const char *func,
-                   const char *file,
-                   int         line,
-                   const char *format, ... )  D_FORMAT_PRINTF(4);
+void DIRECT_API direct_debug_enter( DirectLogDomain *domain,
+                                    const char *func,
+                                    const char *file,
+                                    int         line,
+                                    const char *format, ... )  D_FORMAT_PRINTF(5);
 
-void direct_assertion( const char *exp,
-                       const char *func,
-                       const char *file,
-                       int         line );
+void DIRECT_API direct_debug_exit( DirectLogDomain *domain,
+                                   const char *func,
+                                   const char *file,
+                                   int         line,
+                                   const char *format, ... )  D_FORMAT_PRINTF(5);
 
-void direct_assumption( const char *exp,
-                        const char *func,
-                        const char *file,
-                        int         line );
+void DIRECT_API direct_break( const char *func,
+                              const char *file,
+                              int         line,
+                              const char *format, ... )  D_FORMAT_PRINTF(4);
+
+void DIRECT_API direct_assertion( const char *exp,
+                                  const char *func,
+                                  const char *file,
+                                  int         line );
+
+void DIRECT_API direct_assumption( const char *exp,
+                                   const char *func,
+                                   const char *file,
+                                   int         line );
 #endif
 
 #if DIRECT_BUILD_DEBUG || defined(DIRECT_ENABLE_DEBUG) || defined(DIRECT_FORCE_DEBUG)
 
 #define D_DEBUG_ENABLED  (1)
 
-#define D_DEBUG(x...)                                                                \
+#define D_DEBUG_LOG(_Domain,_level,...)                                              \
      do {                                                                            \
-          if (!direct_config || direct_config->debug)                                \
-               direct_debug( x );                                                    \
+          direct_debug_log( &_Domain, _level, __VA_ARGS__ );                         \
      } while (0)
 
 
-#define D_DEBUG_AT(d,x...)                                                           \
+// TODO: OPT: check domain etc here to avoid probably expensive parameter generation
+#define D_DEBUG_AT(d,...)                                                            \
      do {                                                                            \
-          direct_debug_at( &d, x );                                                  \
+          direct_debug_at( &d, __VA_ARGS__ );                                        \
      } while (0)
 
-#define D_DEBUG_ENTER(d,x...)                                                        \
+#define D_DEBUG_ENTER(d,...)                                                         \
      do {                                                                            \
-          direct_debug_enter( &d, __FUNCTION__, __FILE__, __LINE__, x );             \
+          /*direct_debug_enter( &d, __FUNCTION__, __FILE__, __LINE__, x );*/             \
      } while (0)
 
-#define D_DEBUG_EXIT(d,x...)                                                         \
+#define D_DEBUG_EXIT(d,...)                                                          \
      do {                                                                            \
-          direct_debug_exit( &d, __FUNCTION__, __FILE__, __LINE__, x );              \
+          /*direct_debug_exit( &d, __FUNCTION__, __FILE__, __LINE__, x );*/              \
      } while (0)
 
 #define D_ASSERT(exp)                                                                \
@@ -145,11 +161,14 @@ void direct_assumption( const char *exp,
      } while (0)
 
 
-#define D_BREAK(x...)                                                                \
+#define D_BREAK(...)                                                                 \
      do {                                                                            \
-          direct_break( __FUNCTION__, __FILE__, __LINE__, x );                       \
+          direct_break( __FUNCTION__, __FILE__, __LINE__, __VA_ARGS__ );                       \
      } while (0)
 
+#define D_DEBUG_CHECK(d)                                                             \
+     direct_log_domain_check( &d )
+
 #elif defined(DIRECT_MINI_DEBUG)
 
 /*
@@ -158,9 +177,16 @@ void direct_assumption( const char *exp,
 
 #define D_DEBUG_ENABLED  (2)
 
-#define D_DEBUG_AT(d,x...)                                                           \
+#define D_DEBUG_LOG(_Domain,_level,...)                                              \
      do {                                                                            \
-          if (direct_config->debug) direct_debug_at_always( &d, x );                 \
+          if (direct_config->log_level >= DIRECT_LOG_DEBUG)                          \
+               direct_debug_at_always( &d, __VA_ARGS__ );                            \
+     } while (0)
+
+#define D_DEBUG_AT(d,...)                                                            \
+     do {                                                                            \
+          if (direct_config->log_level >= DIRECT_LOG_DEBUG)                          \
+               direct_debug_at_always( &d, __VA_ARGS__ );                            \
      } while (0)
 
 #define D_CHECK(exp, aa)                                                             \
@@ -181,8 +207,17 @@ void direct_assumption( const char *exp,
 #define D_ASSERT(exp)    D_CHECK(exp, Assertion)
 #define D_ASSUME(exp)    D_CHECK(exp, Assumption)
 
+#define D_DEBUG_CHECK(d)                                                             \
+     direct_config->debug
+
 #endif    /* MINI_DEBUG  / DIRECT_BUILD_DEBUG || DIRECT_ENABLE_DEBUG || DIRECT_FORCE_DEBUG */
 
+
+#define D_DEBUG_AT__(d,...)                                                          \
+     do {                                                                            \
+          direct_log_printf( NULL, __VA_ARGS__ );                                    \
+     } while (0)
+
 #endif    /* DIRECT_BUILD_TEXT */
 
 
@@ -194,20 +229,26 @@ void direct_assumption( const char *exp,
 #define D_DEBUG_ENABLED  (0)
 #endif
 
+#ifndef D_DEBUG_LOG
+#define D_DEBUG_LOG(_Domain,_level,...)                                              \
+     do {                                                                            \
+     } while (0)
+#endif
+
 #ifndef D_DEBUG
-#define D_DEBUG(x...)              do {} while (0)
+#define D_DEBUG(d,...)             do {} while (0)
 #endif
 
 #ifndef D_DEBUG_AT
-#define D_DEBUG_AT(d,x...)         do {} while (0)
+#define D_DEBUG_AT(d,...)          do {} while (0)
 #endif
 
 #ifndef D_DEBUG_ENTER
-#define D_DEBUG_ENTER(d,x...)      do {} while (0)
+#define D_DEBUG_ENTER(d,...)       do {} while (0)
 #endif
 
 #ifndef D_DEBUG_EXIT
-#define D_DEBUG_EXIT(d,x...)       do {} while (0)
+#define D_DEBUG_EXIT(d,...)        do {} while (0)
 #endif
 
 #ifndef D_ASSERT
@@ -218,16 +259,24 @@ void direct_assumption( const char *exp,
 #define D_ASSUME(exp)              do {} while (0)
 #endif
 
+#ifndef D_DEBUG_AT__
+#define D_DEBUG_AT__(d,...)        do {} while (0)
+#endif
+
+#ifndef D_DEBUG_CHECK
+#define D_DEBUG_CHECK(d)           false
+#endif
+
 #ifndef D_BREAK
-#define D_BREAK(x...)              do {} while (0)
+#define D_BREAK(...)               do {} while (0)
 #endif
 
 #ifndef d_debug_at
-#define d_debug_at( domain, x... ) do {} while (0)
+#define d_debug_at( domain, ... )  do {} while (0)
 #endif
 
-#ifndef D_DEBUG_DOMAIN
-#define D_DEBUG_DOMAIN(i,n,d)
+#ifndef D_LOG_DOMAIN
+#define D_LOG_DOMAIN(i,n,d)
 #endif
 
 
@@ -245,6 +294,14 @@ void direct_assumption( const char *exp,
                                       ((spell)[sizeof(spell)*2/9] <<  8) | \
                                       ((spell)[sizeof(spell)*1/9]      )) )
 
+#define D_MAGIC_PRINT(m)           do {                                              \
+                                        D_INFO( "Magic '%s' = '%d'\n", #m, D_MAGIC(#m) );       \
+                                   } while (0)
+
+
+#if DIRECT_BUILD_DEBUGS
+
+#define D_MAGIC_CHECK(o,m)         ((o) != NULL && (o)->magic == D_MAGIC(#m))
 
 #define D_MAGIC_SET(o,m)           do {                                              \
                                         D_ASSERT( (o) != NULL );                     \
@@ -282,6 +339,81 @@ void direct_assumption( const char *exp,
                                         (o)->magic = 0;                              \
                                    } while (0)
 
+#define D_INDEX_ASSERT(index,array)                         \
+     do {                                                   \
+          D_ASSERT( index >= 0 );                           \
+          D_ASSERT( index < D_ARRAY_SIZE(array) );          \
+     } while (0)
+
+#else
+
+
+#define D_MAGIC_CHECK(o,m)         ((o) != NULL)
+
+#define D_MAGIC_SET(o,m)           do {                                              \
+                                   } while (0)
+
+#define D_MAGIC_SET_ONLY(o,m)      do {                                              \
+                                   } while (0)
+
+#define D_MAGIC_ASSERT(o,m)        do {                                              \
+                                   } while (0)
+
+#define D_MAGIC_ASSUME(o,m)        do {                                              \
+                                   } while (0)
+
+#define D_MAGIC_ASSERT_IF(o,m)     do {                                              \
+                                   } while (0)
+
+#define D_MAGIC_CLEAR(o)           do {                                              \
+                                   } while (0)
+
+#define D_INDEX_ASSERT(index,array)                         \
+     do {                                                   \
+     } while (0)
+
+#endif
+
+
+
+#define D_FLAGS_ASSERT(flags,f)    D_ASSERT( D_FLAGS_ARE_IN(flags,f) )
+
+
+
+#if D_DEBUG_ENABLED
+#define D_INFO_LINE()                                                                                    \
+     do {                                                                                                \
+          long long micros = direct_clock_get_micros();                                                  \
+          int       indent = direct_trace_debug_indent() * 2;                                            \
+                                                                                                         \
+          D_INFO( "[%-16.16s %3lld.%03lld,%03lld] (%5d) %*s%*s %s:%d\n",                                 \
+                  direct_thread_self_name()?:"NO NAME",                                                  \
+                  micros / 1000000LL, (micros / 1000LL) % 1000LL, micros % 1000LL, direct_gettid(),      \
+                  indent, "", indent-60, __FUNCTION__, __FILE__, __LINE__ );                             \
+     } while (0)
+
+#define D_INFO_LINE_MSG(...)                                                                             \
+     do {                                                                                                \
+          long long micros = direct_clock_get_micros();                                                  \
+          int       indent = direct_trace_debug_indent() * 2;                                            \
+          char      _buf[200];                                                                           \
+                                                                                                         \
+          direct_snprintf( _buf, sizeof(_buf), __VA_ARGS__ );                                            \
+                                                                                                         \
+          D_INFO( "[%-16.16s %3lld.%03lld,%03lld] (%5d) %*s%*s  '%-40s'  %s:%d\n",                       \
+                  direct_thread_self_name()?:"NO NAME",                                                  \
+                  micros / 1000000LL, (micros / 1000LL) % 1000LL, micros % 1000LL, direct_gettid(),      \
+                  indent, "", indent-60, __FUNCTION__, _buf, __FILE__, __LINE__ );                       \
+     } while (0)
+#else
+#define D_INFO_LINE()                                                                                    \
+     do {                                                                                                \
+     } while (0)
+
+#define D_INFO_LINE_MSG(...)                                                                             \
+     do {                                                                                                \
+     } while (0)
+#endif
 
 #endif
 
