@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,12 +28,10 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
-#include <stdlib.h>
-
-#include <pthread.h>
-
 #include <direct/debug.h>
 #include <direct/direct.h>
 #include <direct/interface.h>
@@ -42,7 +42,7 @@
 #include <direct/thread.h>
 #include <direct/util.h>
 
-D_DEBUG_DOMAIN( Direct_Main, "Direct/Main", "Initialization and shutdown of libdirect" );
+D_LOG_DOMAIN( Direct_Main, "Direct/Main", "Initialization and shutdown of libdirect" );
 
 /**************************************************************************************************/
 
@@ -57,35 +57,42 @@ struct __D_DirectCleanupHandler {
 
 /**************************************************************************************************/
 
-static int              refs      = 0;
-static DirectLink      *handlers  = NULL; 
-static pthread_mutex_t  main_lock = PTHREAD_MUTEX_INITIALIZER;
+static int          refs;
+static DirectLink  *handlers;
+static DirectMutex  main_lock;
 
-/**************************************************************************************************/
-
-static void
-direct_cleanup( void )
+void
+__D_direct_init()
 {
-     DirectCleanupHandler *handler, *temp;
-     
-     if (!refs)
-          return;
-          
-     direct_list_foreach_safe (handler, temp, handlers) {
-          D_DEBUG_AT( Direct_Main, "Calling cleanup func %p...\n", handler->func );
+     direct_mutex_init( &main_lock );
+}
 
-          handler->func( handler->ctx );
-          
-          /*direct_list_remove( &handlers, &handler->link );
-
-          D_MAGIC_CLEAR( handler );
-
-          D_FREE( handler );*/
-     }
-     
+void
+__D_direct_deinit()
+{
      direct_print_memleaks();
 
      direct_print_interface_leaks();
+
+     direct_mutex_deinit( &main_lock );
+}
+
+static void
+direct_atexit_handler( void )
+{
+     DirectCleanupHandler *handler, *temp;
+
+     direct_list_foreach_safe (handler, temp, handlers) {
+          D_DEBUG_AT( Direct_Main, "Calling cleanup func %p( %p )...\n", handler->func, handler->ctx );
+
+          direct_list_remove( &handlers, &handler->link );
+
+          handler->func( handler->ctx );
+
+          D_MAGIC_CLEAR( handler );
+
+          D_FREE( handler );
+     }
 }
 
 /**************************************************************************************************/
@@ -103,7 +110,7 @@ direct_cleanup_handler_add( DirectCleanupHandlerFunc   func,
      D_DEBUG_AT( Direct_Main,
                  "Adding cleanup handler %p with context %p...\n", func, ctx );
      
-     handler = D_CALLOC( 1, sizeof(DirectCleanupHandler) );
+     handler = (DirectCleanupHandler*) D_CALLOC( 1, sizeof(DirectCleanupHandler) );
      if (!handler) {
           D_WARN( "out of memory" );
           return DR_NOLOCALMEMORY;
@@ -114,14 +121,14 @@ direct_cleanup_handler_add( DirectCleanupHandlerFunc   func,
      
      D_MAGIC_SET( handler, DirectCleanupHandler );
 
-     pthread_mutex_lock( &main_lock );
+     direct_mutex_lock( &main_lock );
      
      if (handlers == NULL)
-          atexit( direct_cleanup );
+          atexit( direct_atexit_handler );
      
      direct_list_append( &handlers, &handler->link );
      
-     pthread_mutex_unlock( &main_lock );
+     direct_mutex_unlock( &main_lock );
      
      *ret_handler = handler;
      
@@ -138,59 +145,68 @@ direct_cleanup_handler_remove( DirectCleanupHandler *handler )
      D_DEBUG_AT( Direct_Main, "Removing cleanup handler %p with context %p...\n", 
                  handler->func, handler->ctx );
 
-     pthread_mutex_lock( &main_lock );
-     direct_list_remove( &handlers, &handler->link );
-     pthread_mutex_unlock( &main_lock );
-
-     D_MAGIC_CLEAR( handler );
-
-     D_FREE( handler );
+     /* Safety check, in case handler is removed from within itself being invoked */
+     if (handler->link.prev) {
+          direct_mutex_lock( &main_lock );
+          direct_list_remove( &handlers, &handler->link );
+          direct_mutex_unlock( &main_lock );
+     
+          D_MAGIC_CLEAR( handler );
+     
+          D_FREE( handler );
+     }
 
      return DR_OK;
 }
 
 DirectResult
-direct_initialize( void )
+direct_initialize()
 {
-     pthread_mutex_lock( &main_lock );
+     direct_clock_set_time( DIRECT_CLOCK_SESSION, 0 );
 
-     D_DEBUG_AT( Direct_Main, "direct_initialize() called...\n" );
+     direct_mutex_lock( &main_lock );
+
+     D_DEBUG_AT( Direct_Main, "direct_initialize() called... (from %s)\n",
+                 direct_trace_lookup_symbol_at( direct_trace_get_caller() ) );
 
      if (refs++) {
           D_DEBUG_AT( Direct_Main, "...%d references now.\n", refs );
-          pthread_mutex_unlock( &main_lock );
+          direct_mutex_unlock( &main_lock );
           return DR_OK;
      }
      else if (!direct_thread_self_name())
           direct_thread_set_name( "Main Thread" );
 
-     D_DEBUG_AT( Direct_Main, "...initializing now.\n" );
+     D_DEBUG_AT( Direct_Main, "...first reference, initializing now.\n" );
 
      direct_signals_initialize();
 
-     pthread_mutex_unlock( &main_lock );
+     direct_mutex_unlock( &main_lock );
 
      return DR_OK;
 }
 
 DirectResult
-direct_shutdown( void )
+direct_shutdown()
 {
-     pthread_mutex_lock( &main_lock );
+     direct_mutex_lock( &main_lock );
 
-     D_DEBUG_AT( Direct_Main, "direct_shutdown() called...\n" );
+     D_DEBUG_AT( Direct_Main, "direct_shutdown() called... (from %s)\n",
+                 direct_trace_lookup_symbol_at( direct_trace_get_caller() ) );
 
-     if (--refs) {
-          D_DEBUG_AT( Direct_Main, "...%d references left.\n", refs );
-          pthread_mutex_unlock( &main_lock );
-          return DR_OK;
+     if (refs == 1) {
+          D_DEBUG_AT( Direct_Main, "...shutting down now.\n" );
+
+          direct_signals_shutdown();
+
+          D_DEBUG_AT( Direct_Main, "  -> done.\n" );
      }
+     else
+          D_DEBUG_AT( Direct_Main, "...%d references left.\n", refs - 1 );
 
-     D_DEBUG_AT( Direct_Main, "...shutting down now.\n" );
+     refs--;
 
-     direct_signals_shutdown();
-
-     pthread_mutex_unlock( &main_lock );
+     direct_mutex_unlock( &main_lock );
 
      return DR_OK;
 }
