@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -30,34 +32,35 @@
 
 #include <config.h>
 
-#include <sys/time.h>
 #include <time.h>
 
 #include <stdlib.h>
 #include <string.h>
 
-#include <dfb_types.h>
-
 #include <direct/conf.h>
 #include <direct/debug.h>
 #include <direct/mem.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
 
-#if defined (ARCH_PPC) || (SIZEOF_LONG == 8)
+#if defined (ARCH_PPC) || defined (ARCH_ARM) || (SIZEOF_LONG == 8)
 # define RUN_BENCHMARK  1
 #else
 # define RUN_BENCHMARK  0
 #endif
 
 #if RUN_BENCHMARK
-D_DEBUG_DOMAIN( Direct_Memcpy, "Direct/Memcpy", "Direct's Memcpy Routines" );
+D_LOG_DOMAIN( Direct_Memcpy, "Direct/Memcpy", "Direct's Memcpy Routines" );
 #endif
 
 #ifdef USE_PPCASM
 #include "ppcasm_memcpy.h"
 #endif
 
+#if defined(USE_ARMASM) && !defined(WORDS_BIGENDIAN)
+#include "armasm_memcpy.h"
+#endif
+
 
 #if SIZEOF_LONG == 8
 
@@ -73,7 +76,7 @@ static void * generic64_memcpy( void * to, const void * from, size_t len )
           /* Align destination to 8-byte boundary */
           delta = (unsigned long)d & 7;
           if (delta) {
-               len -= 8 - delta;                 
+               len -= 8 - delta;
 
                if ((unsigned long)d & 1) {
                     *d++ = *s++;
@@ -87,10 +90,10 @@ static void * generic64_memcpy( void * to, const void * from, size_t len )
                     d += 4; s += 4;
                }
           }
-          
+
           n    = len >> 6;
           len &= 63;
-          
+
           for (; n; n--) {
                ((u64*)d)[0] = ((const u64*)s)[0];
                ((u64*)d)[1] = ((const u64*)s)[1];
@@ -108,7 +111,7 @@ static void * generic64_memcpy( void * to, const void * from, size_t len )
       */
      if (len) {
           n = len >> 3;
-          
+
           for (; n; n--) {
                *((u64*)d) = *((const u64*)s);
                d += 8; s += 8;
@@ -124,7 +127,7 @@ static void * generic64_memcpy( void * to, const void * from, size_t len )
           if (len & 1)
                *d = *s;
      }
-     
+
      return to;
 }
 
@@ -133,6 +136,14 @@ static void * generic64_memcpy( void * to, const void * from, size_t len )
 
 typedef void* (*memcpy_func)(void *to, const void *from, size_t len);
 
+
+static void *
+std_memcpy( void *to, const void *from, size_t len )
+{
+     return memcpy( to, from, len );
+}
+
+
 static struct {
      char                 *name;
      char                 *desc;
@@ -142,7 +153,7 @@ static struct {
 } memcpy_method[] =
 {
      { NULL, NULL, NULL, 0, 0},
-     { "libc",     "libc memcpy()",             (memcpy_func) memcpy, 0, 0},
+     { "libc",     "libc memcpy()",             std_memcpy, 0, 0},
 #if SIZEOF_LONG == 8
      { "generic64","Generic 64bit memcpy()",    generic64_memcpy, 0, 0},
 #endif /* SIZEOF_LONG == 8 */
@@ -152,20 +163,15 @@ static struct {
      { "ppccache", "ppcasm_cacheable_memcpy()",  direct_ppcasm_cacheable_memcpy, 0, 0},
 #endif /* __LINUX__ */
 #endif /* USE_PPCASM */
+#if defined(USE_ARMASM) && !defined(WORDS_BIGENDIAN)
+     { "arm",      "armasm_memcpy()",            direct_armasm_memcpy, 0, 0},
+#endif
      { NULL, NULL, NULL, 0, 0}
 };
 
 
-static inline unsigned long long int rdtsc( void )
-{
-     struct timeval tv;
 
-     gettimeofday (&tv, NULL);
-     return (tv.tv_sec * 1000000 + tv.tv_usec);
-}
-
-
-memcpy_func direct_memcpy = (memcpy_func) memcpy;
+memcpy_func direct_memcpy = std_memcpy;
 
 #define BUFSIZE 1024
 
@@ -213,12 +219,12 @@ direct_find_best_memcpy( void )
           if (memcpy_method[i].cpu_require & ~config_flags)
                continue;
 
-          t = rdtsc();
+          t = direct_clock_get_time( DIRECT_CLOCK_MONOTONIC );
 
           for (j=0; j<500; j++)
                memcpy_method[i].function( buf1 + j*BUFSIZE, buf2 + j*BUFSIZE, BUFSIZE );
 
-          t = rdtsc() - t;
+          t = direct_clock_get_time( DIRECT_CLOCK_MONOTONIC ) - t;
           memcpy_method[i].time = t;
 
           D_DEBUG_AT( Direct_Memcpy, "\t%-10s  %20lld\n", memcpy_method[i].name, t );
