@@ -1,29 +1,30 @@
 /*
    GLIB - Library of useful routines for C programming
-   Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
- 
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald 
+
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
-              Ville Syrjälä <syrjala@sci.fi>,
-              Claudio Ciccani <klan@users.sf.net> and
-              Michael Emmel <mike.emmel@gmail.com>.
- 
+              Ville Syrjälä <syrjala@sci.fi> and
+              Claudio Ciccani <klan@users.sf.net>.
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
- 
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
- 
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -53,8 +54,13 @@
 
 D_DEBUG_DOMAIN( Fusion_Hash, "Fusion/Hash", "Hash table implementation" );
 
-
-
+#define HASH_STR(h,p) \
+{\
+    h = *p;\
+    if (h)\
+        for (p += 1; *p != '\0'; p++)\
+            h = (h << 5) - h + *p;\
+}\
 
 static const unsigned int primes[] =
 {
@@ -110,13 +116,46 @@ fusion_hash_node_destroy (FusionHash *hash,FusionHashNode *node,
 static unsigned int
 spaced_primes_closest (unsigned int num)
 {
-     int i;
+     unsigned int i;
      for (i = 0; i < nprimes; i++)
           if (primes[i] > num)
                return primes[i];
      return primes[nprimes - 1];
 }
 
+static __inline__ FusionHashNode**
+fusion_hash_lookup_node (FusionHash *hash,
+                         const void *key)
+{
+     FusionHashNode **node;
+
+     /*TODO We could also optimize pointer hashing*/
+     if (hash->key_type == HASH_STRING )
+     {
+          unsigned int h;
+          const signed char *p = key;
+          HASH_STR(h,p)
+          node = &hash->nodes[h % hash->size];
+     }
+     else
+          node = &hash->nodes[((unsigned long)key) % hash->size];
+
+     /* Hash table lookup needs to be fast.
+     *  We therefore remove the extra conditional of testing
+     *  whether to call the key_equal_func or not from
+     *  the inner loop.
+     */
+     if (hash->key_type == HASH_STRING ) {
+          while(*node && strcmp((const char *)(*node)->key,(const char*)key))
+               node = &(*node)->next;
+     } else
+          while (*node && (*node)->key != key)
+               node = &(*node)->next;
+
+     return node;
+
+}
+
 /**
  * fusion_hash_create_local:
  * @key_type: Type of hash key the hash is optimized for strings ints and pointers 
@@ -349,6 +388,9 @@ fusion_hash_replace (FusionHash *hash,
                          SHFREE(hash->pool, (*node)->value );
                }
           }
+
+          (*node)->key = (void*)key;
+          (*node)->value = (void*)value;
      }
      else {
           if (hash->local)
@@ -359,10 +401,14 @@ fusion_hash_replace (FusionHash *hash,
           if ( !(*node) )
                return hash->local?DR_NOLOCALMEMORY:DR_NOSHAREDMEMORY;
 
+          (*node)->key = (void*)key;
+          (*node)->value = (void*)value;
+
           hash->nnodes++;
+
+          if (fusion_hash_should_resize(hash))
+               fusion_hash_resize(hash);
      }
-     (*node)->key = (void*)key;
-     (*node)->value = (void*)value;
 
      return DR_OK;
 }
