@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -401,8 +405,6 @@ fusion_property_destroy (FusionProperty *property)
 
 #else /* FUSION_BUILD_MULTI */
 
-#include <pthread.h>
-
 /*
  * Initializes the property
  */
@@ -411,8 +413,8 @@ fusion_property_init (FusionProperty *property, const FusionWorld *world)
 {
      D_ASSERT( property != NULL );
 
-     direct_util_recursive_pthread_mutex_init (&property->single.lock);
-     pthread_cond_init (&property->single.cond, NULL);
+     direct_recursive_mutex_init (&property->single.lock);
+     direct_waitqueue_init (&property->single.cond);
 
      property->single.state = FUSION_PROPERTY_AVAILABLE;
 
@@ -429,11 +431,11 @@ fusion_property_lease (FusionProperty *property)
 
      D_ASSERT( property != NULL );
 
-     pthread_mutex_lock (&property->single.lock);
+     direct_mutex_lock (&property->single.lock);
 
      /* Wait as long as the property is leased by another party. */
      while (property->single.state == FUSION_PROPERTY_LEASED)
-          pthread_cond_wait (&property->single.cond, &property->single.lock);
+          direct_waitqueue_wait (&property->single.cond, &property->single.lock);
 
      /* Fail if purchased by another party, otherwise succeed. */
      if (property->single.state == FUSION_PROPERTY_PURCHASED)
@@ -441,7 +443,7 @@ fusion_property_lease (FusionProperty *property)
      else
           property->single.state = FUSION_PROPERTY_LEASED;
 
-     pthread_mutex_unlock (&property->single.lock);
+     direct_mutex_unlock (&property->single.lock);
 
      return ret;
 }
@@ -456,11 +458,11 @@ fusion_property_purchase (FusionProperty *property)
 
      D_ASSERT( property != NULL );
 
-     pthread_mutex_lock (&property->single.lock);
+     direct_mutex_lock (&property->single.lock);
 
      /* Wait as long as the property is leased by another party. */
      while (property->single.state == FUSION_PROPERTY_LEASED)
-          pthread_cond_wait (&property->single.cond, &property->single.lock);
+          direct_waitqueue_wait (&property->single.cond, &property->single.lock);
 
      /* Fail if purchased by another party, otherwise succeed. */
      if (property->single.state == FUSION_PROPERTY_PURCHASED)
@@ -469,10 +471,10 @@ fusion_property_purchase (FusionProperty *property)
           property->single.state = FUSION_PROPERTY_PURCHASED;
 
           /* Wake up any other waiting party. */
-          pthread_cond_broadcast (&property->single.cond);
+          direct_waitqueue_broadcast (&property->single.cond);
      }
 
-     pthread_mutex_unlock (&property->single.lock);
+     direct_mutex_unlock (&property->single.lock);
 
      return ret;
 }
@@ -485,7 +487,7 @@ fusion_property_cede (FusionProperty *property)
 {
      D_ASSERT( property != NULL );
 
-     pthread_mutex_lock (&property->single.lock);
+     direct_mutex_lock (&property->single.lock);
 
      /* Simple error checking, maybe we should also check the owner. */
      D_ASSERT( property->single.state != FUSION_PROPERTY_AVAILABLE );
@@ -494,9 +496,9 @@ fusion_property_cede (FusionProperty *property)
      property->single.state = FUSION_PROPERTY_AVAILABLE;
 
      /* Wake up one waiting party if there are any. */
-     pthread_cond_signal (&property->single.cond);
+     direct_waitqueue_signal (&property->single.cond);
 
-     pthread_mutex_unlock (&property->single.lock);
+     direct_mutex_unlock (&property->single.lock);
 
      return DR_OK;
 }
@@ -520,8 +522,8 @@ fusion_property_destroy (FusionProperty *property)
 {
      D_ASSERT( property != NULL );
 
-     pthread_cond_destroy (&property->single.cond);
-     pthread_mutex_destroy (&property->single.lock);
+     direct_waitqueue_deinit (&property->single.cond);
+     direct_mutex_deinit (&property->single.lock);
 
      return DR_OK;
 }
