@@ -1,15 +1,19 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
+   Fusion shmalloc is based on GNU malloc. Please see below.
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
@@ -26,10 +30,11 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <errno.h>
 
 #include <direct/build.h>
@@ -40,12 +45,13 @@
 #include <fusion/build.h>
 #include <fusion/shmalloc.h>
 
-#include <fusion/fusion_internal.h>
 #include <fusion/shm/shm_internal.h>
 
 
 #if FUSION_BUILD_MULTI
 
+#include <fusion/fusion_internal.h>
+
 /*************************** MULTI APPLICATION CORE ***************************/
 
 #if DIRECT_BUILD_DEBUGS  /* Build with debug support? */
@@ -94,8 +100,8 @@ fill_shmem_desc( SHMemDesc *desc, int bytes, const char *func, const char *file,
      desc->mem   = desc + 1;
      desc->bytes = bytes;
 
-     snprintf( desc->func, SHMEMDESC_FUNC_NAME_LENGTH, func );
-     snprintf( desc->file, SHMEMDESC_FILE_NAME_LENGTH, file );
+     direct_snputs( desc->func, func, SHMEMDESC_FUNC_NAME_LENGTH );
+     direct_snputs( desc->file, file, SHMEMDESC_FILE_NAME_LENGTH );
 
      desc->line = line;
      desc->fid  = fusion_id;
@@ -533,7 +539,7 @@ fusion_dbg_shmalloc( FusionSHMPoolShared *pool,
      D_ASSERT( __size > 0 );
 
      if (pool->debug)
-          return direct_malloc( file, line, func, __size );
+          return direct_dbg_malloc( file, line, func, __size );
 
      return malloc( __size );
 }
@@ -549,7 +555,7 @@ fusion_dbg_shcalloc( FusionSHMPoolShared *pool,
      D_ASSERT( __size > 0 );
 
      if (pool->debug)
-          return direct_calloc( file, line, func, __nmemb, __size );
+          return direct_dbg_calloc( file, line, func, __nmemb, __size );
 
      return calloc( __nmemb, __size );
 }
@@ -565,7 +571,7 @@ fusion_dbg_shrealloc( FusionSHMPoolShared *pool,
      D_MAGIC_ASSERT( pool, FusionSHMPoolShared );
 
      if (pool->debug)
-          return direct_realloc( file, line, func, what, __ptr, __size );
+          return direct_dbg_realloc( file, line, func, what, __ptr, __size );
 
      return realloc( __ptr, __size );
 }
@@ -580,7 +586,7 @@ fusion_dbg_shfree( FusionSHMPoolShared *pool,
      D_ASSERT( __ptr != NULL );
 
      if (pool->debug)
-          direct_free( file, line, func, what, __ptr );
+          direct_dbg_free( file, line, func, what, __ptr );
      else
           free( __ptr );
 }
@@ -595,7 +601,7 @@ fusion_dbg_shstrdup( FusionSHMPoolShared *pool,
      D_ASSERT( string != NULL );
 
      if (pool->debug)
-          return direct_strdup( file, line, func, string );
+          return direct_dbg_strdup( file, line, func, string );
 
      return strdup( string );
 }
