@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __FUSION__VECTOR_H__
 #define __FUSION__VECTOR_H__
 
@@ -46,30 +50,30 @@ typedef struct {
      FusionSHMPoolShared *pool;
 } FusionVector;
 
-void         fusion_vector_init       ( FusionVector        *vector,
-                                        int                  capacity,
-                                        FusionSHMPoolShared *pool );
+void         FUSION_API fusion_vector_init       ( FusionVector        *vector,
+                                                   int                  capacity,
+                                                   FusionSHMPoolShared *pool );
 
-void         fusion_vector_destroy    ( FusionVector        *vector );
+void         FUSION_API fusion_vector_destroy    ( FusionVector        *vector );
 
-DirectResult fusion_vector_add        ( FusionVector        *vector,
-                                        void                *element );
+DirectResult FUSION_API fusion_vector_add        ( FusionVector        *vector,
+                                                   void                *element );
 
-DirectResult fusion_vector_insert     ( FusionVector        *vector,
-                                        void                *element,
-                                        int                  index );
+DirectResult FUSION_API fusion_vector_insert     ( FusionVector        *vector,
+                                                   void                *element,
+                                                   int                  index );
 
-DirectResult fusion_vector_move       ( FusionVector        *vector,
-                                        int                  from,
-                                        int                  to );
+DirectResult FUSION_API fusion_vector_move       ( FusionVector        *vector,
+                                                   int                  from,
+                                                   int                  to );
 
-DirectResult fusion_vector_remove     ( FusionVector        *vector,
-                                        int                  index );
+DirectResult FUSION_API fusion_vector_remove     ( FusionVector        *vector,
+                                                   int                  index );
 
-DirectResult fusion_vector_remove_last( FusionVector        *vector );
+DirectResult FUSION_API fusion_vector_remove_last( FusionVector        *vector );
 
 
-static inline bool
+static __inline__ bool
 fusion_vector_has_elements( const FusionVector *vector )
 {
      D_MAGIC_ASSERT( vector, FusionVector );
@@ -77,7 +81,7 @@ fusion_vector_has_elements( const FusionVector *vector )
      return vector->count > 0;
 }
 
-static inline bool
+static __inline__ bool
 fusion_vector_is_empty( const FusionVector *vector )
 {
      D_MAGIC_ASSERT( vector, FusionVector );
@@ -85,7 +89,7 @@ fusion_vector_is_empty( const FusionVector *vector )
      return vector->count == 0;
 }
 
-static inline int
+static __inline__ int
 fusion_vector_size( const FusionVector *vector )
 {
      D_MAGIC_ASSERT( vector, FusionVector );
@@ -93,7 +97,7 @@ fusion_vector_size( const FusionVector *vector )
      return vector->count;
 }
 
-static inline void *
+static __inline__ void *
 fusion_vector_at( const FusionVector *vector, int index )
 {
      D_MAGIC_ASSERT( vector, FusionVector );
@@ -103,7 +107,7 @@ fusion_vector_at( const FusionVector *vector, int index )
      return vector->elements[index];
 }
 
-static inline bool
+static __inline__ bool
 fusion_vector_contains( const FusionVector *vector, const void *element )
 {
      int           i;
@@ -124,7 +128,7 @@ fusion_vector_contains( const FusionVector *vector, const void *element )
      return false;
 }
 
-static inline int
+static __inline__ int
 fusion_vector_index_of( const FusionVector *vector, const void *element )
 {
      int           i;
@@ -152,12 +156,14 @@ fusion_vector_index_of( const FusionVector *vector, const void *element )
 
 #define fusion_vector_foreach(element, index, vector)                         \
      for ((index) = 0;                                                        \
-          (index) < (vector).count && (element = (vector).elements[index]);   \
+          (index) < (vector).count && (element = (__typeof__(element))(vector).elements[index]);   \
           (index)++)
 
 #define fusion_vector_foreach_reverse(element, index, vector)                 \
      for ((index) = (vector).count - 1;                                       \
-          (index) >= 0 && (element = (vector).elements[index]);               \
+          (index) >= 0 && (vector).count &&                                   \
+               (vector).elements &&                                           \
+               (element = (__typeof__(element))(vector).elements[index]);     \
           (index)--)
 
 #endif
