@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,18 +28,41 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __VOODOO__CONF_H__
 #define __VOODOO__CONF_H__
 
-#include <voodoo/types.h>
+#include <voodoo/play.h>
 
 
 struct __V_VoodooConfig {
-     /* nothing yet */
+     VoodooPlayInfo  play_info;
+     bool            forward_nodes;
+     unsigned int    memory_max;
+     unsigned int    surface_max;
+     unsigned int    layer_mask;
+     unsigned int    stacking_mask;
+     unsigned int    resource_id;
+     bool            server_fork;
+     char           *server_single;
+     char           *play_broadcast;
+     unsigned int    compression_min;
+     bool            link_raw;
+     bool            link_packet;
 };
 
-extern VoodooConfig *voodoo_config;
+extern VoodooConfig VOODOO_API *voodoo_config;
 
 
+DirectResult        VOODOO_API  voodoo_config_set( const char *name, const char *value );
+
+const char          VOODOO_API *voodoo_config_usage( void );
+
+
+
+void __Voodoo_conf_init( void );
+void __Voodoo_conf_deinit( void );
+
 #endif
 
