@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,23 +28,25 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __VOODOO__INTERFACE_H__
 #define __VOODOO__INTERFACE_H__
 
 #include <voodoo/types.h>
 
 
-DirectResult
+DirectResult VOODOO_API
 voodoo_construct_requestor( VoodooManager     *manager,
                             const char        *name,
                             VoodooInstanceID   instance,
                             void              *arg,
                             void             **ret_interface );
 
-DirectResult
+DirectResult VOODOO_API
 voodoo_construct_dispatcher( VoodooManager     *manager,
                              const char        *name,
-                             void              *interface,
+                             void              *interface_ptr,
                              VoodooInstanceID   super,
                              void              *arg,
                              VoodooInstanceID  *ret_instance,
