@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __VOODOO__MESSAGE_H__
 #define __VOODOO__MESSAGE_H__
 
@@ -57,7 +61,10 @@ typedef enum {
 typedef enum {
      VMSG_SUPER,
      VMSG_REQUEST,
-     VMSG_RESPONSE
+     VMSG_RESPONSE,
+
+     VMSG_DISCOVER, // temporary solution for compatibility
+     VMSG_SENDINFO, // temporary solution for compatibility
 } VoodooMessageType;
 
 
@@ -94,23 +101,26 @@ struct __V_VoodooResponseMessage {
 typedef struct {
      int         magic;
 
-     const void *msg;
-     const void *ptr;
+     const char *msg;
+     const char *ptr;
 } VoodooMessageParser;
 
 
 
 #define __VOODOO_PARSER_PROLOG( parser, req_type )          \
-     const void             *_vp_ptr;                       \
+     const char             *_vp_ptr;                       \
      VoodooMessageBlockType  _vp_type;                      \
      int                     _vp_length;                    \
+     VoodooMessageParser    *_parser = &parser;             \
                                                             \
-     D_MAGIC_ASSERT( &(parser), VoodooMessageParser );      \
+     D_MAGIC_ASSERT( _parser, VoodooMessageParser );        \
                                                             \
-     _vp_ptr = (parser).ptr;                                \
+     (void)_vp_type;                                        \
+                                                            \
+     _vp_ptr = _parser->ptr;                                \
                                                             \
      /* Read message block type. */                         \
-     _vp_type = *(const u32*) _vp_ptr;                      \
+     _vp_type = *(const VoodooMessageBlockType*) _vp_ptr;   \
                                                             \
      D_ASSERT( _vp_type == (req_type) );                    \
                                                             \
@@ -120,21 +130,22 @@ typedef struct {
 
 #define __VOODOO_PARSER_EPILOG( parser )                    \
      /* Advance message data pointer. */                    \
-     (parser).ptr += 8 + VOODOO_MSG_ALIGN(_vp_length)
+     _parser->ptr += 8 + VOODOO_MSG_ALIGN(_vp_length)
 
 
 #define VOODOO_PARSER_BEGIN( parser, message )                                                 \
      do {                                                                                      \
           const VoodooMessageHeader *_vp_header = (const VoodooMessageHeader *) (message);     \
+          VoodooMessageParser       *_parser = &parser;                                        \
                                                                                                \
           D_ASSERT( (message) != NULL );                                                       \
           D_ASSERT( _vp_header->type == VMSG_REQUEST || _vp_header->type == VMSG_RESPONSE );   \
                                                                                                \
-          (parser).msg = (message);                                                            \
-          (parser).ptr = (parser).msg + (_vp_header->type == VMSG_REQUEST ?                    \
+          _parser->msg = (const char*)(message);                                               \
+          _parser->ptr = _parser->msg + (_vp_header->type == VMSG_REQUEST ?                    \
                               sizeof(VoodooRequestMessage) : sizeof(VoodooResponseMessage));   \
                                                                                                \
-          D_MAGIC_SET_ONLY( &(parser), VoodooMessageParser );                                  \
+          D_MAGIC_SET_ONLY( _parser, VoodooMessageParser );                                    \
      } while (0)
 
 
@@ -178,10 +189,10 @@ typedef struct {
      do {                                                             \
           __VOODOO_PARSER_PROLOG( parser, VMBT_DATA );                \
                                                                       \
-          D_ASSERT( _vp_length > 0 );                                 \
+          /*D_ASSERT( _vp_length > 0 );*/                                 \
                                                                       \
           /* Return pointer to data. */                               \
-          (ret_data) = _vp_ptr + 8;                                   \
+          (ret_data) = (__typeof__(ret_data))(_vp_ptr + 8);           \
                                                                       \
           __VOODOO_PARSER_EPILOG( parser );                           \
      } while (0)
@@ -190,7 +201,7 @@ typedef struct {
      do {                                                             \
           __VOODOO_PARSER_PROLOG( parser, VMBT_DATA );                \
                                                                       \
-          D_ASSERT( _vp_length > 0 );                                 \
+          /*D_ASSERT( _vp_length > 0 );*/                                 \
           D_ASSERT( _vp_length <= max_len );                          \
                                                                       \
           /* Copy data block. */                                      \
@@ -203,7 +214,7 @@ typedef struct {
      do {                                                             \
           __VOODOO_PARSER_PROLOG( parser, VMBT_DATA );                \
                                                                       \
-          D_ASSERT( _vp_length > 0 );                                 \
+          /*D_ASSERT( _vp_length > 0 );*/                                 \
                                                                       \
           /* Allocate memory on the stack. */                         \
           (ret_data) = alloca( _vp_length );                          \
@@ -222,7 +233,7 @@ typedef struct {
                                                                       \
           /* Return pointer to data or NULL. */                       \
           if (_vp_length)                                             \
-               (ret_data) = _vp_ptr + 8;                              \
+               (ret_data) = (__typeof__(ret_data))(_vp_ptr + 8);      \
           else                                                        \
                (ret_data) = NULL;                                     \
                                                                       \
@@ -244,11 +255,13 @@ typedef struct {
 
 #define VOODOO_PARSER_END( parser )                                   \
      do {                                                             \
-          D_MAGIC_ASSERT( &(parser), VoodooMessageParser );           \
+          VoodooMessageParser *_parser = &parser;                     \
                                                                       \
-          D_ASSUME( *(const u32*) ((parser).ptr) == VMBT_NONE );      \
+          D_MAGIC_ASSERT( _parser, VoodooMessageParser );             \
                                                                       \
-          D_MAGIC_CLEAR( &(parser) );                                 \
+          D_ASSUME( *(const u32*) (_parser->ptr) == VMBT_NONE );      \
+                                                                      \
+          D_MAGIC_CLEAR( _parser );                                   \
      } while (0)
 
 
