@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,17 +28,12 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#include <string.h>
 
 #include <directfb.h>
 
@@ -49,15 +46,18 @@
 #include <direct/thread.h>
 #include <direct/util.h>
 
-#include <core/coredefs.h>
+#include <misc/conf.h>
 
+#include <idirectfb.h>
+
+#include <voodoo/conf.h>
 #include <voodoo/interface.h>
 #include <voodoo/manager.h>
 #include <voodoo/message.h>
 
 #include "idirectfb_dispatcher.h"
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFB        *thiz,
                             VoodooManager    *manager,
                             VoodooInstanceID *ret_instance );
@@ -84,6 +84,8 @@ typedef struct {
 
      IDirectFB             *real;
 
+     CoreDFB               *core;
+
      VoodooInstanceID       self;         /* The instance of this dispatcher itself. */
 
      DirectLink            *data_buffers; /* list of known data buffers */
@@ -107,6 +109,8 @@ IDirectFB_Dispatcher_Destruct( IDirectFB *thiz )
           D_FREE( entry );
      }
 
+     data->real->Release( data->real );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
@@ -189,11 +193,11 @@ IDirectFB_Dispatcher_SetVideoMode( IDirectFB    *thiz,
 static DFBResult
 IDirectFB_Dispatcher_CreateSurface( IDirectFB                    *thiz,
                                     const DFBSurfaceDescription  *desc,
-                                    IDirectFBSurface            **interface )
+                                    IDirectFBSurface            **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!desc || !interface)
+     if (!desc || !interface_ptr)
           return DFB_INVARG;
 
 
@@ -204,11 +208,11 @@ IDirectFB_Dispatcher_CreateSurface( IDirectFB                    *thiz,
 static DFBResult
 IDirectFB_Dispatcher_CreatePalette( IDirectFB                    *thiz,
                                     const DFBPaletteDescription  *desc,
-                                    IDirectFBPalette            **interface )
+                                    IDirectFBPalette            **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -234,11 +238,11 @@ IDirectFB_Dispatcher_EnumScreens( IDirectFB         *thiz,
 static DFBResult
 IDirectFB_Dispatcher_GetScreen( IDirectFB        *thiz,
                                 DFBScreenID       id,
-                                IDirectFBScreen **interface )
+                                IDirectFBScreen **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -264,11 +268,11 @@ IDirectFB_Dispatcher_EnumDisplayLayers( IDirectFB               *thiz,
 static DFBResult
 IDirectFB_Dispatcher_GetDisplayLayer( IDirectFB              *thiz,
                                       DFBDisplayLayerID       id,
-                                      IDirectFBDisplayLayer **interface )
+                                      IDirectFBDisplayLayer **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -294,11 +298,11 @@ IDirectFB_Dispatcher_EnumInputDevices( IDirectFB              *thiz,
 static DFBResult
 IDirectFB_Dispatcher_GetInputDevice( IDirectFB             *thiz,
                                      DFBInputDeviceID       id,
-                                     IDirectFBInputDevice **interface )
+                                     IDirectFBInputDevice **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -308,11 +312,11 @@ IDirectFB_Dispatcher_GetInputDevice( IDirectFB             *thiz,
 
 static DFBResult
 IDirectFB_Dispatcher_CreateEventBuffer( IDirectFB             *thiz,
-                                        IDirectFBEventBuffer **interface)
+                                        IDirectFBEventBuffer **interface_ptr)
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -324,11 +328,11 @@ static DFBResult
 IDirectFB_Dispatcher_CreateInputEventBuffer( IDirectFB                   *thiz,
                                              DFBInputDeviceCapabilities   caps,
                                              DFBBoolean                   global,
-                                             IDirectFBEventBuffer       **interface)
+                                             IDirectFBEventBuffer       **interface_ptr)
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -339,12 +343,12 @@ IDirectFB_Dispatcher_CreateInputEventBuffer( IDirectFB                   *thiz,
 static DFBResult
 IDirectFB_Dispatcher_CreateImageProvider( IDirectFB               *thiz,
                                           const char              *filename,
-                                          IDirectFBImageProvider **interface )
+                                          IDirectFBImageProvider **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
      /* Check arguments */
-     if (!filename || !interface)
+     if (!filename || !interface_ptr)
           return DFB_INVARG;
 
 
@@ -355,12 +359,12 @@ IDirectFB_Dispatcher_CreateImageProvider( IDirectFB               *thiz,
 static DFBResult
 IDirectFB_Dispatcher_CreateVideoProvider( IDirectFB               *thiz,
                                           const char              *filename,
-                                          IDirectFBVideoProvider **interface )
+                                          IDirectFBVideoProvider **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
      /* Check arguments */
-     if (!interface || !filename)
+     if (!interface_ptr || !filename)
           return DFB_INVARG;
 
 
@@ -372,12 +376,12 @@ static DFBResult
 IDirectFB_Dispatcher_CreateFont( IDirectFB                 *thiz,
                                  const char                *filename,
                                  const DFBFontDescription  *desc,
-                                 IDirectFBFont            **interface )
+                                 IDirectFBFont            **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
      /* Check arguments */
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -388,11 +392,11 @@ IDirectFB_Dispatcher_CreateFont( IDirectFB                 *thiz,
 static DFBResult
 IDirectFB_Dispatcher_CreateDataBuffer( IDirectFB                       *thiz,
                                        const DFBDataBufferDescription  *desc,
-                                       IDirectFBDataBuffer            **interface )
+                                       IDirectFBDataBuffer            **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
 
@@ -490,7 +494,7 @@ IDirectFB_Dispatcher_GetInterface( IDirectFB   *thiz,
                                    const char  *type,
                                    const char  *implementation,
                                    void        *arg,
-                                   void       **interface )
+                                   void       **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
@@ -501,6 +505,15 @@ IDirectFB_Dispatcher_GetInterface( IDirectFB   *thiz,
 
 /**************************************************************************************************/
 
+static DirectResult
+Dispatch_Release( IDirectFB *thiz, IDirectFB *real,
+                  VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
+
+     return voodoo_manager_unregister_local( manager, data->self );
+}
+
 static DirectResult
 Dispatch_SetCooperativeLevel( IDirectFB *thiz, IDirectFB *real,
                               VoodooManager *manager, VoodooRequestMessage *msg )
@@ -517,7 +530,7 @@ Dispatch_SetCooperativeLevel( IDirectFB *thiz, IDirectFB *real,
 
      ret = real->SetCooperativeLevel( real, level );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -535,7 +548,7 @@ Dispatch_GetDeviceDescription( IDirectFB *thiz, IDirectFB *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_DATA, sizeof(DFBGraphicsDeviceDescription), &desc,
                                     VMBT_NONE );
@@ -561,7 +574,7 @@ Dispatch_SetVideoMode( IDirectFB *thiz, IDirectFB *real,
 
      ret = real->SetVideoMode( real, width, height, bpp );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -609,7 +622,7 @@ Dispatch_EnumVideoModes( IDirectFB *thiz, IDirectFB *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_INT, context.num,
                                     VMBT_DATA, context.num * sizeof(IDirectFB_Dispatcher_EnumVideoModes_Item), context.items,
@@ -638,12 +651,10 @@ Dispatch_CreateEventBuffer( IDirectFB *thiz, IDirectFB *real,
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
                                        instance, buffer, &requestor );
-     if (ret) {
-          buffer->Release( buffer );
-          return ret;
-     }
 
-     return DFB_OK;
+     buffer->Release( buffer );
+
+     return ret;
 }
 
 static DirectResult
@@ -672,12 +683,10 @@ Dispatch_CreateInputEventBuffer( IDirectFB *thiz, IDirectFB *real,
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
                                        instance, buffer, &requestor );
-     if (ret) {
-          buffer->Release( buffer );
-          return ret;
-     }
 
-     return DFB_OK;
+     buffer->Release( buffer );
+
+     return ret;
 }
 
 static DirectResult
@@ -697,6 +706,14 @@ Dispatch_CreateImageProvider( IDirectFB *thiz, IDirectFB *real,
      VOODOO_PARSER_GET_ID( parser, instance );
      VOODOO_PARSER_END( parser );
 
+     if (1) {
+          ret = voodoo_manager_check_allocation( manager, 0x100000 /*FIXME*/ );
+          if (ret) {
+               D_ERROR( "Allocation not permitted!\n" );
+               return ret;
+          }
+     }
+
      direct_list_foreach (l, data->data_buffers) {
           DataBufferEntry *entry = (DataBufferEntry*) l;
 
@@ -717,7 +734,7 @@ Dispatch_CreateImageProvider( IDirectFB *thiz, IDirectFB *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -726,19 +743,79 @@ static DirectResult
 Dispatch_CreateSurface( IDirectFB *thiz, IDirectFB *real,
                         VoodooManager *manager, VoodooRequestMessage *msg )
 {
-     DirectResult                 ret;
-     const DFBSurfaceDescription *desc;
-     IDirectFBSurface            *surface;
-     VoodooInstanceID             instance;
-     VoodooMessageParser          parser;
+     DirectResult           ret;
+     DFBSurfaceDescription  desc;
+     IDirectFBSurface      *surface;
+     VoodooInstanceID       instance;
+     VoodooMessageParser    parser;
+     bool                   force_system = (voodoo_config->resource_id != 0);
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
      VOODOO_PARSER_BEGIN( parser, msg );
-     VOODOO_PARSER_GET_DATA( parser, desc );
+     VOODOO_PARSER_READ_DFBSurfaceDescription( parser, desc );
      VOODOO_PARSER_END( parser );
 
-     ret = real->CreateSurface( real, desc, &surface );
+     if (1) {
+          unsigned int w = 256, h = 256, b = 2, size;
+
+          if (desc.flags & DSDESC_WIDTH)
+               w = desc.width;
+
+          if (desc.flags & DSDESC_HEIGHT)
+               h = desc.height;
+
+          if (desc.flags & DSDESC_PIXELFORMAT)
+               b = DFB_BYTES_PER_PIXEL( desc.pixelformat ) ? DFB_BYTES_PER_PIXEL( desc.pixelformat ) : 2;
+
+          size = w * h * b;
+
+          D_INFO( "Checking creation of %u kB surface\n", size / 1024 );
+
+          if (voodoo_config->surface_max && voodoo_config->surface_max < size) {
+               D_ERROR( "Allocation of %u kB surface not permitted (limit %u kB)\n",
+                        size / 1024, voodoo_config->surface_max / 1024 );
+               return DR_LIMITEXCEEDED;
+          }
+
+          ret = voodoo_manager_check_allocation( manager, size );
+          if (ret) {
+               D_ERROR( "Allocation not permitted!\n" );
+               return ret;
+          }
+     }
+
+     if (voodoo_config->resource_id) {
+          if (desc.flags & DSDESC_RESOURCE_ID) {
+               if (desc.resource_id == voodoo_config->resource_id) {
+                    force_system = false;
+               }
+          }
+     }
+
+     if (force_system) {
+          DFBSurfaceDescription sd = desc;
+
+          if (sd.flags & DSDESC_CAPS) {
+               sd.caps &= ~DSCAPS_VIDEOONLY;
+               sd.caps |=  DSCAPS_SYSTEMONLY;
+          }
+          else {
+               sd.flags |= DSDESC_CAPS;
+               sd.caps   = DSCAPS_SYSTEMONLY;
+          }
+
+          ret = real->CreateSurface( real, &sd, &surface );
+     }
+     else if (desc.flags & (DSDESC_PALETTE | DSDESC_PREALLOCATED)) {
+          DFBSurfaceDescription sd = desc;
+          sd.flags &= ~(DSDESC_PALETTE | DSDESC_PREALLOCATED);
+
+          ret = real->CreateSurface( real, &sd, &surface );
+     }
+     else {
+          ret = real->CreateSurface( real, &desc, &surface );
+     }
      if (ret)
           return ret;
 
@@ -749,7 +826,7 @@ Dispatch_CreateSurface( IDirectFB *thiz, IDirectFB *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -793,7 +870,7 @@ Dispatch_EnumScreens( IDirectFB *thiz, IDirectFB *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_INT, context.num,
                                     VMBT_DATA, context.num * sizeof(IDirectFB_Dispatcher_EnumScreens_Item), context.items,
@@ -816,6 +893,11 @@ Dispatch_GetDisplayLayer( IDirectFB *thiz, IDirectFB *real,
      VOODOO_PARSER_GET_ID( parser, id );
      VOODOO_PARSER_END( parser );
 
+     if (voodoo_config->layer_mask && !(voodoo_config->layer_mask & (1 << id))) {
+          D_ERROR( "Layer with id %u not allowed!\n", id );
+          return DR_ACCESSDENIED;
+     }
+
      ret = real->GetDisplayLayer( real, id, &layer );
      if (ret)
           return ret;
@@ -827,7 +909,7 @@ Dispatch_GetDisplayLayer( IDirectFB *thiz, IDirectFB *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -873,7 +955,7 @@ Dispatch_EnumInputDevices( IDirectFB *thiz, IDirectFB *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_INT, context.num,
                                     VMBT_DATA, context.num * sizeof(IDirectFB_Dispatcher_EnumInputDevices_Item), context.items,
@@ -907,7 +989,7 @@ Dispatch_GetInputDevice( IDirectFB *thiz, IDirectFB *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -939,7 +1021,7 @@ Dispatch_GetScreen( IDirectFB *thiz, IDirectFB *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -948,21 +1030,43 @@ static DirectResult
 Dispatch_CreateFont( IDirectFB *thiz, IDirectFB *real,
                      VoodooManager *manager, VoodooRequestMessage *msg )
 {
-     DirectResult              ret;
-     IDirectFBFont            *font;
-     VoodooInstanceID          instance;
-     VoodooMessageParser       parser;
-     const char               *filename;
+     DirectResult            ret;
+     DirectLink             *l;
+     VoodooMessageParser     parser;
+     VoodooInstanceID        instance;
+     IDirectFBFont          *font;
+     IDirectFBDataBuffer    *buffer = NULL;
      const DFBFontDescription *desc;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
 
      VOODOO_PARSER_BEGIN( parser, msg );
-     VOODOO_PARSER_GET_STRING( parser, filename );
+     VOODOO_PARSER_GET_ID( parser, instance );
      VOODOO_PARSER_GET_DATA( parser, desc );
      VOODOO_PARSER_END( parser );
 
-     ret = real->CreateFont( real, filename, desc, &font );
+     direct_list_foreach (l, data->data_buffers) {
+          DataBufferEntry *entry = (DataBufferEntry*) l;
+
+          if (entry->instance == instance) {
+               buffer = entry->requestor;
+               break;
+          }
+     }
+
+     if (1) {
+          unsigned int size;
+
+          buffer->GetLength( buffer, &size );
+
+          ret = voodoo_manager_check_allocation( manager, size );
+          if (ret) {
+               D_ERROR( "Allocation not permitted!\n" );
+               return ret;
+          }
+     }
+
+     ret = buffer->CreateFont( buffer, desc, &font );
      if (ret)
           return ret;
 
@@ -973,7 +1077,7 @@ Dispatch_CreateFont( IDirectFB *thiz, IDirectFB *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -996,7 +1100,7 @@ Dispatch_CreateDataBuffer( IDirectFB *thiz, IDirectFB *real,
      VOODOO_PARSER_END( parser );
 
      ret = voodoo_construct_requestor( manager, "IDirectFBDataBuffer",
-                                       instance, NULL, &ptr );
+                                       instance, data->real, &ptr );
      if (ret)
           return ret;
 
@@ -1012,6 +1116,8 @@ Dispatch_CreateDataBuffer( IDirectFB *thiz, IDirectFB *real,
      entry->instance  = instance;
      entry->requestor = requestor;
 
+     entry->requestor->AddRef( entry->requestor );
+
      direct_list_prepend( &data->data_buffers, &entry->link );
 
      return DFB_OK;
@@ -1027,11 +1133,42 @@ Dispatch_WaitIdle( IDirectFB *thiz, IDirectFB *real,
 
      ret = real->WaitIdle( real );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_GetInterface( IDirectFB *thiz, IDirectFB *real,
+                       VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult         ret;
+     VoodooMessageParser  parser;
+     const char          *type;
+     IAny                *interface_ptr;
+     VoodooInstanceID     instance;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFB_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_STRING( parser, type );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->GetInterface( real, type, NULL, NULL, (void**) &interface_ptr );
+     if (ret)
+          return ret;
+
+     ret = voodoo_construct_dispatcher( manager, type, interface_ptr, data->self, NULL, &instance, NULL );
+     if (ret) {
+          interface_ptr->Release( interface_ptr );
+          return ret;
+     }
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
+                                    ret, instance,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMessage *msg )
 {
@@ -1039,6 +1176,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
               "Handling request for instance %u with method %u...\n", msg->instance, msg->method );
 
      switch (msg->method) {
+          case IDIRECTFB_METHOD_ID_Release:
+               return Dispatch_Release( dispatcher, real, manager, msg );
+
           case IDIRECTFB_METHOD_ID_SetCooperativeLevel:
                return Dispatch_SetCooperativeLevel( dispatcher, real, manager, msg );
 
@@ -1086,6 +1226,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFB_METHOD_ID_WaitIdle:
                return Dispatch_WaitIdle( dispatcher, real, manager, msg );
+
+          case IDIRECTFB_METHOD_ID_GetInterface:
+               return Dispatch_GetInterface( dispatcher, real, manager, msg );
      }
 
      return DFB_NOSUCHMETHOD;
@@ -1114,13 +1257,15 @@ Construct( IDirectFB *thiz, VoodooManager *manager, VoodooInstanceID *ret_instan
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFB_Dispatcher)
 
+     dfb_config->no_singleton = true;
+
      ret = DirectFBCreate( &real );
      if (ret) {
           DIRECT_DEALLOCATE_INTERFACE( thiz );
           return ret;
      }
 
-     ret = voodoo_manager_register_local( manager, true, thiz, real, Dispatch, &instance );
+     ret = voodoo_manager_register_local( manager, VOODOO_INSTANCE_NONE, thiz, real, Dispatch, &instance );
      if (ret) {
           real->Release( real );
           DIRECT_DEALLOCATE_INTERFACE( thiz );
@@ -1131,6 +1276,7 @@ Construct( IDirectFB *thiz, VoodooManager *manager, VoodooInstanceID *ret_instan
 
      data->ref  = 1;
      data->real = real;
+     data->core = ((IDirectFB_data*)(real->priv))->core;
      data->self = instance;
 
      thiz->AddRef                  = IDirectFB_Dispatcher_AddRef;
