@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __IDIRECTFBDISPLAYLAYER_DISPATCHER_H__
 #define __IDIRECTFBDISPLAYLAYER_DISPATCHER_H__
 
@@ -61,6 +65,8 @@
 #define IDIRECTFBDISPLAYLAYER_METHOD_ID_SetCursorOpacity          30
 #define IDIRECTFBDISPLAYLAYER_METHOD_ID_SetFieldParity            31
 #define IDIRECTFBDISPLAYLAYER_METHOD_ID_WaitForSync               32
+#define IDIRECTFBDISPLAYLAYER_METHOD_ID_GetWindowByResourceID     33
+#define IDIRECTFBDISPLAYLAYER_METHOD_ID_GetRotation               34
 
 /*
  * private data struct of IDirectFBDisplayLayer_Dispatcher
