@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,11 +28,12 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 
 #include <math.h>
@@ -38,39 +41,22 @@
 
 #include <directfb.h>
 
-#include <core/core.h>
-#include <core/coredefs.h>
-#include <core/coretypes.h>
-
-#include <core/gfxcard.h>
-#include <core/fonts.h>
-#include <core/state.h>
-#include <core/palette.h>
-#include <core/surface.h>
-
-#include <media/idirectfbfont.h>
-
-#include <display/idirectfbsurface.h>
-#include <display/idirectfbpalette.h>
-
-#include <misc/util.h>
-
 #include <direct/interface.h>
 #include <direct/mem.h>
 #include <direct/memcpy.h>
 #include <direct/messages.h>
 #include <direct/util.h>
 
-#include <gfx/convert.h>
-#include <gfx/util.h>
-
+#include <voodoo/conf.h>
 #include <voodoo/interface.h>
 #include <voodoo/manager.h>
 #include <voodoo/message.h>
 
+#include <idirectfbsurface_requestor.h>
+
 #include "idirectfbsurface_dispatcher.h"
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFBSurface *thiz,
                             IDirectFBSurface *real,
                             VoodooManager    *manager,
@@ -93,6 +79,11 @@ typedef struct {
      IDirectFBSurface      *real;
 
      VoodooInstanceID       super;
+     VoodooInstanceID       self;
+
+     VoodooManager         *manager;
+
+     VoodooInstanceID       remote;
 } IDirectFBSurface_Dispatcher_data;
 
 /**************************************************************************************************/
@@ -100,8 +91,14 @@ typedef struct {
 static void
 IDirectFBSurface_Dispatcher_Destruct( IDirectFBSurface *thiz )
 {
+     IDirectFBSurface_Dispatcher_data *data;
+
      D_DEBUG( "%s (%p)\n", __FUNCTION__, thiz );
 
+     data = thiz->priv;
+
+     data->real->Release( data->real );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
@@ -205,11 +202,11 @@ IDirectFBSurface_Dispatcher_GetAccelerationMask( IDirectFBSurface    *thiz,
 
 static DFBResult
 IDirectFBSurface_Dispatcher_GetPalette( IDirectFBSurface  *thiz,
-                                        IDirectFBPalette **interface )
+                                        IDirectFBPalette **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
      return DFB_UNIMPLEMENTED;
@@ -230,7 +227,7 @@ IDirectFBSurface_Dispatcher_SetPalette( IDirectFBSurface *thiz,
 static DFBResult
 IDirectFBSurface_Dispatcher_SetAlphaRamp( IDirectFBSurface *thiz,
                                           u8 a0, u8 a1, u8 a2, u8 a3 )
- 
+
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
@@ -262,6 +259,18 @@ IDirectFBSurface_Dispatcher_GetFramebufferOffset( IDirectFBSurface *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_GetPhysicalAddress( IDirectFBSurface *thiz,
+                                                unsigned long    *addr )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     if (!addr)
+          return DFB_INVARG;
+
+      return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_Unlock( IDirectFBSurface *thiz )
 {
@@ -302,7 +311,7 @@ IDirectFBSurface_Dispatcher_Clear( IDirectFBSurface *thiz,
 }
 
 static DFBResult
-IDirectFBSurface_Dispatcher_SetClip( IDirectFBSurface *thiz, 
+IDirectFBSurface_Dispatcher_SetClip( IDirectFBSurface *thiz,
                                      const DFBRegion  *clip )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
@@ -311,11 +320,11 @@ IDirectFBSurface_Dispatcher_SetClip( IDirectFBSurface *thiz,
 }
 
 static DFBResult
-IDirectFBSurface_Dispatcher_GetClip( IDirectFBSurface *thiz, 
+IDirectFBSurface_Dispatcher_GetClip( IDirectFBSurface *thiz,
                                      DFBRegion        *clip )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
-     
+
      if (!clip)
           return DFB_INVARG;
 
@@ -453,6 +462,16 @@ IDirectFBSurface_Dispatcher_FillRectangles( IDirectFBSurface   *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_FillTrapezoids( IDirectFBSurface   *thiz,
+                                            const DFBTrapezoid *traps,
+                                            unsigned int        num_traps )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_FillSpans( IDirectFBSurface *thiz,
                                        int               y,
@@ -578,6 +597,21 @@ IDirectFBSurface_Dispatcher_StretchBlit( IDirectFBSurface   *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBSurface_Dispatcher_BatchStretchBlit( IDirectFBSurface   *thiz,
+                                              IDirectFBSurface   *source,
+                                              const DFBRectangle *source_rects,
+                                              const DFBRectangle *destination_rects,
+                                              int                 num )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     if (!source || num < 1 || !source_rects || !destination_rects)
+          return DFB_INVARG;
+
+     return DFB_UNIMPLEMENTED;
+}
+
 static DFBResult
 IDirectFBSurface_Dispatcher_TextureTriangles( IDirectFBSurface     *thiz,
                                               IDirectFBSurface     *source,
@@ -645,11 +679,11 @@ IDirectFBSurface_Dispatcher_GetSubSurface( IDirectFBSurface    *thiz,
 
 static DFBResult
 IDirectFBSurface_Dispatcher_GetGL( IDirectFBSurface   *thiz,
-                                   IDirectFBGL       **interface )
+                                   IDirectFBGL       **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
      return DFB_UNIMPLEMENTED;
@@ -681,7 +715,7 @@ static DFBResult
 IDirectFBSurface_Dispatcher_ReleaseSource( IDirectFBSurface *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
-     
+
      return DFB_UNIMPLEMENTED;
 }
 
@@ -691,12 +725,21 @@ IDirectFBSurface_Dispatcher_SetIndexTranslation( IDirectFBSurface *thiz,
                                                  int               num_indices )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
-     
+
      return DFB_UNIMPLEMENTED;
 }
 
 /**************************************************************************************************/
 
+static DirectResult
+Dispatch_Release( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                  VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     return voodoo_manager_unregister_local( manager, data->self );
+}
+
 static DirectResult
 Dispatch_GetPosition( IDirectFBSurface *thiz, IDirectFBSurface *real,
                       VoodooManager *manager, VoodooRequestMessage *msg )
@@ -710,7 +753,7 @@ Dispatch_GetPosition( IDirectFBSurface *thiz, IDirectFBSurface *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_DATA, sizeof(DFBPoint), &position,
                                     VMBT_NONE );
@@ -729,7 +772,7 @@ Dispatch_GetSize( IDirectFBSurface *thiz, IDirectFBSurface *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_DATA, sizeof(DFBDimension), &dimension,
                                     VMBT_NONE );
@@ -748,7 +791,7 @@ Dispatch_GetVisibleRectangle( IDirectFBSurface *thiz, IDirectFBSurface *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_DATA, sizeof(DFBRectangle), &rect,
                                     VMBT_NONE );
@@ -767,7 +810,7 @@ Dispatch_GetPixelFormat( IDirectFBSurface *thiz, IDirectFBSurface *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_INT, format,
                                     VMBT_NONE );
@@ -799,7 +842,7 @@ Dispatch_GetAccelerationMask( IDirectFBSurface *thiz, IDirectFBSurface *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_INT, mask,
                                     VMBT_NONE );
@@ -826,7 +869,7 @@ Dispatch_GetPalette( IDirectFBSurface *thiz, IDirectFBSurface *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -863,18 +906,27 @@ Dispatch_Flip( IDirectFBSurface *thiz, IDirectFBSurface *real,
      VoodooMessageParser  parser;
      const DFBRegion     *region;
      DFBSurfaceFlipFlags  flags;
+     unsigned int         millis = 0;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
      VOODOO_PARSER_BEGIN( parser, msg );
      VOODOO_PARSER_GET_ODATA( parser, region );
      VOODOO_PARSER_GET_INT( parser, flags );
+     if (data->remote)
+          VOODOO_PARSER_GET_UINT( parser, millis );
      VOODOO_PARSER_END( parser );
 
      ret = real->Flip( real, region, flags );
 
+     if (data->remote)
+          voodoo_manager_request( manager, data->remote,
+                                  IDIRECTFBSURFACE_REQUESTOR_METHOD_ID_FlipNotify, VREQ_NONE, NULL,
+                                  VMBT_UINT, millis,
+                                  VMBT_NONE );
+
      if (flags & DSFLIP_WAIT)
-          return voodoo_manager_respond( manager, msg->header.serial,
+          return voodoo_manager_respond( manager, true, msg->header.serial,
                                          ret, VOODOO_INSTANCE_NONE,
                                          VMBT_NONE );
 
@@ -1139,6 +1191,36 @@ Dispatch_StretchBlit( IDirectFBSurface *thiz, IDirectFBSurface *real,
      return DFB_OK;
 }
 
+static DirectResult
+Dispatch_BatchStretchBlit( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                           VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult         ret;
+     VoodooMessageParser  parser;
+     VoodooInstanceID     instance;
+     unsigned int         num;
+     const DFBRectangle  *srects;
+     const DFBRectangle  *drects;
+     void                *surface;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_ID( parser, instance );
+     VOODOO_PARSER_GET_UINT( parser, num );
+     VOODOO_PARSER_GET_DATA( parser, srects );
+     VOODOO_PARSER_GET_DATA( parser, drects );
+     VOODOO_PARSER_END( parser );
+
+     ret = voodoo_manager_lookup_local( manager, instance, NULL, &surface );
+     if (ret)
+          return ret;
+
+     real->BatchStretchBlit( real, surface, srects, drects, num );
+
+     return DFB_OK;
+}
+
 static DirectResult
 Dispatch_TextureTriangles( IDirectFBSurface *thiz, IDirectFBSurface *real,
                            VoodooManager *manager, VoodooRequestMessage *msg )
@@ -1227,6 +1309,26 @@ Dispatch_FillRectangles( IDirectFBSurface *thiz, IDirectFBSurface *real,
      return DFB_OK;
 }
 
+static DirectResult
+Dispatch_FillTrapezoids( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                         VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     VoodooMessageParser  parser;
+     unsigned int         num_traps;
+     const DFBTrapezoid  *traps;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_UINT( parser, num_traps );
+     VOODOO_PARSER_GET_DATA( parser, traps );
+     VOODOO_PARSER_END( parser );
+
+     real->FillTrapezoids( real, traps, num_traps );
+
+     return DFB_OK;
+}
+
 static DirectResult
 Dispatch_FillSpans( IDirectFBSurface *thiz, IDirectFBSurface *real,
                     VoodooManager *manager, VoodooRequestMessage *msg )
@@ -1413,7 +1515,7 @@ Dispatch_SetEncoding( IDirectFBSurface *thiz, IDirectFBSurface *real,
      VOODOO_PARSER_END( parser );
 
      real->SetEncoding( real, encoding );
-     
+
      return DFB_OK;
 }
 
@@ -1444,7 +1546,37 @@ Dispatch_GetSubSurface( IDirectFBSurface *thiz, IDirectFBSurface *real,
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
+                                    DFB_OK, instance,
+                                    VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_MakeSubSurface( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                         VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult         ret;
+     VoodooMessageParser  parser;
+     VoodooInstanceID     instance;
+     const DFBRectangle  *rect;
+     void                *surface;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_ID( parser, instance );
+     VOODOO_PARSER_GET_ODATA( parser, rect );
+     VOODOO_PARSER_END( parser );
+
+     ret = voodoo_manager_lookup_local( manager, instance, NULL, &surface );
+     if (ret)
+          return ret;
+
+     ret = real->MakeSubSurface( real, surface, rect );
+     if (ret)
+          return ret;
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -1459,7 +1591,7 @@ Dispatch_DisableAcceleration( IDirectFBSurface *thiz, IDirectFBSurface *real,
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
      VOODOO_PARSER_BEGIN( parser, msg );
-     VOODOO_PARSER_GET_UINT( parser, mask );
+     VOODOO_PARSER_GET_INT( parser, mask );
      VOODOO_PARSER_END( parser );
 
      real->DisableAcceleration( real, mask );
@@ -1478,11 +1610,112 @@ Dispatch_ReleaseSource( IDirectFBSurface *thiz, IDirectFBSurface *real,
      return DFB_OK;
 }
 
+#define RLE16_KEY   0xf001
+
+static void
+rle16_decode( const u16    *src,
+              u16          *dst,
+              unsigned int  num )
+{
+     unsigned int n = 0, last, count, out = 0;
+
+     while (out < num) {
+          last = src[n++];
+
+          if (last == RLE16_KEY) {
+               count = src[n++];
+
+               if (count == RLE16_KEY) {
+                    dst[out++] = RLE16_KEY;
+               }
+               else {
+                    last = src[n++];
+
+                    while (count >= 4) {
+                         dst[out+0] =
+                         dst[out+1] =
+                         dst[out+2] =
+                         dst[out+3] = last;
+
+                         out   += 4;
+                         count -= 4;
+                    }
+
+                    while (count >= 2) {
+                         dst[out+0] =
+                         dst[out+1] = last;
+
+                         out   += 2;
+                         count -= 2;
+                    }
+
+                    while (count--)
+                         dst[out++] = last;
+               }
+          }
+          else
+               dst[out++] = last;
+     }
+
+     D_ASSERT( out == num );
+}
+
+#define RLE32_KEY   0xf0012345
+
+static void
+rle32_decode( const u32    *src,
+              u32          *dst,
+              unsigned int  num )
+{
+     unsigned int n = 0, last, count, out = 0;
+
+     while (out < num) {
+          last = src[n++];
+
+          if (last == RLE32_KEY) {
+               count = src[n++];
+
+               if (count == RLE32_KEY) {
+                    dst[out++] = RLE32_KEY;
+               }
+               else {
+                    last = src[n++];
+
+                    while (count >= 4) {
+                         dst[out+0] =
+                         dst[out+1] =
+                         dst[out+2] =
+                         dst[out+3] = last;
+
+                         out   += 4;
+                         count -= 4;
+                    }
+
+                    while (count >= 2) {
+                         dst[out+0] =
+                         dst[out+1] = last;
+
+                         out   += 2;
+                         count -= 2;
+                    }
+
+                    while (count--)
+                         dst[out++] = last;
+               }
+          }
+          else
+               dst[out++] = last;
+     }
+
+     D_ASSERT( out == num );
+}
+
 static DirectResult
 Dispatch_Write( IDirectFBSurface *thiz, IDirectFBSurface *real,
                 VoodooManager *manager, VoodooRequestMessage *msg )
 {
      VoodooMessageParser  parser;
+     unsigned int         encoded;
      const DFBRectangle  *rect;
      const void          *ptr;
      int                  pitch;
@@ -1490,12 +1723,256 @@ Dispatch_Write( IDirectFBSurface *thiz, IDirectFBSurface *real,
      DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
 
      VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_UINT( parser, encoded );
      VOODOO_PARSER_GET_DATA( parser, rect );
      VOODOO_PARSER_GET_DATA( parser, ptr );
      VOODOO_PARSER_GET_INT( parser, pitch );
      VOODOO_PARSER_END( parser );
 
-     real->Write( real, rect, ptr, pitch );
+     if (encoded) {
+          switch (encoded) {
+               case 2: {
+                    if (rect->w > 2048) {
+                         u16 *buf = D_MALLOC( rect->w * 2 );
+
+                         if (buf) {
+                              rle16_decode( ptr, buf, rect->w );
+
+                              real->Write( real, rect, buf, pitch );
+
+                              D_FREE( buf );
+                         }
+                         else
+                              D_OOM();
+                    }
+                    else {
+                         u16 buf[2048];
+
+                         rle16_decode( ptr, buf, rect->w );
+
+                         real->Write( real, rect, buf, pitch );
+                    }
+                    break;
+               }
+
+               case 4: {
+                    if (rect->w > 1024) {
+                         u32 *buf = D_MALLOC( rect->w * 4 );
+
+                         if (buf) {
+                              rle32_decode( ptr, buf, rect->w );
+
+                              real->Write( real, rect, buf, pitch );
+
+                              D_FREE( buf );
+                         }
+                         else
+                              D_OOM();
+                    }
+                    else {
+                         u32 buf[1024];
+
+                         rle32_decode( ptr, buf, rect->w );
+
+                         real->Write( real, rect, buf, pitch );
+                    }
+                    break;
+               }
+
+               default:
+                    D_UNIMPLEMENTED();
+                    break;
+          }
+     }
+     else
+          real->Write( real, rect, ptr, pitch );
+
+     return DFB_OK;
+}
+
+#define RLE16_KEY   0xf001
+
+static bool
+rle16_encode( const u16    *src,
+              u16          *dst,
+              unsigned int  num,
+              unsigned int *ret_num )
+{
+     unsigned int n, last, count = 0, out = 0;
+
+     for (n=0; n<num; n++) {
+          if (out + 3 > num) {
+               *ret_num = num;
+               return false;
+          }
+
+          if (count > 0) {
+               D_ASSERT( src[n] == last );
+
+               count++;
+          }
+          else {
+               count = 1;
+               last  = src[n];
+          }
+
+          if (n == num-1 || src[n+1] != last) {
+               if (count > 2 || (count > 1 && last == RLE16_KEY)) {
+                    dst[out++] = RLE16_KEY;
+                    dst[out++] = count;
+                    dst[out++] = last;
+               }
+               else {
+                    if (count > 1 || last == RLE16_KEY)
+                         dst[out++] = last;
+
+                    dst[out++] = last;
+               }
+
+               count = 0;
+          }
+     }
+
+     *ret_num = out;
+
+     return true;
+}
+
+#define RLE32_KEY   0xf0012345
+
+static bool
+rle32_encode( const u32    *src,
+              u32          *dst,
+              unsigned int  num,
+              unsigned int *ret_num )
+{
+     unsigned int n, last, count = 0, out = 0;
+
+     for (n=0; n<num; n++) {
+          if (out + 3 > num) {
+               *ret_num = num;
+               return false;
+          }
+
+          if (count > 0) {
+               D_ASSERT( src[n] == last );
+
+               count++;
+          }
+          else {
+               count = 1;
+               last  = src[n];
+          }
+
+          if (n == num-1 || src[n+1] != last) {
+               if (count > 2 || (count > 1 && last == RLE32_KEY)) {
+                    dst[out++] = RLE32_KEY;
+                    dst[out++] = count;
+                    dst[out++] = last;
+               }
+               else {
+                    if (count > 1 || last == RLE32_KEY)
+                         dst[out++] = last;
+
+                    dst[out++] = last;
+               }
+
+               count = 0;
+          }
+     }
+
+     *ret_num = out;
+
+     return true;
+}
+
+static DirectResult
+Dispatch_Read( IDirectFBSurface *thiz, IDirectFBSurface *real,
+               VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult           ret;
+     VoodooMessageParser    parser;
+     const DFBRectangle    *rect;
+     int                    len;
+     int                    y;
+     void                  *buf;
+     DFBSurfacePixelFormat  format;
+     unsigned int           encoded;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, rect );
+     VOODOO_PARSER_END( parser );
+
+     real->GetPixelFormat( real, &format );
+
+     len = DFB_BYTES_PER_LINE( format, rect->w );
+     buf = alloca( len );
+
+
+     switch (voodoo_config->compression_min ? DSPF_UNKNOWN : format) {
+          case DSPF_RGB16: {
+               u16 tmp[rect->w];
+
+               for (y=0; y<rect->h; y++) {
+                    unsigned int num;
+                    DFBRectangle r = { rect->x, rect->y + y, rect->w, 1 };
+
+                    real->Read( real, &r, buf, len );
+
+                    encoded = rle16_encode( buf, tmp, rect->w, &num );
+
+                    ret = voodoo_manager_respond( manager, y == rect->h - 1, msg->header.serial,
+                                                  DFB_OK, VOODOO_INSTANCE_NONE,
+                                                  VMBT_UINT, encoded ? 2 : 0,
+                                                  VMBT_DATA, DFB_BYTES_PER_LINE( format, num ),
+                                                             encoded ? tmp : buf,
+                                                  VMBT_NONE );
+                    if (ret)
+                         break;
+               }
+               break;
+          }
+
+          case DSPF_RGB32:
+          case DSPF_ARGB: {
+               u32 tmp[rect->w];
+
+               for (y=0; y<rect->h; y++) {
+                    unsigned int num;
+                    DFBRectangle r = { rect->x, rect->y + y, rect->w, 1 };
+
+                    real->Read( real, &r, buf, len );
+
+                    encoded = rle32_encode( buf, tmp, rect->w, &num );
+
+                    ret = voodoo_manager_respond( manager, y == rect->h - 1, msg->header.serial,
+                                                  DFB_OK, VOODOO_INSTANCE_NONE,
+                                                  VMBT_UINT, encoded ? 4 : 0,
+                                                  VMBT_DATA, DFB_BYTES_PER_LINE( format, num ),
+                                                             encoded ? tmp : buf,
+                                                  VMBT_NONE );
+                    if (ret)
+                         break;
+               }
+               break;
+          }
+
+          default:
+               for (y=0; y<rect->h; y++) {
+                    DFBRectangle r = { rect->x, rect->y + y, rect->w, 1 };
+
+                    real->Read( real, &r, buf, len );
+
+                    voodoo_manager_respond( manager, y == rect->h - 1, msg->header.serial,
+                                            DFB_OK, VOODOO_INSTANCE_NONE,
+                                            VMBT_UINT, 0,
+                                            VMBT_DATA, len, buf,
+                                            VMBT_NONE );
+               }
+               break;
+     }
 
      return DFB_OK;
 }
@@ -1536,6 +2013,44 @@ Dispatch_SetMatrix( IDirectFBSurface *thiz, IDirectFBSurface *real,
      return DFB_OK;
 }
 
+static DirectResult
+Dispatch_SetRemoteInstance( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                            VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     VoodooMessageParser parser;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_ID( parser, data->remote );
+     VOODOO_PARSER_END( parser );
+
+     return voodoo_manager_respond( manager, true, msg->header.serial, DR_OK, VOODOO_INSTANCE_NONE, VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_GetFrameTime( IDirectFBSurface *thiz, IDirectFBSurface *real,
+                       VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DFBResult ret;
+     long long micros;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBSurface_Dispatcher)
+
+     ret = real->GetFrameTime( real, &micros );
+     if (ret)
+          return ret;
+
+     /* Compensate to achieve same behaviour as with a local app calling GetFrameTime+Flip */
+     micros += voodoo_manager_connection_delay( data->manager );
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
+                                    DFB_OK, VOODOO_INSTANCE_NONE,
+                                    VMBT_UINT, (unsigned long long) micros >> 32,
+                                    VMBT_UINT, (unsigned long long) micros,
+                                    VMBT_NONE );
+}
+
 static DirectResult
 Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMessage *msg )
 {
@@ -1543,9 +2058,12 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
               "Handling request for instance %u with method %u...\n", msg->instance, msg->method );
 
      switch (msg->method) {
+          case IDIRECTFBSURFACE_METHOD_ID_Release:
+               return Dispatch_Release( dispatcher, real, manager, msg );
+
           case IDIRECTFBSURFACE_METHOD_ID_GetPosition:
                return Dispatch_GetPosition( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBSURFACE_METHOD_ID_GetSize:
                return Dispatch_GetSize( dispatcher, real, manager, msg );
 
@@ -1603,6 +2121,9 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
           case IDIRECTFBSURFACE_METHOD_ID_StretchBlit:
                return Dispatch_StretchBlit( dispatcher, real, manager, msg );
 
+          case IDIRECTFBSURFACE_METHOD_ID_BatchStretchBlit:
+               return Dispatch_BatchStretchBlit( dispatcher, real, manager, msg );
+
           case IDIRECTFBSURFACE_METHOD_ID_TextureTriangles:
                return Dispatch_TextureTriangles( dispatcher, real, manager, msg );
 
@@ -1632,7 +2153,7 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBSURFACE_METHOD_ID_DrawGlyph:
                return Dispatch_DrawGlyph( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBSURFACE_METHOD_ID_SetEncoding:
                return Dispatch_SetEncoding( dispatcher, real, manager, msg );
 
@@ -1642,23 +2163,38 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
           case IDIRECTFBSURFACE_METHOD_ID_FillRectangles:
                return Dispatch_FillRectangles( dispatcher, real, manager, msg );
 
+          case IDIRECTFBSURFACE_METHOD_ID_FillTrapezoids:
+               return Dispatch_FillTrapezoids( dispatcher, real, manager, msg );
+
           case IDIRECTFBSURFACE_METHOD_ID_FillSpans:
                return Dispatch_FillSpans( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBSURFACE_METHOD_ID_DisableAcceleration:
                return Dispatch_DisableAcceleration( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBSURFACE_METHOD_ID_ReleaseSource:
                return Dispatch_ReleaseSource( dispatcher, real, manager, msg );
 
           case IDIRECTFBSURFACE_METHOD_ID_Write:
                return Dispatch_Write( dispatcher, real, manager, msg );
 
+          case IDIRECTFBSURFACE_METHOD_ID_Read:
+               return Dispatch_Read( dispatcher, real, manager, msg );
+
           case IDIRECTFBSURFACE_METHOD_ID_SetRenderOptions:
                return Dispatch_SetRenderOptions( dispatcher, real, manager, msg );
 
           case IDIRECTFBSURFACE_METHOD_ID_SetMatrix:
                return Dispatch_SetMatrix( dispatcher, real, manager, msg );
+
+          case IDIRECTFBSURFACE_METHOD_ID_MakeSubSurface:
+               return Dispatch_MakeSubSurface( dispatcher, real, manager, msg );
+
+          case IDIRECTFBSURFACE_METHOD_ID_SetRemoteInstance:
+               return Dispatch_SetRemoteInstance( dispatcher, real, manager, msg );
+
+          case IDIRECTFBSURFACE_METHOD_ID_GetFrameTime:
+               return Dispatch_GetFrameTime( dispatcher, real, manager, msg );
      }
 
      return DFB_NOSUCHMETHOD;
@@ -1685,7 +2221,7 @@ Construct( IDirectFBSurface *thiz,
 
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBSurface_Dispatcher)
 
-     ret = voodoo_manager_register_local( manager, false, thiz, real, Dispatch, ret_instance );
+     ret = voodoo_manager_register_local( manager, super, thiz, real, Dispatch, ret_instance );
      if (ret) {
           DIRECT_DEALLOCATE_INTERFACE( thiz );
           return ret;
@@ -1694,6 +2230,8 @@ Construct( IDirectFBSurface *thiz,
      data->ref   = 1;
      data->real  = real;
      data->super = super;
+     data->self    = *ret_instance;
+     data->manager = manager;
 
      thiz->AddRef = IDirectFBSurface_Dispatcher_AddRef;
      thiz->Release = IDirectFBSurface_Dispatcher_Release;
@@ -1711,6 +2249,7 @@ Construct( IDirectFBSurface *thiz,
 
      thiz->Lock = IDirectFBSurface_Dispatcher_Lock;
      thiz->GetFramebufferOffset = IDirectFBSurface_Dispatcher_GetFramebufferOffset;
+     thiz->GetPhysicalAddress = IDirectFBSurface_Dispatcher_GetPhysicalAddress;
      thiz->Unlock = IDirectFBSurface_Dispatcher_Unlock;
      thiz->Flip = IDirectFBSurface_Dispatcher_Flip;
      thiz->SetField = IDirectFBSurface_Dispatcher_SetField;
@@ -1733,11 +2272,13 @@ Construct( IDirectFBSurface *thiz,
      thiz->TileBlit = IDirectFBSurface_Dispatcher_TileBlit;
      thiz->BatchBlit = IDirectFBSurface_Dispatcher_BatchBlit;
      thiz->StretchBlit = IDirectFBSurface_Dispatcher_StretchBlit;
+     thiz->BatchStretchBlit = IDirectFBSurface_Dispatcher_BatchStretchBlit;
      thiz->TextureTriangles = IDirectFBSurface_Dispatcher_TextureTriangles;
 
      thiz->SetDrawingFlags = IDirectFBSurface_Dispatcher_SetDrawingFlags;
      thiz->FillRectangle = IDirectFBSurface_Dispatcher_FillRectangle;
      thiz->FillRectangles = IDirectFBSurface_Dispatcher_FillRectangles;
+     thiz->FillTrapezoids = IDirectFBSurface_Dispatcher_FillTrapezoids;
      thiz->FillSpans = IDirectFBSurface_Dispatcher_FillSpans;
      thiz->DrawLine = IDirectFBSurface_Dispatcher_DrawLine;
      thiz->DrawLines = IDirectFBSurface_Dispatcher_DrawLines;
@@ -1755,11 +2296,11 @@ Construct( IDirectFBSurface *thiz,
      thiz->GetGL = IDirectFBSurface_Dispatcher_GetGL;
 
      thiz->Dump = IDirectFBSurface_Dispatcher_Dump;
-     
+
      thiz->DisableAcceleration = IDirectFBSurface_Dispatcher_DisableAcceleration;
-     
+
      thiz->ReleaseSource = IDirectFBSurface_Dispatcher_ReleaseSource;
-     
+
      thiz->SetIndexTranslation = IDirectFBSurface_Dispatcher_SetIndexTranslation;
 
      return DFB_OK;
