@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __IDIRECTFBSURFACE_DISPATCHER_H__
 #define __IDIRECTFBSURFACE_DISPATCHER_H__
 
@@ -87,5 +91,9 @@
 #define IDIRECTFBSURFACE_METHOD_ID_Read                      56
 #define IDIRECTFBSURFACE_METHOD_ID_SetColors                 57
 #define IDIRECTFBSURFACE_METHOD_ID_BatchBlit2                58
+#define IDIRECTFBSURFACE_METHOD_ID_SetRemoteInstance         59
+#define IDIRECTFBSURFACE_METHOD_ID_FillTrapezoids            60
+#define IDIRECTFBSURFACE_METHOD_ID_BatchStretchBlit          61
+#define IDIRECTFBSURFACE_METHOD_ID_GetFrameTime              62
 
 #endif
