@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <directfb.h>
@@ -36,6 +40,7 @@
 #include <direct/messages.h>
 #include <direct/util.h>
 
+#include <voodoo/conf.h>
 #include <voodoo/interface.h>
 #include <voodoo/manager.h>
 
@@ -44,7 +49,7 @@
 #include "idirectfbwindow_dispatcher.h"
 
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFBWindow  *thiz,
                             IDirectFBWindow  *real,
                             VoodooManager    *manager,
@@ -62,14 +67,20 @@ DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBWindow, Dispatcher )
 static void
 IDirectFBWindow_Dispatcher_Destruct( IDirectFBWindow *thiz )
 {
+     IDirectFBWindow_Dispatcher_data *data;
+
      D_DEBUG( "%s (%p)\n", __FUNCTION__, thiz );
 
+     data = thiz->priv;
+
+     data->real->Release( data->real );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 /**************************************************************************************************/
 
-static DFBResult
+static DirectResult
 IDirectFBWindow_Dispatcher_AddRef( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
@@ -79,7 +90,7 @@ IDirectFBWindow_Dispatcher_AddRef( IDirectFBWindow *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBWindow_Dispatcher_Release( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
@@ -117,7 +128,7 @@ IDirectFBWindow_Dispatcher_DetachEventBuffer( IDirectFBWindow       *thiz,
                                               IDirectFBEventBuffer  *buffer )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
-     
+
      D_UNIMPLEMENTED();
 
      return DFB_UNIMPLEMENTED;
@@ -548,6 +559,15 @@ IDirectFBWindow_Dispatcher_ResizeSurface( IDirectFBWindow *thiz,
 
 /**************************************************************************************************/
 
+static DirectResult
+Dispatch_Release( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                  VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     return voodoo_manager_unregister_local( data->manager, data->self );
+}
+
 static DirectResult
 Dispatch_CreateEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
                             VoodooManager *manager, VoodooRequestMessage *msg )
@@ -570,10 +590,10 @@ Dispatch_CreateEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = voodoo_construct_requestor( manager, "IDirectFBEventBuffer",
                                        instance, buffer, &requestor );
-     if (ret)
-          buffer->Release( buffer );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     buffer->Release( buffer );
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -605,11 +625,27 @@ Dispatch_AttachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->AttachEventBuffer( real, buffer_data->src );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_EnableEvents( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                       VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     VoodooMessageParser parser;
+     DFBWindowEventType  mask;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_UINT( parser, mask );
+     VOODOO_PARSER_END( parser );
+
+     return real->EnableEvents( real, mask );
+}
+
 static DirectResult
 Dispatch_DetachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
                             VoodooManager *manager, VoodooRequestMessage *msg )
@@ -637,7 +673,7 @@ Dispatch_DetachEventBuffer( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->DetachEventBuffer( real, buffer_data->src );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -655,7 +691,7 @@ Dispatch_GetID( IDirectFBWindow *thiz, IDirectFBWindow *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_ID, id,
                                     VMBT_NONE );
@@ -674,7 +710,7 @@ Dispatch_GetPosition( IDirectFBWindow *thiz, IDirectFBWindow *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_DATA, sizeof(DFBPoint), &position,
                                     VMBT_NONE );
@@ -693,7 +729,7 @@ Dispatch_GetSize( IDirectFBWindow *thiz, IDirectFBWindow *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_DATA, sizeof(DFBDimension), &size,
                                     VMBT_NONE );
@@ -714,13 +750,13 @@ Dispatch_GetSurface( IDirectFBWindow *thiz, IDirectFBWindow *real,
           return ret;
 
      ret = voodoo_construct_dispatcher( manager, "IDirectFBSurface",
-                                        surface, data->self, NULL, &instance, NULL );
+                                        surface, data->super, NULL, &instance, NULL );
      if (ret) {
           surface->Release( surface );
           return ret;
      }
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, instance,
                                     VMBT_NONE );
 }
@@ -729,7 +765,6 @@ static DirectResult
 Dispatch_SetOptions( IDirectFBWindow *thiz, IDirectFBWindow *real,
                      VoodooManager *manager, VoodooRequestMessage *msg )
 {
-     DFBResult           ret;
      VoodooMessageParser parser;
      DFBWindowOptions    options;
 
@@ -739,9 +774,7 @@ Dispatch_SetOptions( IDirectFBWindow *thiz, IDirectFBWindow *real,
      VOODOO_PARSER_GET_INT( parser, options );
      VOODOO_PARSER_END( parser );
 
-     ret = real->SetOptions( real, options );
-
-     return DFB_OK;
+     return real->SetOptions( real, options );
 }
 
 static DirectResult
@@ -757,7 +790,7 @@ Dispatch_GetOptions( IDirectFBWindow *thiz, IDirectFBWindow *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_INT, options,
                                     VMBT_NONE );
@@ -794,7 +827,7 @@ Dispatch_GetOpacity( IDirectFBWindow *thiz, IDirectFBWindow *real,
      if (ret)
           return ret;
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     DFB_OK, VOODOO_INSTANCE_NONE,
                                     VMBT_UINT, opacity,
                                     VMBT_NONE );
@@ -823,7 +856,7 @@ Dispatch_SetCursorShape( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->SetCursorShape( real, surface, hot->x, hot->y );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -838,7 +871,7 @@ Dispatch_RequestFocus( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->RequestFocus( real );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -853,7 +886,7 @@ Dispatch_GrabPointer( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->GrabPointer( real );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -868,7 +901,7 @@ Dispatch_UngrabPointer( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->UngrabPointer( real );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -925,7 +958,7 @@ Dispatch_Resize( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->Resize( real, size->w, size->h );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -934,7 +967,6 @@ static DirectResult
 Dispatch_SetStackingClass( IDirectFBWindow *thiz, IDirectFBWindow *real,
                            VoodooManager *manager, VoodooRequestMessage *msg )
 {
-     DFBResult              ret;
      VoodooMessageParser    parser;
      DFBWindowStackingClass stacking_class;
 
@@ -944,9 +976,14 @@ Dispatch_SetStackingClass( IDirectFBWindow *thiz, IDirectFBWindow *real,
      VOODOO_PARSER_GET_INT( parser, stacking_class );
      VOODOO_PARSER_END( parser );
 
-     ret = real->SetStackingClass( real, stacking_class );
+     if (1) {
+          if (voodoo_config->stacking_mask && !(voodoo_config->stacking_mask & (1 << stacking_class))) {
+               D_ERROR( "Stacking class not permitted!\n" );
+               return DR_ACCESSDENIED;
+          }
+     }
 
-     return DFB_OK;
+     return real->SetStackingClass( real, stacking_class );
 }
 
 static DirectResult
@@ -1014,7 +1051,7 @@ Dispatch_Destroy( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->Destroy( real );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -1035,7 +1072,7 @@ Dispatch_SetBounds( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->SetBounds( real, bounds->x, bounds->y, bounds->w, bounds->h );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
@@ -1056,11 +1093,97 @@ Dispatch_ResizeSurface( IDirectFBWindow *thiz, IDirectFBWindow *real,
 
      ret = real->ResizeSurface( real, size->w, size->h );
 
-     return voodoo_manager_respond( manager, msg->header.serial,
+     return voodoo_manager_respond( manager, true, msg->header.serial,
                                     ret, VOODOO_INSTANCE_NONE,
                                     VMBT_NONE );
 }
 
+static DirectResult
+Dispatch_SetSrcGeometry( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                         VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DFBResult                ret;
+     VoodooMessageParser      parser;
+     const DFBWindowGeometry *geometry;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, geometry );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->SetSrcGeometry( real, geometry );
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_SetDstGeometry( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                         VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DFBResult                ret;
+     VoodooMessageParser      parser;
+     const DFBWindowGeometry *geometry;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, geometry );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->SetDstGeometry( real, geometry );
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
+                                    ret, VOODOO_INSTANCE_NONE,
+                                    VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_GetProperty( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                      VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     DirectResult         ret;
+     VoodooMessageParser  parser;
+     const char          *key;
+     void                *value = NULL;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_STRING( parser, key );
+     VOODOO_PARSER_END( parser );
+
+     ret = real->GetProperty( real, key, &value );
+     if (ret)
+          return ret;
+
+     if (!value)
+          value = (void*) "";
+
+     return voodoo_manager_respond( manager, true, msg->header.serial,
+                                    DFB_OK, VOODOO_INSTANCE_NONE,
+                                    VMBT_STRING, value,
+                                    VMBT_NONE );
+}
+
+static DirectResult
+Dispatch_SendEvent( IDirectFBWindow *thiz, IDirectFBWindow *real,
+                    VoodooManager *manager, VoodooRequestMessage *msg )
+{
+     VoodooMessageParser   parser;
+     const DFBWindowEvent *event;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Dispatcher)
+
+     VOODOO_PARSER_BEGIN( parser, msg );
+     VOODOO_PARSER_GET_DATA( parser, event );
+     VOODOO_PARSER_END( parser );
+
+     return real->SendEvent( real, event );
+}
+
 
 static DirectResult
 Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMessage *msg )
@@ -1069,21 +1192,27 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
               "Handling request for instance %u with method %u...\n", msg->instance, msg->method );
 
      switch (msg->method) {
+          case IDIRECTFBWINDOW_METHOD_ID_Release:
+               return Dispatch_Release( dispatcher, real, manager, msg );
+
           case IDIRECTFBWINDOW_METHOD_ID_CreateEventBuffer:
                return Dispatch_CreateEventBuffer( dispatcher, real, manager, msg );
 
           case IDIRECTFBWINDOW_METHOD_ID_AttachEventBuffer:
                return Dispatch_AttachEventBuffer( dispatcher, real, manager, msg );
-               
+
+          case IDIRECTFBWINDOW_METHOD_ID_EnableEvents:
+               return Dispatch_EnableEvents( dispatcher, real, manager, msg );
+
           case IDIRECTFBWINDOW_METHOD_ID_DetachEventBuffer:
                return Dispatch_DetachEventBuffer( dispatcher, real, manager, msg );
 
           case IDIRECTFBWINDOW_METHOD_ID_GetID:
                return Dispatch_GetID( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBWINDOW_METHOD_ID_GetPosition:
                return Dispatch_GetPosition( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBWINDOW_METHOD_ID_GetSize:
                return Dispatch_GetSize( dispatcher, real, manager, msg );
 
@@ -1143,12 +1272,24 @@ Dispatch( void *dispatcher, void *real, VoodooManager *manager, VoodooRequestMes
 
           case IDIRECTFBWINDOW_METHOD_ID_Destroy:
                return Dispatch_Destroy( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBWINDOW_METHOD_ID_SetBounds:
                return Dispatch_SetBounds( dispatcher, real, manager, msg );
-               
+
           case IDIRECTFBWINDOW_METHOD_ID_ResizeSurface:
                return Dispatch_ResizeSurface( dispatcher, real, manager, msg );
+
+          case IDIRECTFBWINDOW_METHOD_ID_SetSrcGeometry:
+               return Dispatch_SetSrcGeometry( dispatcher, real, manager, msg );
+
+          case IDIRECTFBWINDOW_METHOD_ID_SetDstGeometry:
+               return Dispatch_SetDstGeometry( dispatcher, real, manager, msg );
+
+          case IDIRECTFBWINDOW_METHOD_ID_GetProperty:
+               return Dispatch_GetProperty( dispatcher, real, manager, msg );
+
+          case IDIRECTFBWINDOW_METHOD_ID_SendEvent:
+               return Dispatch_SendEvent( dispatcher, real, manager, msg );
      }
 
      return DFB_NOSUCHMETHOD;
@@ -1182,7 +1323,7 @@ Construct( IDirectFBWindow  *thiz,     /* Dispatcher interface */
      D_ASSERT( ret_instance != NULL );
 
      /* Register the dispatcher, getting a new instance ID that refers to it. */
-     ret = voodoo_manager_register_local( manager, false, thiz, real, Dispatch, &instance );
+     ret = voodoo_manager_register_local( manager, super, thiz, real, Dispatch, &instance );
      if (ret) {
           DIRECT_DEALLOCATE_INTERFACE( thiz );
           return ret;
@@ -1196,6 +1337,7 @@ Construct( IDirectFBWindow  *thiz,     /* Dispatcher interface */
      data->real  = real;
      data->self  = instance;
      data->super = super;
+     data->manager = manager;
 
      /* Initialize interface methods. */
      thiz->AddRef             = IDirectFBWindow_Dispatcher_AddRef;
