@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <directfb.h>
@@ -40,13 +44,16 @@
 #include <voodoo/interface.h>
 #include <voodoo/manager.h>
 
+#include <misc/conf.h>
+
 #include <input/idirectfbinputbuffer.h>
 #include <media/idirectfbdatabuffer.h>
 
+#include <idirectfb.h>
 #include <idirectfb_dispatcher.h>
 
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFB *thiz, const char *host, int session );
 
 #include <direct/interface_implementation.h>
@@ -75,8 +82,16 @@ typedef struct {
 static void
 IDirectFB_Requestor_Destruct( IDirectFB *thiz )
 {
+     IDirectFB_Requestor_data *data = thiz->priv;
+
      D_DEBUG( "%s (%p)\n", __FUNCTION__, thiz );
 
+     voodoo_manager_request( data->manager, data->instance,
+                             IDIRECTFB_METHOD_ID_Release, VREQ_NONE, NULL,
+                             VMBT_NONE );
+
+     voodoo_client_destroy( data->client );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
@@ -115,6 +130,21 @@ IDirectFB_Requestor_SetCooperativeLevel( IDirectFB           *thiz,
      if (level == data->level)
           return DFB_OK;
 
+     switch (level) {
+          case DFSCL_NORMAL:
+               break;
+
+          case DFSCL_FULLSCREEN:
+          case DFSCL_EXCLUSIVE:
+               if (dfb_config->force_windowed || dfb_config->force_desktop)
+                    return DFB_ACCESSDENIED;
+
+               break;
+
+          default:
+               return DFB_INVARG;
+     }
+
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFB_METHOD_ID_SetCooperativeLevel, VREQ_RESPOND, &response,
                                    VMBT_INT, level,
@@ -123,7 +153,7 @@ IDirectFB_Requestor_SetCooperativeLevel( IDirectFB           *thiz,
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK)
+     if (ret == DR_OK)
           data->level = level;
 
      voodoo_manager_finish_request( data->manager, response );
@@ -147,7 +177,7 @@ IDirectFB_Requestor_GetDeviceDescription( IDirectFB                    *thiz,
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK) {
+     if (ret == DR_OK) {
           VoodooMessageParser parser;
 
           VOODOO_PARSER_BEGIN( parser, response );
@@ -190,17 +220,27 @@ IDirectFB_Requestor_EnumVideoModes( IDirectFB            *thiz,
 
      VOODOO_PARSER_BEGIN( parser, response );
      VOODOO_PARSER_GET_INT( parser, num );
-     VOODOO_PARSER_COPY_DATA( parser, items );
+
+     items = D_MALLOC( sizeof(*items) * num );
+     if (items)
+          VOODOO_PARSER_READ_DATA( parser, items, sizeof(*items) * num );
+     else
+          ret = D_OOM();
+
      VOODOO_PARSER_END( parser );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     for (i=0; i<num; i++) {
-          if (callbackfunc( items[i].width, items[i].height, items[i].bpp, callbackdata ) == DFENUM_CANCEL)
-               return DFB_OK;
+     if (items) {
+          for (i=0; i<num; i++) {
+               if (callbackfunc( items[i].width, items[i].height, items[i].bpp, callbackdata ) == DFENUM_CANCEL)
+                    break;
+          }
+
+          D_FREE( items );
      }
 
-     return DFB_OK;
+     return ret;
 }
 
 static DFBResult
@@ -235,36 +275,71 @@ IDirectFB_Requestor_SetVideoMode( IDirectFB    *thiz,
 
 static DFBResult
 IDirectFB_Requestor_CreateSurface( IDirectFB                    *thiz,
-                                   const DFBSurfaceDescription  *desc,
+                                   const DFBSurfaceDescription  *real_desc,
                                    IDirectFBSurface            **ret_interface )
 {
-     DirectResult           ret;
-     VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     DirectResult            ret;
+     VoodooResponseMessage  *response;
+     VoodooInstanceID        instance_id;
+     DFBSurfaceDescription   desc;
+     DFBSurfaceCapabilities  caps          = DSCAPS_NONE;
+     void                   *interface_ptr = NULL;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
-     if (!desc || !ret_interface)
+     if (!real_desc || !ret_interface)
           return DFB_INVARG;
 
-     if (desc->flags & (DSDESC_PALETTE | DSDESC_PREALLOCATED))
+     direct_memcpy( &desc, real_desc, sizeof(DFBSurfaceDescription) );
+
+     if (desc.flags & (DSDESC_PALETTE | DSDESC_PREALLOCATED))
           D_ONCE( "DSDESC_PALETTE and DSDESC_PREALLOCATED not supported yet" );
 
+     if (desc.flags & DSDESC_CAPS)
+          caps = desc.caps;
+
+     if (caps & DSCAPS_PRIMARY) {
+          if (!(desc.flags & DSDESC_WIDTH) && dfb_config->mode.width) {
+               desc.flags |= DSDESC_WIDTH;
+               desc.width  = dfb_config->mode.width;
+          }
+
+          if (!(desc.flags & DSDESC_HEIGHT) && dfb_config->mode.height) {
+               desc.flags  |= DSDESC_HEIGHT;
+               desc.height  = dfb_config->mode.height;
+          }
+     }
+
+     if (!(desc.flags & DSDESC_PIXELFORMAT) && dfb_config->mode.format != DSPF_UNKNOWN) {
+          desc.flags       |= DSDESC_PIXELFORMAT;
+          desc.pixelformat  = dfb_config->mode.format;
+     }
+
+     if (!(desc.flags & DSDESC_RESOURCE_ID)) {
+          desc.flags       |= DSDESC_RESOURCE_ID;
+          desc.resource_id  = dfb_config->resource_id;
+     }
+
+     D_INFO( "IDirectFB_Requestor_CreateSurface: Using resource ID %lu\n", desc.resource_id );
+
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFB_METHOD_ID_CreateSurface, VREQ_RESPOND, &response,
-                                   VMBT_DATA, sizeof(DFBSurfaceDescription), desc,
+                                   VMBT_DFBSurfaceDescription( desc ),
                                    VMBT_NONE );
      if (ret)
           return ret;
 
-     ret = response->result;
-     if (ret == DFB_OK)
-          ret = voodoo_construct_requestor( data->manager, "IDirectFBSurface",
-                                            response->instance, NULL, &interface );
+     /* Copy and finish as we do our next request in surface constructor already! */
+     instance_id = response->instance;
 
      voodoo_manager_finish_request( data->manager, response );
 
-     *ret_interface = interface;
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, "IDirectFBSurface",
+                                            instance_id, thiz, &interface_ptr );
+
+     *ret_interface = interface_ptr;
 
      return ret;
 }
@@ -272,11 +347,11 @@ IDirectFB_Requestor_CreateSurface( IDirectFB                    *thiz,
 static DFBResult
 IDirectFB_Requestor_CreatePalette( IDirectFB                    *thiz,
                                    const DFBPaletteDescription  *desc,
-                                   IDirectFBPalette            **interface )
+                                   IDirectFBPalette            **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
      D_UNIMPLEMENTED();
@@ -314,17 +389,27 @@ IDirectFB_Requestor_EnumScreens( IDirectFB         *thiz,
 
      VOODOO_PARSER_BEGIN( parser, response );
      VOODOO_PARSER_GET_INT( parser, num );
-     VOODOO_PARSER_COPY_DATA( parser, items );
+
+     items = D_MALLOC( sizeof(*items) * num );
+     if (items)
+          VOODOO_PARSER_READ_DATA( parser, items, sizeof(*items) * num );
+     else
+          ret = D_OOM();
+
      VOODOO_PARSER_END( parser );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     for (i=0; i<num; i++) {
-          if (callbackfunc( items[i].screen_id, items[i].desc, callbackdata ) == DFENUM_CANCEL)
-               return DFB_OK;
+     if (items) {
+          for (i=0; i<num; i++) {
+               if (callbackfunc( items[i].screen_id, items[i].desc, callbackdata ) == DFENUM_CANCEL)
+                    break;
+          }
+
+          D_FREE( items );
      }
 
-     return DFB_OK;
+     return ret;
 }
 
 static DFBResult
@@ -334,13 +419,12 @@ IDirectFB_Requestor_GetScreen( IDirectFB        *thiz,
 {
      DirectResult           ret;
      VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     void                  *interface_ptr = NULL;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
      if (!ret_interface)
           return DFB_INVARG;
-
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFB_METHOD_ID_GetScreen, VREQ_RESPOND, &response,
                                    VMBT_ID, id,
@@ -349,13 +433,13 @@ IDirectFB_Requestor_GetScreen( IDirectFB        *thiz,
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK)
+     if (ret == DR_OK)
           ret = voodoo_construct_requestor( data->manager, "IDirectFBScreen",
-                                            response->instance, NULL, &interface );
+                                            response->instance, thiz, &interface_ptr );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     *ret_interface = interface;
+     *ret_interface = interface_ptr;
 
      return ret;
 }
@@ -382,7 +466,7 @@ IDirectFB_Requestor_GetDisplayLayer( IDirectFB              *thiz,
 {
      DirectResult           ret;
      VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     void                  *interface_ptr = NULL;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
@@ -397,13 +481,13 @@ IDirectFB_Requestor_GetDisplayLayer( IDirectFB              *thiz,
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK)
+     if (ret == DR_OK)
           ret = voodoo_construct_requestor( data->manager, "IDirectFBDisplayLayer",
-                                            response->instance, NULL, &interface );
+                                            response->instance, thiz, &interface_ptr );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     *ret_interface = interface;
+     *ret_interface = interface_ptr;
 
      return ret;
 }
@@ -437,14 +521,24 @@ IDirectFB_Requestor_EnumInputDevices( IDirectFB              *thiz,
 
      VOODOO_PARSER_BEGIN( parser, response );
      VOODOO_PARSER_GET_INT( parser, num );
-     VOODOO_PARSER_COPY_DATA( parser, items );
+
+     items = D_MALLOC( sizeof(*items) * num );
+     if (items)
+          VOODOO_PARSER_READ_DATA( parser, items, sizeof(*items) * num );
+     else
+          ret = D_OOM();
+
      VOODOO_PARSER_END( parser );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     for (i=0; i<num; i++) {
-          if (callbackfunc( items[i].device_id, items[i].desc, callbackdata ) == DFENUM_CANCEL)
-               return DFB_OK;
+     if (items) {
+          for (i=0; i<num; i++) {
+               if (callbackfunc( items[i].device_id, items[i].desc, callbackdata ) == DFENUM_CANCEL)
+                    break;
+          }
+
+          D_FREE( items );
      }
 
      return ret;
@@ -457,7 +551,7 @@ IDirectFB_Requestor_GetInputDevice( IDirectFB             *thiz,
 {
      DirectResult           ret;
      VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     void                  *interface_ptr = NULL;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
@@ -472,13 +566,13 @@ IDirectFB_Requestor_GetInputDevice( IDirectFB             *thiz,
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK)
+     if (ret == DR_OK)
           ret = voodoo_construct_requestor( data->manager, "IDirectFBInputDevice",
-                                            response->instance, NULL, &interface );
+                                            response->instance, thiz, &interface_ptr );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     *ret_interface = interface;
+     *ret_interface = interface_ptr;
 
      return ret;
 }
@@ -622,12 +716,12 @@ IDirectFB_Requestor_CreateImageProvider( IDirectFB               *thiz,
 static DFBResult
 IDirectFB_Requestor_CreateVideoProvider( IDirectFB               *thiz,
                                          const char              *filename,
-                                         IDirectFBVideoProvider **interface )
+                                         IDirectFBVideoProvider **interface_ptr )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
      /* Check arguments */
-     if (!interface || !filename)
+     if (!interface_ptr || !filename)
           return DFB_INVARG;
 
      D_UNIMPLEMENTED();
@@ -641,36 +735,37 @@ IDirectFB_Requestor_CreateFont( IDirectFB                 *thiz,
                                 const DFBFontDescription  *desc,
                                 IDirectFBFont            **ret_interface )
 {
-     DirectResult           ret;
-     VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     DFBResult                 ret;
+     DFBDataBufferDescription  bdesc;
+     IDirectFBDataBuffer      *buffer;
+     IDirectFBFont            *font;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
-     if (!ret_interface)
+     /* Check arguments */
+     if (!filename || !ret_interface)
           return DFB_INVARG;
 
-     if (!filename || !desc) {
-          D_ONCE( "unimplemented" );
-          return DFB_UNSUPPORTED;
-     }
+     /* Create a data buffer. */
+     bdesc.flags = DBDESC_FILE;
+     bdesc.file  = filename;
 
-     ret = voodoo_manager_request( data->manager, data->instance,
-                                   IDIRECTFB_METHOD_ID_CreateFont, VREQ_RESPOND, &response,
-                                   VMBT_STRING, filename,
-                                   VMBT_DATA, sizeof(DFBFontDescription), desc,
-                                   VMBT_NONE );
+     ret = thiz->CreateDataBuffer( thiz, &bdesc, &buffer );
      if (ret)
           return ret;
 
-     ret = response->result;
-     if (ret == DFB_OK)
-          ret = voodoo_construct_requestor( data->manager, "IDirectFBFont",
-                                            response->instance, NULL, &interface );
+     /* Create (probing) the font. */
+     ret = buffer->CreateFont( buffer, desc, &font );
+     if (ret) {
+          buffer->Release( buffer );
+          return ret;
+     }
 
-     voodoo_manager_finish_request( data->manager, response );
+     /* We don't need it anymore, font has its own reference. */
+     buffer->Release( buffer );
 
-     *ret_interface = interface;
+     /* Return the new font. */
+     *ret_interface = font;
 
      return ret;
 }
@@ -695,7 +790,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
      if (!desc) {
           DIRECT_ALLOCATE_INTERFACE( buffer, IDirectFBDataBuffer );
 
-          ret = IDirectFBDataBuffer_Streamed_Construct( buffer, NULL );
+          ret = IDirectFBDataBuffer_Streamed_Construct( buffer, NULL, thiz );
      }
      else if (desc->flags & DBDESC_FILE) {
           if (!desc->file)
@@ -703,7 +798,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
 
           DIRECT_ALLOCATE_INTERFACE( buffer, IDirectFBDataBuffer );
 
-          ret = IDirectFBDataBuffer_File_Construct( buffer, desc->file, NULL );
+          ret = IDirectFBDataBuffer_File_Construct( buffer, desc->file, NULL, thiz );
      }
      else if (desc->flags & DBDESC_MEMORY) {
           if (!desc->memory.data || !desc->memory.length)
@@ -714,7 +809,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
           ret = IDirectFBDataBuffer_Memory_Construct( buffer,
                                                       desc->memory.data,
                                                       desc->memory.length,
-                                                      NULL );
+                                                      NULL, thiz );
      }
      else
           return DFB_INVARG;
@@ -724,7 +819,7 @@ IDirectFB_Requestor_CreateDataBuffer( IDirectFB                       *thiz,
 
      /* Create the dispatcher. */
      ret = voodoo_construct_dispatcher( data->manager, "IDirectFBDataBuffer",
-                                        buffer, data->instance, NULL, &instance, &ptr );
+                                        buffer, data->instance, thiz, &instance, &ptr );
      if (ret) {
           buffer->Release( buffer );
           return ret;
@@ -849,13 +944,36 @@ IDirectFB_Requestor_GetInterface( IDirectFB   *thiz,
                                   const char  *type,
                                   const char  *implementation,
                                   void        *arg,
-                                  void       **interface )
+                                  void       **ret_interface )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     void                  *interface_ptr = NULL;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFB_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!type || !ret_interface)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     if (implementation || arg)
+          return DFB_UNSUPPORTED;
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFB_METHOD_ID_GetInterface, VREQ_RESPOND, &response,
+                                   VMBT_STRING, type,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, type, response->instance, thiz, &interface_ptr );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     *ret_interface = interface_ptr;
+
+     return ret;
 }
 
 /**************************************************************************************************/
@@ -873,7 +991,7 @@ Probe()
  * Fills in function pointers and intializes data structure.
  */
 static DFBResult
-Construct( IDirectFB *thiz, const char *host, int session )
+Construct( IDirectFB *thiz, const char *host, int port )
 {
      DFBResult ret;
 
@@ -882,7 +1000,7 @@ Construct( IDirectFB *thiz, const char *host, int session )
      data->ref   = 1;
      data->level = DFSCL_NORMAL;
 
-     ret = voodoo_client_create( host, session, &data->client );
+     ret = voodoo_client_create( host, port, &data->client );
      if (ret) {
           DIRECT_DEALLOCATE_INTERFACE( thiz );
           return ret;
