@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2007  The DirectFB Organization (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <directfb.h>
@@ -36,6 +40,8 @@
 #include <direct/messages.h>
 #include <direct/util.h>
 
+#include <misc/conf.h>
+
 #include <voodoo/client.h>
 #include <voodoo/interface.h>
 #include <voodoo/manager.h>
@@ -43,9 +49,10 @@
 #include <idirectfbdisplaylayer_dispatcher.h>
 
 #include "idirectfbdisplaylayer_requestor.h"
+#include "idirectfbsurface_requestor.h"
 
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFBDisplayLayer *thiz,
                             VoodooManager         *manager,
                             VoodooInstanceID       instance,
@@ -61,14 +68,20 @@ DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBDisplayLayer, Requestor )
 static void
 IDirectFBDisplayLayer_Requestor_Destruct( IDirectFBDisplayLayer *thiz )
 {
+     IDirectFBDisplayLayer_Requestor_data *data = thiz->priv;
+
      D_DEBUG( "%s (%p)\n", __FUNCTION__, thiz );
 
+     voodoo_manager_request( data->manager, data->instance,
+                             IDIRECTFBDISPLAYLAYER_METHOD_ID_Release, VREQ_NONE, NULL,
+                             VMBT_NONE );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 /**************************************************************************************************/
 
-static DFBResult
+static DirectResult
 IDirectFBDisplayLayer_Requestor_AddRef( IDirectFBDisplayLayer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
@@ -78,7 +91,7 @@ IDirectFBDisplayLayer_Requestor_AddRef( IDirectFBDisplayLayer *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBDisplayLayer_Requestor_Release( IDirectFBDisplayLayer *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
@@ -91,13 +104,39 @@ IDirectFBDisplayLayer_Requestor_Release( IDirectFBDisplayLayer *thiz )
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_GetID( IDirectFBDisplayLayer *thiz,
-                                       DFBDisplayLayerID     *id )
+                                       DFBDisplayLayerID     *ret_id )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     VoodooMessageParser    parser;
+     DFBDisplayLayerID      id;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!ret_id)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_GetID, VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret) {
+          voodoo_manager_finish_request( data->manager, response );
+          return ret;
+     }
+
+     VOODOO_PARSER_BEGIN( parser, response );
+     VOODOO_PARSER_GET_ID( parser, id );
+     VOODOO_PARSER_END( parser );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     *ret_id = id;
+
+     return ret;
 }
 
 static DFBResult
@@ -113,35 +152,87 @@ IDirectFBDisplayLayer_Requestor_GetDescription( IDirectFBDisplayLayer      *thiz
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_GetSurface( IDirectFBDisplayLayer  *thiz,
-                                            IDirectFBSurface      **interface )
+                                            IDirectFBSurface      **ret_interface )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     void                  *interface_ptr = NULL;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!ret_interface)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_GetSurface, VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, "IDirectFBSurface",
+                                            response->instance, data->idirectfb, &interface_ptr );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     *ret_interface = interface_ptr;
+
+     return ret;
 }
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_GetScreen( IDirectFBDisplayLayer  *thiz,
-                                           IDirectFBScreen       **interface )
+                                           IDirectFBScreen       **ret_interface )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     void                  *interface_ptr = NULL;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!ret_interface)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_GetScreen, VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, "IDirectFBScreen",
+                                            response->instance, data->idirectfb, &interface_ptr );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     *ret_interface = interface_ptr;
+
+     return ret;
 }
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_SetCooperativeLevel( IDirectFBDisplayLayer           *thiz,
                                                      DFBDisplayLayerCooperativeLevel  level )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_SetCooperativeLevel, VREQ_RESPOND, &response,
+                                   VMBT_UINT, level,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
 
-     return DFB_UNIMPLEMENTED;
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
 }
 
 static DFBResult
@@ -253,7 +344,7 @@ IDirectFBDisplayLayer_Requestor_GetConfiguration( IDirectFBDisplayLayer *thiz,
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK) {
+     if (ret == DR_OK) {
           VoodooMessageParser parser;
 
           VOODOO_PARSER_BEGIN( parser, response );
@@ -269,24 +360,61 @@ IDirectFBDisplayLayer_Requestor_GetConfiguration( IDirectFBDisplayLayer *thiz,
 static DFBResult
 IDirectFBDisplayLayer_Requestor_TestConfiguration( IDirectFBDisplayLayer       *thiz,
                                                    const DFBDisplayLayerConfig *config,
-                                                   DFBDisplayLayerConfigFlags  *failed )
+                                                   DFBDisplayLayerConfigFlags  *ret_failed )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     VoodooMessageParser    parser;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!config)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_TestConfiguration, VREQ_RESPOND, &response,
+                                   VMBT_DATA, sizeof(DFBDisplayLayerConfig), config,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     if (ret_failed) {
+          VOODOO_PARSER_BEGIN( parser, response );
+          VOODOO_PARSER_GET_UINT( parser, *ret_failed );
+          VOODOO_PARSER_END( parser );
+     }
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
 }
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_SetConfiguration( IDirectFBDisplayLayer       *thiz,
                                                   const DFBDisplayLayerConfig *config )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!config)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_SetConfiguration, VREQ_RESPOND, &response,
+                                   VMBT_DATA, sizeof(DFBDisplayLayerConfig), config,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
 }
 
 static DFBResult
@@ -295,20 +423,29 @@ IDirectFBDisplayLayer_Requestor_SetBackgroundMode( IDirectFBDisplayLayer
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
-
-     return DFB_UNIMPLEMENTED;
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBDISPLAYLAYER_METHOD_ID_SetBackgroundMode, VREQ_NONE, NULL,
+                                    VMBT_INT, background_mode,
+                                    VMBT_NONE );
 }
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_SetBackgroundImage( IDirectFBDisplayLayer *thiz,
                                                     IDirectFBSurface      *surface )
 {
+     IDirectFBSurface_Requestor_data *surface_data;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!surface)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     DIRECT_INTERFACE_GET_DATA_FROM( surface, surface_data, IDirectFBSurface_Requestor );
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBDISPLAYLAYER_METHOD_ID_SetBackgroundImage, VREQ_NONE, NULL,
+                                    VMBT_ID, surface_data->instance,
+                                    VMBT_NONE );
 }
 
 static DFBResult
@@ -324,33 +461,43 @@ IDirectFBDisplayLayer_Requestor_SetBackgroundColor( IDirectFBDisplayLayer *thiz,
 
 static DFBResult
 IDirectFBDisplayLayer_Requestor_CreateWindow( IDirectFBDisplayLayer       *thiz,
-                                              const DFBWindowDescription  *desc,
+                                              const DFBWindowDescription  *real_desc,
                                               IDirectFBWindow            **ret_interface )
 {
      DirectResult           ret;
      VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     DFBWindowDescription   desc;
+     void                  *interface_ptr = NULL;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
      if (!ret_interface)
           return DFB_INVARG;
 
+     direct_memcpy( &desc, real_desc, sizeof(DFBWindowDescription) );
+
+     if (!(desc.flags & DWDESC_RESOURCE_ID)) {
+          desc.flags       |= DWDESC_RESOURCE_ID;
+          desc.resource_id  = dfb_config->resource_id;
+     }
+
+     D_INFO( "IDirectFBDisplayLayer_Requestor_CreateWindow: Using resource ID %lu\n", desc.resource_id );
+
      ret = voodoo_manager_request( data->manager, data->instance,
                                    IDIRECTFBDISPLAYLAYER_METHOD_ID_CreateWindow, VREQ_RESPOND, &response,
-                                   VMBT_DATA, sizeof(DFBWindowDescription), desc,
+                                   VMBT_DATA, sizeof(DFBWindowDescription), &desc,
                                    VMBT_NONE );
      if (ret)
           return ret;
 
      ret = response->result;
-     if (ret == DFB_OK)
+     if (ret == DR_OK)
           ret = voodoo_construct_requestor( data->manager, "IDirectFBWindow",
-                                            response->instance, NULL, &interface );
+                                            response->instance, data->idirectfb, &interface_ptr );
 
      voodoo_manager_finish_request( data->manager, response );
 
-     *ret_interface = interface;
+     *ret_interface = interface_ptr;
 
      return ret;
 }
@@ -358,13 +505,34 @@ IDirectFBDisplayLayer_Requestor_CreateWindow( IDirectFBDisplayLayer       *thiz,
 static DFBResult
 IDirectFBDisplayLayer_Requestor_GetWindow( IDirectFBDisplayLayer  *thiz,
                                            DFBWindowID             id,
-                                           IDirectFBWindow       **window )
+                                           IDirectFBWindow       **ret_interface )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     void                  *interface_ptr = NULL;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!ret_interface)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_GetWindow, VREQ_RESPOND, &response,
+                                   VMBT_ID, id,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, "IDirectFBWindow",
+                                            response->instance, data->idirectfb, &interface_ptr );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     *ret_interface = interface_ptr;
+
+     return ret;
 }
 
 static DFBResult
@@ -467,6 +635,71 @@ IDirectFBDisplayLayer_Requestor_WaitForSync( IDirectFBDisplayLayer *thiz )
      return DFB_UNIMPLEMENTED;
 }
 
+static DFBResult
+IDirectFBDisplayLayer_Requestor_GetWindowByResourceID( IDirectFBDisplayLayer  *thiz,
+                                                       unsigned long           resource_id,
+                                                       IDirectFBWindow       **ret_interface )
+{
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     void                  *interface_ptr = NULL;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
+
+     if (!ret_interface)
+          return DFB_INVARG;
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_GetWindowByResourceID, VREQ_RESPOND, &response,
+                                   VMBT_ID, resource_id,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, "IDirectFBWindow",
+                                            response->instance, data->idirectfb, &interface_ptr );
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     *ret_interface = interface_ptr;
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBDisplayLayer_Requestor_GetRotation( IDirectFBDisplayLayer *thiz,
+                                             int                   *ret_rotation )
+{
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBDisplayLayer_Requestor)
+
+     if (!ret_rotation)
+          return DFB_INVARG;
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBDISPLAYLAYER_METHOD_ID_GetRotation, VREQ_RESPOND, &response,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret == DR_OK) {
+          VoodooMessageParser parser;
+
+          VOODOO_PARSER_BEGIN( parser, response );
+          VOODOO_PARSER_GET_INT( parser, *ret_rotation );
+          VOODOO_PARSER_END( parser );
+     }
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
+}
+
 /**************************************************************************************************/
 
 static DFBResult
@@ -484,9 +717,10 @@ Construct( IDirectFBDisplayLayer *thiz,
 {
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBDisplayLayer_Requestor)
 
-     data->ref      = 1;
-     data->manager  = manager;
-     data->instance = instance;
+     data->ref       = 1;
+     data->manager   = manager;
+     data->instance  = instance;
+     data->idirectfb = arg;
 
      thiz->AddRef                = IDirectFBDisplayLayer_Requestor_AddRef;
      thiz->Release               = IDirectFBDisplayLayer_Requestor_Release;
@@ -520,6 +754,8 @@ Construct( IDirectFBDisplayLayer *thiz,
      thiz->SetCursorOpacity      = IDirectFBDisplayLayer_Requestor_SetCursorOpacity;
      thiz->SetFieldParity        = IDirectFBDisplayLayer_Requestor_SetFieldParity;
      thiz->WaitForSync           = IDirectFBDisplayLayer_Requestor_WaitForSync;
+     thiz->GetWindowByResourceID = IDirectFBDisplayLayer_Requestor_GetWindowByResourceID;
+     thiz->GetRotation           = IDirectFBDisplayLayer_Requestor_GetRotation;
 
      return DFB_OK;
 }
