@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <directfb.h>
@@ -40,12 +44,13 @@
 #include <voodoo/interface.h>
 #include <voodoo/manager.h>
 
+#include <idirectfb_dispatcher.h>
 #include <idirectfbimageprovider_dispatcher.h>
 
 #include "idirectfbsurface_requestor.h"
 
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFBImageProvider *thiz,
                             VoodooManager          *manager,
                             VoodooInstanceID        instance,
@@ -66,6 +71,8 @@ typedef struct {
 
      VoodooManager         *manager;
      VoodooInstanceID       instance;
+
+     IDirectFBDataBuffer   *buffer;
 } IDirectFBImageProvider_Requestor_data;
 
 /**************************************************************************************************/
@@ -73,8 +80,16 @@ typedef struct {
 static void
 IDirectFBImageProvider_Requestor_Destruct( IDirectFBImageProvider *thiz )
 {
+     IDirectFBImageProvider_Requestor_data *data = thiz->priv;
+
      D_DEBUG( "%s (%p)\n", __FUNCTION__, thiz );
 
+     data->buffer->Release( data->buffer );
+
+     voodoo_manager_request( data->manager, data->instance,
+                             IDIRECTFBIMAGEPROVIDER_METHOD_ID_Release, VREQ_NONE, NULL,
+                             VMBT_NONE );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
@@ -128,7 +143,7 @@ IDirectFBImageProvider_Requestor_GetSurfaceDescription( IDirectFBImageProvider *
      }
 
      VOODOO_PARSER_BEGIN( parser, response );
-     VOODOO_PARSER_READ_DATA( parser, desc, sizeof(DFBSurfaceDescription) );
+     VOODOO_PARSER_READ_DFBSurfaceDescription( parser, *desc );
      VOODOO_PARSER_END( parser );
 
      voodoo_manager_finish_request( data->manager, response );
@@ -230,11 +245,16 @@ Construct( IDirectFBImageProvider *thiz,
            VoodooInstanceID        instance,
            void                   *arg )
 {
+     IDirectFBDataBuffer *buffer = arg;
+
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBImageProvider_Requestor)
 
+     buffer->AddRef( buffer );
+
      data->ref      = 1;
      data->manager  = manager;
      data->instance = instance;
+     data->buffer   = buffer;
 
      thiz->AddRef                = IDirectFBImageProvider_Requestor_AddRef;
      thiz->Release               = IDirectFBImageProvider_Requestor_Release;
