@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <directfb.h>
@@ -49,7 +53,7 @@
 #include "idirectfbwindow_requestor.h"
 
 
-static DFBResult Probe();
+static DFBResult Probe( void );
 static DFBResult Construct( IDirectFBWindow  *thiz,
                             VoodooManager    *manager,
                             VoodooInstanceID  instance,
@@ -60,19 +64,27 @@ static DFBResult Construct( IDirectFBWindow  *thiz,
 DIRECT_INTERFACE_IMPLEMENTATION( IDirectFBWindow, Requestor )
 
 
+D_DEBUG_DOMAIN( IDirectFBWindow_Requestor, "IDirectFBWindow/Requestor", "IDirectFBWindow Requestor" );
+
 /**************************************************************************************************/
 
 static void
 IDirectFBWindow_Requestor_Destruct( IDirectFBWindow *thiz )
 {
+     IDirectFBWindow_Requestor_data *data = thiz->priv;
+
      D_DEBUG( "%s (%p)\n", __FUNCTION__, thiz );
 
+     voodoo_manager_request( data->manager, data->instance,
+                             IDIRECTFBWINDOW_METHOD_ID_Release, VREQ_NONE, NULL,
+                             VMBT_NONE );
+
      DIRECT_DEALLOCATE_INTERFACE( thiz );
 }
 
 /**************************************************************************************************/
 
-static DFBResult
+static DirectResult
 IDirectFBWindow_Requestor_AddRef( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
@@ -82,7 +94,7 @@ IDirectFBWindow_Requestor_AddRef( IDirectFBWindow *thiz )
      return DFB_OK;
 }
 
-static DFBResult
+static DirectResult
 IDirectFBWindow_Requestor_Release( IDirectFBWindow *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
@@ -158,6 +170,8 @@ IDirectFBWindow_Requestor_AttachEventBuffer( IDirectFBWindow      *thiz,
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
+     D_DEBUG_AT( IDirectFBWindow_Requestor, "%s( buffer %p )\n", __FUNCTION__, buffer );
+
      if (!buffer)
           return DFB_INVARG;
 
@@ -214,9 +228,10 @@ IDirectFBWindow_Requestor_EnableEvents( IDirectFBWindow       *thiz,
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
-     D_UNIMPLEMENTED();
-
-     return DFB_UNIMPLEMENTED;
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBWINDOW_METHOD_ID_EnableEvents, VREQ_NONE, NULL,
+                                    VMBT_UINT, mask,
+                                    VMBT_NONE );
 }
 
 static DFBResult
@@ -357,7 +372,8 @@ IDirectFBWindow_Requestor_GetSurface( IDirectFBWindow   *thiz,
 {
      DirectResult           ret;
      VoodooResponseMessage *response;
-     void                  *interface = NULL;
+     VoodooInstanceID       instance_id;
+     void                  *interface_ptr = NULL;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
@@ -370,14 +386,17 @@ IDirectFBWindow_Requestor_GetSurface( IDirectFBWindow   *thiz,
      if (ret)
           return ret;
 
-     ret = response->result;
-     if (ret == DFB_OK)
-          ret = voodoo_construct_requestor( data->manager, "IDirectFBSurface",
-                                            response->instance, NULL, &interface );
+     /* Copy and finish as we do our next request in surface constructor already! */
+     instance_id = response->instance;
 
      voodoo_manager_finish_request( data->manager, response );
 
-     *ret_interface = interface;
+     ret = response->result;
+     if (ret == DR_OK)
+          ret = voodoo_construct_requestor( data->manager, "IDirectFBSurface",
+                                            instance_id, data->idirectfb, &interface_ptr );
+
+     *ret_interface = interface_ptr;
 
      return ret;
 }
@@ -398,13 +417,40 @@ IDirectFBWindow_Requestor_SetProperty( IDirectFBWindow  *thiz,
 static DFBResult
 IDirectFBWindow_Requestor_GetProperty( IDirectFBWindow  *thiz,
                                        const char       *key,
-                                       void            **value )
+                                       void            **ret_value )
 {
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+     VoodooMessageParser    parser;
+     const char            *value;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
 
-     D_UNIMPLEMENTED();
+     if (!ret_value)
+          return DFB_INVARG;
 
-     return DFB_UNIMPLEMENTED;
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_GetProperty, VREQ_RESPOND, &response,
+                                   VMBT_STRING, key,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+     if (ret) {
+          voodoo_manager_finish_request( data->manager, response );
+          return ret;
+     }
+
+     VOODOO_PARSER_BEGIN( parser, response );
+     VOODOO_PARSER_GET_STRING( parser, value );
+     VOODOO_PARSER_END( parser );
+
+     *ret_value = D_STRDUP( value );    // FIXME: LEAK
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return DFB_OK;
 }
 
 static DFBResult
@@ -419,7 +465,6 @@ IDirectFBWindow_Requestor_RemoveProperty( IDirectFBWindow  *thiz,
      return DFB_UNIMPLEMENTED;
 }
 
-
 static DFBResult
 IDirectFBWindow_Requestor_SetOptions( IDirectFBWindow  *thiz,
                                       DFBWindowOptions  options )
@@ -904,6 +949,106 @@ IDirectFBWindow_Requestor_ResizeSurface( IDirectFBWindow *thiz,
      return ret;
 }
 
+static DFBResult
+IDirectFBWindow_Requestor_Bind( IDirectFBWindow *thiz,
+                                IDirectFBWindow *source,
+                                int              x,
+                                int              y )
+{
+     D_UNIMPLEMENTED();
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_Unbind( IDirectFBWindow *thiz,
+                                  IDirectFBWindow *source )
+{
+     D_UNIMPLEMENTED();
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_SetKeySelection( IDirectFBWindow               *thiz,
+                                           DFBWindowKeySelection          selection,
+                                           const DFBInputDeviceKeySymbol *keys,
+                                           unsigned int                   num_keys )
+{
+     D_UNIMPLEMENTED();
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_GrabUnselectedKeys( IDirectFBWindow *thiz )
+{
+     D_UNIMPLEMENTED();
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_UngrabUnselectedKeys( IDirectFBWindow *thiz )
+{
+     D_UNIMPLEMENTED();
+     return DFB_UNIMPLEMENTED;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_SetSrcGeometry( IDirectFBWindow         *thiz,
+                                          const DFBWindowGeometry *geometry )
+{
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_SetSrcGeometry, VREQ_RESPOND, &response,
+                                   VMBT_DATA, sizeof(DFBWindowGeometry), geometry,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_SetDstGeometry( IDirectFBWindow         *thiz,
+                                          const DFBWindowGeometry *geometry )
+{
+     DirectResult           ret;
+     VoodooResponseMessage *response;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     ret = voodoo_manager_request( data->manager, data->instance,
+                                   IDIRECTFBWINDOW_METHOD_ID_SetDstGeometry, VREQ_RESPOND, &response,
+                                   VMBT_DATA, sizeof(DFBWindowGeometry), geometry,
+                                   VMBT_NONE );
+     if (ret)
+          return ret;
+
+     ret = response->result;
+
+     voodoo_manager_finish_request( data->manager, response );
+
+     return ret;
+}
+
+static DFBResult
+IDirectFBWindow_Requestor_SendEvent( IDirectFBWindow      *thiz,
+                                     const DFBWindowEvent *event )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBWindow_Requestor)
+
+     return voodoo_manager_request( data->manager, data->instance,
+                                    IDIRECTFBWINDOW_METHOD_ID_SendEvent, VREQ_NONE, NULL,
+                                    VMBT_DATA, sizeof(DFBWindowEvent), event,
+                                    VMBT_NONE );
+}
+
 /**************************************************************************************************/
 
 static DFBResult
@@ -921,9 +1066,10 @@ Construct( IDirectFBWindow  *thiz,
 {
      DIRECT_ALLOCATE_INTERFACE_DATA(thiz, IDirectFBWindow_Requestor)
 
-     data->ref      = 1;
-     data->manager  = manager;
-     data->instance = instance;
+     data->ref       = 1;
+     data->manager   = manager;
+     data->instance  = instance;
+     data->idirectfb = arg;
 
      thiz->AddRef             = IDirectFBWindow_Requestor_AddRef;
      thiz->Release            = IDirectFBWindow_Requestor_Release;
@@ -968,6 +1114,14 @@ Construct( IDirectFBWindow  *thiz,
      thiz->Destroy            = IDirectFBWindow_Requestor_Destroy;
      thiz->SetBounds          = IDirectFBWindow_Requestor_SetBounds;
      thiz->ResizeSurface      = IDirectFBWindow_Requestor_ResizeSurface;
+     thiz->Bind               = IDirectFBWindow_Requestor_Bind;
+     thiz->Unbind             = IDirectFBWindow_Requestor_Unbind;
+     thiz->SetKeySelection    = IDirectFBWindow_Requestor_SetKeySelection;
+     thiz->GrabUnselectedKeys = IDirectFBWindow_Requestor_GrabUnselectedKeys;
+     thiz->UngrabUnselectedKeys = IDirectFBWindow_Requestor_UngrabUnselectedKeys;
+     thiz->SetSrcGeometry      = IDirectFBWindow_Requestor_SetSrcGeometry;
+     thiz->SetDstGeometry      = IDirectFBWindow_Requestor_SetDstGeometry;
+     thiz->SendEvent          = IDirectFBWindow_Requestor_SendEvent;
 
      return DFB_OK;
 }
