@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __DFB__CORE__CORE_SYSTEM_H__
 #define __DFB__CORE__CORE_SYSTEM_H__
 
@@ -100,6 +104,15 @@ system_get_busid( int *ret_bus, int *ret_dev, int *ret_func );
 static void
 system_get_deviceid( unsigned int *ret_vendor_id, unsigned int *ret_device_id );
 
+static int
+system_surface_data_size( void );
+
+static void
+system_surface_data_init( CoreSurface *surface, void *data );
+
+static void
+system_surface_data_destroy( CoreSurface *surface, void *data );
+
 
 static CoreSystemFuncs system_funcs = {
      .GetSystemInfo       = system_get_info,
@@ -123,7 +136,10 @@ static CoreSystemFuncs system_funcs = {
      .AuxMemoryVirtual    = system_aux_memory_virtual,
      .AuxRamLength        = system_auxram_length,
      .GetBusID            = system_get_busid,
-     .GetDeviceID         = system_get_deviceid
+     .GetDeviceID         = system_get_deviceid,
+     .SurfaceDataSize     = system_surface_data_size,
+     .SurfaceDataInit     = system_surface_data_init,
+     .SurfaceDataDestroy  = system_surface_data_destroy
 };
 
 #define DFB_CORE_SYSTEM(shortname)                              \
