@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __CORETYPES_H__
 #define __CORETYPES_H__
 
@@ -47,6 +51,9 @@ typedef struct __DFB_DFBWMCore               DFBWMCore;
 
 typedef struct __DFB_DFBGraphicsCore         CoreGraphicsDevice; /* FIXME */
 
+typedef struct __DFB_CoreGraphicsState       CoreGraphicsState;
+typedef struct __DFB_CoreGraphicsStateClient CoreGraphicsStateClient;
+
 
 typedef struct _CoreCleanup                  CoreCleanup;
 
@@ -57,6 +64,11 @@ typedef struct _CorePalette                  CorePalette;
 typedef struct _CardState                    CardState;
 
 
+typedef struct __DFB_DFBFontManager          DFBFontManager;
+typedef struct __DFB_DFBFontCache            DFBFontCache;
+typedef struct __DFB_DFBFontCacheRow         DFBFontCacheRow;
+
+
 typedef struct __DFB_CoreGraphicsSerial      CoreGraphicsSerial;
 
 typedef struct __DFB_CoreScreen              CoreScreen;
@@ -73,6 +85,7 @@ typedef struct __DFB_CoreSurfaceAccessor     CoreSurfaceAccessor;
 typedef struct __DFB_CoreSurfaceAllocation   CoreSurfaceAllocation;
 typedef struct __DFB_CoreSurfaceBuffer       CoreSurfaceBuffer;
 typedef struct __DFB_CoreSurfaceBufferLock   CoreSurfaceBufferLock;
+typedef struct __DFB_CoreSurfaceClient       CoreSurfaceClient;
 typedef struct __DFB_CoreSurfacePool         CoreSurfacePool;
 typedef struct __DFB_CoreSurfacePoolBridge   CoreSurfacePoolBridge;
 typedef struct __DFB_CoreSurfacePoolTransfer CoreSurfacePoolTransfer;
@@ -85,5 +98,55 @@ typedef struct __DFB_CoreWindowStack         CoreWindowStack;
 typedef unsigned int CoreSurfacePoolID;
 typedef unsigned int CoreSurfacePoolBridgeID;
 
+
+
+#ifdef __cplusplus
+
+#include <direct/Types++.h>
+
+namespace DirectFB {
+namespace Util {
+     class FPS;
+}
+class Renderer;
+class Task;
+class Throttle;
+class SurfaceTask;
+class DisplayTask;
+}
+#define DFB_Util_FPS               DirectFB::Util::FPS
+#define DFB_Renderer               DirectFB::Renderer
+#define DFB_Task                   DirectFB::Task
+#define DFB_TaskList               Direct::List<DirectFB::Task*>
+#define DFB_TaskListLocked         Direct::ListLocked<DirectFB::Task*>
+#define DFB_TaskListSimple         Direct::ListSimple<DirectFB::Task*>
+#define DFB_Throttle               DirectFB::Throttle
+#define DFB_SurfaceTask            DirectFB::SurfaceTask
+#define DFB_SurfaceTaskList        Direct::List<DirectFB::SurfaceTask*>
+#define DFB_SurfaceTaskListLocked  Direct::ListLocked<DirectFB::SurfaceTask*>
+#define DFB_SurfaceTaskListSimple  Direct::ListSimple<DirectFB::SurfaceTask*>
+#define DFB_DisplayTask            DirectFB::DisplayTask
+#define DFB_DisplayTaskList        Direct::List<DirectFB::DisplayTask*>
+#define DFB_DisplayTaskListLocked  Direct::ListLocked<DirectFB::DisplayTask*>
+#define DFB_DisplayTaskListSimple  Direct::ListSimple<DirectFB::DisplayTask*>
+#else
+typedef void DFB_Util_FPS;
+typedef void DFB_Renderer;
+typedef void DFB_Task;
+typedef void DFB_TaskList;
+typedef void DFB_TaskListLocked;
+typedef void DFB_TaskListSimple;
+typedef void DFB_Throttle;
+typedef void DFB_SurfaceTask;
+typedef void DFB_SurfaceTaskList;
+typedef void DFB_SurfaceTaskListLocked;
+typedef void DFB_SurfaceTaskListSimple;
+typedef void DFB_DisplayTask;
+typedef void DFB_DisplayTaskList;
+typedef void DFB_DisplayTaskListLocked;
+typedef void DFB_DisplayTaskListSimple;
+#endif
+
+
 #endif
 
