@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __DFB__CORE__SCREEN_H__
 #define __DFB__CORE__SCREEN_H__
 
@@ -45,6 +49,8 @@ DFBResult dfb_screen_set_powermode   ( CoreScreen                  *screen,
                                        DFBScreenPowerMode           mode );
 
 DFBResult dfb_screen_wait_vsync      ( CoreScreen                  *screen );
+DFBResult dfb_screen_get_vsync_count ( CoreScreen                  *screen,
+                                       unsigned long               *ret_count );
 
 
 /* Mixers */
@@ -118,5 +124,8 @@ DFBResult dfb_screen_get_layer_dimension( CoreScreen                 *screen,
                                           int                        *ret_width,
                                           int                        *ret_height );
 
+DFBResult dfb_screen_get_frame_interval ( CoreScreen                 *screen,
+                                          long long                  *ret_micros );
+
 #endif
 
