@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,14 +28,18 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __CORE__SURFACE_BUFFER_H__
 #define __CORE__SURFACE_BUFFER_H__
 
 #include <direct/debug.h>
 
+#include <fusion/object.h>
 #include <fusion/vector.h>
 
 #include <core/surface.h>
+#include <core/surface_allocation.h>
 
 #include <directfb.h>
 
@@ -46,58 +52,21 @@ typedef enum {
 
      CSBF_STICKED   = 0x00000001,  /* Sticked to one Surface Pool, e.g. system only. */
 
-     CSBF_ALL       = 0x00000001   /* All of these. */
+     CSBF_DECOUPLE  = 0x00000002,  /* Buffer is about to be deallocated and removed from surface. */
+     CSBF_RIGHT     = 0x00000004,  /* Buffer is for right eye */
+
+     CSBF_ALL       = 0x00000007   /* All of these. */
 } CoreSurfaceBufferFlags;
 
-/*
- * Configuration and State flags of a Surface Buffer Allocation
- */
+
 typedef enum {
-     CSALF_NONE          = 0x00000000,  /* None of these. */
+     CSBNF_NONE     = 0x00000000
+} CoreSurfaceBufferNotificationFlags;
 
-     CSALF_ONEFORALL     = 0x00000001,  /* Only one allocation in pool for all buffers. */
-     CSALF_VOLATILE      = 0x00000002,  /* Allocation should be freed when no longer up to date. */
-     CSALF_PREALLOCATED  = 0x00000004,  /* Preallocated memory, don't zap when "thrifty-surface-buffers" is active. */
+typedef struct {
+     CoreSurfaceBufferNotificationFlags  flags;
+} CoreSurfaceBufferNotification;
 
-     CSALF_MUCKOUT       = 0x00001000,  /* Indicates surface pool being in the progress of mucking out this and possibly
-                                           other allocations to have enough space for a new allocation to be made. */
-
-     CSALF_ALL           = 0x00001007   /* All of these. */
-} CoreSurfaceAllocationFlags;
-
-/*
- * An Allocation of a Surface Buffer
- */
-struct __DFB_CoreSurfaceAllocation {
-     int                            magic;
-
-     DirectSerial                   serial;       /* Equals serial of buffer if content is up to date. */
-
-     CoreSurfaceBuffer             *buffer;       /* Surface Buffer owning this allocation. */
-     CoreSurface                   *surface;      /* Surface owning the Buffer of this allocation. */
-     CoreSurfacePool               *pool;         /* Surface Pool providing the allocation. */
-     void                          *data;         /* Pool's private data for this allocation. */
-     int                            size;         /* Amount of data used by this allocation. */
-     unsigned long                  offset;       /* Offset within address range of pool if contiguous. */
-
-     CoreSurfaceAllocationFlags     flags;        /* Pool can return CSALF_ONEFORALL upon allocation of first buffer. */
-
-     const CoreSurfaceAccessFlags  *access;                 /* Possible access flags (pointer to pool description). */
-     CoreSurfaceAccessFlags         accessed[_CSAID_NUM];   /* Access since last synchronization. */
-};
-
-#define CORE_SURFACE_ALLOCATION_ASSERT(alloc)                                                  \
-     do {                                                                                      \
-          D_MAGIC_ASSERT( alloc, CoreSurfaceAllocation );                                      \
-          D_ASSUME( (alloc)->size > 0 );                                                       \
-          D_ASSERT( (alloc)->size >= 0 );                                                      \
-          D_ASSERT( (alloc)->offset + (alloc)->size <= ((alloc)->pool->desc.size ?:~0UL) );    \
-          D_FLAGS_ASSERT( (alloc)->access[CSAID_CPU], CSAF_ALL );                              \
-          D_FLAGS_ASSERT( (alloc)->access[CSAID_GPU], CSAF_ALL );                              \
-          D_FLAGS_ASSERT( (alloc)->flags, CSALF_ALL );                                         \
-          D_FLAGS_ASSERT( (alloc)->accessed[CSAID_CPU], CSAF_ALL );                            \
-          D_FLAGS_ASSERT( (alloc)->accessed[CSAID_GPU], CSAF_ALL );                            \
-     } while (0)
 
 /*
  * A Lock on a Surface Buffer
@@ -117,6 +86,8 @@ struct __DFB_CoreSurfaceBufferLock {
      unsigned int             pitch;              /* " */
 
      void                    *handle;             /* " */
+
+     DFB_SurfaceTask         *task;
 };
 
 static inline void
@@ -130,13 +101,14 @@ dfb_surface_buffer_lock_reset( CoreSurfaceBufferLock *lock )
      lock->phys       = 0;
      lock->offset     = ~0;
      lock->pitch      = 0;
-     lock->handle     = NULL;
+     lock->handle     = 0;
+     lock->task       = NULL;
 }
 
 static inline void
 dfb_surface_buffer_lock_init( CoreSurfaceBufferLock *lock, CoreSurfaceAccessorID accessor, CoreSurfaceAccessFlags access )
 {
-     D_MAGIC_SET( lock, CoreSurfaceBufferLock );
+     D_MAGIC_SET_ONLY( lock, CoreSurfaceBufferLock );
 
      lock->accessor = accessor;
      lock->access   = access;
@@ -159,15 +131,14 @@ dfb_surface_buffer_lock_deinit( CoreSurfaceBufferLock *lock )
      do {                                                                                           \
           D_MAGIC_ASSERT( lock, CoreSurfaceBufferLock );                                            \
           D_FLAGS_ASSERT( (lock)->access, CSAF_ALL );                                               \
-          if ((lock)->buffer) {                                                                     \
-               D_ASSERT( (lock)->allocation != NULL );                                              \
-               D_ASSERT( (lock)->buffer == (lock)->allocation->buffer );                            \
+          if ((lock)->allocation) {                                                                 \
+               /*D_ASSERT( (lock)->buffer == (lock)->allocation->buffer );*/                            \
                D_ASSUME( (lock)->addr != NULL || (lock)->phys != 0 || (lock)->offset != ~0 || (lock)->handle != NULL );\
                D_ASSUME( (lock)->offset == (lock)->allocation->offset || (lock)->offset == ~0 );    \
                D_ASSERT( (lock)->pitch > 0 || ((lock)->addr == NULL && (lock)->phys == 0) );        \
           }                                                                                         \
           else {                                                                                    \
-               D_ASSERT( (lock)->allocation == NULL );                                              \
+               D_ASSERT( (lock)->buffer == NULL );                                                  \
                D_ASSERT( (lock)->addr == NULL );                                                    \
                D_ASSERT( (lock)->phys == 0 );                                                       \
                D_ASSERT( (lock)->offset == ~0 );                                                    \
@@ -180,6 +151,8 @@ dfb_surface_buffer_lock_deinit( CoreSurfaceBufferLock *lock )
  * A Surface Buffer of a Surface
  */
 struct __DFB_CoreSurfaceBuffer {
+     FusionObject             object;
+
      int                      magic;
 
      DirectSerial             serial;        /* Increased when content is written. */
@@ -194,18 +167,28 @@ struct __DFB_CoreSurfaceBuffer {
 
      FusionVector             allocs;        /* Allocations within Surface Pools. */
 
-#if 1
-     unsigned int             locked;        /* Lock count. FIXME: Add fail safe cleanup! */
-#endif
+     CoreSurfaceConfig        config;        /* Configuration of its surface at the time of the buffer creation */
+     CoreSurfaceTypeFlags     type;
+     unsigned long            resource_id;   /* layer id, window id, or user specified */
+     
+     int                      index;
 };
 
+#define CORE_SURFACE_BUFFER_ASSERT(buffer)                                                     \
+     do {                                                                                      \
+          D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );                                         \
+     } while (0)
 
-DFBResult dfb_surface_buffer_new    ( CoreSurface             *surface,
+
+DFBResult dfb_surface_buffer_create ( CoreDFB                 *core,
+                                      CoreSurface             *surface,
                                       CoreSurfaceBufferFlags   flags,
+                                      int                      index,
                                       CoreSurfaceBuffer      **ret_buffer );
 
-DFBResult dfb_surface_buffer_destroy( CoreSurfaceBuffer       *buffer );
+DFBResult dfb_surface_buffer_decouple( CoreSurfaceBuffer       *buffer );
 
+DFBResult dfb_surface_buffer_deallocate( CoreSurfaceBuffer    *buffer );
 
 DFBResult dfb_surface_buffer_lock   ( CoreSurfaceBuffer       *buffer,
                                       CoreSurfaceAccessorID    accessor,
@@ -228,29 +211,41 @@ DFBResult dfb_surface_buffer_dump   ( CoreSurfaceBuffer       *buffer,
                                       const char              *directory,
                                       const char              *prefix );
 
+DFBResult dfb_surface_buffer_dump_raw( CoreSurfaceBuffer       *buffer,
+                                       const char              *directory,
+                                       const char              *prefix );
+
+DFBResult dfb_surface_buffer_dump_type_locked( CoreSurfaceBuffer     *buffer,
+                                               const char            *directory,
+                                               const char            *prefix,
+                                               bool                   raw,
+                                               CoreSurfaceBufferLock *lock );
+
+DFBResult dfb_surface_buffer_dump_type_locked2( CoreSurfaceBuffer     *buffer,
+                                                const char            *directory,
+                                                const char            *prefix,
+                                                bool                   raw,
+                                                void                  *addr,
+                                                int                    pitch );
+
+CoreSurfaceAllocation *
+dfb_surface_buffer_find_allocation( CoreSurfaceBuffer       *buffer,
+                                    CoreSurfaceAccessorID    accessor,
+                                    CoreSurfaceAccessFlags   flags,
+                                    bool                     lock );
+
 static inline int
 dfb_surface_buffer_index( CoreSurfaceBuffer *buffer )
 {
-     int          index;
-     CoreSurface *surface;
-
      D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );
 
-     surface = buffer->surface;
-     D_MAGIC_ASSERT( surface, CoreSurface );
-
-     for (index=0; index<MAX_SURFACE_BUFFERS; index++) {
-          if (surface->buffers[index] == buffer)
-               return index;
-     }
-
-     D_ASSERT( index<MAX_SURFACE_BUFFERS );
-
-     return 0;
+     return buffer->index;
 }
 
-DFBResult dfb_surface_allocation_update( CoreSurfaceAllocation  *allocation,
-                                         CoreSurfaceAccessFlags  access );
+
+FUSION_OBJECT_METHODS( CoreSurfaceBuffer, dfb_surface_buffer )
+
+FusionObjectPool *dfb_surface_buffer_pool_create( const FusionWorld *world );
 
 #endif
 
