@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,10 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
+//#define DIRECT_ENABLE_DEBUG
+
 #include <config.h>
 
 #include <stdio.h>
@@ -55,6 +61,10 @@
 #include <core/windowstack.h>
 #include <core/wm.h>
 
+#include <core/CoreLayerRegion.h>
+#include <core/CoreSurface.h>
+#include <core/CoreWindow.h>
+
 #include <misc/conf.h>
 #include <misc/util.h>
 
@@ -71,7 +81,8 @@
 #include <core/windows_internal.h>
 
 
-D_DEBUG_DOMAIN( Core_Windows, "Core/Windows", "DirectFB Window Core" );
+D_DEBUG_DOMAIN( Core_Windows,        "Core/Windows",        "DirectFB Window Core" );
+D_DEBUG_DOMAIN( Core_Windows_Events, "Core/Windows/Events", "DirectFB Window Core Events" );
 
 
 typedef struct {
@@ -100,17 +111,62 @@ static const ReactionFunc dfb_window_globals[] = {
 static void
 window_destructor( FusionObject *object, bool zombie, void *ctx )
 {
-     CoreWindow *window = (CoreWindow*) object;
+     CoreWindow      *window = (CoreWindow*) object;
+     CoreWindowStack *stack  = window->stack;
 
      D_DEBUG_AT( Core_Windows, "destroying %p (%d,%d - %dx%d%s)\n", window,
                  DFB_RECTANGLE_VALS( &window->config.bounds ), zombie ? " ZOMBIE" : "");
 
+     D_ASSUME( window->stack != NULL );
+
+     if (!stack) {
+          fusion_object_destroy( object );
+          return;
+     }
+
+     CoreWindow_Deinit_Dispatch( &window->call );
+
+     dfb_windowstack_lock( stack );
+
      dfb_window_destroy( window );
 
+
+     if (window->cursor.surface)
+          dfb_surface_unlink( &window->cursor.surface );
+
+     if (window->caps & DWCAPS_SUBWINDOW) {
+          int         index;
+          CoreWindow *toplevel;
+
+          toplevel = window->toplevel;
+          D_ASSERT( toplevel != NULL );
+
+          index = fusion_vector_index_of( &toplevel->subwindows, window );
+          D_ASSERT( index >= 0 );
+
+          fusion_vector_remove( &toplevel->subwindows, index );
+
+          window->toplevel = NULL;
+     }
+     else {
+          CoreWindow *sub;
+          int         i;
+
+          fusion_vector_foreach (sub, i, window->subwindows) {
+               sub->toplevel = NULL;
+          }
+
+          fusion_vector_destroy( &window->subwindows );
+     }
+
+     dfb_windowstack_unlock( stack );
+
      /* Unlink the primary region of the context. */
      if (window->primary_region)
           dfb_layer_region_unlink( &window->primary_region );
 
+     D_MAGIC_CLEAR( window );
+
      fusion_object_destroy( object );
 }
 
@@ -123,26 +179,24 @@ dfb_window_pool_create( const FusionWorld *world )
                                        window_destructor, NULL, world );
 }
 
-/**************************************************************************************************/
-
-static DFBResult
-create_region( CoreDFB                 *core,
-               CoreLayerContext        *context,
-               CoreWindow              *window,
-               DFBSurfacePixelFormat    format,
-               DFBSurfaceCapabilities   surface_caps,
-               CoreLayerRegion        **ret_region,
-               CoreSurface            **ret_surface )
+DFBResult
+dfb_window_create_region( CoreWindow              *window,
+                          CoreLayerContext        *context,
+                          CoreSurface             *window_surface,
+                          DFBSurfacePixelFormat    format,
+                          DFBSurfaceColorSpace     colorspace,
+                          DFBSurfaceCapabilities   surface_caps,
+                          CoreLayerRegion        **ret_region,
+                          CoreSurface            **ret_surface )
 {
      DFBResult              ret;
      CoreLayerRegionConfig  config;
      CoreLayerRegion       *region;
-     CoreSurface           *surface;
+     CoreSurface           *surface = window_surface;
      CoreSurfaceConfig      scon;
 
-     D_ASSERT( core != NULL );
-     D_ASSERT( context != NULL );
      D_ASSERT( window != NULL );
+     D_ASSERT( context != NULL );
      D_ASSERT( ret_region != NULL );
      D_ASSERT( ret_surface != NULL );
 
@@ -151,6 +205,7 @@ create_region( CoreDFB                 *core,
      config.width         = window->config.bounds.w;
      config.height        = window->config.bounds.h;
      config.format        = format;
+     config.colorspace    = colorspace;
      config.options       = context->config.options & DLOP_FLICKER_FILTERING;
      config.source        = (DFBRectangle) { 0, 0, config.width, config.height };
      config.dest          = window->config.bounds;
@@ -181,6 +236,7 @@ create_region( CoreDFB                 *core,
      if (ret)
           return ret;
 
+     region->config.keep_buffers = true;
 
      do {
           ret = dfb_layer_region_set_configuration( region, &config, CLRCF_ALL );
@@ -197,30 +253,35 @@ create_region( CoreDFB                 *core,
           }
      } while (ret);
 
-     scon.flags  = CSCONF_SIZE | CSCONF_FORMAT | CSCONF_CAPS;
-     scon.size.w = config.width;
-     scon.size.h = config.height;
-     scon.format = format;
-     scon.caps   = surface_caps | DSCAPS_VIDEOONLY;
+     if (!surface) {
+          scon.flags          = CSCONF_SIZE | CSCONF_FORMAT | CSCONF_COLORSPACE | CSCONF_CAPS;
+          scon.size.w         = config.width;
+          scon.size.h         = config.height;
+          scon.format         = format;
+          scon.colorspace     = colorspace;
+          scon.caps           = surface_caps | DSCAPS_VIDEOONLY;
 
-     ret = dfb_surface_create( core, &scon, CSTF_SHARED | CSTF_LAYER, context->layer_id, NULL, &surface );
-     if (ret) {
-          dfb_layer_region_unref( region );
-          return ret;
+          ret = dfb_surface_create( core_dfb, &scon, CSTF_SHARED | CSTF_LAYER, context->layer_id, NULL, &surface );
+          if (ret) {
+               dfb_layer_region_unref( region );
+               return ret;
+          }
      }
 
-     ret = dfb_layer_region_set_surface( region, surface );
+     ret = dfb_layer_region_set_surface( region, surface, false );
      if (ret) {
           dfb_surface_unref( surface );
           dfb_layer_region_unref( region );
           return ret;
      }
 
-     ret = dfb_layer_region_enable( region );
-     if (ret) {
-          dfb_surface_unref( surface );
-          dfb_layer_region_unref( region );
-          return ret;
+     if (!window_surface) {
+          ret = dfb_layer_region_enable( region );
+          if (ret) {
+               dfb_surface_unref( surface );
+               dfb_layer_region_unref( region );
+               return ret;
+          }
      }
 
      *ret_region  = region;
@@ -229,6 +290,48 @@ create_region( CoreDFB                 *core,
      return DFB_OK;
 }
 
+/**************************************************************************************************/
+
+static DFBResult
+init_subwindow( CoreWindow      *window,
+                CoreWindowStack *stack,
+                DFBWindowID      toplevel_id )
+{
+     DFBResult   ret;
+     CoreWindow *toplevel;
+
+     /* Lookup top level window */
+     ret = dfb_wm_window_lookup( stack, toplevel_id, &toplevel );
+     if (ret)
+          return ret;
+
+     /* Make sure chosen top level window is not a sub window */
+     if (toplevel->caps & DWCAPS_SUBWINDOW) {
+          D_ASSERT( toplevel->toplevel != NULL );
+          D_ASSERT( toplevel->toplevel_id != 0 );
+
+          return DFB_INVARG;
+     }
+     else {
+          D_ASSERT( toplevel->toplevel == NULL );
+          D_ASSERT( toplevel->toplevel_id == 0 );
+     }
+
+     /* Link top level window into sub window structure */
+     window->toplevel = toplevel;
+     if (ret)
+          return ret;
+
+     /* Add window to sub window list of top level window */
+     ret = fusion_vector_add( &toplevel->subwindows, window );
+     if (ret) {
+          dfb_window_unlink( &window->toplevel );
+          return ret;
+     }
+
+     return DFB_OK;
+}
+
 DFBResult
 dfb_window_create( CoreWindowStack             *stack,
                    const DFBWindowDescription  *desc,
@@ -245,6 +348,8 @@ dfb_window_create( CoreWindowStack             *stack,
      DFBWindowCapabilities   caps;
      DFBSurfaceCapabilities  surface_caps;
      DFBSurfacePixelFormat   pixelformat;
+     DFBSurfaceColorSpace    colorspace;
+     DFBWindowID             toplevel_id;
 
      D_DEBUG_AT( Core_Windows, "%s( %p )\n", __FUNCTION__, stack );
 
@@ -255,23 +360,39 @@ dfb_window_create( CoreWindowStack             *stack,
      D_ASSERT( desc->height > 0 );
      D_ASSERT( ret_window != NULL );
 
-     if (desc->width > 4096 || desc->height > 4096)
+     if (desc->width > 4096 || desc->height > 4096) {
+          D_DEBUG_AT( Core_Windows, "  -> LIMIT EXCEEDED (%dx%d / 4096x4096)\n", desc->width, desc->height );
           return DFB_LIMITEXCEEDED;
+     }
 
      /* Lock the window stack. */
-     if (dfb_windowstack_lock( stack ))
+     if (dfb_windowstack_lock( stack )) {
+          D_DEBUG_AT( Core_Windows, "  -> WINDOWSTACK LOCK FAILED\n" );
           return DFB_FUSION;
+     }
 
      context = stack->context;
      layer   = dfb_layer_at( context->layer_id );
 
+     D_DEBUG_AT( Core_Windows, "  -> caps 0x%08x\n", desc->caps );
 
      caps         = desc->caps;
      pixelformat  = desc->pixelformat;
+     colorspace   = desc->colorspace;
      surface_caps = desc->surface_caps & (DSCAPS_INTERLACED    | DSCAPS_SEPARATED  |
                                           DSCAPS_PREMULTIPLIED | DSCAPS_DEPTH      |
                                           DSCAPS_STATIC_ALLOC  | DSCAPS_SYSTEMONLY |
-                                          DSCAPS_VIDEOONLY);
+                                          DSCAPS_VIDEOONLY     | DSCAPS_TRIPLE     |
+                                          DSCAPS_GL);
+     toplevel_id  = (desc->flags & DWDESC_TOPLEVEL_ID) ? desc->toplevel_id : 0;
+
+     if (toplevel_id != 0)
+          caps |= DWCAPS_SUBWINDOW;
+     else
+          caps &= ~DWCAPS_SUBWINDOW;
+
+     if (caps & DWCAPS_STEREO)
+          surface_caps |= DSCAPS_STEREO;
 
      if (!dfb_config->translucent_windows) {
           caps &= ~DWCAPS_ALPHACHANNEL;
@@ -290,6 +411,7 @@ dfb_window_create( CoreWindowStack             *stack,
                     pixelformat = DSPF_ARGB;
           }
           else if (! DFB_PIXELFORMAT_HAS_ALPHA(pixelformat)) {
+               D_DEBUG_AT( Core_Windows, "  -> PIXELFORMAT '%s' HAS NO ALPHA\n", dfb_pixelformat_name(pixelformat) );
                dfb_windowstack_unlock( stack );
                return DFB_INVARG;
           }
@@ -304,6 +426,17 @@ dfb_window_create( CoreWindowStack             *stack,
           }
      }
 
+     /* Set the color space. */
+     if (colorspace == DSCS_UNKNOWN) {
+          colorspace = DFB_COLORSPACE_DEFAULT(pixelformat);
+     }
+     else if (! DFB_COLORSPACE_IS_COMPATIBLE(colorspace, pixelformat)) {
+          D_DEBUG_AT( Core_Windows, "  -> COLORSPACE '%s' IS NOT COMPATIBLE with FORMAT '%s'\n",
+                      dfb_colorspace_name(colorspace), dfb_pixelformat_name(pixelformat) );
+          dfb_windowstack_unlock( stack );
+          return DFB_INVARG;
+     }
+
      /* Choose window surface policy */
      if ((surface_caps & DSCAPS_VIDEOONLY) ||
          (context->config.buffermode == DLBM_WINDOWS))
@@ -331,8 +464,10 @@ dfb_window_create( CoreWindowStack             *stack,
 
      dfb_surface_caps_apply_policy( surface_policy, &surface_caps );
 
-     if (caps & DWCAPS_DOUBLEBUFFER)
-          surface_caps |= DSCAPS_DOUBLE;
+     if (caps & DWCAPS_DOUBLEBUFFER) {
+          if (!(surface_caps & DSCAPS_TRIPLE))
+               surface_caps |= DSCAPS_DOUBLE;
+     }
 
 
      memset( &config, 0, sizeof(CoreWindowConfig) );
@@ -357,18 +492,32 @@ dfb_window_create( CoreWindowStack             *stack,
 
      /* Create the window object. */
      window = dfb_core_create_window( layer->core );
+     if (!window) {
+          D_DEBUG_AT( Core_Windows, "  -> CORE CREATE WINDOW FAILED\n" );
+          dfb_windowstack_unlock( stack );
+          return DFB_FUSION;
+     }
 
-     window->id                 = ++stack->id_pool;
-     window->caps               = caps;
-     window->stack              = stack;
-     window->config             = config;
-     window->config.association = (desc->flags & DWDESC_PARENT) ? desc->parent_id : 0;
+     window->id                  = ++stack->id_pool;
+     window->caps                = caps | DWCAPS_NOFOCUS;
+     window->requested_caps      = caps;
+     window->stack               = stack;
+     window->config              = config;
+     window->config.association  = (desc->flags & DWDESC_PARENT) ? desc->parent_id : 0;
+     window->config.cursor_flags = dfb_config->default_cursor_flags;
+
+     /* Set toplevel window ID (new sub window feature) */
+     window->toplevel_id = toplevel_id;
 
      if (desc->flags & DWDESC_RESOURCE_ID)
           window->resource_id = desc->resource_id;
 
+     D_MAGIC_SET( window, CoreWindow );
+
      ret = dfb_wm_preconfigure_window( stack, window );
      if(ret) {
+          D_DEBUG_AT( Core_Windows, "  -> WINDOW PRECONFIGURE FAILED (%s)\n", DirectResultString(ret) );
+          D_MAGIC_CLEAR( window );
           fusion_object_destroy( &window->object );
           dfb_windowstack_unlock( stack );
           return ret;
@@ -378,6 +527,25 @@ dfb_window_create( CoreWindowStack             *stack,
      config = window->config;
      caps   = window->caps;
 
+     /* Initialize sub window... */
+     if (caps & DWCAPS_SUBWINDOW) {
+          ret = init_subwindow( window, stack, toplevel_id );
+          if (ret) {
+               D_DEBUG_AT( Core_Windows, "  -> SUBWINDOW INIT FAILED (%s)\n", DirectResultString(ret) );
+               D_MAGIC_CLEAR( window );
+               fusion_object_destroy( &window->object );
+               dfb_windowstack_unlock( stack );
+               return ret;
+          }
+     }
+     else {
+          /* ...or initialize top level window */
+          fusion_vector_init( &window->subwindows, 3, stack->shmpool );
+
+          /* In case WM forbids sub window request, clear the toplevel window ID */
+          window->toplevel_id = 0;
+     }
+
      if (dfb_config->warn.flags & DCWF_CREATE_WINDOW)
           D_WARN( "create-window   %4dx%4d %6s, caps 0x%08x, surface-caps 0x%08x, ID %u",
                   window->config.bounds.w, window->config.bounds.h, dfb_pixelformat_name(pixelformat),
@@ -389,9 +557,11 @@ dfb_window_create( CoreWindowStack             *stack,
                CoreLayerRegion *region = NULL;
 
                /* Create a region for the window. */
-               ret = create_region( layer->core, context, window,
-                                    pixelformat, surface_caps, &region, &surface );
+               ret = dfb_window_create_region( window, context, NULL,
+                                               pixelformat, colorspace, surface_caps, &region, &surface );
                if (ret) {
+                    D_DEBUG_AT( Core_Windows, "  -> REGION CREATE FAILED (%s)\n", DirectResultString(ret) );
+                    D_MAGIC_CLEAR( window );
                     fusion_object_destroy( &window->object );
                     dfb_windowstack_unlock( stack );
                     return ret;
@@ -411,6 +581,8 @@ dfb_window_create( CoreWindowStack             *stack,
                /* Get the primary region of the layer context. */
                ret = dfb_layer_context_get_primary_region( context, true, &region );
                if (ret) {
+                    D_DEBUG_AT( Core_Windows, "  -> LAYER CONTEXT PRIMARY REGION FAILED (%s)\n", DirectResultString(ret) );
+                    D_MAGIC_CLEAR( window );
                     fusion_object_destroy( &window->object );
                     dfb_windowstack_unlock( stack );
                     return ret;
@@ -433,13 +605,16 @@ dfb_window_create( CoreWindowStack             *stack,
                if (!window->surface) {
                     /* Create the surface for the window. */
                     ret = dfb_surface_create_simple( layer->core, config.bounds.w, config.bounds.h,
-                                                     pixelformat, surface_caps, CSTF_SHARED | CSTF_WINDOW,
+                                                     pixelformat, colorspace, surface_caps, 
+                                                     CSTF_SHARED | CSTF_WINDOW,
                                                      (desc->flags & DWDESC_RESOURCE_ID) ?
                                                      desc->resource_id : window->id,
                                                      region->surface ?
                                                      region->surface->palette : NULL, &surface );
                     if (ret) {
+                         D_DEBUG_AT( Core_Windows, "  -> SURFACE CREATE FAILED (%s)\n", DirectResultString(ret) );
                          D_DERROR( ret, "Core/Windows: Failed to create window surface!\n" );
+                         D_MAGIC_CLEAR( window );
                          dfb_layer_region_unlink( &window->primary_region );
                          fusion_object_destroy( &window->object );
                          dfb_windowstack_unlock( stack );
@@ -461,8 +636,12 @@ dfb_window_create( CoreWindowStack             *stack,
      /* Pass the new window to the window manager. */
      ret = dfb_wm_add_window( stack, window );
      if (ret) {
+          D_DEBUG_AT( Core_Windows, "  -> WINDOW ADD FAILED (%s)\n", DirectResultString(ret) );
+
           D_DERROR( ret, "Core/Windows: Failed to add window to manager!\n" );
 
+          D_MAGIC_CLEAR( window );
+
           if (window->surface)
                dfb_surface_unlink( &window->surface );
 
@@ -483,6 +662,8 @@ dfb_window_create( CoreWindowStack             *stack,
      /* Increase number of windows. */
      stack->num++;
 
+     CoreWindow_Init_Dispatch( layer->core, window, &window->call );
+
      /* Finally activate the object. */
      fusion_object_activate( &window->object );
 
@@ -500,11 +681,13 @@ dfb_window_create( CoreWindowStack             *stack,
 void
 dfb_window_destroy( CoreWindow *window )
 {
+     int              i;
      DFBWindowEvent   evt;
      CoreWindowStack *stack;
      BoundWindow     *bound, *next;
+     CoreWindow      *subwindow;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( DFB_WINDOW_INITIALIZED( window ) );
 
      D_DEBUG_AT( Core_Windows, "dfb_window_destroy (%p) [%4d,%4d - %4dx%4d]\n",
@@ -520,6 +703,14 @@ dfb_window_destroy( CoreWindow *window )
      if (dfb_windowstack_lock( stack ))
           return;
 
+     /* Destroy sub windows first. */
+     fusion_vector_foreach_reverse (subwindow, i, window->subwindows) {
+          D_ASSERT( subwindow != NULL );
+          D_ASSERT( DFB_WINDOW_INITIALIZED( subwindow ) );
+
+          dfb_window_destroy( subwindow );
+     }
+
      /* Avoid multiple destructions. */
      if (DFB_WINDOW_DESTROYED( window )) {
           D_DEBUG_AT( Core_Windows, "%p already destroyed.\n", window );
@@ -560,6 +751,7 @@ dfb_window_destroy( CoreWindow *window )
 
      /* Unlink the window's surface. */
      if (window->surface) {
+          dfb_surface_destroy_buffers( window->surface );
           dfb_surface_unlink( &window->surface );
      }
 
@@ -583,7 +775,7 @@ dfb_window_change_stacking( CoreWindow             *window,
      CoreWindowStack  *stack;
      CoreWindowConfig  config;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -615,7 +807,7 @@ dfb_window_raise( CoreWindow *window )
      DFBResult        ret;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -645,7 +837,7 @@ dfb_window_lower( CoreWindow *window )
      DFBResult        ret;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -675,7 +867,7 @@ dfb_window_raisetotop( CoreWindow *window )
      DFBResult        ret;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -705,7 +897,7 @@ dfb_window_lowertobottom( CoreWindow *window )
      DFBResult        ret;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -736,7 +928,7 @@ dfb_window_putatop( CoreWindow *window,
      DFBResult        ret;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -767,7 +959,7 @@ dfb_window_putbelow( CoreWindow *window,
      DFBResult        ret;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
      stack = window->stack;
@@ -799,6 +991,8 @@ dfb_window_set_config( CoreWindow             *window,
      DFBResult         ret;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -817,6 +1011,47 @@ dfb_window_set_config( CoreWindow             *window,
      return ret;
 }
 
+DFBResult
+dfb_window_set_cursor_shape( CoreWindow   *window,
+                             CoreSurface  *surface,
+                             unsigned int  hot_x,
+                             unsigned int  hot_y )
+{
+     DFBResult         ret   = DFB_OK;
+     CoreWindowStack  *stack = window->stack;
+
+     D_MAGIC_ASSERT( window, CoreWindow );
+
+     /* Lock the window stack. */
+     if (dfb_windowstack_lock( stack ))
+          return DFB_FUSION;
+
+     /* Never call WM after destroying the window. */
+     if (DFB_WINDOW_DESTROYED( window )) {
+          dfb_windowstack_unlock( stack );
+          return DFB_DESTROYED;
+     }
+
+     window->cursor.hot_x = hot_x;
+     window->cursor.hot_y = hot_y;
+
+     if (window->cursor.surface)
+          dfb_surface_unlink( &window->cursor.surface );
+
+     if (surface) {
+          ret = dfb_surface_link( &window->cursor.surface, surface );
+          if (ret == DFB_OK) {
+               if (window->flags & CWF_FOCUSED)
+                    dfb_windowstack_cursor_set_shape( stack, surface, hot_x, hot_y );
+          }
+     }
+
+     /* Unlock the window stack. */
+     dfb_windowstack_unlock( stack );
+
+     return ret;
+}
+
 static DFBResult
 move_window( CoreWindow *window,
              int         x,
@@ -826,6 +1061,8 @@ move_window( CoreWindow *window,
      CoreWindowConfig  config;
      BoundWindow      *bound;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      config.bounds.x = x;
      config.bounds.y = y;
 
@@ -851,6 +1088,8 @@ dfb_window_move( CoreWindow *window,
      DFBResult        ret;    
      CoreWindowStack *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -897,6 +1136,8 @@ dfb_window_set_bounds( CoreWindow *window,
      int               old_x;
      int               old_y;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -962,6 +1203,8 @@ dfb_window_resize( CoreWindow   *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      D_DEBUG_AT( Core_Windows, "dfb_window_resize (%p) [%4d,%4d - %4dx%4d -> %dx%d]\n",
                  window, DFB_RECTANGLE_VALS( &window->config.bounds ), width, height );
 
@@ -1007,6 +1250,8 @@ dfb_window_bind( CoreWindow *window,
      CoreWindowStack *stack = window->stack;
      BoundWindow     *bound;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      if (window == source)
           return DFB_UNSUPPORTED;
      
@@ -1063,6 +1308,8 @@ dfb_window_unbind( CoreWindow *window,
      CoreWindowStack *stack = window->stack;
      BoundWindow     *bound;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1115,6 +1362,8 @@ dfb_window_set_color( CoreWindow *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1149,6 +1398,8 @@ dfb_window_set_colorkey( CoreWindow *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1182,6 +1433,8 @@ dfb_window_set_opacity( CoreWindow *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1216,6 +1469,8 @@ dfb_window_change_options( CoreWindow       *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      D_ASSUME( disable | enable );
 
      if (!disable && !enable)
@@ -1249,6 +1504,8 @@ dfb_window_set_opaque( CoreWindow      *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      DFB_REGION_ASSERT_IF( region );
 
      /* Lock the window stack. */
@@ -1286,6 +1543,8 @@ dfb_window_change_events( CoreWindow         *window,
      CoreWindowConfig  config;
      CoreWindowStack  *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      D_ASSUME( disable | enable );
 
      if (!disable && !enable)
@@ -1321,6 +1580,8 @@ dfb_window_set_key_selection( CoreWindow                    *window,
     CoreWindowConfig  config;
     CoreWindowStack  *stack = window->stack;
 
+    D_MAGIC_ASSERT( window, CoreWindow );
+
     D_ASSERT( selection == DWKS_ALL || selection == DWKS_NONE || selection == DWKS_LIST );
     D_ASSERT( keys != NULL || selection != DWKS_LIST );
     D_ASSERT( num_keys > 0 || selection != DWKS_LIST );
@@ -1356,7 +1617,7 @@ dfb_window_change_grab( CoreWindow       *window,
      CoreWMGrab       wmgrab;
      CoreWindowStack *stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
 
      stack = window->stack;
 
@@ -1392,6 +1653,8 @@ dfb_window_grab_key( CoreWindow                 *window,
      CoreWMGrab       grab;
      CoreWindowStack *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1423,6 +1686,8 @@ dfb_window_ungrab_key( CoreWindow                 *window,
      CoreWMGrab       grab;
      CoreWindowStack *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1447,28 +1712,50 @@ dfb_window_ungrab_key( CoreWindow                 *window,
 
 DFBResult
 dfb_window_repaint( CoreWindow          *window,
-                    const DFBRegion     *region,
-                    DFBSurfaceFlipFlags  flags )
+                    const DFBRegion     *left_region,
+                    const DFBRegion     *right_region,
+                    DFBSurfaceFlipFlags  flags,
+                    long long            timestamp )
 {
      DFBResult        ret;
      CoreWindowStack *stack = window->stack;
 
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( window->stack != NULL );
 
-     DFB_REGION_ASSERT_IF( region );
+     DFB_REGION_ASSERT_IF( left_region );
+     DFB_REGION_ASSERT_IF( right_region );
+
+     D_DEBUG_AT( Core_Windows, "%s( %p, left %d,%d-%dx%d, right %d,%d-%dx%d, flags 0x%08x, timestamp %lld )\n", __FUNCTION__, window,
+                 DFB_RECTANGLE_VALS_FROM_REGION( left_region ),
+                 DFB_RECTANGLE_VALS_FROM_REGION( right_region ), flags, timestamp );
 
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
 
+     if (window->region && window->region->state & CLRSF_ENABLED) {
+          D_DEBUG_AT( Core_Windows, "  -> updating single window region\n" );
+
+          ret = CoreLayerRegion_FlipUpdate2( window->region, left_region, right_region, flags, timestamp );
+     }
+     else {
+          D_DEBUG_AT( Core_Windows, "  -> updating composed window region\n" );
+
+          ret = CoreSurface_Flip2( window->surface, DFB_FALSE, left_region, right_region, flags, timestamp );
+     }
+
      /* Never call WM after destroying the window. */
      if (DFB_WINDOW_DESTROYED( window )) {
           dfb_windowstack_unlock( stack );
           return DFB_DESTROYED;
      }
 
-     ret = dfb_wm_update_window( window, region, flags );
+     if (!dfb_config->single_window || fusion_vector_size( &window->stack->visible_windows ) != 1) {
+          D_DEBUG_AT( Core_Windows, "  -> dispatching update to window manager\n" );
+
+          ret = dfb_wm_update_window( window, left_region, right_region, flags );
+     }
 
      /* Unlock the window stack. */
      dfb_windowstack_unlock( stack );
@@ -1536,11 +1823,27 @@ void
 dfb_window_post_event( CoreWindow     *window,
                        DFBWindowEvent *event )
 {
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
      D_ASSERT( event != NULL );
 
      D_ASSUME( !DFB_WINDOW_DESTROYED( window ) || event->type == DWET_DESTROYED );
 
+     switch (event->type) {
+          case DWET_BUTTONDOWN:
+          case DWET_BUTTONUP:
+               D_DEBUG_AT( Core_Windows_Events, "%s( window id %u )\n", __FUNCTION__, window->object.id );
+               D_DEBUG_AT( Core_Windows_Events, "  -> TYPE    0x%08x\n", event->type );
+
+               D_DEBUG_AT( Core_Windows_Events, "  => BUTTON%s\n", event->type == DWET_BUTTONDOWN ? "DOWN" : "UP" );
+               D_DEBUG_AT( Core_Windows_Events, "     -> button  %d\n", event->button );
+               D_DEBUG_AT( Core_Windows_Events, "     -> x, y    %d,%d\n", event->x, event->y );
+               D_DEBUG_AT( Core_Windows_Events, "     -> cx, cy  %d,%d\n", event->cx, event->cy );
+               break;
+
+          default:
+               break;
+     }
+
      if (! (event->type & window->config.events))
           return;
 
@@ -1549,15 +1852,6 @@ dfb_window_post_event( CoreWindow     *window,
      event->clazz     = DFEC_WINDOW;
      event->window_id = window->id;
 
-     D_ASSUME( window->stack != NULL );
-
-     if (window->stack) {
-          CoreWindowStack *stack = window->stack;
-
-          event->cx = stack->cursor.x;
-          event->cy = stack->cursor.y;
-     }
-
      if (!core_window_filter( window, event ))
           dfb_window_dispatch( window, event, dfb_window_globals );
 }
@@ -1567,6 +1861,8 @@ dfb_window_send_configuration( CoreWindow *window )
 {
      DFBWindowEvent event;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      D_ASSUME( !DFB_WINDOW_DESTROYED( window ) );
 
      event.type = DWET_POSITION_SIZE;
@@ -1586,6 +1882,8 @@ dfb_window_request_focus( CoreWindow *window )
      DFBResult        ret;
      CoreWindowStack *stack = window->stack;
 
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      /* Lock the window stack. */
      if (dfb_windowstack_lock( stack ))
           return DFB_FUSION;
@@ -1607,7 +1905,7 @@ dfb_window_request_focus( CoreWindow *window )
 DFBWindowID
 dfb_window_id( const CoreWindow *window )
 {
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
 
      return window->id;
 }
@@ -1619,7 +1917,7 @@ dfb_window_id( const CoreWindow *window )
 CoreSurface *
 dfb_window_surface( const CoreWindow *window )
 {
-     D_ASSERT( window != NULL );
+     D_MAGIC_ASSERT( window, CoreWindow );
 
      return window->surface;
 }
@@ -1629,6 +1927,8 @@ dfb_window_surface( const CoreWindow *window )
 static bool
 core_window_filter( CoreWindow *window, const DFBWindowEvent *event )
 {
+     D_MAGIC_ASSERT( window, CoreWindow );
+
      switch (event->type) {
           case DWET_GOTFOCUS:
                D_FLAGS_SET( window->flags, CWF_FOCUSED );
@@ -1653,3 +1953,50 @@ core_window_filter( CoreWindow *window, const DFBWindowEvent *event )
      return false;
 }
 
+DFBResult
+dfb_window_set_rotation( CoreWindow *window,
+                         int         rotation )
+{
+     DFBResult        ret   = DFB_OK;
+     CoreWindowStack *stack = window->stack;
+
+     D_MAGIC_ASSERT( window, CoreWindow );
+
+     stack = window->stack;
+     D_MAGIC_ASSERT( stack, CoreWindowStack );
+     
+     /* Lock the window stack. */
+     if (dfb_windowstack_lock( stack ))
+          return DFB_FUSION;
+
+     /* Never call WM after destroying the window. */
+     if (DFB_WINDOW_DESTROYED( window )) {
+          dfb_windowstack_unlock( stack );
+          return DFB_DESTROYED;
+     }
+     
+     /* Do nothing if the rotation didn't change. */
+     if (window->config.rotation != rotation) {
+          CoreWindowConfig config;
+
+          switch (rotation) {
+               case 0:
+               case 90:
+               case 180:
+               case 270:
+                    config.rotation = rotation;
+
+                    dfb_wm_set_window_config( window, &config, CWCF_ROTATION );
+                    break;
+
+               default:
+                    ret = DFB_UNSUPPORTED;
+          }
+     }
+
+     /* Unlock the window stack. */
+     dfb_windowstack_unlock( stack );
+
+     return ret;
+}
+
