@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __CORE__WINDOWSTACK_H__
 #define __CORE__WINDOWSTACK_H__
 
@@ -52,6 +56,10 @@ DirectResult     dfb_windowstack_lock   ( CoreWindowStack  *stack );
 
 DirectResult     dfb_windowstack_unlock ( CoreWindowStack  *stack );
 
+
+
+
+
 /*
  * repaints all window on a window stack
  */
@@ -102,4 +110,5 @@ DFBResult dfb_windowstack_get_cursor_position (CoreWindowStack *stack,
                                                int             *x,
                                                int             *y);
 
+
 #endif
