@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2015  DirectFB integrated media GmbH
+   (c) Copyright 2001-2015  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,19 +28,32 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-
 #include <string.h>
 
+#include <direct/conf.h>
+#include <direct/direct.h>
+#include <direct/interface.h>
+#include <direct/log.h>
+#include <direct/mem.h>
+#include <direct/messages.h>
+#include <direct/util.h>
+
 #include <directfb.h>
 #include <directfb_version.h>
 
 #include <misc/conf.h>
 
+#if !DIRECTFB_BUILD_PURE_VOODOO
+#include <unistd.h>
+
+#include <direct/thread.h>
+
 #include <core/core.h>
 #include <core/coredefs.h>
 #include <core/coretypes.h>
@@ -52,27 +67,29 @@
 #include <core/surface.h>
 #include <core/windows.h>
 #include <core/windowstack.h>
-#include <core/wm.h>
 
 #include <gfx/convert.h>
 
-#include <direct/conf.h>
-#include <direct/direct.h>
-#include <direct/interface.h>
-#include <direct/log.h>
-#include <direct/mem.h>
-#include <direct/messages.h>
-#include <direct/util.h>
-
 #include <display/idirectfbsurface.h>
+#endif
 
 #include <idirectfb.h>
 
 
-IDirectFB *idirectfb_singleton = NULL;
+#ifndef DIRECTFB_VERSION_VENDOR
+#define DIRECTFB_VERSION_VENDOR
+#endif
+
+
+D_DEBUG_DOMAIN( DirectFB_Main, "DirectFB/Main", "DirectFB Main Functions" );
+
 
 static DFBResult CreateRemote( const char *host, int session, IDirectFB **ret_interface );
 
+
+IDirectFB *idirectfb_singleton = NULL;
+
+
 /*
  * Version checking
  */
@@ -114,6 +131,8 @@ DirectFBInit( int *argc, char *(*argv[]) )
 {
      DFBResult ret;
 
+     D_DEBUG_AT( DirectFB_Main, "%s( %p, %p )\n", __FUNCTION__, argc, argv );
+
      ret = dfb_config_init( argc, argv );
      if (ret)
           return ret;
@@ -126,18 +145,14 @@ DirectFBSetOption( const char *name, const char *value )
 {
      DFBResult ret;
 
+     D_DEBUG_AT( DirectFB_Main, "%s( '%s', '%s' )\n", __FUNCTION__, name, value );
+
      if (dfb_config == NULL) {
           D_ERROR( "DirectFBSetOption: DirectFBInit has to be "
                    "called before DirectFBSetOption!\n" );
           return DFB_INIT;
      }
 
-     if (idirectfb_singleton) {
-          D_ERROR( "DirectFBSetOption: DirectFBSetOption has to be "
-                   "called before DirectFBCreate!\n" );
-          return DFB_INIT;
-     }
-
      if (!name)
           return DFB_INVARG;
 
@@ -153,11 +168,14 @@ DirectFBSetOption( const char *name, const char *value )
  * which is needed to access other functions
  */
 DFBResult
-DirectFBCreate( IDirectFB **interface )
+DirectFBCreate( IDirectFB **interface_ptr )
 {
+#if !DIRECTFB_BUILD_PURE_VOODOO
      DFBResult  ret;
      IDirectFB *dfb;
-     CoreDFB   *core_dfb;
+#endif
+
+     D_DEBUG_AT( DirectFB_Main, "%s( %p )\n", __FUNCTION__, interface_ptr );
 
      if (!dfb_config) {
           /*  don't use D_ERROR() here, it uses dfb_config  */
@@ -166,12 +184,17 @@ DirectFBCreate( IDirectFB **interface )
           return DFB_INIT;
      }
 
-     if (!interface)
+     if (!interface_ptr)
           return DFB_INVARG;
 
-     if (idirectfb_singleton) {
+
+     if (!dfb_config->no_singleton && idirectfb_singleton) {
+          D_DEBUG_AT( DirectFB_Main, "  -> using singleton %p\n", idirectfb_singleton );
+
           idirectfb_singleton->AddRef( idirectfb_singleton );
-          *interface = idirectfb_singleton;
+
+          *interface_ptr = idirectfb_singleton;
+
           return DFB_OK;
      }
 
@@ -180,40 +203,70 @@ DirectFBCreate( IDirectFB **interface )
      if ( !(direct_config->quiet & DMT_BANNER) && dfb_config->banner) {
           direct_log_printf( NULL,
                              "\n"
-                             "   ~~~~~~~~~~~~~~~~~~~~~~~~~~| DirectFB " DIRECTFB_VERSION " |~~~~~~~~~~~~~~~~~~~~~~~~~~\n"
-                             "        (c) 2001-2008  The world wide DirectFB Open Source Community\n"
+                             "   ~~~~~~~~~~~~~~~~~~~~~~~~~| DirectFB " DIRECTFB_VERSION DIRECTFB_VERSION_VENDOR " |~~~~~~~~~~~~~~~~~~~~~~~~~~\n"
+                             "        (c) 2012-2015  DirectFB integrated media GmbH\n"
+                             "        (c) 2001-2015  The world wide DirectFB Open Source Community\n"
                              "        (c) 2000-2004  Convergence (integrated media) GmbH\n"
                              "      ----------------------------------------------------------------\n"
                              "\n" );
      }
 
+#if !DIRECTFB_BUILD_PURE_VOODOO
      if (dfb_config->remote.host)
-          return CreateRemote( dfb_config->remote.host, dfb_config->remote.session, interface );
+          return CreateRemote( dfb_config->remote.host, dfb_config->remote.port, interface_ptr );
 
-     ret = dfb_core_create( &core_dfb );
-     if (ret)
-          return ret;
+     static DirectMutex lock = DIRECT_RECURSIVE_MUTEX_INITIALIZER(lock);
+
+     direct_mutex_lock( &lock );
+
+     if (!dfb_config->no_singleton && idirectfb_singleton) {
+          D_DEBUG_AT( DirectFB_Main, "  -> using (new) singleton %p\n", idirectfb_singleton );
+
+          idirectfb_singleton->AddRef( idirectfb_singleton );
+
+          *interface_ptr = idirectfb_singleton;
+
+          direct_mutex_unlock( &lock );
+          return DFB_OK;
+     }
 
      DIRECT_ALLOCATE_INTERFACE( dfb, IDirectFB );
 
-     ret = IDirectFB_Construct( dfb, core_dfb );
+     if (!dfb_config->no_singleton) {
+          D_DEBUG_AT( DirectFB_Main, "  -> setting singleton to %p (was %p)\n", dfb, idirectfb_singleton );
+
+          idirectfb_singleton = dfb;
+     }
+
+     ret = IDirectFB_Construct( dfb );
      if (ret) {
-          dfb_core_destroy( core_dfb, false );
+          if (!dfb_config->no_singleton) {
+               D_DEBUG_AT( DirectFB_Main, "  -> ERROR, resetting singleton to NULL\n" );
+               idirectfb_singleton = NULL;
+          }
+
+          direct_mutex_unlock( &lock );
           return ret;
      }
 
-     if (dfb_core_is_master( core_dfb )) {
-          /* not fatal */
-          ret = dfb_wm_post_init( core_dfb );
-          if (ret)
-               D_DERROR( ret, "DirectFBCreate: Post initialization of WM failed!\n" );
+     direct_mutex_unlock( &lock );
 
-          dfb_core_activate( core_dfb );
+     ret = IDirectFB_WaitInitialised( dfb );
+     if (ret) {
+          if (!dfb_config->no_singleton)
+               idirectfb_singleton = NULL;
+          dfb->Release( dfb );
+          return ret;
      }
 
-     *interface = idirectfb_singleton = dfb;
+     D_DEBUG_AT( DirectFB_Main, "  -> Done\n" );
+
+     *interface_ptr = dfb;
 
      return DFB_OK;
+#else
+     return CreateRemote( dfb_config->remote.host ? dfb_config->remote.host : "", dfb_config->remote.port, interface_ptr );
+#endif
 }
 
 DFBResult
@@ -232,19 +285,6 @@ DirectFBError( const char *msg, DFBResult error )
 const char *
 DirectFBErrorString( DFBResult error )
 {
-     if (D_RESULT_TYPE_IS( error, 'D','F','B' )) {
-          switch (error) {
-               case DFB_NOVIDEOMEMORY:
-                    return "Out of video memory!";
-               case DFB_MISSINGFONT:
-                    return "No font has been set!";
-               case DFB_MISSINGIMAGE:
-                    return "No image has been set!";
-               default:
-                    return "UKNOWN DIRECTFB RESULT!";
-          }
-     }
-
      return DirectResultString( error );
 }
 
@@ -262,11 +302,11 @@ DirectFBErrorFatal( const char *msg, DFBResult error )
 /**************************************************************************************************/
 
 static DFBResult
-CreateRemote( const char *host, int session, IDirectFB **ret_interface )
+CreateRemote( const char *host, int port, IDirectFB **ret_interface )
 {
      DFBResult             ret;
      DirectInterfaceFuncs *funcs;
-     void                 *interface;
+     void                 *interface_ptr;
 
      D_ASSERT( host != NULL );
      D_ASSERT( ret_interface != NULL );
@@ -275,15 +315,15 @@ CreateRemote( const char *host, int session, IDirectFB **ret_interface )
      if (ret)
           return ret;
 
-     ret = funcs->Allocate( &interface );
+     ret = funcs->Allocate( &interface_ptr );
      if (ret)
           return ret;
 
-     ret = funcs->Construct( interface, host, session );
+     ret = funcs->Construct( interface_ptr, host, port );
      if (ret)
           return ret;
 
-     *ret_interface = idirectfb_singleton = interface;
+     *ret_interface = interface_ptr;
 
      return DFB_OK;
 }
