@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -36,6 +40,8 @@
 
 #include <directfb.h>
 
+#include <core/CoreScreen.h>
+
 #include <core/layers.h>
 #include <core/screen.h>
 #include <core/screens.h>
@@ -143,22 +149,21 @@ IDirectFBScreen_GetSize( IDirectFBScreen *thiz,
                          int             *ret_width,
                          int             *ret_height )
 {
-     DFBResult ret;
-     int       width  = 0;
-     int       height = 0;
+     DFBResult    ret;
+     DFBDimension size;
 
      DIRECT_INTERFACE_GET_DATA(IDirectFBScreen)
 
      if (!ret_width && !ret_height)
           return DFB_INVARG;
 
-     ret = dfb_screen_get_screen_size( data->screen, &width, &height );
+     ret = CoreScreen_GetScreenSize( data->screen, &size );
 
      if (ret_width)
-          *ret_width = width;
+          *ret_width = size.w;
 
      if (ret_height)
-          *ret_height = height;
+          *ret_height = size.h;
 
      return ret;
 }
@@ -201,7 +206,7 @@ IDirectFBScreen_SetPowerMode( IDirectFBScreen    *thiz,
                return DFB_INVARG;
      }
 
-     return dfb_screen_set_powermode( data->screen, mode );
+     return CoreScreen_SetPowerMode( data->screen, mode );
 }
 
 static DFBResult
@@ -209,7 +214,27 @@ IDirectFBScreen_WaitForSync( IDirectFBScreen *thiz )
 {
      DIRECT_INTERFACE_GET_DATA(IDirectFBScreen)
 
-     return dfb_screen_wait_vsync( data->screen );
+     return CoreScreen_WaitVSync( data->screen );
+}
+
+static DFBResult
+IDirectFBScreen_GetVSyncCount( IDirectFBScreen *thiz,
+                               unsigned long   *ret_count )
+{
+     DFBResult ret;
+     u64       count;
+
+     DIRECT_INTERFACE_GET_DATA(IDirectFBScreen)
+
+     D_ASSERT( ret_count != NULL );
+
+     ret = CoreScreen_GetVSyncCount( data->screen, &count );
+     if (ret)
+          return ret;
+
+     *ret_count = count;
+
+     return DFB_OK;
 }
 
 static DFBResult
@@ -282,8 +307,8 @@ IDirectFBScreen_TestMixerConfiguration( IDirectFBScreen            *thiz,
           return ret;
 
      /* Test the patched configuration. */
-     return dfb_screen_test_mixer_config( data->screen,
-                                          mixer, &patched, failed );
+     return CoreScreen_TestMixerConfig( data->screen,
+                                        mixer, &patched, failed );
 }
 
 static DFBResult
@@ -316,7 +341,7 @@ IDirectFBScreen_SetMixerConfiguration( IDirectFBScreen            *thiz,
           return ret;
 
      /* Set the patched configuration. */
-     return dfb_screen_set_mixer_config( data->screen, mixer, &patched );
+     return CoreScreen_SetMixerConfig( data->screen, mixer, &patched );
 }
 
 static DFBResult
@@ -389,8 +414,8 @@ IDirectFBScreen_TestEncoderConfiguration( IDirectFBScreen              *thiz,
           return ret;
 
      /* Test the patched configuration. */
-     return dfb_screen_test_encoder_config( data->screen,
-                                            encoder, &patched, failed );
+     return CoreScreen_TestEncoderConfig( data->screen,
+                                          encoder, &patched, failed );
 }
 
 static DFBResult
@@ -423,7 +448,7 @@ IDirectFBScreen_SetEncoderConfiguration( IDirectFBScreen              *thiz,
           return ret;
 
      /* Set the patched configuration. */
-     return dfb_screen_set_encoder_config( data->screen, encoder, &patched );
+     return CoreScreen_SetEncoderConfig( data->screen, encoder, &patched );
 }
 
 static DFBResult
@@ -437,7 +462,7 @@ IDirectFBScreen_GetOutputDescriptions( IDirectFBScreen            *thiz,
      if (!descriptions)
           return DFB_INVARG;
 
-     if (!data->description.caps & DSCCAPS_OUTPUTS)
+     if (! (data->description.caps & DSCCAPS_OUTPUTS))
           return DFB_UNSUPPORTED;
 
      for (i=0; i<data->description.outputs; i++)
@@ -496,8 +521,8 @@ IDirectFBScreen_TestOutputConfiguration( IDirectFBScreen             *thiz,
           return ret;
 
      /* Test the patched configuration. */
-     return dfb_screen_test_output_config( data->screen,
-                                           output, &patched, failed );
+     return CoreScreen_TestOutputConfig( data->screen,
+                                         output, &patched, failed );
 }
 
 static DFBResult
@@ -530,7 +555,7 @@ IDirectFBScreen_SetOutputConfiguration( IDirectFBScreen             *thiz,
           return ret;
 
      /* Set the patched configuration. */
-     return dfb_screen_set_output_config( data->screen, output, &patched );
+     return CoreScreen_SetOutputConfig( data->screen, output, &patched );
 }
 
 /******************************************************************************/
@@ -555,6 +580,7 @@ IDirectFBScreen_Construct( IDirectFBScreen *thiz,
      thiz->EnumDisplayLayers        = IDirectFBScreen_EnumDisplayLayers;
      thiz->SetPowerMode             = IDirectFBScreen_SetPowerMode;
      thiz->WaitForSync              = IDirectFBScreen_WaitForSync;
+     thiz->GetVSyncCount            = IDirectFBScreen_GetVSyncCount;
      thiz->GetMixerDescriptions     = IDirectFBScreen_GetMixerDescriptions;
      thiz->GetMixerConfiguration    = IDirectFBScreen_GetMixerConfiguration;
      thiz->TestMixerConfiguration   = IDirectFBScreen_TestMixerConfiguration;
@@ -631,7 +657,7 @@ PatchEncoderConfig( DFBScreenEncoderConfig       *patched,
                case DSETV_PAL_60:
                case DSETV_PAL_M:
                     patched->resolution = DSOR_720_480;
-                    patched->frequency = DSEF_60HZ;
+                    patched->frequency = DSEF_59_94HZ;
                     break;
 
                case DSETV_SECAM:
@@ -641,8 +667,9 @@ PatchEncoderConfig( DFBScreenEncoderConfig       *patched,
 
                case DSETV_NTSC:
                case DSETV_NTSC_M_JPN:
+               case DSETV_NTSC_443:
                     patched->resolution = DSOR_720_480;
-                    patched->frequency = DSEF_60HZ;
+                    patched->frequency = DSEF_59_94HZ;
                     break;
 
                default:
@@ -670,6 +697,12 @@ PatchEncoderConfig( DFBScreenEncoderConfig       *patched,
      if (patch->flags & DSECONF_SLOW_BLANKING)
           patched->slow_blanking = patch->slow_blanking;
 
+     if (patch->flags & DSECONF_FRAMING)
+          patched->framing = patch->framing;
+
+     if (patch->flags & DSECONF_ASPECT_RATIO)
+          patched->aspect_ratio = patch->aspect_ratio;
+
      return DFB_OK;
 }
 
