@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -27,6 +29,8 @@
 */
 
 
+
+
 #define __aligned( n )  __attribute__ ((aligned((n))))
 
 
@@ -90,6 +94,7 @@ static void Sacc_add_to_Dacc_MMX( GenefxState *gfxs )
                : "%st", "memory");
 }
 
+__attribute__((unused))
 static void Sacc_to_Aop_rgb16_MMX( GenefxState *gfxs )
 {
      static const u32 preload[] = { 0xFF00FF00, 0x0000FF00 };
@@ -127,6 +132,7 @@ static void Sacc_to_Aop_rgb16_MMX( GenefxState *gfxs )
                : "%eax", "%st", "memory");
 }
 
+__attribute__((unused))
 static void Sacc_to_Aop_rgb32_MMX( GenefxState *gfxs )
 {
      static const u32 preload[]  = { 0xFF00FF00, 0x0000FF00 };
@@ -162,6 +168,7 @@ static void Sacc_to_Aop_rgb32_MMX( GenefxState *gfxs )
                : "%st", "memory");
 }
 
+__attribute__((unused))
 __attribute__((no_instrument_function))
 static void Sop_argb_Sto_Dacc_MMX( GenefxState *gfxs )
 {
@@ -201,6 +208,7 @@ static void Sop_argb_Sto_Dacc_MMX( GenefxState *gfxs )
                : "%edx", "%st", "memory");
 }
 
+__attribute__((unused))
 static void Sop_argb_to_Dacc_MMX( GenefxState *gfxs )
 {
      static const u32 zeros[]  = { 0, 0 };
@@ -223,6 +231,7 @@ static void Sop_argb_to_Dacc_MMX( GenefxState *gfxs )
                : "%st", "memory");
 }
 
+__attribute__((unused))
 static void Sop_rgb16_to_Dacc_MMX( GenefxState *gfxs )
 {
      static const u32 mask[]  = { 0x07E0001F, 0x0000F800 };
@@ -301,6 +310,7 @@ static void Sop_rgb16_to_Dacc_MMX( GenefxState *gfxs )
                : "%st", "memory");
 }
 
+__attribute__((unused))
 static void Sop_rgb32_to_Dacc_MMX( GenefxState *gfxs )
 {
      static const u32 alpha[]  = { 0, 0x00FF0000 };
@@ -334,7 +344,7 @@ static void Xacc_blend_invsrcalpha_MMX( GenefxState *gfxs )
      __asm__ __volatile__ (
 	       "movq     %3, %%mm7\n\t"
                "cmp      $0, %2\n\t"
-               "jne      3f\n\t"
+               "jne      1f\n\t"
                "movq     %4, %%mm6\n\t"
                "movd     %5, %%mm0\n\t"
                "punpcklbw %%mm6, %%mm0\n\t" /* mm0 = 00aa 00rr 00gg 00bb */
@@ -344,23 +354,28 @@ static void Xacc_blend_invsrcalpha_MMX( GenefxState *gfxs )
                "psubw     %%mm0, %%mm1\n\t"
 
                ".align   16\n"
-               "4:\n\t"                 /* blend from color */
+               "2:\n\t"                 /* blend from color */
                "testw    $0xF000, 6(%0)\n\t"
-               "jnz      1f\n\t"
+               "jnz      3f\n\t"
                "movq     (%0), %%mm0\n\t"
                "pmullw   %%mm1, %%mm0\n\t"
                "psrlw    $8, %%mm0\n\t"
-               "movq     %%mm0, (%0)\n"
-               "1:\n\t"
+               "movq     %%mm0, (%6)\n\t"
+               "jmp      4f\n\t"
+               "3:\n\t"
+               "movq     (%0), %%mm0\n\t"
+               "movq     %%mm0, (%6)\n\t"
+               "4:\n\t"
 	       "add      $8, %0\n\t"
+	       "add      $8, %6\n\t"
                "dec      %1\n\t"
-               "jnz      4b\n\t"
-               "jmp      2f\n\t"
+               "jnz      2b\n\t"
+               "jmp      9f\n\t"
 
                ".align   16\n"
-               "3:\n\t"                      /* blend from Sacc */
+               "1:\n\t"                      /* blend from Sacc */
                "testw    $0xF000, 6(%0)\n\t"
-               "jnz      1f\n\t"
+               "jnz      5f\n\t"
                "movq     (%2), %%mm2\n\t"
                "movq     (%0), %%mm0\n\t"
 	       "punpckhwd %%mm2, %%mm2\n\t" /* mm2 = 00aa 00aa xxxx xxxx */
@@ -369,17 +384,22 @@ static void Xacc_blend_invsrcalpha_MMX( GenefxState *gfxs )
                "psubw    %%mm2, %%mm1\n\t"
                "pmullw   %%mm1, %%mm0\n\t"
                "psrlw    $8, %%mm0\n\t"
-               "movq     %%mm0, (%0)\n"
-               "1:\n\t"
+               "movq     %%mm0, (%6)\n\t"
+               "jmp      6f\n\t"
+               "5:\n\t"
+               "movq     (%0), %%mm0\n\t"
+               "movq     %%mm0, (%6)\n\t"
+               "6:\n\t"
 	       "add      $8, %2\n\t"
 	       "add      $8, %0\n\t"
+	       "add      $8, %6\n\t"
                "dec      %1\n\t"
-               "jnz      3b\n\t"
-               "2:\n\t"
+               "jnz      1b\n\t"
+               "9:\n\t"
                "emms"
                : /* no outputs */
-               : "D" (gfxs->Xacc), "c" (gfxs->length), "S" (gfxs->Sacc),
-                 "m" (*einser), "m" (*zeros), "m" (gfxs->color)
+               : "D" (gfxs->Yacc), "c" (gfxs->length), "S" (gfxs->Sacc),
+                 "m" (*einser), "m" (*zeros), "m" (gfxs->color), "r" (gfxs->Xacc)
                : "%st", "memory");
 }
 
@@ -399,24 +419,29 @@ static void Xacc_blend_srcalpha_MMX( GenefxState *gfxs )
                "punpckldq %%mm0, %%mm0\n\t" /* mm0 = 00aa 00aa 00aa 00aa */
                "paddw     %%mm7, %%mm0\n\t"
 
-               ".align   16\n"
+               ".align   16\n\t"
                "4:\n\t"                 /* blend from color */
                "testw    $0xF000, 6(%0)\n\t"
-               "jnz      1f\n\t"
+               "jnz      6f\n\t"
                "movq     (%0), %%mm1\n\t"
                "pmullw   %%mm0, %%mm1\n\t"
                "psrlw    $8, %%mm1\n\t"
-               "movq     %%mm1, (%0)\n"
+               "movq     %%mm1, (%6)\n\t"
+               "jmp      1f\n\t"
+               "6:\n\t"
+               "movq     (%0), %%mm1\n\t"
+               "movq     %%mm1, (%6)\n\t"
                "1:\n\t"
 	       "add      $8, %0\n\t"
+	       "add      $8, %6\n\t"
                "dec      %1\n\t"
                "jnz      4b\n\t"
                "jmp      2f\n\t"
 
-               ".align   16\n"
+               ".align   16\n\t"
                "3:\n\t"                      /* blend from Sacc */
                "testw    $0xF000, 6(%0)\n\t"
-               "jnz      1f\n\t"
+               "jnz      5f\n\t"
                "movq     (%2), %%mm0\n\t"
                "movq     (%0), %%mm1\n\t"
 	       "punpckhwd %%mm0, %%mm0\n\t" /* mm2 = 00aa 00aa xxxx xxxx */
@@ -424,20 +449,26 @@ static void Xacc_blend_srcalpha_MMX( GenefxState *gfxs )
                "paddw    %%mm7, %%mm0\n\t"
                "pmullw   %%mm0, %%mm1\n\t"
                "psrlw    $8, %%mm1\n\t"
-               "movq     %%mm1, (%0)\n"
-               "1:\n\t"
+               "movq     %%mm1, (%6)\n\t"
+               "jmp      7f\n\t"
+               "5:\n\t"
+               "movq     (%0), %%mm1\n\t"
+               "movq     %%mm1, (%6)\n\t"
+               "7:\n\t"
 	       "add      $8, %2\n\t"
 	       "add      $8, %0\n\t"
+	       "add      $8, %6\n\t"
                "dec      %1\n\t"
                "jnz      3b\n\t"
                "2:\n\t"
                "emms"
                : /* no outputs */
-               : "D" (gfxs->Xacc), "c" (gfxs->length), "S" (gfxs->Sacc),
-                 "m" (*ones), "m" (*zeros), "m" (gfxs->color)
+               : "D" (gfxs->Yacc), "c" (gfxs->length), "S" (gfxs->Sacc),
+                 "m" (*ones), "m" (*zeros), "m" (gfxs->color), "r" (gfxs->Xacc)
                : "%st", "memory");
 }
 
+#if 0
 static void Dacc_YCbCr_to_RGB_MMX( GenefxState *gfxs )
 {  
      static const u16 __aligned(8) sub0[4] = {  16,  16,  16,  16 }; 
@@ -636,4 +667,4 @@ static void Dacc_RGB_to_YCbCr_MMX( GenefxState *gfxs )
           w--;
      }
 }
-
+#endif
