@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 /*
  * Example:
  * #define A_SHIFT 15
@@ -70,18 +74,22 @@
 
 static void Sop_PFI_OP_Dacc(Sto)( GenefxState *gfxs )
 {
-     int                l     = gfxs->length;
+     int                l     = gfxs->length+1;
      int                i     = gfxs->Xphase;
      int                SperD = gfxs->SperD;
      u16               *S     = gfxs->Sop[0];
+     int                Ostep = gfxs->Ostep;
      GenefxAccumulator *D     = gfxs->Dacc;
 
-     while (l--) {
+     if (Ostep != 1)
+          D_UNIMPLEMENTED();
+
+     while (--l) {
           u16 s = S[i>>16];
 
           EXPAND( *D, s );
 
-          D++;
+          ++D;
           i += SperD;
      }
 }
@@ -90,14 +98,18 @@ static void Sop_PFI_OP_Dacc(Sto)( GenefxState *gfxs )
 
 static void Sop_PFI_OP_Dacc(SKto)( GenefxState *gfxs )
 {
-     int                l     = gfxs->length;
+     int                l     = gfxs->length+1;
      int                i     = gfxs->Xphase;
      int                SperD = gfxs->SperD;
      u16               *S     = gfxs->Sop[0];
+     int                Ostep = gfxs->Ostep;
      GenefxAccumulator *D     = gfxs->Dacc;
      u16                Skey  = gfxs->Skey;
 
-     while (l--) {
+     if (Ostep != 1)
+          D_UNIMPLEMENTED();
+
+     while (--l) {
           u16 s = S[i>>16];
 
           if (MASK_RGB( s ) != Skey)
@@ -105,7 +117,7 @@ static void Sop_PFI_OP_Dacc(SKto)( GenefxState *gfxs )
           else
                D->RGB.a = 0xF000;
 
-          D++;
+          ++D;
           i += SperD;
      }
 }
@@ -117,18 +129,33 @@ static void Sop_PFI_OP_Dacc(to)( GenefxState *gfxs )
      int             w, l = gfxs->length;
      u16               *S = gfxs->Sop[0];
      GenefxAccumulator *D = gfxs->Dacc;
+     int                Ostep = gfxs->Ostep;
+
+     if (Ostep != 1) {
+          l++;
+          while (--l) {
+              u16 s = *S;
+
+              EXPAND( *D, s );
+
+              S += Ostep;
+
+              ++D;
+          }
+          return;
+     }
 
      if ((long)S & 2) {
           u16 s = *S++;
 
           EXPAND( *D, s );
 
-          D++;
-          l--;
+          ++D;
+          --l;
      }
 
-     w = l >> 1;
-     while (w--) {
+     w = (l >> 1) +1;
+     while (--w) {
           u32 s = *(u32 *) S;
 
 #ifdef WORDS_BIGENDIAN
@@ -154,20 +181,22 @@ static void Sop_PFI_OP_Dacc(to)( GenefxState *gfxs )
 
 static void Sop_PFI_OP_Dacc(Kto)( GenefxState *gfxs )
 {
-     int                l    = gfxs->length;
+     int                l    = gfxs->length+1;
      u16               *S    = gfxs->Sop[0];
      GenefxAccumulator *D    = gfxs->Dacc;
      u16                Skey = gfxs->Skey;
+     int                Ostep = gfxs->Ostep;
 
-     while (l--) {
-          u16 s = *S++;
+     while (--l) {
+          u16 s = *S;
 
           if (MASK_RGB( s ) != Skey)
                EXPAND( *D, s );
           else
                D->RGB.a = 0xF000;
 
-          D++;
+          S += Ostep;
+          ++D;
      }
 }
 
@@ -178,18 +207,31 @@ static void Sacc_OP_Aop_PFI(to)( GenefxState *gfxs )
      int             w, l = gfxs->length;
      GenefxAccumulator *S = gfxs->Sacc;
      u16               *D = gfxs->Aop[0];
+     int                Dstep = gfxs->Astep;
+
+     if (Dstep != 1) {
+          l++;
+          while (--l) {
+              if (!(S->RGB.a & 0xF000))
+                   *D = PIXEL( *S );
+
+              ++S;
+              D += Dstep;
+          }
+          return;
+     }
 
      if ((long)D & 2) {
           if (!(S->RGB.a & 0xF000))
                *D = PIXEL( *S );
 
-          S++;
-          D++;
-          l--;
+          ++S;
+          ++D;
+          --l;
      }
 
-     w = l >> 1;
-     while (w--) {
+     w = (l >> 1) +1;
+     while (--w) {
           u32 *D2 = (u32 *) D;
 
           if (!(S[0].RGB.a & 0xF000) && !(S[1].RGB.a & 0xF000)) {
@@ -224,20 +266,34 @@ static void Sacc_OP_Aop_PFI(Sto)( GenefxState *gfxs )
      int                SperD = gfxs->SperD;
      GenefxAccumulator *Sacc  = gfxs->Sacc;
      u16               *D     = gfxs->Aop[0];
+     int                Dstep = gfxs->Astep;
 
+     if (Dstep != 1) {
+         l++;
+         while (--l) {
+              GenefxAccumulator *S = &Sacc[i>>16];
+
+              if (!(S->RGB.a & 0xF000))
+                   *D = PIXEL( *S );
+
+              D += Dstep;
+              i += SperD;
+         }
+         return;
+     }
      if ((long)D & 2) {
           GenefxAccumulator *S = Sacc;
 
           if (!(S->RGB.a & 0xF000))
                *D = PIXEL( *S );
 
-          D++;
-          l--;
+          ++D;
+          --l;
           i += SperD;
      }
 
-     w = l >> 1;
-     while (w--) {
+     w = (l >> 1) +1;
+     while (--w) {
           GenefxAccumulator *S0 = &Sacc[i>>16];
           GenefxAccumulator *S1 = &Sacc[(i+SperD)>>16];
           u32               *D2 = (u32 *) D;
@@ -271,17 +327,18 @@ static void Sacc_OP_Aop_PFI(Sto)( GenefxState *gfxs )
 
 static void Sacc_OP_Aop_PFI(toK)( GenefxState *gfxs )
 {
-     int                l    = gfxs->length;
+     int                l    = gfxs->length+1;
      GenefxAccumulator *S    = gfxs->Sacc;
      u16               *D    = gfxs->Aop[0];
      u16                Dkey = gfxs->Dkey;
+     int                Dstep = gfxs->Astep;
 
-     while (l--) {
+     while (--l) {
           if (!(S->RGB.a & 0xF000) && MASK_RGB( *D ) == Dkey)
                *D = PIXEL( *S );
 
-          S++;
-          D++;
+          ++S;
+          D += Dstep;
      }
 }
 
@@ -289,24 +346,85 @@ static void Sacc_OP_Aop_PFI(toK)( GenefxState *gfxs )
 
 static void Sacc_OP_Aop_PFI(StoK)( GenefxState *gfxs )
 {
-     int                l     = gfxs->length;
+     int                l     = gfxs->length+1;
      int                i     = gfxs->Xphase;
      int                SperD = gfxs->SperD;
      GenefxAccumulator *Sacc  = gfxs->Sacc;
      u16               *D     = gfxs->Aop[0];
      u16                Dkey  = gfxs->Dkey;
+     int                Dstep = gfxs->Astep;
 
-     while (l--) {
+     while (--l) {
           GenefxAccumulator *S = &Sacc[i>>16];
 
           if (!(S->RGB.a & 0xF000) && MASK_RGB( *D ) == Dkey)
                *D = PIXEL( *S );
 
-          D++;
+          D += Dstep;
           i += SperD;
      }
 }
 
+/********************************* Sop_PFI_TEX_to_Dacc ***************************/
+
+static void Sop_PFI_OP_Dacc(TEX_to)( GenefxState *gfxs )
+{
+     int                l     = gfxs->length+1;
+     int                s     = gfxs->s;
+     int                t     = gfxs->t;
+     int                SperD = gfxs->SperD;
+     int                TperD = gfxs->TperD;
+     u16               *S     = gfxs->Sop[0];
+     int                Ostep = gfxs->Ostep;
+     GenefxAccumulator *D     = gfxs->Dacc;
+     int                sp2   = gfxs->src_pitch / 2;
+
+     if (Ostep != 1)
+          D_UNIMPLEMENTED();
+
+     while (--l) {
+          u16 p = S[(s>>16) + (t>>16) * sp2];
+
+          EXPAND( *D, p );
+
+          ++D;
+          s += SperD;
+          t += TperD;
+     }
+}
+
+/********************************* Sop_PFI_TEX_Kto_Dacc ***************************/
+
+static void Sop_PFI_OP_Dacc(TEX_Kto)( GenefxState *gfxs )
+{
+     int                l     = gfxs->length+1;
+     int                s     = gfxs->s;
+     int                t     = gfxs->t;
+     int                SperD = gfxs->SperD;
+     int                TperD = gfxs->TperD;
+     u16               *S     = gfxs->Sop[0];
+     u16                Skey  = gfxs->Skey;
+     int                Ostep = gfxs->Ostep;
+     GenefxAccumulator *D     = gfxs->Dacc;
+     int                sp2   = gfxs->src_pitch / 2;
+
+     if (Ostep != 1)
+          D_UNIMPLEMENTED();
+
+     while (--l) {
+          u16 p = S[(s>>16) + (t>>16) * sp2];
+
+          if ((p & RGB_MASK) != Skey)
+               EXPAND( *D, p );
+          else
+               D->RGB.a = 0xF000;
+
+          ++D;
+          s += SperD;
+          t += TperD;
+     }
+}
+
 /******************************************************************************/
 
 #undef RGB_MASK
