@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -38,6 +42,8 @@
 
 #include <directfb.h>
 
+#include <core/CoreInputDevice.h>
+
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 
@@ -201,6 +207,38 @@ IDirectFBInputDevice_GetKeymapEntry( IDirectFBInputDevice      *thiz,
      return dfb_input_device_get_keymap_entry( data->device, keycode, entry );
 }
 
+static DFBResult
+IDirectFBInputDevice_SetKeymapEntry( IDirectFBInputDevice      *thiz,
+                                     int                        keycode,
+                                     DFBInputDeviceKeymapEntry *entry )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
+
+     if (!entry)
+          return DFB_INVARG;
+
+     if (data->desc.min_keycode < 0 || data->desc.max_keycode < 0)
+          return DFB_UNSUPPORTED;
+
+     if (keycode < data->desc.min_keycode ||
+         keycode > data->desc.max_keycode)
+          return DFB_INVARG;
+
+     return CoreInputDevice_SetKeymapEntry( data->device, keycode, entry );
+}
+
+static DFBResult
+IDirectFBInputDevice_LoadKeymap ( IDirectFBInputDevice          *thiz,
+                                  char                          *filename )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
+
+     if (!filename)
+          return DFB_INVARG;
+
+     return dfb_input_device_load_keymap( data->device, filename );
+}
+
 static DFBResult
 IDirectFBInputDevice_GetKeyState( IDirectFBInputDevice        *thiz,
                                   DFBInputDeviceKeyIdentifier  key_id,
@@ -221,12 +259,19 @@ static DFBResult
 IDirectFBInputDevice_GetModifiers( IDirectFBInputDevice       *thiz,
                                    DFBInputDeviceModifierMask *modifiers )
 {
+     DFBResult            ret;
+     CoreInputDeviceState state;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
 
      if (!modifiers)
           return DFB_INVARG;
 
-     *modifiers = data->modifiers;
+     ret = dfb_input_device_get_state( data->device, &state );
+     if (ret)
+          return ret;
+
+     *modifiers = state.modifiers_l | state.modifiers_r;
 
      return DFB_OK;
 }
@@ -235,12 +280,19 @@ static DFBResult
 IDirectFBInputDevice_GetLockState( IDirectFBInputDevice    *thiz,
                                    DFBInputDeviceLockState *locks )
 {
+     DFBResult            ret;
+     CoreInputDeviceState state;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
 
      if (!locks)
           return DFB_INVARG;
 
-     *locks = data->locks;
+     ret = dfb_input_device_get_state( data->device, &state );
+     if (ret)
+          return ret;
+
+     *locks = state.locks;
 
      return DFB_OK;
 }
@@ -249,12 +301,19 @@ static DFBResult
 IDirectFBInputDevice_GetButtons( IDirectFBInputDevice     *thiz,
                                  DFBInputDeviceButtonMask *buttons )
 {
+     DFBResult            ret;
+     CoreInputDeviceState state;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
 
      if (!buttons)
           return DFB_INVARG;
 
-     *buttons = data->buttonmask;
+     ret = dfb_input_device_get_state( data->device, &state );
+     if (ret)
+          return ret;
+
+     *buttons = state.buttons;
 
      return DFB_OK;
 }
@@ -262,14 +321,21 @@ IDirectFBInputDevice_GetButtons( IDirectFBInputDevice     *thiz,
 static DFBResult
 IDirectFBInputDevice_GetButtonState( IDirectFBInputDevice           *thiz,
                                      DFBInputDeviceButtonIdentifier  button,
-                                     DFBInputDeviceButtonState      *state)
+                                     DFBInputDeviceButtonState      *ret_state)
 {
+     DFBResult            ret;
+     CoreInputDeviceState state;
+
      DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
 
-     if (!state || (int)button < DIBI_FIRST || button > DIBI_LAST)
+     ret = dfb_input_device_get_state( data->device, &state );
+     if (ret)
+          return ret;
+
+     if (!ret_state || (int)button < DIBI_FIRST || button > DIBI_LAST)
           return DFB_INVARG;
 
-     *state = (data->buttonmask & (1 << button)) ? DIBS_DOWN : DIBS_UP;
+     *ret_state = (state.buttons & (1 << button)) ? DIBS_DOWN : DIBS_UP;
 
      return DFB_OK;
 }
@@ -308,6 +374,15 @@ IDirectFBInputDevice_GetXY( IDirectFBInputDevice *thiz,
      return DFB_OK;
 }
 
+static DFBResult
+IDirectFBInputDevice_SetConfiguration( IDirectFBInputDevice       *thiz,
+                                       const DFBInputDeviceConfig *config )
+{
+     DIRECT_INTERFACE_GET_DATA(IDirectFBInputDevice)
+
+     return CoreInputDevice_SetConfiguration( data->device, config );
+}
+
 DFBResult
 IDirectFBInputDevice_Construct( IDirectFBInputDevice *thiz,
                                 CoreInputDevice      *device )
@@ -327,6 +402,8 @@ IDirectFBInputDevice_Construct( IDirectFBInputDevice *thiz,
      thiz->GetID = IDirectFBInputDevice_GetID;
      thiz->GetDescription = IDirectFBInputDevice_GetDescription;
      thiz->GetKeymapEntry = IDirectFBInputDevice_GetKeymapEntry;
+     thiz->SetKeymapEntry = IDirectFBInputDevice_SetKeymapEntry;
+     thiz->LoadKeymap = IDirectFBInputDevice_LoadKeymap;
      thiz->CreateEventBuffer = IDirectFBInputDevice_CreateEventBuffer;
      thiz->AttachEventBuffer = IDirectFBInputDevice_AttachEventBuffer;
      thiz->DetachEventBuffer = IDirectFBInputDevice_DetachEventBuffer;
@@ -337,6 +414,7 @@ IDirectFBInputDevice_Construct( IDirectFBInputDevice *thiz,
      thiz->GetButtonState = IDirectFBInputDevice_GetButtonState;
      thiz->GetAxis = IDirectFBInputDevice_GetAxis;
      thiz->GetXY = IDirectFBInputDevice_GetXY;
+     thiz->SetConfiguration = IDirectFBInputDevice_SetConfiguration;
 
      return DFB_OK;
 }
