@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <fcntl.h>
@@ -35,7 +39,6 @@
 
 #include <direct/mem.h>
 
-#include <fusion/arena.h>
 #include <fusion/shmalloc.h>
 
 #include <core/core.h>
@@ -49,6 +52,10 @@
 
 #include <core/core_system.h>
 
+#ifdef HAVE_GFX_SH772X
+#include <uiomux/uiomux.h>
+#endif
+
 DFB_CORE_SYSTEM( devmem )
 
 /**********************************************************************************************************************/
@@ -88,6 +95,10 @@ MapMemAndReg( DevMemData    *data,
           }
      }
 
+#ifdef HAVE_GFX_SH772X
+	uiomux_register (data->mem, mem_phys, mem_length);
+#endif
+
      close( fd );
 
      return DFB_OK;
@@ -165,7 +176,7 @@ system_initialize( CoreDFB *core, void **ret_data )
 
      dfb_surface_pool_initialize( core, &devmemSurfacePoolFuncs, &shared->pool );
 
-     fusion_arena_add_shared_field( dfb_core_arena( core ), "devmem", shared );
+     core_arena_add_shared_field( core, "devmem", shared );
 
      return DFB_OK;
 }
@@ -194,7 +205,7 @@ system_join( CoreDFB *core, void **ret_data )
      if (!data)
           return D_OOM();
 
-     ret = fusion_arena_get_shared_field( dfb_core_arena( core ), "devmem", &tmp );
+     ret = core_arena_get_shared_field( core, "devmem", &tmp );
      if (ret) {
           D_FREE( data );
           return ret;
@@ -365,6 +376,27 @@ system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
      return;
 }
 
+static int
+system_surface_data_size( void )
+{
+     /* Return zero because shared surface data is unneeded. */
+     return 0;
+}
+
+static void
+system_surface_data_init( CoreSurface *surface, void *data )
+{
+     /* Ignore since unneeded. */
+     return;
+}
+
+static void
+system_surface_data_destroy( CoreSurface *surface, void *data )
+{
+     /* Ignore since unneeded. */
+     return;
+}
+
 static void
 system_get_deviceid( unsigned int *ret_vendor_id,
                      unsigned int *ret_device_id )
