@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <direct/debug.h>
@@ -131,6 +135,14 @@ devmemInitPool( CoreDFB                    *core,
      ret_desc->access[CSAID_LAYER5] = CSAF_READ;
      ret_desc->access[CSAID_LAYER6] = CSAF_READ;
      ret_desc->access[CSAID_LAYER7] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER8] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER9] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER10] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER11] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER12] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER13] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER14] = CSAF_READ;
+     ret_desc->access[CSAID_LAYER15] = CSAF_READ;
 
      snprintf( ret_desc->name, DFB_SURFACE_POOL_DESC_NAME_LENGTH, "/dev/mem" );
 
@@ -265,6 +277,8 @@ devmemAllocateBuffer( CoreSurfacePool       *pool,
      DevMemPoolLocalData  *local = pool_local;
      DevMemAllocationData *alloc = alloc_data;
 
+     (void)surface;
+
      D_DEBUG_AT( DevMem_Surfaces, "%s( %p )\n", __FUNCTION__, buffer );
 
      D_MAGIC_ASSERT( pool, CoreSurfacePool );
@@ -312,7 +326,6 @@ devmemDeallocateBuffer( CoreSurfacePool       *pool,
 
      D_MAGIC_ASSERT( pool, CoreSurfacePool );
      D_MAGIC_ASSERT( data, DevMemPoolData );
-     D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );
      D_MAGIC_ASSERT( alloc, DevMemAllocationData );
 
      if (alloc->chunk)
@@ -363,7 +376,6 @@ devmemUnlock( CoreSurfacePool       *pool,
      DevMemAllocationData *alloc = alloc_data;
 
      D_MAGIC_ASSERT( pool, CoreSurfacePool );
-     D_MAGIC_ASSERT( allocation, CoreSurfaceAllocation );
      D_MAGIC_ASSERT( alloc, DevMemAllocationData );
      D_MAGIC_ASSERT( lock, CoreSurfaceBufferLock );
 
