@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <asm/types.h>    /* Needs to be included before dfb_types.h */
@@ -96,6 +100,10 @@ static int       vt_get_fb( int vt );
 static void      vt_set_fb( int vt, int fb );
 static void     *vt_thread( DirectThread *thread, void *arg );
 
+static void      vt_start_flushing( void );
+static void      vt_stop_flushing( void );
+static void     *vt_flush_thread( DirectThread *thread, void *arg );
+
 DFBResult
 dfb_vt_initialize( void )
 {
@@ -221,6 +229,8 @@ dfb_vt_initialize( void )
           return ret;
      }
 
+     vt_start_flushing();
+
      dfb_fbdev->vt = dfb_vt;
 
      return DFB_OK;
@@ -239,14 +249,19 @@ dfb_vt_join( void )
 DFBResult
 dfb_vt_shutdown( bool emergency )
 {
+     int res;
      const char cursoron_str[] = "\033[?0;0;0c";
      const char blankon_str[] = "\033[9;10]";
 
+     (void)res;
+
      D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
 
      if (!dfb_vt)
           return DFB_OK;
 
+     vt_stop_flushing();
+
      if (dfb_config->vt_switching) {
           if (ioctl( dfb_vt->fd, VT_SETMODE, &dfb_vt->vt_mode ) < 0)
                D_PERROR( "DirectFB/fbdev/vt: Unable to restore VT mode!!!\n" );
@@ -264,12 +279,18 @@ dfb_vt_shutdown( bool emergency )
 
      if (dfb_config->kd_graphics) {
           if (ioctl( dfb_vt->fd, KDSETMODE, KD_TEXT ) < 0)
-               D_PERROR( "DirectFB/Keyboard: KD_TEXT failed!\n" );
+               D_PERROR( "DirectFB/fbdev/vt: KD_TEXT failed!\n" );
      }
      else {
-          write( dfb_vt->fd, blankon_str, sizeof(blankon_str) );
+          res = write( dfb_vt->fd, blankon_str, sizeof(blankon_str) );
      }
-     write( dfb_vt->fd, cursoron_str, sizeof(cursoron_str) );
+     res = write( dfb_vt->fd, cursoron_str, sizeof(cursoron_str) );
+
+     if (tcsetattr( dfb_vt->fd, TCSAFLUSH, &dfb_vt->old_ts ) < 0)
+          D_PERROR("DirectFB/fbdev/vt: tcsetattr for original values failed!\n");
+
+     if (ioctl( dfb_vt->fd, KDSKBMODE, K_XLATE ) < 0)
+          D_PERROR( "DirectFB/fbdev/vt: K_XLATE failed!\n" );
 
      if (dfb_config->vt_switch) {
           D_DEBUG_AT( VT, "  -> switching back...\n" );
@@ -443,10 +464,14 @@ vt_switch_handler( int signum )
 static DFBResult
 vt_init_switching( void )
 {
+     struct termios ts;
      const char cursoroff_str[] = "\033[?1;0;0c";
      const char blankoff_str[] = "\033[9;0]";
      char buf[32];
 
+     int res;
+     (void)res;
+
      D_DEBUG_AT( VT, "%s()\n", __FUNCTION__ );
 
      /* FIXME: Opening the device should be moved out of this function. */
@@ -481,16 +506,42 @@ vt_init_switching( void )
         otherwise we'd get access denied error: */
      ioctl( dfb_vt->fd, TIOCSCTTY, 0 );
 
-     write( dfb_vt->fd, cursoroff_str, sizeof(cursoroff_str) );
+     if (ioctl( dfb_vt->fd, KDSKBMODE, K_MEDIUMRAW ) < 0) {
+          D_PERROR( "DirectFB/fbdev/vt: K_MEDIUMRAW failed!\n" );
+          close( dfb_vt->fd );
+          return DFB_INIT;
+     }
+
+     if (tcgetattr( dfb_vt->fd, &dfb_vt->old_ts ) < 0) {
+          D_PERROR( "DirectFB/fbdev/vt: tcgetattr failed!\n" );
+          ioctl( dfb_vt->fd, KDSKBMODE, K_XLATE );
+          close( dfb_vt->fd );
+          return DFB_INIT;
+     }
+     ts = dfb_vt->old_ts;
+     ts.c_cc[VTIME] = 0;
+     ts.c_cc[VMIN] = 1;
+     ts.c_lflag &= ~(ICANON|ECHO|ISIG);
+     ts.c_iflag = 0;
+     if (tcsetattr( dfb_vt->fd, TCSAFLUSH, &ts ) < 0) {
+          D_PERROR( "DirectFB/fbdev/vt: tcsetattr for new values failed!\n" );
+          ioctl( dfb_vt->fd, KDSKBMODE, K_XLATE );
+          close( dfb_vt->fd );
+          return DFB_INIT;
+     }
+
+     res = write( dfb_vt->fd, cursoroff_str, sizeof(cursoroff_str) );
      if (dfb_config->kd_graphics) {
           if (ioctl( dfb_vt->fd, KDSETMODE, KD_GRAPHICS ) < 0) {
                D_PERROR( "DirectFB/fbdev/vt: KD_GRAPHICS failed!\n" );
+               tcsetattr( dfb_vt->fd, TCSAFLUSH, &dfb_vt->old_ts );
+               ioctl( dfb_vt->fd, KDSKBMODE, K_XLATE );
                close( dfb_vt->fd );
                return DFB_INIT;
           }
      }
      else {
-          write( dfb_vt->fd, blankoff_str, sizeof(blankoff_str) );
+          res = write( dfb_vt->fd, blankoff_str, sizeof(blankoff_str) );
      }
 
      if (dfb_config->vt_switching) {
@@ -504,6 +555,8 @@ vt_init_switching( void )
           if (sigaction( SIG_SWITCH_FROM, &sig_tty, &dfb_vt->sig_usr1 ) ||
               sigaction( SIG_SWITCH_TO, &sig_tty, &dfb_vt->sig_usr2 )) {
                D_PERROR( "DirectFB/fbdev/vt: sigaction failed!\n" );
+               tcsetattr( dfb_vt->fd, TCSAFLUSH, &dfb_vt->old_ts );
+               ioctl( dfb_vt->fd, KDSKBMODE, K_XLATE );
                close( dfb_vt->fd );
                return DFB_INIT;
           }
@@ -520,6 +573,8 @@ vt_init_switching( void )
                sigaction( SIG_SWITCH_FROM, &dfb_vt->sig_usr1, NULL );
                sigaction( SIG_SWITCH_TO, &dfb_vt->sig_usr2, NULL );
 
+               tcsetattr( dfb_vt->fd, TCSAFLUSH, &dfb_vt->old_ts );
+               ioctl( dfb_vt->fd, KDSKBMODE, K_XLATE );
                close( dfb_vt->fd );
 
                return DFB_INIT;
@@ -583,3 +638,48 @@ vt_set_fb( int vt, int fb )
      }
 }
 
+static void
+vt_start_flushing( void )
+{
+     dfb_vt->flush = true;
+     dfb_vt->flush_thread = direct_thread_create( DTT_DEFAULT, vt_flush_thread, NULL, "VT Flusher" );
+}
+
+static void
+vt_stop_flushing( void )
+{
+     dfb_vt->flush = false;
+     direct_thread_cancel( dfb_vt->flush_thread );
+     direct_thread_join( dfb_vt->flush_thread );
+     direct_thread_destroy( dfb_vt->flush_thread );
+     dfb_vt->flush_thread = NULL;
+}
+
+/*
+ * If the vt buffer in not kept clean the kernel may stop sleeping.
+ */
+static void *
+vt_flush_thread( DirectThread *thread, void *arg )
+{
+     D_DEBUG_AT( VT, "%s( %p, %p )\n", __FUNCTION__, thread, arg );
+
+     while (dfb_vt->flush) {
+          fd_set set;
+          int ret;
+
+          FD_ZERO( &set );
+          FD_SET( dfb_vt->fd, &set );
+
+          ret = select( dfb_vt->fd + 1, &set, NULL, NULL, NULL );
+
+          if (ret < 0 && errno == EINTR)
+               continue;
+
+          if (ret < 0 || !dfb_vt->flush)
+               break;
+
+          tcflush( dfb_vt->fd, TCIFLUSH );
+     }
+
+     return NULL;
+}
