@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -116,11 +120,13 @@ primaryGetScreenSize( CoreScreen *screen,
      return DFB_OK;
 }
 
-ScreenFuncs osxPrimaryScreenFuncs = {
+static ScreenFuncs primaryScreenFuncs = {
      .InitScreen    = primaryInitScreen,
      .GetScreenSize = primaryGetScreenSize,
 };
 
+ScreenFuncs *osxPrimaryScreenFuncs = &primaryScreenFuncs;
+
 /******************************************************************************/
 
 static int
@@ -398,7 +404,7 @@ primaryReallocateSurface( CoreLayer             *layer,
      return DFB_OK;
 }
 
-DisplayLayerFuncs osxPrimaryLayerFuncs = {
+static DisplayLayerFuncs primaryLayerFuncs = {
      .LayerDataSize     = primaryLayerDataSize,
      .RegionDataSize    = primaryRegionDataSize,
      .InitLayer         = primaryInitLayer,
@@ -414,6 +420,8 @@ DisplayLayerFuncs osxPrimaryLayerFuncs = {
      .ReallocateSurface = primaryReallocateSurface,
 };
 
+DisplayLayerFuncs *osxPrimaryLayerFuncs = &primaryLayerFuncs;
+
 /******************************************************************************/
 
 static DFBResult
