@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <stdio.h>
@@ -38,7 +42,6 @@
 #include <direct/messages.h>
 #include <direct/thread.h>
 
-#include <fusion/arena.h>
 #include <fusion/shmalloc.h>
 
 #include <core/core.h>
@@ -122,7 +125,7 @@ system_initialize( CoreDFB *core, void **data )
 
      dfb_layers_register( screen, NULL, sdlPrimaryLayerFuncs );
 
-     fusion_arena_add_shared_field( dfb_core_arena( core ), "sdl", dfb_sdl );
+     core_arena_add_shared_field( core, "sdl", dfb_sdl );
 
      dfb_surface_pool_initialize( core, &sdlSurfacePoolFuncs, &dfb_sdl->sdl_pool );
 
@@ -139,7 +142,7 @@ system_join( CoreDFB *core, void **data )
 
      D_ASSERT( dfb_sdl == NULL );
 
-     fusion_arena_get_shared_field( dfb_core_arena( core ), "sdl", &ret );
+     core_arena_get_shared_field( core, "sdl", &ret );
 
      dfb_sdl = ret;
      dfb_sdl_core = core;
@@ -311,6 +314,27 @@ system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
      return;
 }
 
+static int
+system_surface_data_size( void )
+{
+     /* Return zero because shared surface data is unneeded. */
+     return 0;
+}
+
+static void
+system_surface_data_init( CoreSurface *surface, void *data )
+{
+     /* Ignore since unneeded. */
+     return;
+}
+
+static void
+system_surface_data_destroy( CoreSurface *surface, void *data )
+{
+     /* Ignore since unneeded. */
+     return;
+}
+
 static void
 system_get_deviceid( unsigned int *ret_vendor_id,
                      unsigned int *ret_device_id )
