@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <fusion/shmalloc.h>
@@ -251,6 +255,7 @@ DFBResult dfb_surfacemanager_displace( CoreDFB           *core,
      /* FIXME: Only one global device at the moment. */
      device = dfb_core_get_part( core, DFCP_GRAPHICS );
      D_ASSERT( device != NULL );
+     D_UNUSED_P( device );
 
      dfb_gfxcard_calc_buffer_size( dfb_core_get_part( core, DFCP_GRAPHICS ), buffer, NULL, &length );
 
@@ -267,7 +272,7 @@ DFBResult dfb_surfacemanager_displace( CoreDFB           *core,
           allocation = chunk->allocation;
           if (allocation) {
                CoreSurfaceBuffer *other;
-               int                size;
+               int                size, locks;
 
                D_MAGIC_ASSERT( allocation, CoreSurfaceAllocation );
                D_ASSERT( chunk->buffer == allocation->buffer );
@@ -276,8 +281,9 @@ DFBResult dfb_surfacemanager_displace( CoreDFB           *core,
                other = allocation->buffer;
                D_MAGIC_ASSERT( other, CoreSurfaceBuffer );
 
-               if (other->locked) {
-                    D_DEBUG_AT( SurfMan, "  ++ %7d locked %dx\n", allocation->size, other->locked );
+               locks = dfb_surface_allocation_locks( allocation );
+               if (locks) {
+                    D_DEBUG_AT( SurfMan, "  ++ %7d locked %dx\n", allocation->size, locks );
                     goto next_reset;
                }
 
@@ -428,6 +434,7 @@ DFBResult dfb_surfacemanager_deallocate( SurfaceManager *manager,
      buffer = chunk->buffer;
      D_MAGIC_ASSERT( buffer, CoreSurfaceBuffer );
      D_MAGIC_ASSERT( buffer->surface, CoreSurface );
+     D_UNUSED_P( buffer );
 
      D_DEBUG_AT( SurfMan, "%s( %p ) <- %dx%d %s\n", __FUNCTION__, buffer,
                  buffer->surface->config.size.w, buffer->surface->config.size.h,
