@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <fusion/types.h>
@@ -36,7 +40,6 @@
 
 #include <directfb.h>
 
-#include <fusion/arena.h>
 #include <fusion/shmalloc.h>
 #include <fusion/lock.h>
 
@@ -44,6 +47,7 @@
 #include <core/coredefs.h>
 #include <core/coretypes.h>
 #include <core/layers.h>
+#include <core/layers_internal.h>
 #include <core/palette.h>
 #include <core/surface.h>
 #include <core/system.h>
@@ -59,11 +63,17 @@
 #include "xwindow.h"
 #include "x11.h"
 #include "x11_surface_pool.h"
+#include "x11_surface_pool_bridge.h"
+
+#ifdef USE_GLX
+#include "glx_surface_pool.h"
+#endif
 
 #include "vpsmem_surface_pool.h"
 
 #include <core/core_system.h>
 
+D_DEBUG_DOMAIN( X11_Core, "X11/Core", "Main X11 system functions" );
 
 DFB_CORE_SYSTEM( x11 )
 
@@ -106,13 +116,52 @@ static FusionCallHandlerResult call_handler( int           caller,
 
 /**********************************************************************************************************************/
 
+static void
+sync_display( DFBX11 *x11 )
+{
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+
+     XSync( x11->display, False );
+}
+
+static void
+sync_display_noop( DFBX11 *x11 )
+{
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+}
+
+/**********************************************************************************************************************/
+
+static DFBX11Shared *shared_for_error_handler;
+
+static int
+error_handler( Display *display, XErrorEvent *event )
+{
+     char buf[512];
+
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+
+     XGetErrorText( display, event->error_code, buf, sizeof(buf) );
+
+     D_ERROR( "X11/Core: Error! %s\n", buf );
+
+     shared_for_error_handler->x_error = True;
+
+     return 0;
+}
+
 static DFBResult
 InitLocal( DFBX11 *x11, DFBX11Shared *shared, CoreDFB *core )
 {
-     int i, n;
+     int i, n, d;
 
      XInitThreads();
 
+     if (direct_config_get_int_value( "x11-sync" ))
+          x11->Sync = sync_display;
+     else
+          x11->Sync = sync_display_noop;
+
      x11->shared = shared;
      x11->core   = core;
 
@@ -124,6 +173,7 @@ InitLocal( DFBX11 *x11, DFBX11Shared *shared, CoreDFB *core )
 
      x11->screenptr = DefaultScreenOfDisplay(x11->display);
      x11->screennum = DefaultScreen(x11->display);
+     d              = DefaultDepthOfScreen(x11->screenptr);
 
      for (i=0; i<x11->screenptr->ndepths; i++) {
           const Depth *depth = &x11->screenptr->depths[i];
@@ -131,11 +181,14 @@ InitLocal( DFBX11 *x11, DFBX11Shared *shared, CoreDFB *core )
           for (n=0; n<depth->nvisuals; n++) {
                Visual *visual = &depth->visuals[n];
 
-               D_DEBUG( "X11/Visual: [%2d] ID 0x%02lx, depth %d, red 0x%06lx, green 0x%06lx, blue 0x%06lx, %d bits/rgb, %d entries\n",
+               D_DEBUG_AT( X11_Core, "[Visual %d] ID 0x%02lx, depth %d, RGB 0x%06lx/0x%06lx/0x%06lx, %d bpRGB, %d entr.\n",
                         n, visual->visualid, depth->depth,
                         visual->red_mask, visual->green_mask, visual->blue_mask,
                         visual->bits_per_rgb, visual->map_entries );
 
+               if (depth->depth != d)
+                    continue;
+
                switch (depth->depth) {
                     case 32:
                          if (visual->red_mask   == 0xff0000 &&
@@ -176,9 +229,11 @@ InitLocal( DFBX11 *x11, DFBX11Shared *shared, CoreDFB *core )
           XShmQueryVersion( x11->display, &x11->xshm_major, &x11->xshm_minor, &x11->use_shm );
 
 
-     x11->screen = dfb_screens_register( NULL, x11, &x11PrimaryScreenFuncs );
+     x11->screen = dfb_screens_register( NULL, x11, x11PrimaryScreenFuncs );
 
-     dfb_layers_register( x11->screen, x11, &x11PrimaryLayerFuncs );
+     dfb_layers_register( x11->screen, x11, x11PrimaryLayerFuncs );
+     dfb_layers_register( x11->screen, x11, x11PrimaryLayerFuncs );
+     dfb_layers_register( x11->screen, x11, x11PrimaryLayerFuncs );
 
      return DFB_OK;
 }
@@ -188,8 +243,11 @@ InitLocal( DFBX11 *x11, DFBX11Shared *shared, CoreDFB *core )
 static void
 system_get_info( CoreSystemInfo *info )
 {
-     info->type = CORE_X11;   
-     info->caps = CSCAPS_ACCELERATION;
+     info->type = CORE_X11;
+     info->caps = CSCAPS_ACCELERATION  | CSCAPS_PREFER_SHM | CSCAPS_SYSMEM_EXTERNAL |
+                  CSCAPS_DISPLAY_TASKS | CSCAPS_NOTIFY_DISPLAY;
+
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
 
      snprintf( info->name, DFB_CORE_SYSTEM_INFO_NAME_LENGTH, "X11" );
 }
@@ -201,6 +259,8 @@ system_initialize( CoreDFB *core, void **data )
      DFBX11       *x11;
      DFBX11Shared *shared;
 
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+
      x11 = D_CALLOC( 1, sizeof(DFBX11) );
      if (!x11)
           return D_OOM();
@@ -211,6 +271,10 @@ system_initialize( CoreDFB *core, void **data )
           return D_OOSHM();
      }
 
+     /* we need the error handler to signal the error to us, so
+        use a global static */
+     shared_for_error_handler = shared;
+     XSetErrorHandler( error_handler );
 
      /*
       * Local init (master and slave)
@@ -231,8 +295,6 @@ system_initialize( CoreDFB *core, void **data )
      shared->screen_size.w = x11->screenptr->width;
      shared->screen_size.h = x11->screenptr->height;
 
-     fusion_skirmish_init( &shared->lock, "X11 System", dfb_core_world(core) );
-
      fusion_call_init( &shared->call, call_handler, x11, dfb_core_world(core) );
 
 
@@ -247,14 +309,21 @@ system_initialize( CoreDFB *core, void **data )
       */
      dfb_surface_pool_initialize( core, &x11SurfacePoolFuncs, &shared->x11image_pool );
 
+#ifdef USE_GLX
+     dfb_surface_pool_initialize( core, &glxSurfacePoolFuncs, &shared->glx_pool );
+#endif
+
      if (dfb_config->video_length) {
           shared->vpsmem_length = dfb_config->video_length;
 
           dfb_surface_pool_initialize( core, &vpsmemSurfacePoolFuncs, &shared->vpsmem_pool );
      }
 
+#ifdef USE_GLX
+     dfb_surface_pool_bridge_initialize( core, &x11SurfacePoolBridgeFuncs, x11, &shared->x11_pool_bridge );
+#endif
 
-     fusion_arena_add_shared_field( dfb_core_arena( core ), "x11", shared );
+     core_arena_add_shared_field( core, "x11", shared );
 
      return DFB_OK;
 }
@@ -267,11 +336,13 @@ system_join( CoreDFB *core, void **data )
      DFBX11       *x11;
      DFBX11Shared *shared;
 
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+
      x11 = D_CALLOC( 1, sizeof(DFBX11) );
      if (!x11)
           return D_OOM();
 
-     fusion_arena_get_shared_field( dfb_core_arena( core ), "x11", &ptr );
+     core_arena_get_shared_field( core, "x11", &ptr );
      shared = ptr;
 
 
@@ -297,9 +368,19 @@ system_join( CoreDFB *core, void **data )
      if (shared->x11image_pool)
           dfb_surface_pool_join( core, shared->x11image_pool, &x11SurfacePoolFuncs );
 
+#ifdef USE_GLX
+     if (shared->glx_pool)
+          dfb_surface_pool_join( core, shared->glx_pool, &glxSurfacePoolFuncs );
+#endif
+
      if (shared->vpsmem_pool)
           dfb_surface_pool_join( core, shared->vpsmem_pool, &vpsmemSurfacePoolFuncs );
 
+#ifdef USE_GLX
+     if (shared->x11_pool_bridge)
+          dfb_surface_pool_bridge_join( core, shared->x11_pool_bridge, &x11SurfacePoolBridgeFuncs, x11 );
+#endif
+
      return DFB_OK;
 }
 
@@ -309,12 +390,22 @@ system_shutdown( bool emergency )
      DFBX11       *x11    = dfb_system_data();
      DFBX11Shared *shared = x11->shared;
 
+     int i;
+
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+
      /*
       * Master deinit
       */
+     if (shared->x11_pool_bridge)
+          dfb_surface_pool_bridge_destroy( shared->x11_pool_bridge );
+
      if (shared->vpsmem_pool)
           dfb_surface_pool_destroy( shared->vpsmem_pool );
 
+     if (shared->glx_pool)
+          dfb_surface_pool_destroy( shared->glx_pool );
+
      if (shared->x11image_pool)
           dfb_surface_pool_destroy( shared->x11image_pool );
 
@@ -324,12 +415,19 @@ system_shutdown( bool emergency )
       */
      fusion_call_destroy( &shared->call );
 
-     fusion_skirmish_prevail( &shared->lock );
+     /* close remaining windows */
+     for( i=0; i<dfb_layer_num(); i++ ) {
+          CoreLayer    *layer;
+          X11LayerData *lds;
 
-     if (shared->xw)
-         dfb_x11_close_window( x11, shared->xw );
-
-     fusion_skirmish_destroy( &shared->lock );
+          layer = dfb_layer_at( i );
+          lds   = layer->layer_data;
+          if( lds && lds->xw ) {
+              dfb_x11_close_window( x11, lds->xw );
+              lds->xw = 0;
+              shared->window_count--;
+          }
+     }
 
 
      SHFREE( dfb_core_shmpool( x11->core ), shared );
@@ -352,12 +450,20 @@ system_leave( bool emergency )
      DFBX11       *x11    = dfb_system_data();
      DFBX11Shared *shared = x11->shared;
 
+     D_DEBUG_AT( X11_Core, "%s()\n", __FUNCTION__ );
+
      /*
       * Slave deinit
       */
+     if (shared->x11_pool_bridge)
+          dfb_surface_pool_bridge_leave( shared->x11_pool_bridge );
+
      if (shared->vpsmem_pool)
           dfb_surface_pool_leave( shared->vpsmem_pool );
 
+     if (shared->glx_pool)
+          dfb_surface_pool_leave( shared->glx_pool );
+
      if (shared->x11image_pool)
           dfb_surface_pool_leave( shared->x11image_pool );
 
@@ -470,6 +576,25 @@ system_get_busid( int *ret_bus, int *ret_dev, int *ret_func )
 {
 }
 
+static int
+system_surface_data_size( void )
+{
+     /* Return zero because shared surface data is unneeded. */
+     return 0;
+}
+
+static void
+system_surface_data_init( CoreSurface *surface, void *data )
+{
+     /* Ignore since unneeded. */
+}
+
+static void
+system_surface_data_destroy( CoreSurface *surface, void *data )
+{
+     /* Ignore since unneeded. */
+}
+
 static void
 system_get_deviceid( unsigned int *ret_vendor_id,
                      unsigned int *ret_device_id )
@@ -492,7 +617,7 @@ call_handler( int           caller,
                break;
 
           case X11_DESTROY_WINDOW:
-               *ret_val = dfb_x11_destroy_window_handler( x11 );
+               *ret_val = dfb_x11_destroy_window_handler( x11, call_ptr );
                break;
 
           case X11_UPDATE_SCREEN:
