@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __X11SYSTEM__X11_H__
 #define __X11SYSTEM__X11_H__
 
@@ -40,33 +44,49 @@
 #include "xwindow.h"
 
 typedef struct {
-     CoreLayerRegionConfig  config;
+     CoreLayerRegionConfig   config;
+     XWindow               **xw;
 } SetModeData;
 
 typedef struct {
-     DFBRegion              region;
-
-     CoreSurfaceBufferLock *lock;
+     bool                   stereo;
+     DFBRegion              left_region;
+     DFBRegion              right_region;
+     CoreSurfaceBufferLock  left_lock;
+     CoreSurfaceBufferLock  right_lock;
+     XWindow               *xw;
 } UpdateScreenData;
 
+typedef struct {
+     XWindow               **xw;
+} DestroyData;
 
 typedef struct {
      UpdateScreenData     update;
      SetModeData          setmode;
+     DestroyData          destroy;
                          
-     FusionSkirmish       lock;
      FusionCall           call;
 
      FusionSHMPoolShared *data_shmpool;
  
      CoreSurfacePool     *x11image_pool;
 
+     CoreSurfacePool     *glx_pool; /* only used for GL */
+
      CoreSurfacePool     *vpsmem_pool;
      unsigned int         vpsmem_length;
+ 
+     CoreSurfacePoolBridge *x11_pool_bridge;
 
      DFBDimension         screen_size;
 
-     XWindow             *xw;
+     int                  window_count; /* merely for optimizing wait loop */
+
+     Bool                 x_error;
+
+     bool                 stereo;
+     int                  stereo_width;
 } DFBX11Shared;
 
 struct __DFB_X11 {
@@ -82,8 +102,10 @@ struct __DFB_X11 {
      Display             *display;
      Screen              *screenptr;
      int                  screennum;
-                     
+
      Visual              *visuals[DFB_NUM_PIXELFORMATS];
+     
+     void               (*Sync)( DFBX11 *x11 );
 };
 
 typedef enum {
@@ -97,8 +119,8 @@ typedef enum {
 
 
 
-DFBResult dfb_x11_create_window_handler ( DFBX11 *x11, CoreLayerRegionConfig *config );
-DFBResult dfb_x11_destroy_window_handler( DFBX11 *x11 );
+DFBResult dfb_x11_create_window_handler ( DFBX11 *x11, SetModeData *setmode );
+DFBResult dfb_x11_destroy_window_handler( DFBX11 *x11, DestroyData *destroy );
 
 DFBResult dfb_x11_update_screen_handler ( DFBX11 *x11, UpdateScreenData *data );
 DFBResult dfb_x11_set_palette_handler   ( DFBX11 *x11, CorePalette *palette );
