@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #ifndef __X11SYSTEM__X11IMAGE_H__
 #define __X11SYSTEM__X11IMAGE_H__
 
@@ -72,5 +76,8 @@ DFBResult x11ImageDestroy( DFBX11                *x11,
 DFBResult x11ImageAttach ( x11Image              *image,
                            void                 **ret_addr );
 
+DFBResult x11ImageDetach ( x11Image              *image,
+                           void                  *addr );
+
 #endif /* __X11SYSTEM__X11IMAGE_H__ */
 
