@@ -1,17 +1,17 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
 
-   All rights reserved.
-
    This file is subject to the terms and conditions of the MIT License:
 
    Permission is hereby granted, free of charge, to any person
@@ -55,7 +55,7 @@
 #if DIRECT_BUILD_DEBUGS
 #define MSG(x...)                       \
 do {                                    \
-     direct_debug( x ); \
+     direct_log_printf( NULL, x );      \
 } while (0)
 #else
 #define MSG(x...)                       \
@@ -106,7 +106,7 @@ main( int argc, char *argv[] )
      MSG( "Waiting at skirmish...\n" );
 
      ret = fusion_skirmish_wait( &skirmish, 10 );
-     if (ret != DFB_TIMEOUT) {
+     if (ret != DR_TIMEOUT) {
           D_DERROR( ret, "fusion_skirmish_wait() did not timeout!\n" );
           return -3;
      }
