@@ -94,18 +94,23 @@ Inspector_Run( Inspector *inspector )
           DSPF_ARGB4444,
           DSPF_ARGB8565,
           DSPF_ARGB,
+          DSPF_ABGR,
           DSPF_AiRGB,
           DSPF_A1,
+          DSPF_A1_LSB,
           DSPF_A8,
           DSPF_YUY2,
           DSPF_UYVY,
           DSPF_I420,
           DSPF_YV12,
+          DSPF_YV16,
           DSPF_NV12,
           DSPF_NV21,
           DSPF_NV16,
           DSPF_AYUV,
           DSPF_YUV444P,
+          DSPF_AVYU,
+          DSPF_VYU,
      };
 
      int                    i, j, n;
