@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -49,6 +51,7 @@ static const DirectFBScreenEncoderScanModeNames( scan_mode_names );
 static const DirectFBScreenEncoderTestPictureNames( test_picture_names );
 static const DirectFBScreenEncoderTVStandardsNames( tv_standard_names );
 static const DirectFBScreenEncoderFrequencyNames( frequency_names );
+static const DirectFBScreenEncoderPictureFramingNames( framing_names );
 static const DirectFBScreenOutputSignalsNames( signal_names );
 static const DirectFBScreenOutputResolutionNames( resolution_names );
 
@@ -304,6 +307,10 @@ static const AnyOption options[] = {
        &encoder_config.frequency, &encoder_config.flags,
        DSECONF_FREQUENCY, parse_enum, frequency_names },
 
+     { "-ea", "--framing",    "<framing>", "Set picture framing",
+       &encoder_config.framing, &encoder_config.flags,
+       DSECONF_FRAMING, parse_enum, framing_names },
+
 
      { "-o",  "--output",       "<index>",    "Index of output to use",
        &output,  NULL, 0, parse_int, NULL }
@@ -593,8 +600,8 @@ dump_mixer_config( const DFBScreenMixerConfig *config )
      }
 
      if (config->flags & DSMCONF_BACKGROUND)
-          printf( "Background:    0x%02x, 0x%02x, 0x%02x (RGB)\n",
-                  config->background.r, config->background.g, config->background.b );
+          printf( "Background:    0x%02x, 0x%02x, 0x%02x, 0x%02x (ARGB)\n",
+                  config->background.a, config->background.r, config->background.g, config->background.b );
 
      printf( "\n" );
 }
@@ -628,7 +635,7 @@ resolution_name( DFBScreenOutputResolution resolution )
 }
 
 static const char *
-scan_mode_name( DFBScreenEncoderTestPicture scan_mode )
+scan_mode_name( DFBScreenEncoderScanMode scan_mode )
 {
      int i;
 
@@ -653,6 +660,19 @@ frequency_name( DFBScreenEncoderFrequency frequency )
      return "<invalid>";
 }
 
+static const char *
+framing_name( DFBScreenEncoderPictureFraming framing )
+{
+     int i;
+
+     for (i=0; i<D_ARRAY_SIZE(framing_names); i++) {
+          if (framing_names[i].framing == framing)
+               return framing_names[i].name;
+     }
+
+     return "<invalid>";
+}
+
 static void
 dump_encoder_config( const DFBScreenEncoderConfig *config )
 {
@@ -697,6 +717,9 @@ dump_encoder_config( const DFBScreenEncoderConfig *config )
      if (config->flags & DSECONF_FREQUENCY)
           printf( "Frequency:     %s\n", frequency_name( config->frequency ) );
 
+     if (config->flags & DSECONF_FRAMING)
+          printf( "Framing:       %s\n", framing_name( config->framing ) );
+
      printf( "\n" );
 }
 
