@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -78,6 +80,7 @@ static DFBResult  load_image      (const char             *filename,
                                    DFBSurfaceDescription  *desc,
                                    DFBColor               *palette,
                                    int                    *palette_size,
+                                   const DFBColor         *transparent,
                                    DFBSurfacePixelFormat   rgbformat,
                                    bool                    dither565);
 static void       dither_rgb16    (const u32              *src,
@@ -118,6 +121,8 @@ int main (int         argc,
      int         i, n;
      bool        rawmode        = 0;
      bool        dither565      = 0;
+     DFBColor   *transparent    = NULL;
+     int         res;
 
      /*  parse command line  */
 
@@ -162,9 +167,24 @@ int main (int         argc,
                     if (*name)
                          continue;
                }
+               if (strncmp (arg, "transparent=", 12) == 0 && !transparent) {
+                    char *error;
+                    u32   argb = strtoul (arg + 12, &error, 16);
 
-               print_usage (argv[0]);
-               return EXIT_FAILURE;
+                    if (! error) {
+                         transparent = alloca(sizeof (DFBColor));
+
+                         transparent->b = argb & 0xFF;
+                         argb >>= 8;
+                         transparent->g = argb & 0xFF;
+                         argb >>= 8;
+                         transparent->r = argb & 0xFF;
+                         argb >>= 8;
+                         transparent->a = argb & 0xFF;
+                    }
+                    if (transparent)
+                         continue;
+               }
           }
 
           filename[num_images++] = argv[n];
@@ -203,7 +223,8 @@ int main (int         argc,
                f = fopen(filename[0], "r");
                if (f)
                {
-                    fread(data, statbuf.st_size, 1, f);
+                    res = fread(data, statbuf.st_size, 1, f);
+                    (void)res;
                     fclose(f);
                }
 
@@ -219,7 +240,7 @@ int main (int         argc,
           }
 
           if (load_image (filename[0],
-                          &desc, palette, &palette_size,
+                          &desc, palette, &palette_size, transparent,
                           rgbformat, dither565) != DFB_OK)
                return EXIT_FAILURE;
 
@@ -244,8 +265,8 @@ int main (int         argc,
                image[i].pixelformat = desc.pixelformat;
 
                if (load_image (filename[i],
-                               image + i, foo, &foo_size, rgbformat,
-                               dither565) != DFB_OK)
+                               image + i, foo, &foo_size, transparent,
+                               rgbformat, dither565) != DFB_OK)
                     return EXIT_FAILURE;
           }
 
@@ -282,6 +303,7 @@ static DFBResult load_image (const char            *filename,
                              DFBSurfaceDescription *desc,
                              DFBColor              *palette,
                              int                   *palette_size,
+                             const DFBColor        *transparent,
                              DFBSurfacePixelFormat  rgbformat,
                              bool                   dither565)
 {
@@ -320,7 +342,7 @@ static DFBResult load_image (const char            *filename,
      if (!png_ptr)
           goto cleanup;
 
-     if (setjmp (png_ptr->jmpbuf)) {
+     if (setjmp (png_jmpbuf(png_ptr))) {
           if (desc->preallocated[0].data) {
                free (desc->preallocated[0].data);
                desc->preallocated[0].data = NULL;
@@ -388,16 +410,22 @@ static DFBResult load_image (const char            *filename,
 
      switch (src_format) {
           case DSPF_LUT8:
-               if (info_ptr->num_palette) {
+          {
+               png_colorp info_palette;
+               int num_palette;
+
+               png_get_PLTE(png_ptr,info_ptr,&info_palette,&num_palette);
+
+               if (num_palette) {
                     png_byte *alpha;
                     int       i, num;
 
-                    *palette_size = MIN (info_ptr->num_palette, 256);
+                    *palette_size = MIN (num_palette, 256);
                     for (i = 0; i < *palette_size; i++) {
                          palette[i].a = 0xFF;
-                         palette[i].r = info_ptr->palette[i].red;
-                         palette[i].g = info_ptr->palette[i].green;
-                         palette[i].b = info_ptr->palette[i].blue;
+                         palette[i].r = info_palette[i].red;
+                         palette[i].g = info_palette[i].green;
+                         palette[i].b = info_palette[i].blue;
                     }
                     if (png_get_valid (png_ptr, info_ptr, PNG_INFO_tRNS)) {
                          png_get_tRNS (png_ptr, info_ptr, &alpha, &num, NULL);
@@ -406,6 +434,7 @@ static DFBResult load_image (const char            *filename,
                     }
                }
                break;
+          }
           case DSPF_RGB32:
                 png_set_filler (png_ptr, 0xFF,
 #ifdef WORDS_BIGENDIAN
@@ -430,7 +459,7 @@ static DFBResult load_image (const char            *filename,
 
      data  = malloc (height * pitch);
      if (!data) {
-          fprintf (stderr, "Failed to allocate %ld bytes.\n", height * pitch);
+          fprintf (stderr, "Failed to allocate %lu bytes.\n", (unsigned long)(height * pitch));
           goto cleanup;
      }
 
@@ -447,6 +476,22 @@ static DFBResult load_image (const char            *filename,
      if (!dest_format)
           dest_format = src_format;
 
+     /*  replace color in completely transparent pixels  */
+     if (transparent && DFB_PIXELFORMAT_HAS_ALPHA(src_format)) {
+          unsigned char *row;
+          int            h;
+
+          for (row = data, h = height; h; h--, row += pitch) {
+                 u32 *pixel;
+                 int  w;
+
+                 for (pixel = (u32 *) row, w = width; w; w--, pixel++) {
+                      if ((*pixel & 0xff000000) == 0x0)
+                           *pixel = dfb_color_to_argb (transparent);
+                 }
+          }
+     }
+
      if (DFB_BYTES_PER_PIXEL(src_format) != DFB_BYTES_PER_PIXEL(dest_format)) {
           unsigned char *s, *d, *dest;
           int            d_pitch, h;
@@ -459,8 +504,7 @@ static DFBResult load_image (const char            *filename,
 
           dest = malloc (height * d_pitch);
           if (!dest) {
-               fprintf (stderr, "Failed to allocate %ld bytes.\n",
-                        height * d_pitch);
+               fprintf (stderr, "Failed to allocate %lu bytes.\n", (unsigned long)(height * d_pitch));
                goto cleanup;
           }
 
@@ -619,7 +663,7 @@ static DFBResult merge_images (DFBSurfaceDescription *images,
 static void
 dither_rgb16 (const u32 *src, u16 *dest, int y, int width)
 {
-     const u32 *dm = DM_565 + ((y & (DM_HEIGHT - 1)) << DM_WIDTH_SHIFT);
+     const u32 *dm = DM_565 + ((y & (DM_565_HEIGHT - 1)) << DM_565_WIDTH_SHIFT);
      int        x;
 
      for (x = 0; x < width; x++) {
@@ -627,7 +671,7 @@ dither_rgb16 (const u32 *src, u16 *dest, int y, int width)
                      (src[x] & 0xFF00)   << 2 |
                      (src[x] & 0xFF0000) << 4);
 
-          rgb += dm[x & (DM_WIDTH - 1)];
+          rgb += dm[x & (DM_565_WIDTH - 1)];
           rgb += (0x10040100
                   - ((rgb & 0x1e0001e0) >> 5)
                   - ((rgb & 0x00070000) >> 6));
@@ -842,7 +886,7 @@ static DFBResult dump_rectangles (const char    *name,
           if (len < 0) {
                int l = fprintf (fp, "  { \"%s\", ", v);
 
-               fprintf (fp, blanks - len + l);
+               fprintf (fp, "%s", blanks - len + l);
                fprintf (fp, "{ x : %4d, y : %4d, w : %4d, h : %4d } }",
                         rect->x, rect->y, rect->w, rect->h);
           }
