@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -335,8 +337,16 @@ mutex_lock_unlock_loop( void *arg )
 static void
 bench_mutex_threaded( void )
 {
-     int             i;
-     pthread_mutex_t lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+     int                 i;
+     pthread_mutex_t     lock;
+     pthread_mutexattr_t attr;
+
+     pthread_mutexattr_init( &attr );
+     pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
+
+     pthread_mutex_init( &lock, &attr );
+
+     pthread_mutexattr_destroy( &attr );
 
 
      /* mutex lock/unlock (2-5 threads) */
@@ -366,8 +376,16 @@ bench_mutex_threaded( void )
 static void
 bench_mutex( void )
 {
-     pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-     pthread_mutex_t rmutex = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+     pthread_mutex_t     mutex = PTHREAD_MUTEX_INITIALIZER;
+     pthread_mutex_t     rmutex;
+     pthread_mutexattr_t attr;
+
+     pthread_mutexattr_init( &attr );
+     pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
+
+     pthread_mutex_init( &rmutex, &attr );
+
+     pthread_mutexattr_destroy( &attr );
 
 
      /* pthread_mutex lock/unlock */
