@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -39,12 +41,13 @@
 
 int main( void )
 {
+     int res;
      unsigned char  byt;
      unsigned short wrd;
 
      do {
-          fread (&wrd, 2, 1, stdin);
-
+          res = fread (&wrd, 2, 1, stdin);
+          (void)res;
 #ifdef WORDS_BIGENDIAN
           swab (&wrd, &wrd, 2);
 #endif
