@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2009  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <unistd.h>
@@ -847,6 +851,22 @@ wm_begin_updates( CoreWindow      *window,
      return DFB_OK;
 }
 
+static DFBResult
+wm_set_cursor_position( CoreWindow      *window,
+                        void            *wm_data,
+                        void            *window_data,
+                        int              x,
+                        int              y )
+{
+     D_ASSERT( window != NULL );
+     D_ASSERT( wm_data != NULL );
+     D_ASSERT( window_data != NULL );
+
+     D_UNIMPLEMENTED();
+
+     return DFB_UNIMPLEMENTED;
+}
+
 /**************************************************************************************************/
 
 static DFBResult
@@ -876,7 +896,8 @@ static DFBResult
 wm_update_window( CoreWindow          *window,
                   void                *wm_data,
                   void                *window_data,
-                  const DFBRegion     *region,
+                  const DFBRegion     *left_region,
+                  const DFBRegion     *right_region,
                   DFBSurfaceFlipFlags  flags )
 {
      WindowData *data = window_data;
@@ -885,14 +906,14 @@ wm_update_window( CoreWindow          *window,
      D_ASSERT( wm_data != NULL );
      D_ASSERT( window_data != NULL );
 
-     DFB_REGION_ASSERT_IF( region );
+     DFB_REGION_ASSERT_IF( left_region );
 
      D_MAGIC_ASSERT( data, WindowData );
 
      if (!data->window)
           return DFB_DESTROYED;
 
-     return unique_window_update( data->window, region, flags );
+     return unique_window_update( data->window, left_region, flags );
 }
 
 /**************************************************************************************************/
@@ -950,7 +971,8 @@ wm_update_cursor( CoreWindowStack       *stack,
 
           /* Create the cursor backing store surface. */
           ret = dfb_surface_create_simple( wmdata->core, stack->cursor.size.w, stack->cursor.size.h,
-                                           region->config.format, caps, CSTF_SHARED | CSTF_CURSOR,
+                                           region->config.format, region->config.colorspace, 
+                                           caps, CSTF_SHARED | CSTF_CURSOR,
                                            0, /* FIXME: no shared cursor objects, no cursor id */
                                            NULL, &cursor_bs );
           if (ret) {
