@@ -1,11 +1,13 @@
 /*
-   (c) Copyright 2001-2008  The world wide DirectFB Open Source Community (directfb.org)
+   (c) Copyright 2012-2013  DirectFB integrated media GmbH
+   (c) Copyright 2001-2013  The world wide DirectFB Open Source Community (directfb.org)
    (c) Copyright 2000-2004  Convergence (integrated media) GmbH
 
    All rights reserved.
 
    Written by Denis Oliver Kropp <dok@directfb.org>,
-              Andreas Hundt <andi@fischlustig.de>,
+              Andreas Shimokawa <andi@directfb.org>,
+              Marek Pikarski <mass@directfb.org>,
               Sven Neumann <neo@directfb.org>,
               Ville Syrjälä <syrjala@sci.fi> and
               Claudio Ciccani <klan@users.sf.net>.
@@ -26,6 +28,8 @@
    Boston, MA 02111-1307, USA.
 */
 
+
+
 #include <config.h>
 
 #include <directfb.h>
@@ -738,6 +742,7 @@ dispatch_key( UniqueWindow           *window,
      D_ASSERT( event != NULL );
 
      evt.type       = event->keyboard.press ? DWET_KEYDOWN : DWET_KEYUP;
+     evt.flags      = (event->keyboard.flags & UIKF_REPEAT) ? DWEF_REPEAT : 0;
      evt.key_code   = event->keyboard.key_code;
      evt.key_id     = event->keyboard.key_id;
      evt.key_symbol = event->keyboard.key_symbol;
